@@ -53,12 +53,17 @@ import org.apache.commons.logging.Log;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientBound2ClipBoard;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientHTTP;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientJMS;
+import com.openexchange.office.calcengine.client.impl.CalcEngineClientJMSLBEmbedded;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientLocal;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientMonitoring;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientNative;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientPerformanceMeasurement;
+import com.openexchange.office.calcengine.client.impl.CalcEngineClientPerformanceMeasurement2;
+import com.openexchange.office.calcengine.client.impl.CalcEngineClientSimulator;
 import com.openexchange.office.calcengine.client.impl.CalcEngineErrorSimulator;
+import com.openexchange.office.calcengine.client.impl.CalcEngineRecorder;
 import com.openexchange.office.tools.logging.LogFactory;
+import com.openxchange.office_communication.configuration.configitems.ELoadBalancerType;
 
 //=============================================================================
 /** factory where you can retrieve instances of type ICalcEngine from.
@@ -159,8 +164,10 @@ public class CalcEngineClientFactory
         throws Exception
     {
         LOG.info("create calc engine client : NATIVE ...");
-        final CalcEngineClientNative aClient = CalcEngineClientNative.create ();
-        return aClient;
+              ICalcEngineClient iClient = null;
+                                iClient = CalcEngineClientNative.create ();
+    	                      //iClient = CalcEngineClientPerformanceMeasurement2.create(iClient);
+        return iClient;
     }
 
     //-------------------------------------------------------------------------
@@ -168,17 +175,28 @@ public class CalcEngineClientFactory
         throws Exception
     {
         LOG.info("create calc engine client : INLINE ...");
+		final CalcEngineConfig                 aCEConfig          = CalcEngineConfig               .get    ();
         final CalcEngineClientNative           aClient            = CalcEngineClientNative         .create (                 );
         final CalcEngineClientBound2ClipBoard  aClipBoardWrapper  = CalcEngineClientBound2ClipBoard.create (aClient          );
         final CalcEngineClientMonitoring       aMonitoringWrapper = CalcEngineClientMonitoring     .create (aClipBoardWrapper);
+              ICalcEngineClient                iClient            = null;
         
+        iClient = aMonitoringWrapper;
+              
         if (CalcEngineConfig.get ().isPerformanceMeasurementOn())
         {
-        	final CalcEngineClientPerformanceMeasurement aRecorder = CalcEngineClientPerformanceMeasurement.create(aMonitoringWrapper);
-        	return aRecorder;
+        	final CalcEngineClientPerformanceMeasurement2 aMeasure = CalcEngineClientPerformanceMeasurement2.create(aMonitoringWrapper);
+        	iClient = aMeasure;
+        }
+
+        if (aCEConfig.isRecordingOn())
+        {
+        	final String             sRecordingPath = aCEConfig.getRecordingPath();
+        	final CalcEngineRecorder aRecorder      = CalcEngineRecorder.create(iClient, sRecordingPath);
+        	iClient = aRecorder;
         }
         
-        return aMonitoringWrapper;
+        return iClient;
     }
 
     //-------------------------------------------------------------------------
@@ -186,18 +204,36 @@ public class CalcEngineClientFactory
         throws Exception
     {
         LOG.info("create calc engine client : LOCAL ...");
+		final CalcEngineConfig                aCEConfig          = CalcEngineConfig               .get    ();
         final CalcEngineClientLocal           aClient            = CalcEngineClientLocal          .create (                 );
         final CalcEngineClientBound2ClipBoard aClipBoardWrapper  = CalcEngineClientBound2ClipBoard.create (aClient          );
         final CalcEngineClientMonitoring      aMonitoringWrapper = CalcEngineClientMonitoring     .create (aClipBoardWrapper);
+              ICalcEngineClient               iClient            = null;
+
         aClient.describeEnvironment(aDesc);
+
+        iClient = aMonitoringWrapper;
         
-        if (CalcEngineConfig.get ().isPerformanceMeasurementOn())
+        if (aCEConfig.isSimulatorOn())
         {
-        	final CalcEngineClientPerformanceMeasurement aRecorder = CalcEngineClientPerformanceMeasurement.create(aMonitoringWrapper);
-        	return aRecorder;
+        	final CalcEngineErrorSimulator aSimulator = CalcEngineErrorSimulator.create(iClient);
+        	iClient = aSimulator;
         }
         
-        return aMonitoringWrapper;
+        if (aCEConfig.isPerformanceMeasurementOn())
+        {
+        	final CalcEngineClientPerformanceMeasurement2 aMeasure = CalcEngineClientPerformanceMeasurement2.create(iClient);
+        	iClient = aMeasure;
+        }
+        
+        if (aCEConfig.isRecordingOn())
+        {
+        	final String             sRecordingPath = aCEConfig.getRecordingPath();
+        	final CalcEngineRecorder aRecorder      = CalcEngineRecorder.create(iClient, sRecordingPath);
+        	iClient = aRecorder;
+        }
+
+        return iClient;
     }
 
     //-------------------------------------------------------------------------
@@ -231,7 +267,7 @@ public class CalcEngineClientFactory
         
         if (CalcEngineConfig.get ().isPerformanceMeasurementOn())
         {
-        	final CalcEngineClientPerformanceMeasurement aRecorder = CalcEngineClientPerformanceMeasurement.create(aClipBoardWrapper);
+        	final CalcEngineClientPerformanceMeasurement2 aRecorder = CalcEngineClientPerformanceMeasurement2.create(aClipBoardWrapper);
         	return aRecorder;
         }
         
@@ -264,8 +300,7 @@ public class CalcEngineClientFactory
         throws Exception
     {
         LOG.info("create calc engine client : SIMULATOR ...");
-        //final CalcEngineClientSimulator aClient = CalcEngineClientSimulator.create ();
-        final CalcEngineErrorSimulator aClient = CalcEngineErrorSimulator.create ();
+        final CalcEngineClientSimulator aClient = CalcEngineClientSimulator.create ();
         return aClient;
     }
 
@@ -294,19 +329,40 @@ public class CalcEngineClientFactory
         throws Exception
     {
 		LOG.info("create calc engine client : JMS-CLIENT ...");
-		final com.openxchange.office_communication.configuration.configitems.CalcEngineConfig aCEConfig = com.openxchange.office_communication.configuration.configitems.CalcEngineConfig.access();
-		final CalcEngineClientJMS             aJMSClient        = CalcEngineClientJMS.create (aCEConfig.getInQueue());
-        final CalcEngineClientBound2ClipBoard aClipBoardWrapper = CalcEngineClientBound2ClipBoard.create (aJMSClient);
-        final CalcEngineClientMonitoring      aMonitoringWrapper = CalcEngineClientMonitoring    .create (aClipBoardWrapper);
-        aJMSClient.start ();
+		final CalcEngineConfig aCEConfig = CalcEngineConfig.get ();
+		
+		final ELoadBalancerType eLBType = aCEConfig.getLoadBalancerType ();
+		      ICalcEngineClient iClient = null;
+
+		if (eLBType == ELoadBalancerType.E_CLIENT_EMBEDDED)
+		{
+			final CalcEngineClientJMSLBEmbedded aJMSClient = CalcEngineClientJMSLBEmbedded.create ();
+	        aJMSClient.start ();
+	        iClient = aJMSClient;
+		}
+		else
+		{
+			final CalcEngineClientJMS aJMSClient = CalcEngineClientJMS.create (aCEConfig.getInQueue());
+	        aJMSClient.start ();
+	        iClient = aJMSClient;
+		}
+		
+        if (aCEConfig.isSimulatorOn())
+        	iClient = CalcEngineErrorSimulator.create(iClient);
+
+        iClient = CalcEngineClientBound2ClipBoard.create (iClient);
+		iClient = CalcEngineClientMonitoring     .create (iClient);
         
-        if (CalcEngineConfig.get ().isPerformanceMeasurementOn())
+        if (aCEConfig.isPerformanceMeasurementOn())
+        	iClient = CalcEngineClientPerformanceMeasurement2.create(iClient);
+
+        if (aCEConfig.isRecordingOn())
         {
-        	final CalcEngineClientPerformanceMeasurement aRecorder = CalcEngineClientPerformanceMeasurement.create(aMonitoringWrapper);
-        	return aRecorder;
+        	final String sRecordingPath = aCEConfig.getRecordingPath();
+        	             iClient        = CalcEngineRecorder.create(iClient, sRecordingPath);
         }
         
-		return aMonitoringWrapper;
+		return iClient;
     }
 
     //-------------------------------------------------------------------------
