@@ -49,17 +49,18 @@
 package com.openexchange.office.calcengine.client.impl;
 
 import java.util.Map;
+import java.util.concurrent.TimeoutException;
 
 import org.apache.commons.lang.ObjectUtils;
 import org.apache.commons.lang.StringUtils;
 
 import com.openexchange.office.calcengine.client.CalcEngineClipBoardEvent;
 import com.openexchange.office.calcengine.client.CalcEngineConfig;
-import com.openexchange.office.calcengine.client.CalcEngineHandleGenerator;
 import com.openexchange.office.calcengine.client.ECalcEngineError;
 import com.openexchange.office.calcengine.client.ICalcEngineClient;
 import com.openxchange.office_communication.jms.core.plain.EErrorCode;
 import com.openxchange.office_communication.jms.core.plain.JMSClientBase;
+import com.openxchange.office_communication.jms.core.plain.JMSRuntimeStats;
 import com.openxchange.office_communication.office.jms.messages.CalcEngineMessageBean;
 import com.openxchange.office_communication.office.jms.messages.ECalcEngineMethod;
 import com.openxchange.office_communication.office.jms.messages.ECalcEngineResult;
@@ -88,12 +89,22 @@ public class CalcEngineClientJMS extends    JMSClientBase
 	public synchronized static CalcEngineClientJMS create(final String sOutQueue)
 	    throws Exception
 	{
+		System.err.println ("#### DBG : new instance : CalcEngineClientJMS");
 		final CalcEngineConfig    aConfig  = CalcEngineConfig.get();
 		final int                 nTimeout = aConfig.getRequestTimeout();
 		final CalcEngineClientJMS aInst    = new CalcEngineClientJMS  ();
+		final long                nTTL     = 10 * 60 * 1000; // 10min
 		
 	  	aInst.setFullQualifiedRequestQueue(sOutQueue);
-	  	aInst.setTimeout                  (nTimeout );
+//	  	aInst.setTimeout                  (nTimeout );
+//	  	aInst.setMessageTTL               (nTTL     );
+
+	  	if (aConfig.isJmsStatisticsEnabled())
+	  	{
+			System.err.println ("#### DBG : enable JMSRuntimeStats ...");
+	  		JMSRuntimeStats.setEnabled             (true );
+			JMSRuntimeStats.setAutoDumpIntervalInMS(10000);
+	  	}
 
 	  	return aInst;
 	}
@@ -150,7 +161,7 @@ public class CalcEngineClientJMS extends    JMSClientBase
 
 	//-------------------------------------------------------------------------
 	@Override
-	public /*no synchronize*/ String createDocument()
+	public /*no synchronize*/ void createDocument(final String sDocHandle)
 		throws Exception
 	{
 		final LogContext aLog = impl_newLogContext4Method (ECalcEngineMethod.E_CREATE_DOCUMENT);
@@ -161,7 +172,6 @@ public class CalcEngineClientJMS extends    JMSClientBase
 		
     	//-----
     	
-		final String                sDocHandle = CalcEngineHandleGenerator.newHandle();
 		final CalcEngineMessageBean aRequest   = new CalcEngineMessageBean();
     	aRequest.setMethod   (ECalcEngineMethod.E_CREATE_DOCUMENT);
     	aRequest.setDocHandle(sDocHandle);
@@ -181,21 +191,24 @@ public class CalcEngineClientJMS extends    JMSClientBase
 
     	//-----
 
-    	final CalcEngineMessageBean aResponse = super.requestSynchronous(aRequest);
-    	final EErrorCode            eResult   = aResponse.getErrorCode();
+    	EErrorCode            eResult   = null;
+    	CalcEngineMessageBean aResponse = null;
+    	try
+    	{
+	    	aResponse = super.requestSynchronous(aRequest);
+	    	eResult   = aResponse.getErrorCode();
+    	}
+    	catch (final TimeoutException exTimeout)
+    	{
+    		eResult = EErrorCode.E_TIMEOUT;
+    	}
 
     	//-----
 
     	aLog.forLevel   (ELogLevel.E_INFO)
 		    .withMessage("got response"  )
-		    .withVars   (new LogVars()
-			{
-				public void defineVars(final Map<String, Object> lVars) throws Exception
-				{
-					lVars.put (LogConst.DOC_HANDLE, sDocHandle);
-					lVars.put (LogConst.ERROR_CODE, eResult   );
-				}
-			})
+		    .setVar     (LogConst.DOC_HANDLE, sDocHandle)
+		    .setVar     (LogConst.ERROR_CODE, eResult   )
 		    .log();
     	
     	//-----
@@ -212,10 +225,7 @@ public class CalcEngineClientJMS extends    JMSClientBase
     	aLog.forLevel   (ELogLevel.E_INFO)
 		    .withMessage(LogConst.END    )
 		    .log();
-    	
-    	return sDocHandle;
 	}
-
 	
 	//-------------------------------------------------------------------------
 	@Override
@@ -247,23 +257,26 @@ public class CalcEngineClientJMS extends    JMSClientBase
 			})
 		    .log();
 	
-		//-----
+    	//-----
 
-    	final CalcEngineMessageBean aResponse = super.requestSynchronous(aRequest);
-    	final EErrorCode            eResult   = aResponse.getErrorCode();
+    	EErrorCode            eResult   = null;
+    	CalcEngineMessageBean aResponse = null;
+    	try
+    	{
+	    	aResponse = super.requestSynchronous(aRequest);
+	    	eResult   = aResponse.getErrorCode();
+    	}
+    	catch (final TimeoutException exTimeout)
+    	{
+    		eResult = EErrorCode.E_TIMEOUT;
+    	}
 
-		//-----
+    	//-----
 
     	aLog.forLevel   (ELogLevel.E_INFO)
 		    .withMessage("got response"  )
-		    .withVars   (new LogVars()
-			{
-				public void defineVars(final Map<String, Object> lVars) throws Exception
-				{
-					lVars.put (LogConst.DOC_HANDLE, sDocHandle);
-					lVars.put (LogConst.ERROR_CODE, eResult   );
-				}
-			})
+		    .setVar     (LogConst.DOC_HANDLE, sDocHandle)
+		    .setVar     (LogConst.ERROR_CODE, eResult   )
 		    .log();
 		
 		//-----
@@ -313,15 +326,25 @@ public class CalcEngineClientJMS extends    JMSClientBase
 
     	//-----
 
-    	final CalcEngineMessageBean aResponse = super.requestSynchronous(aRequest);
-    	final EErrorCode            eResult   = aResponse.getErrorCode();
+    	EErrorCode            eResult   = null;
+    	CalcEngineMessageBean aResponse = null;
+    	try
+    	{
+	    	aResponse = super.requestSynchronous(aRequest);
+	    	eResult   = aResponse.getErrorCode();
+    	}
+    	catch (final TimeoutException exTimeout)
+    	{
+    		eResult = EErrorCode.E_TIMEOUT;
+    	}
 
-		//-----
+    	//-----
 
-    	aLog.forLevel   (ELogLevel.E_INFO            )
-		    .withMessage("got response"              )
-		    .setVar     (LogConst.ERROR_CODE, eResult)
-		    .log        ();
+    	aLog.forLevel   (ELogLevel.E_INFO)
+		    .withMessage("got response"  )
+		    .setVar     (LogConst.DOC_HANDLE, sDocHandle)
+		    .setVar     (LogConst.ERROR_CODE, eResult   )
+		    .log();
 
     	//-----
     	
@@ -425,17 +448,27 @@ public class CalcEngineClientJMS extends    JMSClientBase
 			.setVar     (LogConst.CONTENT , sCopyJSON)
 			.log        ();
 		
-		//-----
-		
-		final CalcEngineMessageBean aResponse = super.requestSynchronous(aRequest);
-		final EErrorCode            eResult   = aResponse.getErrorCode();
-		
-		//-----
-		
-		aLog.forLevel   (ELogLevel.E_INFO            )
-			.withMessage("got response"              )
-			.setVar     (LogConst.ERROR_CODE, eResult)
-			.log        ();
+    	//-----
+
+    	EErrorCode            eResult   = null;
+    	CalcEngineMessageBean aResponse = null;
+    	try
+    	{
+	    	aResponse = super.requestSynchronous(aRequest);
+	    	eResult   = aResponse.getErrorCode();
+    	}
+    	catch (final TimeoutException exTimeout)
+    	{
+    		eResult = EErrorCode.E_TIMEOUT;
+    	}
+
+    	//-----
+
+    	aLog.forLevel   (ELogLevel.E_INFO)
+		    .withMessage("got response"  )
+		    .setVar     (LogConst.DOC_HANDLE, sDocHandle)
+		    .setVar     (LogConst.ERROR_CODE, eResult   )
+		    .log();
 		
 		//-----
 		
@@ -530,17 +563,27 @@ public class CalcEngineClientJMS extends    JMSClientBase
 			.setVar     (LogConst.CONTENT , sPasteJSON)
 			.log        ();
 
-		//-----
-		
-		final CalcEngineMessageBean aResponse = super.requestSynchronous(aRequest);
-		final EErrorCode            eResult   = aResponse.getErrorCode();
+    	//-----
 
-		//-----
-		
-		aLog.forLevel   (ELogLevel.E_INFO            )
-			.withMessage("got response"              )
-			.setVar     (LogConst.ERROR_CODE, eResult)
-			.log        ();
+    	EErrorCode            eResult   = null;
+    	CalcEngineMessageBean aResponse = null;
+    	try
+    	{
+	    	aResponse = super.requestSynchronous(aRequest);
+	    	eResult   = aResponse.getErrorCode();
+    	}
+    	catch (final TimeoutException exTimeout)
+    	{
+    		eResult = EErrorCode.E_TIMEOUT;
+    	}
+
+    	//-----
+
+    	aLog.forLevel   (ELogLevel.E_INFO)
+		    .withMessage("got response"  )
+		    .setVar     (LogConst.DOC_HANDLE, sDocHandle)
+		    .setVar     (LogConst.ERROR_CODE, eResult   )
+		    .log();
 
 		//-----
 		
@@ -611,7 +654,8 @@ public class CalcEngineClientJMS extends    JMSClientBase
 	public /*no synchronize*/ String getVersion()
 		throws Exception
 	{
-		throw new UnsupportedOperationException("not implemented yet");
+		//throw new UnsupportedOperationException("not implemented yet");
+		return "JMS Client 1.0.0";
 	}
 
 	//-------------------------------------------------------------------------
