@@ -87,12 +87,9 @@ public class CalcEngineClientMonitoring implements ICalcEngineClient {
      * @see com.openexchange.office.calcengine.client.ICalcEngineClient#createDocument()
      */
     @Override
-    public String createDocument() throws Exception {
-        final String ret = m_iWrapped.createDocument();
-
+    public void createDocument(final String sDocHandle) throws Exception {
+        m_iWrapped.createDocument(sDocHandle);
         Statistics.handleDocumentEvent(new CalcEngineEvent(CalcEngineEventType.DOC_INIT));
-
-        return ret;
     };
 
     /* (non-Javadoc)
