@@ -85,14 +85,13 @@ public class CalcEngineClientPerformanceMeasurement implements ICalcEngineClient
 
     //-------------------------------------------------------------------------
     @Override
-    public String createDocument()
+    public void createDocument(final String sHandle)
         throws Exception
     {
     	final long nStart = System.currentTimeMillis();
     	try
     	{
-	    	final String sHandle = m_iWrapped.createDocument();
-	    	return sHandle;
+	    	m_iWrapped.createDocument(sHandle);
     	}
     	finally
     	{
