@@ -68,10 +68,6 @@ public class CalcEngineClientSimulator implements ICalcEngineClient
     //-------------------------------------------------------------------------
     private static final Log LOG = LogFactory.getJclLog(CalcEngineClientSimulator.class);
 
-    //-------------------------------------------------------------------------
-    public static final String OPERATION_4_CRASH    = "{crash:true}";
-    public static final String OPERATION_4_LOST_DOC = "{doc:lost-in-space}";
-
     //-------------------------------------------------------------------------
     private CalcEngineClientSimulator ()
         throws Exception
@@ -86,20 +82,18 @@ public class CalcEngineClientSimulator implements ICalcEngineClient
 
     //-------------------------------------------------------------------------
     @Override
-    public String createDocument()
+    public void createDocument(final String sHandle)
         throws Exception
     {
     	INFO("created new document ...");
 
     	final Map< String, Document > lDocs   = mem_Documents ();
-    	final String                  sHandle = CalcEngineHandleGenerator.newHandle();
     	final Document                aDoc    = new Document ();
     	
     	aDoc.sHandle = sHandle;
         lDocs.put (sHandle, aDoc);
 
     	INFO("... new document with handle ["+sHandle+"] created.");
-        return sHandle;
     };
     
     //-------------------------------------------------------------------------
@@ -133,24 +127,11 @@ public class CalcEngineClientSimulator implements ICalcEngineClient
         if ( ! impl_isHandleKnown (sHandle))
         	return ECalcEngineError.E_DOCUMENT_LOST;
 
-        // special simulation mode : simulate a missing document ... even if its well known .-)
-        
-        if (impl_isSpecialOp(sOperationJSON, OPERATION_4_LOST_DOC))
-        {
-        	INFO ("... simulate 'lost doc' by return E_DOCUMENT_LOST !");
-        	return ECalcEngineError.E_DOCUMENT_LOST;
-        }
-        
         final Map< String, Document > lDocs = mem_Documents ();
         final Document                aDoc  = lDocs.get(sHandle);
         
         aDoc.lOperations.append (sOperationJSON);
 
-        // special simulation mode : simulate crash by throwing an exception
-        
-        if (impl_isSpecialOp(sOperationJSON, OPERATION_4_CRASH))
-        	throw new Exception ("Crash was forced by special simulator operation.");
-        
         // put the incoming JSON back as result for testing purposes ... might be an unit test rely on that .-)
         sResultJSON.setLength (0);
         sResultJSON.append    (sOperationJSON);
