@@ -77,19 +77,21 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
     {}
     
     //-------------------------------------------------------------------------
-    public synchronized static CalcEngineErrorSimulator create()
+    public synchronized static CalcEngineErrorSimulator create(final ICalcEngineClient iImpl)
         throws Exception
     {
-    	return new CalcEngineErrorSimulator ();
+    	final CalcEngineErrorSimulator aSimulator = new CalcEngineErrorSimulator ();
+    	aSimulator.m_iImpl = iImpl;
+    	return aSimulator;
     }
 
     //-------------------------------------------------------------------------
     @Override
-    public String createDocument()
+    public void createDocument(final String sHandle)
         throws Exception
     {
     	INFO("created new document ...");
-    	return mem_Impl ().createDocument();
+    	m_iImpl.createDocument(sHandle);
     };
     
     //-------------------------------------------------------------------------
@@ -97,7 +99,7 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
     public void destroyDocument(final String sHandle)
         throws Exception
     {
-    	mem_Impl ().destroyDocument(sHandle);
+    	m_iImpl.destroyDocument(sHandle);
     };
 
     //-------------------------------------------------------------------------
@@ -112,7 +114,7 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
         // special simulation mode : simulate a missing document ... even if its well known .-)
         
         INFO("... check for OP_LOST_DOC");
-        if (impl_isSpecialOp(sOperationJSON, OPERATION_4_LOST_DOC))
+        if (impl_isSimulatorOp(sOperationJSON, OPERATION_4_LOST_DOC))
         {
         	INFO ("... simulate 'lost document' by returning E_DOCUMENT_LOST !");
         	return ECalcEngineError.E_DOCUMENT_LOST;
@@ -122,12 +124,12 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
         // and to simulate e.g. crashes during/after that !
         
         INFO("... forward request to real engine");
-        final ECalcEngineError eError = mem_Impl ().executeOperation(sHandle, sOperationJSON, sResultJSON);
+        final ECalcEngineError eError = m_iImpl.executeOperation(sHandle, sOperationJSON, sResultJSON);
         
         // special simulation mode : simulate crash by throwing an exception
         
         INFO("... check for OP_CRASH");
-        if (impl_isSpecialOp(sOperationJSON, OPERATION_4_CRASH))
+        if (impl_isSimulatorOp(sOperationJSON, OPERATION_4_CRASH))
         {
         	INFO ("... simulate 'crash' by throwing runtime exception !");
         	throw new RuntimeException ("Crash was forced by special simulator operation.");
@@ -142,7 +144,7 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
 			 				                 final String... lPagedOperationJSONs)
         throws Exception
     {
-    	return mem_Impl ().restoreDocument(sHandle, lPagedOperationJSONs);
+    	return m_iImpl.restoreDocument(sHandle, lPagedOperationJSONs);
     }
 
     //-------------------------------------------------------------------------
@@ -151,7 +153,7 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
 								 final CalcEngineClipBoardEvent aEvent          )
 		throws Exception
 	{
-    	return mem_Impl ().copy(sSourceDocHandle, aEvent);
+    	return m_iImpl.copy(sSourceDocHandle, aEvent);
 	}
 
     //-------------------------------------------------------------------------
@@ -160,14 +162,14 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
 								  final CalcEngineClipBoardEvent aEvent          )
 		throws Exception
 	{
-    	return mem_Impl ().paste(sTargetDocHandle, aEvent);
+    	return m_iImpl.paste(sTargetDocHandle, aEvent);
 	}
 
     //-------------------------------------------------------------------------
     @Override
     public String getVersion()
     {
-        return "Calc Engine Simulator 1.0 :-)";
+        return "Calc Engine Error Simulator 1.0 :-)";
     }
 
     //-------------------------------------------------------------------------
@@ -175,7 +177,7 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
     public void setLogLevel (final ELogLevel eLevel) 
     	throws Exception
     {
-    	mem_Impl ().setLogLevel(eLevel);
+    	m_iImpl.setLogLevel(eLevel);
     }
 
     //-------------------------------------------------------------------------
@@ -189,12 +191,12 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
     }
 
     //-------------------------------------------------------------------------
-    private static boolean impl_isSpecialOp (final String sOp     ,
-    										 final String sSpecial)
+    private static boolean impl_isSimulatorOp (final String sAllOps,
+    										   final String sSimuOp)
         throws Exception
     {
-    	// we use contains() instead of equals to be prepared for quoting .-)
-    	final boolean bIs = StringUtils.containsIgnoreCase(sOp, sSpecial);
+    	final String  sSimuOpSearch = "\""+sSimuOp+"\"";
+    	final boolean bIs           = StringUtils.containsIgnoreCase(sAllOps, sSimuOpSearch);
     	return bIs;
     }
     
@@ -206,19 +208,6 @@ public class CalcEngineErrorSimulator implements ICalcEngineClient
     	LOG.info(sMessage);
     }
     
-    //-------------------------------------------------------------------------
-    private ICalcEngineClient mem_Impl ()
-    	throws Exception
-    {
-    	if (m_iImpl == null)
-    	{
-    		final CalcEngineDescriptor aDesc = new CalcEngineDescriptor ();
-    		aDesc.m_eMode = ECalcEngineMode.E_NATIVE;
-    		m_iImpl = CalcEngineClientFactory.getDirect(aDesc);
-    	}
-    	return m_iImpl;
-    }
-    
     //-------------------------------------------------------------------------
     private ICalcEngineClient m_iImpl = null;
 }
