@@ -65,8 +65,6 @@ import com.openexchange.office.calcengine.client.ECalcEngineError;
 import com.openexchange.office.calcengine.client.ECalcEngineMode;
 import com.openexchange.office.calcengine.client.ICalcEngineClient;
 import com.openexchange.office.calcengine.client.impl.CalcEngineClientHTTP.EAliveState;
-import com.openexchange.office.tools.executablewrapper.Executable;
-import com.openexchange.office.tools.executablewrapper.SimpleExecutableWatcher;
 import com.openexchange.office.tools.jobpool.JobArguments;
 import com.openexchange.office.tools.jobpool.JobBase;
 import com.openexchange.office.tools.jobpool.JobEnvironment;
@@ -74,6 +72,8 @@ import com.openexchange.office.tools.jobpool.JobResults;
 import com.openexchange.office.tools.logging.ContextAwareLogHelp;
 import com.openexchange.office.tools.logging.ELogLevel;
 import com.openexchange.office.tools.logging.LogFactory;
+import com.openxchange.office_communication.tools.exec.Executable;
+import com.openxchange.office_communication.tools.exec.SimpleExecutableWatcher;
 
 //=============================================================================
 public class CalcEngineJob extends JobBase
@@ -122,6 +122,7 @@ public class CalcEngineJob extends JobBase
 	public static final String ENV_WORKERBIN          = "env.worker-bin";
 	public static final String ENV_PORT               = "env.port";
 	public static final String ENV_MAXMEM             = "env.worker-maxmem";
+	public static final String ENV_CACHE_PATH         = "env.cache-path";
 
 	//-------------------------------------------------------------------------
 	public static final String ARG_OPERATION          = "arg.operation"      ;
@@ -159,9 +160,10 @@ public class CalcEngineJob extends JobBase
 		
 		// read configuration first to know the connection details
 		
-		final String  sWorkerBin = aEnv.get (ENV_WORKERBIN, null);
-		final Integer nPort      = aEnv.get (ENV_PORT     , 0   );
-		final Long    nMaxMemKB  = aEnv.get (ENV_MAXMEM   , 0L  ); // optional !
+		final String  sWorkerBin = aEnv.get (ENV_WORKERBIN , null);
+		final Integer nPort      = aEnv.get (ENV_PORT      , 0   );
+		final Long    nMaxMemKB  = aEnv.get (ENV_MAXMEM    , 0L  ); // optional !
+		final String  sCachePath = aEnv.get (ENV_CACHE_PATH, null); // optional !
 		
 		Validate.notEmpty (sWorkerBin , "Miss environment var '"+ENV_WORKERBIN+"'.");
 		Validate.isTrue   (nPort>0    , "Miss environment var '"+ENV_PORT     +"'.");
@@ -202,6 +204,12 @@ public class CalcEngineJob extends JobBase
 		aExe.addArgument   ("-i"                    );
 		aExe.addArgument   ("localhost"             );
 		
+		if ( ! StringUtils.isEmpty(sCachePath))
+		{
+			aExe.addArgument ("-arp"    );
+			aExe.addArgument (sCachePath);
+		}
+		
 		if (nMaxMemKB > 0) // optional
 		{
 			aExe.addArgument ("-m"                    );
@@ -282,13 +290,16 @@ public class CalcEngineJob extends JobBase
 		if (m_aProcess == null)
 			return;
 
+		final ContextAwareLogHelp aLog = mem_Log ();
+		LOG.info (aLog.forLevel(ELogLevel.E_INFO).toLog("deactivate worker ..."));
+		impl_shutdownMightRunningWorker ();
+		LOG.info (aLog.forLevel(ELogLevel.E_INFO).toLog("... OK"));
+
 		final Executable aProcess = m_aProcess;
 		m_aProcess = null;
 		
-		final ContextAwareLogHelp aLog = mem_Log ();
-		LOG.info (aLog.forLevel(ELogLevel.E_INFO).toLog("deactivate worker ..."));
-		aProcess.kill();
-		LOG.info (aLog.forLevel(ELogLevel.E_INFO).toLog("... OK"));
+		if (aProcess.isAlive())
+			aProcess.kill ();
 	}
 
 	//-------------------------------------------------------------------------
@@ -329,11 +340,12 @@ public class CalcEngineJob extends JobBase
 			   						 final JobResults   lResults  )
         throws Exception
     {
-		LOG.info (mem_Log ().forLevel(ELogLevel.E_INFO                )
-							.toLog   ("execute : createDocument ...."));
+		final String sDocHandle = lArguments.get(ARG_DOCUMENT_HANDLE);
 
-		final String sId = m_iAPI.createDocument();
-		lResults.set(RESULT_DOCUMENT_HANDLE, sId);
+		LOG.info (mem_Log ().forLevel(ELogLevel.E_INFO                )
+							.toLog   ("execute : createDocument ["+CalcEngineLogContextConst.CONTEXT_DOC_HANDLE+":"+sDocHandle+"] ...."));
+
+		m_iAPI.createDocument(sDocHandle);
     }
 	
 	//-------------------------------------------------------------------------
