@@ -50,7 +50,7 @@ package com.openexchange.office.calcengine.worker;
 
 import org.apache.commons.lang.Validate;
 
-import com.openexchange.office.tools.commandline.CommandLineBase;
+import com.openxchange.office_communication.tools.cmdline.CommandLineBase;
 
 //==============================================================================
 public class WorkerCommandLine extends CommandLineBase
@@ -71,6 +71,10 @@ public class WorkerCommandLine extends CommandLineBase
 	public static final String OPT_SHORT_DEBUG = "d";
 	public static final String OPT_LONG_DEBUG  = "debug";
 
+    //--------------------------------------------------------------------------
+	public static final String OPT_SHORT_AUTORESTORE_PATH = "arp";
+	public static final String OPT_LONG_AUTORESTORE_PATH  = "auto-restore-path";
+	
     //--------------------------------------------------------------------------
 	public static final String OPT_SHORT_MEMORY = "m";
 	public static final String OPT_LONG_MEMORY  = "memory";
@@ -103,6 +107,13 @@ public class WorkerCommandLine extends CommandLineBase
                    "All operation will be simulated."     +
                    "Not thought for real production mode.");
 
+        addOption (OPT_SHORT_AUTORESTORE_PATH                      ,
+		 		   OPT_LONG_AUTORESTORE_PATH                       ,
+		           CommandLineBase.HAS_VALUE                       ,
+		           CommandLineBase.NOT_REQUIRED                    ,
+		           "define path where documents can be cached for."+
+		           "auto-restore mode."                            );
+
         addOption (OPT_SHORT_DEBUG    		                   ,
         		   OPT_LONG_DEBUG                              ,
                    CommandLineBase.HAS_NO_VALUE                ,
@@ -159,6 +170,22 @@ public class WorkerCommandLine extends CommandLineBase
 		return bIsSimulationOn;
 	}
 
+	//--------------------------------------------------------------------------
+	public boolean isAutoRestoreOn ()
+	    throws Exception
+	{
+		final boolean bIsOn = hasOption(OPT_SHORT_AUTORESTORE_PATH);
+		return bIsOn;
+	}
+
+	//--------------------------------------------------------------------------
+	public String getAutoRestorePath ()
+	    throws Exception
+	{
+		final String sPath = getOptionValue(OPT_SHORT_AUTORESTORE_PATH);
+		return sPath;
+	}
+
 	//--------------------------------------------------------------------------
 	public boolean isDebugOn ()
 	    throws Exception
