@@ -484,7 +484,7 @@ public class Docx4jDriver {
 	}
 
 	private static Document getDocument(File f) throws Exception {
-			DocumentBuilder db = XmlUtils.getDocumentBuilderFactory().newDocumentBuilder();
-			return db.parse(f);
+
+		return XmlUtils.getDocumentBuilderFactory().newDocumentBuilder().parse(f);
 	}
 }
