@@ -77,32 +77,28 @@ public class IndexedNodeList<E> implements List<E> {
 		return list.add(arg0);
 	}
 
-	public void addNode(int arg0, IndexedNode<E> arg1) {
-		arg1.setIndex(arg0);
-		if(arg0==list.size()) {
-			addNode(arg1);
-			return;
-		}
-		list.add(arg0, arg1);
-		for(int i=arg0+1;i<list.size();i++) {
+	public void addNode(IndexedNode<E> refNode, IndexedNode<E> newNode, boolean before) {
+		final int index = refNode==null?list.size():before?refNode.getIndex():refNode.getIndex()+1;
+		newNode.setIndex(index);
+		list.add(index, newNode);
+		for(int i=index+1;i<list.size();i++) {
 			list.get(i).setIndex(i);
 		}
 	}
 
-	public IndexedNode<E> setNode(int arg0, IndexedNode<E> arg1) {
-		arg1.setIndex(arg0);
-		return list.set(arg0, arg1);
+	public IndexedNode<E> setNode(IndexedNode<E> oldNode, IndexedNode<E> newNode) {
+		final int index = oldNode.getIndex();
+		newNode.setIndex(index);
+		oldNode.setIndex(-1);
+		return list.set(index, newNode);
 	}
 
-	public IndexedNode<E> removeNode(int arg0) {
-		final IndexedNode<E> node = list.remove(arg0);
-		for(int i=arg0;i<list.size();i++) {
-			list.get(i).setIndex(i);
-		}
-		return node;
+	public void removeNodes(IndexedNode<E> startNode, IndexedNode<E> endNode) {
+		final int startIndex = startNode.getIndex();
+		removeNodes(startIndex, endNode!=null?(endNode.getIndex()-startIndex)+1:1);
 	}
 
-	public void removeNodes(int arg0, int size) {
+	private void removeNodes(int arg0, int size) {
 		for(int i=0;i<size;i++) {
 			list.remove(arg0);
 		}
@@ -110,8 +106,8 @@ public class IndexedNodeList<E> implements List<E> {
 			list.get(i).setIndex(i);
 		}
 	}
-	
-	public static void  moveNodes(IndexedNodeList<Object> source, IndexedNodeList<Object> dest, int sourceIndex, int destIndex, int count, Object newParent) {
+
+	public static void moveNodes(IndexedNodeList<Object> source, IndexedNodeList<Object> dest, int sourceIndex, int destIndex, int count, Object newParent) {
 
 		final boolean sourceIndexIncreasing = source==dest&&destIndex<sourceIndex;
 		for(int i=0;i<count;i++) {
