@@ -1,19 +1,19 @@
 /*
  *  Copyright 2007-2008, Plutext Pty Ltd.
- *   
+ *
  *  This file is part of docx4j.
 
-    docx4j is licensed under the Apache License, Version 2.0 (the "License"); 
-    you may not use this file except in compliance with the License. 
+    docx4j is licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
 
-    You may obtain a copy of the License at 
+    You may obtain a copy of the License at
 
-        http://www.apache.org/licenses/LICENSE-2.0 
+        http://www.apache.org/licenses/LICENSE-2.0
 
-    Unless required by applicable law or agreed to in writing, software 
-    distributed under the License is distributed on an "AS IS" BASIS, 
-    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
-    See the License for the specific language governing permissions and 
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
     limitations under the License.
 
  */
@@ -30,9 +30,9 @@ import javax.xml.bind.annotation.XmlType;
 
 /**
  * <p>Java class for CT_ShapeProperties complex type.
- * 
+ *
  * <p>The following schema fragment specifies the expected content contained within this class.
- * 
+ *
  * <pre>
  * &lt;complexType name="CT_ShapeProperties">
  *   &lt;complexContent>
@@ -52,8 +52,8 @@ import javax.xml.bind.annotation.XmlType;
  *   &lt;/complexContent>
  * &lt;/complexType>
  * </pre>
- * 
- * 
+ *
+ *
  */
 @XmlAccessorType(XmlAccessType.FIELD)
 @XmlType(name = "CT_ShapeProperties", propOrder = {
@@ -73,7 +73,7 @@ import javax.xml.bind.annotation.XmlType;
     "sp3D",
     "extLst"
 })
-public class CTShapeProperties {
+public class CTShapeProperties implements IFillProperties {
 
     protected CTTransform2D xfrm;
     protected CTCustomGeometry2D custGeom;
@@ -97,11 +97,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the xfrm property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTTransform2D }
-     *     
+     *
      */
     public CTTransform2D getXfrm() {
         return xfrm;
@@ -109,11 +109,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the xfrm property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTTransform2D }
-     *     
+     *
      */
     public void setXfrm(CTTransform2D value) {
         this.xfrm = value;
@@ -121,11 +121,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the custGeom property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTCustomGeometry2D }
-     *     
+     *
      */
     public CTCustomGeometry2D getCustGeom() {
         return custGeom;
@@ -133,11 +133,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the custGeom property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTCustomGeometry2D }
-     *     
+     *
      */
     public void setCustGeom(CTCustomGeometry2D value) {
         this.custGeom = value;
@@ -145,11 +145,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the prstGeom property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTPresetGeometry2D }
-     *     
+     *
      */
     public CTPresetGeometry2D getPrstGeom() {
         return prstGeom;
@@ -157,11 +157,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the prstGeom property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTPresetGeometry2D }
-     *     
+     *
      */
     public void setPrstGeom(CTPresetGeometry2D value) {
         this.prstGeom = value;
@@ -169,11 +169,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the noFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTNoFillProperties }
-     *     
+     *
      */
     public CTNoFillProperties getNoFill() {
         return noFill;
@@ -181,11 +181,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the noFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTNoFillProperties }
-     *     
+     *
      */
     public void setNoFill(CTNoFillProperties value) {
         this.noFill = value;
@@ -193,11 +193,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the solidFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTSolidColorFillProperties }
-     *     
+     *
      */
     public CTSolidColorFillProperties getSolidFill() {
         return solidFill;
@@ -205,11 +205,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the solidFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTSolidColorFillProperties }
-     *     
+     *
      */
     public void setSolidFill(CTSolidColorFillProperties value) {
         this.solidFill = value;
@@ -217,11 +217,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the gradFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTGradientFillProperties }
-     *     
+     *
      */
     public CTGradientFillProperties getGradFill() {
         return gradFill;
@@ -229,11 +229,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the gradFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTGradientFillProperties }
-     *     
+     *
      */
     public void setGradFill(CTGradientFillProperties value) {
         this.gradFill = value;
@@ -241,11 +241,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the blipFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTBlipFillProperties }
-     *     
+     *
      */
     public CTBlipFillProperties getBlipFill() {
         return blipFill;
@@ -253,11 +253,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the blipFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTBlipFillProperties }
-     *     
+     *
      */
     public void setBlipFill(CTBlipFillProperties value) {
         this.blipFill = value;
@@ -265,11 +265,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the pattFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTPatternFillProperties }
-     *     
+     *
      */
     public CTPatternFillProperties getPattFill() {
         return pattFill;
@@ -277,11 +277,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the pattFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTPatternFillProperties }
-     *     
+     *
      */
     public void setPattFill(CTPatternFillProperties value) {
         this.pattFill = value;
@@ -289,11 +289,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the grpFill property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTGroupFillProperties }
-     *     
+     *
      */
     public CTGroupFillProperties getGrpFill() {
         return grpFill;
@@ -301,11 +301,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the grpFill property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTGroupFillProperties }
-     *     
+     *
      */
     public void setGrpFill(CTGroupFillProperties value) {
         this.grpFill = value;
@@ -313,11 +313,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the ln property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTLineProperties }
-     *     
+     *
      */
     public CTLineProperties getLn() {
         return ln;
@@ -325,11 +325,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the ln property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTLineProperties }
-     *     
+     *
      */
     public void setLn(CTLineProperties value) {
         this.ln = value;
@@ -337,11 +337,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the effectLst property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTEffectList }
-     *     
+     *
      */
     public CTEffectList getEffectLst() {
         return effectLst;
@@ -349,11 +349,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the effectLst property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTEffectList }
-     *     
+     *
      */
     public void setEffectLst(CTEffectList value) {
         this.effectLst = value;
@@ -361,11 +361,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the effectDag property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTEffectContainer }
-     *     
+     *
      */
     public CTEffectContainer getEffectDag() {
         return effectDag;
@@ -373,11 +373,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the effectDag property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTEffectContainer }
-     *     
+     *
      */
     public void setEffectDag(CTEffectContainer value) {
         this.effectDag = value;
@@ -385,11 +385,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the scene3D property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTScene3D }
-     *     
+     *
      */
     public CTScene3D getScene3D() {
         return scene3D;
@@ -397,11 +397,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the scene3D property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTScene3D }
-     *     
+     *
      */
     public void setScene3D(CTScene3D value) {
         this.scene3D = value;
@@ -409,11 +409,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the sp3D property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTShape3D }
-     *     
+     *
      */
     public CTShape3D getSp3D() {
         return sp3D;
@@ -421,11 +421,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the sp3D property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTShape3D }
-     *     
+     *
      */
     public void setSp3D(CTShape3D value) {
         this.sp3D = value;
@@ -433,11 +433,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the extLst property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link CTOfficeArtExtensionList }
-     *     
+     *
      */
     public CTOfficeArtExtensionList getExtLst() {
         return extLst;
@@ -445,11 +445,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the extLst property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link CTOfficeArtExtensionList }
-     *     
+     *
      */
     public void setExtLst(CTOfficeArtExtensionList value) {
         this.extLst = value;
@@ -457,11 +457,11 @@ public class CTShapeProperties {
 
     /**
      * Gets the value of the bwMode property.
-     * 
+     *
      * @return
      *     possible object is
      *     {@link STBlackWhiteMode }
-     *     
+     *
      */
     public STBlackWhiteMode getBwMode() {
         return bwMode;
@@ -469,11 +469,11 @@ public class CTShapeProperties {
 
     /**
      * Sets the value of the bwMode property.
-     * 
+     *
      * @param value
      *     allowed object is
      *     {@link STBlackWhiteMode }
-     *     
+     *
      */
     public void setBwMode(STBlackWhiteMode value) {
         this.bwMode = value;
