@@ -63,6 +63,7 @@ import org.docx4j.dml.CTOfficeArtExtensionList;
 import org.docx4j.dml.CTShapeProperties;
 import org.docx4j.dml.CTShapeStyle;
 import org.docx4j.dml.CTTextBodyProperties;
+import org.docx4j.dml.IShape;
 
 
 /**
@@ -110,7 +111,7 @@ import org.docx4j.dml.CTTextBodyProperties;
     "linkedTxbx",
     "bodyPr"
 })
-public class CTWordprocessingShape {
+public class CTWordprocessingShape implements IShape {
 
     protected CTNonVisualDrawingProps cNvPr;
     protected CTNonVisualDrawingShapeProps cNvSpPr;
