@@ -19,19 +19,9 @@
  */
 package org.docx4j.jaxb;
 
-
-import java.io.IOException;
-import java.io.InputStream;
-import java.net.URL;
-import java.util.Enumeration;
-import java.util.jar.Attributes;
-import java.util.jar.JarFile;
-import java.util.jar.Manifest;
-
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.JAXBException;
 
-import org.docx4j.utils.ResourceUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -64,51 +54,19 @@ public class Context {
 	private static boolean lowMemoryAbort = false;
 
 	static {
-
-	    // Display diagnostic info about version of JAXB being used.
-		log.info("java.vendor="+System.getProperty("java.vendor"));
-		log.info("java.version="+System.getProperty("java.version"));
-
-		// This stuff is just debugging diagnostics
-		try {
-			searchManifestsForJAXBImplementationInfo( ClassLoader.getSystemClassLoader());
-			if (Thread.currentThread().getContextClassLoader()==null) {
-				log.warn("ContextClassLoader is null for current thread");
-				// Happens with IKVM
-			} else if (ClassLoader.getSystemClassLoader()!=Thread.currentThread().getContextClassLoader()) {
-				searchManifestsForJAXBImplementationInfo(Thread.currentThread().getContextClassLoader());
-			}
-		} catch ( java.security.AccessControlException e) {
-			// Google AppEngine throws this, at com.google.apphosting.runtime.security.CustomSecurityManager.checkPermission
-			log.warn("Caught/ignored " + e.getMessage());
-		}
-
 		// Diagnostics regarding JAXB implementation
-		InputStream jaxbPropsIS=null;
 		try {
-			// Is MOXy configured?
-			jaxbPropsIS = ResourceUtils.getResource("org/docx4j/wml/jaxb.properties");
-			log.info("MOXy JAXB implementation intended..");
-		} catch (Exception e3) {
-			log.info("No MOXy JAXB config found; assume not intended..");
-			log.debug(e3.getMessage());
-		}
-		if (jaxbPropsIS==null) {
-			// Only probe for other implementations if no MOXy
-			try {
-				Object namespacePrefixMapper = NamespacePrefixMapperUtils.getPrefixMapper();
-				if ( namespacePrefixMapper.getClass().getName().equals("org.docx4j.jaxb.NamespacePrefixMapperSunInternal") ) {
-					// Java 6
-					log.info("Using Java 6/7 JAXB implementation");
-				} else {
-					log.info("Using JAXB Reference Implementation");
-				}
-
-			} catch (JAXBException e) {
-				log.error("PANIC! No suitable JAXB implementation available");
-				log.error(e.getMessage(), e);
-				e.printStackTrace();
+			Object namespacePrefixMapper = NamespacePrefixMapperUtils.getPrefixMapper();
+			if ( namespacePrefixMapper.getClass().getName().equals("org.docx4j.jaxb.NamespacePrefixMapperSunInternal") ) {
+				// Java 6
+				log.info("Using Java 6/7 JAXB implementation");
+			} else {
+				log.info("Using JAXB Reference Implementation");
 			}
+		} catch (JAXBException e) {
+			log.error("PANIC! No suitable JAXB implementation available");
+			log.error(e.getMessage(), e);
+			e.printStackTrace();
 		}
         jcThread = new Thread(
             new Runnable() {
@@ -128,7 +86,7 @@ public class Context {
 
                         jc = JAXBContext.newInstance("org.docx4j.wml:org.docx4j.w14:org.docx4j.w15:" +
                                 "org.docx4j.schemas.microsoft.com.office.word_2006.wordml:" +
-                                "org.docx4j.dml:org.docx4j.dml.chart:org.docx4j.dml.chartDrawing:org.docx4j.dml.compatibility:org.docx4j.dml.diagram:org.docx4j.dml.lockedCanvas:org.docx4j.dml.picture:org.docx4j.dml.wordprocessingDrawing:org.docx4j.dml.spreadsheetdrawing:org.docx4j.dml.diagram2008:" +
+                                "org.docx4j.dml:org.docx4j.dml.chart:org.docx4j.dml.chartStyle2012:org.docx4j.dml.chartDrawing:org.docx4j.dml.compatibility:org.docx4j.dml.diagram:org.docx4j.dml.lockedCanvas:org.docx4j.dml.picture:org.docx4j.dml.wordprocessingDrawing:org.docx4j.dml.spreadsheetdrawing:org.docx4j.dml.diagram2008:" +
                                 // All VML stuff is here, since compiling it requires WML and DML (and MathML), but not PML or SML
                                 "org.docx4j.vml:org.docx4j.vml.officedrawing:org.docx4j.vml.wordprocessingDrawing:org.docx4j.vml.presentationDrawing:org.docx4j.vml.spreadsheetDrawing:org.docx4j.vml.root:" +
                                 "org.docx4j.docProps.coverPageProps:" +
@@ -138,9 +96,9 @@ public class Context {
                                 "org.docx4j.dml2010:org.docx4j.dml.wordprocessingCanvas2010:org.docx4j.dml.wordprocessingGroup2010:org.docx4j.dml.wordprocessingShape2010", classLoader );
 
                         if (jc.getClass().getName().equals("org.eclipse.persistence.jaxb.JAXBContext")) {
-                            log.info("MOXy JAXB implementation is in use!");
+                            log.debug("MOXy JAXB implementation is in use!");
                         } else {
-                            log.info("Not using MOXy; using " + jc.getClass().getName());
+                            log.debug("Not using MOXy; using " + jc.getClass().getName());
                         }
                         jcDocPropsCore = JAXBContext.newInstance("org.docx4j.docProps.core:org.docx4j.docProps.core.dc.elements:org.docx4j.docProps.core.dc.terms",classLoader );
                         jcDocPropsCustom = JAXBContext.newInstance("org.docx4j.docProps.custom",classLoader );
@@ -325,56 +283,4 @@ public class Context {
 		}
 		return jcXslFo;
 	}
-
-	public static void searchManifestsForJAXBImplementationInfo(ClassLoader loader) {
-	    Enumeration resEnum;
-	    try {
-	        resEnum = loader.getResources(JarFile.MANIFEST_NAME);
-	        while (resEnum.hasMoreElements()) {
-	            try {
-	                URL url = (URL)resEnum.nextElement();
-//	                System.out.println("\n\n" + url);
-	                InputStream is = url.openStream();
-	                if (is != null) {
-	                    Manifest manifest = new Manifest(is);
-
-                    	Attributes mainAttribs = manifest.getMainAttributes();
-                    	String impTitle = mainAttribs.getValue("Implementation-Title");
-                    	if (impTitle!=null
-                    			&& impTitle.contains("JAXB Reference Implementation")
-                    					|| impTitle.contains("org.eclipse.persistence") ) {
-
-        	                log.debug("\n" + url);
-		                    for(Object key2  : mainAttribs.keySet() ) {
-
-		                    	log.debug(key2 + " : " + mainAttribs.getValue((java.util.jar.Attributes.Name)key2));
-		                    }
-                    	}
-
-	                    // In 2.1.3, it is in here
-	                    for(String key  :  manifest.getEntries().keySet() ) {
-	    	                //System.out.println(key);
-	    	                if (key.equals("com.sun.xml.bind.v2.runtime")) {
-		    	                log.debug("Found JAXB reference implementation in " + url);
-		                    	mainAttribs = manifest.getAttributes(key);
-
-			                    for(Object key2  : mainAttribs.keySet() ) {
-			                    	log.debug(key2 + " : " + mainAttribs.getValue((java.util.jar.Attributes.Name)key2));
-			                    }
-		                    }
-	                    }
-
-	                }
-	            }
-	            catch (Exception e) {
-	                // Silently ignore
-//	            	log.error(e.getMessage(), e);
-	            }
-	        }
-	    } catch (IOException e1) {
-	        // Silently ignore
-//        	log.error(e1);
-	    }
-
-	}
 }
