@@ -136,12 +136,9 @@ public class JaxbValidationEventHandler implements ValidationEventHandler {
 	            }
 	        }
 	        // print message from validation event
-	        if (log.isDebugEnabled() || ve.getMessage().length() < 120 ) {
+	        if (log.isDebugEnabled()) {
 	            log.warn( printSeverity(ve) + ": " + ve.getMessage() );
 	        }
-	        else {
-	            log.warn( printSeverity(ve) + ": " + ve.getMessage().substring(0, 120));
-	        }
 	    }
 	    else if (ve.getSeverity()==ve.WARNING) {
 	        log.warn(printSeverity(ve) + "Message is " + ve.getMessage());
