@@ -140,38 +140,36 @@ public class Load {
 
 				part = ctm.getPart("/" + resolvedPartUri, rel);
 
-
 				if (part instanceof org.docx4j.openpackaging.parts.ThemePart) {
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJc());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
-
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJc());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 				} else if (part instanceof org.docx4j.openpackaging.parts.DocPropsCorePart ) {
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJcDocPropsCore());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJcDocPropsCore());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 
 				} else if (part instanceof org.docx4j.openpackaging.parts.DocPropsCustomPart ) {
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJcDocPropsCustom());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJcDocPropsCustom());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 
 				} else if (part instanceof org.docx4j.openpackaging.parts.DocPropsExtendedPart ) {
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJcDocPropsExtended());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJcDocPropsExtended());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 
 				} else if (part instanceof org.docx4j.openpackaging.parts.CustomXmlDataStoragePropertiesPart ) {
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJcCustomXmlProperties());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJcCustomXmlProperties());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 
 				} else if (part instanceof org.docx4j.openpackaging.parts.JaxbXmlPart) {
 
 					// MainDocument part, Styles part, Font part etc
 
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).setJAXBContext(Context.getJc());
-					((org.docx4j.openpackaging.parts.JaxbXmlPart)part).unmarshal( is );
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).setJAXBContext(Context.getJc());
+					((org.docx4j.openpackaging.parts.JaxbXmlPart<?>)part).unmarshal( is );
 
 				} else if (part instanceof org.docx4j.openpackaging.parts.WordprocessingML.BinaryPart) {
 
@@ -292,13 +290,13 @@ public class Load {
 		// which has a data store item ID, even if it isn't in document.xml.rels
 
 		//Iterator iterator = parts.entrySet().iterator();
-		Collection col = parts.values();
-		Iterator iterator = col.iterator();
+		final Collection<Part> col = parts.values();
+		final Iterator<Part> iterator = col.iterator();
 		while( iterator.hasNext() ) {
 			Part entry = (Part)iterator.next();
 
 			if (entry instanceof CustomXmlPart) {
-				log.info("Found a CustomXmlPart, named " + entry.getPartName().getName() );
+				log.debug("Found a CustomXmlPart, named " + entry.getPartName().getName() );
 				String itemId = null;
 				if (entry.getRelationshipsPart()==null) {
 					continue;
@@ -340,7 +338,7 @@ public class Load {
 						itemId = customXmlProps.getItemId().toLowerCase();
 					}
 				}
-				log.info("Identified/registered ds:itemId " + itemId);
+				log.debug("Identified/registered ds:itemId " + itemId);
 				if (pkg.getCustomXmlDataStorageParts().get(itemId.toLowerCase())!=null) {
 					log.warn("Duplicate CustomXML itemId " + itemId + "; check your source docx!");
 				}
