@@ -22,7 +22,6 @@ package org.docx4j.openpackaging.parts;
 
 
 import javax.xml.bind.JAXBException;
-import javax.xml.bind.Unmarshaller;
 import javax.xml.xpath.XPath;
 
 import org.docx4j.XmlUtils;
@@ -116,6 +115,9 @@ public class DocPropsExtendedPart extends JaxbXmlPart<Properties> {
 //			}
 
 			setJAXBContext(org.docx4j.jaxb.Context.getJcDocPropsExtended());
+
+			log.debug("unmarshalling " + this.getClass().getName() );
+
 			jaxbElement = (Properties)XmlUtils.unmarshal(is, jc, new org.docx4j.jaxb.JaxbValidationEventHandler());
 
 		} catch (Exception e ) {
