@@ -611,26 +611,17 @@ public final class PartName implements Comparable<PartName> {
 	 *  implementation which I only found after writing this
 	 */
 	static public String getRelationshipsPartName(String partName) {
-	//		word/document.xml --->
-	//		word/_rels/document.xml.rels 
-			String relationshipsPartName = null;
-	
-//			log.info("Splitting " + partName );
-	
 			String rightBit = partName; 
-			
 			// Split partName at its last "/"
 			int pos = partName.lastIndexOf("/");
 			if (pos>0) {
 				String leftBit = partName.substring(0, pos);
 				rightBit = partName.substring(pos);
-//				log.info("**" + leftBit + "/_rels" + rightBit + ".rels" );
 				return leftBit + "/_rels" + rightBit + ".rels" ;
 			} else {
 				// eg partname: foo.ext (ie in root)
 				if (!rightBit.startsWith("/"))
 					rightBit="/" + rightBit;
-//				log.info("**" + "/_rels" + rightBit + ".rels" );
 				return "/_rels" + rightBit + ".rels" ;				
 			}
 		}
