@@ -22,6 +22,7 @@ package org.docx4j.openpackaging.parts.WordprocessingML;
 
 
 
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -174,7 +175,7 @@ public class MainDocumentPart extends DocumentPart<org.docx4j.wml.Document> impl
 
 		if (refresh || styleTree==null) {
 
-			log.info("Preparing StyleTree");
+			log.debug("Preparing StyleTree");
 
 		    try {
 				getStyleDefinitionsPart().createVirtualStylesForDocDefaults();
@@ -211,7 +212,7 @@ public class MainDocumentPart extends DocumentPart<org.docx4j.wml.Document> impl
      */
     public Set<String> fontsInUse() {
 
-    	log.info("fontsInUse..");
+    	log.debug("fontsInUse..");
 
     	getPropertyResolver();  // this inits our virtual DocDefaults style
 
@@ -451,7 +452,6 @@ public class MainDocumentPart extends DocumentPart<org.docx4j.wml.Document> impl
 
         Style defaultParagraphStyle;
         Style defaultCharacterStyle;
-    	String defaultTableStyle = "TableNormal";
 
     	private boolean defaultParagraphStyleUsed = false;
     	private boolean defaultCharacterStyleUsed = false;
@@ -615,10 +615,6 @@ public class MainDocumentPart extends DocumentPart<org.docx4j.wml.Document> impl
 	public org.docx4j.wml.P createStyledParagraphOfText(String styleId, String text) {
 
 		org.docx4j.wml.P p = createParagraphOfText(text);
-
-		StyleDefinitionsPart styleDefinitionsPart
-			= this.getStyleDefinitionsPart();
-
 		if (getPropertyResolver().activateStyle(styleId)) {
 			// Style is available
 			org.docx4j.wml.ObjectFactory factory = Context.getWmlObjectFactory();
@@ -694,7 +690,7 @@ public class MainDocumentPart extends DocumentPart<org.docx4j.wml.Document> impl
 		// is activated
     	Set<String> stylesInUse = new java.util.HashSet<String>();
 //    	Set<String> fontsDiscovered = new java.util.HashSet<String>();
-		List list = new java.util.ArrayList<Object>();
+		List<Object> list = new java.util.ArrayList<Object>();
 		list.add(o);
 
 //		FontDiscoveryCharacterVisitor visitor = new FontDiscoveryCharacterVisitor(fontsDiscovered);
