@@ -23,8 +23,10 @@ package org.docx4j.wml;
 
 import org.jvnet.jaxb2_commons.ppp.Child;
 
+import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.List;
+
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.Unmarshaller;
 import javax.xml.bind.annotation.XmlAccessType;
@@ -35,6 +37,8 @@ import javax.xml.bind.annotation.XmlElementRefs;
 import javax.xml.bind.annotation.XmlRootElement;
 import javax.xml.bind.annotation.XmlTransient;
 import javax.xml.bind.annotation.XmlType;
+
+import org.docx4j.IndexedNodeList;
 import org.docx4j.math.CTOMath;
 import org.docx4j.math.CTOMathPara;
 
@@ -74,12 +78,13 @@ import org.docx4j.math.CTOMathPara;
     "comment"
 })
 @XmlRootElement(name = "comments")
-public class Comments implements Child
+public class Comments implements Child, ContentAccessor
 {
-
     protected List<Comments.Comment> comment;
     @XmlTransient
     private Object parent;
+    @XmlTransient
+    private BigInteger commentId;
 
     /**
      * Gets the value of the comment property.
@@ -110,6 +115,23 @@ public class Comments implements Child
         return this.comment;
     }
 
+    // returns null or the content of the corresponding comment that belongs to the commentId
+	@Override
+	public List<Object> getContent() {
+		if(commentId!=null) {
+			for(Comment c:comment) {
+				if(commentId.equals(c.id)) {
+					return c.getContent();
+				}
+			}
+		}
+		return null;
+	}
+
+	public void setCommentId(BigInteger value) {
+		commentId = value;
+	}
+
     /**
      * Gets the parent object in the object tree representing the unmarshalled xml document.
      * 
@@ -162,7 +184,7 @@ public class Comments implements Child
         "content"
     })
     public static class Comment
-        extends CTTrackChange
+        extends CTTrackChange implements ContentAccessor
     {
 
         @XmlElementRefs({
@@ -197,7 +219,7 @@ public class Comments implements Child
             @XmlElementRef(name = "customXmlMoveToRangeEnd", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class),
             @XmlElementRef(name = "altChunk", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main", type = JAXBElement.class)
         })
-        protected List<Object> content;
+        protected IndexedNodeList<Object> content;
         @XmlAttribute(name = "initials", namespace = "http://schemas.openxmlformats.org/wordprocessingml/2006/main")
         protected String initials;
 
@@ -252,9 +274,9 @@ public class Comments implements Child
          * 
          * 
          */
-        public List<Object> getContent() {
+        public IndexedNodeList<Object> getContent() {
             if (content == null) {
-                content = new ArrayList<Object>();
+                content = new IndexedNodeList<Object>();
             }
             return this.content;
         }
@@ -303,5 +325,4 @@ public class Comments implements Child
         }
 
     }
-
 }
