@@ -19,19 +19,15 @@
  */
 package org.xlsx4j.sml;
 
-import javax.xml.bind.Marshaller;
 import javax.xml.bind.Unmarshaller;
 import javax.xml.bind.annotation.XmlAccessType;
 import javax.xml.bind.annotation.XmlAccessorType;
 import javax.xml.bind.annotation.XmlAttribute;
+import javax.xml.bind.annotation.XmlElement;
 import javax.xml.bind.annotation.XmlSchemaType;
 import javax.xml.bind.annotation.XmlTransient;
 import javax.xml.bind.annotation.XmlType;
 
-import org.jvnet.jaxb2_commons.ppp.Child;
-import org.xlsx4j.sml.SmlUtils.CellRef;
-
-
 /**
  * <p>Java class for CT_Cell complex type.
  *
@@ -73,32 +69,20 @@ import org.xlsx4j.sml.SmlUtils.CellRef;
     "vm",
     "ph"
 })
-public class Cell implements Child
+public class Cell
 {
+    @XmlTransient
+    protected CellRep cellRep = new CellRep();
+    @XmlTransient
+    private CellDataReadonly cellData = CellDataReadonly.EmptyCell;
 
-    protected CTCellFormula f;
-    protected String v;
-    protected CTRst is;
-    protected CTExtensionList extLst;
-//  @XmlAttribute(name = "r")
-//  protected String r;
-    @XmlTransient
-    protected Integer column;
-    @XmlAttribute(name = "s")
-    @XmlSchemaType(name = "unsignedInt")
-    protected Long s;
-    @XmlAttribute(name = "t")
-    protected STCellType t;
-    @XmlAttribute(name = "cm")
-    @XmlSchemaType(name = "unsignedInt")
-    protected Long cm;
-    @XmlAttribute(name = "vm")
-    @XmlSchemaType(name = "unsignedInt")
-    protected Long vm;
-    @XmlAttribute(name = "ph")
-    protected Boolean ph;
-    @XmlTransient
-    private Object parent;
+    public CellDataReadonly getCellData() {
+    	return cellData;
+    }
+
+    public void setCellData(CellDataReadonly value) {
+    	cellData = value;
+    }
 
     /**
      * Gets the value of the f property.
@@ -108,8 +92,12 @@ public class Cell implements Child
      *     {@link CTCellFormula }
      *
      */
+    @XmlElement
     public CTCellFormula getF() {
-        return f;
+    	if(cellRep instanceof iCellRepF) {
+    		return ((iCellRepF)cellRep).getF();
+    	}
+        return null;
     }
 
     /**
@@ -121,7 +109,18 @@ public class Cell implements Child
      *
      */
     public void setF(CTCellFormula value) {
-        this.f = value;
+    	if(cellRep instanceof iCellRepF) {
+    		((iCellRepF)cellRep).setF(value);
+    	}
+    	else if(value!=null) {
+    		if(cellRep instanceof CellRepV) {
+    			cellRep = new CellRepVF(cellRep);
+    		}
+    		else {
+    			cellRep = new CellRepF(cellRep);
+    		}
+    		((iCellRepF)cellRep).setF(value);
+    	}
     }
 
     /**
@@ -132,8 +131,12 @@ public class Cell implements Child
      *     {@link String }
      *
      */
+    @XmlElement
     public String getV() {
-        return v;
+    	if(cellRep instanceof iCellRepV) {
+    		return ((iCellRepV)cellRep).getV();
+    	}
+        return null;
     }
 
     /**
@@ -145,7 +148,18 @@ public class Cell implements Child
      *
      */
     public void setV(String value) {
-        this.v = value;
+       	if(cellRep instanceof iCellRepV) {
+    		((iCellRepV)cellRep).setV(value);
+    	}
+    	else if(value!=null) {
+    		if(cellRep instanceof CellRepF) {
+    			cellRep = new CellRepVF(cellRep);
+    		}
+    		else {
+    			cellRep = new CellRepV(cellRep);
+    		}
+    		((iCellRepV)cellRep).setV(value);
+    	}
     }
 
     /**
@@ -156,8 +170,9 @@ public class Cell implements Child
      *     {@link CTRst }
      *
      */
+    @XmlElement
     public CTRst getIs() {
-        return is;
+        return cellData.getIs();
     }
 
     /**
@@ -169,7 +184,10 @@ public class Cell implements Child
      *
      */
     public void setIs(CTRst value) {
-        this.is = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setIs(value);
     }
 
     /**
@@ -180,8 +198,9 @@ public class Cell implements Child
      *     {@link CTExtensionList }
      *
      */
+    @XmlElement
     public CTExtensionList getExtLst() {
-        return extLst;
+        return cellData.getExtLst();
     }
 
     /**
@@ -193,24 +212,27 @@ public class Cell implements Child
      *
      */
     public void setExtLst(CTExtensionList value) {
-        this.extLst = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setExtLst(value);
     }
 
     public int getColumn() {
-        return column;
+        return cellRep.getColumn();
     }
-    
-    public void setColumn(int _column) {
-    	column = _column;
+
+    public void setColumn(int column) {
+    	cellRep.setColumn(column);
     }
 
     @XmlAttribute
     public String getR() {
-		return SmlUtils.getCellRef(column, ((Row)parent).getRow());
+    	return cellRep.getR();
     }
 
     public void setR(String r) {
-    	column = SmlUtils.createCellRef(r).getColumn();
+    	cellRep.setR(r);
     }
 
     /**
@@ -221,12 +243,19 @@ public class Cell implements Child
      *     {@link Long }
      *
      */
-    public long getS() {
-        if (s == null) {
-            return  0L;
-        } else {
-            return s;
-        }
+    @XmlAttribute(name = "s")
+    @XmlSchemaType(name = "unsignedInt")
+    public Long getS() {
+        return cellData.getS();
+    }
+
+    @XmlTransient
+    public long getStyle() {
+    	final Long s = cellData.getS();
+    	if(s==null) {
+    		return 0;
+    	}
+    	return s.longValue();
     }
 
     /**
@@ -238,7 +267,10 @@ public class Cell implements Child
      *
      */
     public void setS(Long value) {
-        this.s = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setS(value);
     }
 
     /**
@@ -249,12 +281,9 @@ public class Cell implements Child
      *     {@link STCellType }
      *
      */
+    @XmlAttribute(name = "t")
     public STCellType getT() {
-        if (t == null) {
-            return STCellType.N;
-        } else {
-            return t;
-        }
+    	return cellData.getT();
     }
 
     /**
@@ -266,7 +295,10 @@ public class Cell implements Child
      *
      */
     public void setT(STCellType value) {
-        this.t = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setT(value);
     }
 
     /**
@@ -277,12 +309,10 @@ public class Cell implements Child
      *     {@link Long }
      *
      */
-    public long getCm() {
-        if (cm == null) {
-            return  0L;
-        } else {
-            return cm;
-        }
+    @XmlAttribute(name = "cm")
+    @XmlSchemaType(name = "unsignedInt")
+    public Long getCm() {
+        return cellData.getCm();
     }
 
     /**
@@ -294,7 +324,10 @@ public class Cell implements Child
      *
      */
     public void setCm(Long value) {
-        this.cm = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setCm(value);
     }
 
     /**
@@ -305,12 +338,10 @@ public class Cell implements Child
      *     {@link Long }
      *
      */
-    public long getVm() {
-        if (vm == null) {
-            return  0L;
-        } else {
-            return vm;
-        }
+    @XmlAttribute(name = "vm")
+    @XmlSchemaType(name = "unsignedInt")
+    public Long getVm() {
+    	return cellData.getVm();
     }
 
     /**
@@ -322,7 +353,10 @@ public class Cell implements Child
      *
      */
     public void setVm(Long value) {
-        this.vm = value;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setVm(value);
     }
 
     /**
@@ -333,40 +367,24 @@ public class Cell implements Child
      *     {@link Boolean }
      *
      */
-    public boolean isPh() {
-        if (ph == null) {
-            return false;
-        } else {
-            return ph;
-        }
+    @XmlAttribute(name = "ph")
+    public Boolean isPh() {
+    	return cellData.isPh();
     }
 
     /**
      * Sets the value of the ph property.
      *
+     *     {@link Boolean }
      * @param value
      *     allowed object is
-     *     {@link Boolean }
      *
      */
     public void setPh(Boolean value) {
-        this.ph = value;
-    }
-
-    /**
-     * Gets the parent object in the object tree representing the unmarshalled xml document.
-     *
-     * @return
-     *     The parent object.
-     */
-    @Override
-    public Object getParent() {
-        return this.parent;
-    }
-
-    @Override
-    public void setParent(Object parent) {
-        this.parent = parent;
+    	if(!(cellData instanceof CellDataWritable)) {
+    		cellData = new CellDataWritable(cellData);
+    	}
+    	((CellDataWritable)cellData).setPh(value);
     }
 
     /**
@@ -377,14 +395,7 @@ public class Cell implements Child
      * @param unmarshaller
      *     The unmarshaller that generated the instance.
      */
-    public void afterUnmarshal(Unmarshaller unmarshaller, Object _parent) {
-        setParent(_parent);
-        if(f!=null&&f.getSi()!=null&&_parent instanceof Row) {
-            ((Row)_parent).setSi(f.getSi());
-        }
-        if(column==null) {
-        	column = ((Row)_parent).getLastColumnNumber()+1;
-        }
-    	org.docx4j.jaxb.Context.checkMemory();
+    public void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
+        org.docx4j.jaxb.Context.abortOnLowMemory();
     }
 }
