@@ -62,11 +62,12 @@ import com.openexchange.documentation.RequestMethod;
 import com.openexchange.documentation.annotations.Action;
 import com.openexchange.documentation.annotations.Parameter;
 import com.openexchange.groupware.upload.UploadFile;
-import com.openexchange.office.tools.DocFileHelper;
-import com.openexchange.office.tools.Resource;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.json.tools.RequestDataHelper;
 import com.openexchange.office.tools.StorageHelper;
 import com.openexchange.office.tools.StreamInfo;
+import com.openexchange.office.tools.files.DocFileHelper;
+import com.openexchange.office.tools.resource.Resource;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.encoding.Base64;
 import com.openexchange.tools.session.ServerSession;
@@ -112,12 +113,13 @@ public class AddFileAction extends DocumentFilterAction {
      */
     @Override
     public AJAXRequestResult perform(AJAXRequestData request, ServerSession session) {
-        AJAXRequestResult ajaxResult = null;
+        final RequestDataHelper requestDataHelper = new RequestDataHelper(request);
         final JSONObject jsonObj = new JSONObject();
+        AJAXRequestResult ajaxResult = null;
         Integer crc32 = null;
 
         try {
-            final String p = request.getParameter("add_crc32");
+            final String p = requestDataHelper.getParameter("add_crc32");
             if (p != null) {
                 crc32 = Integer.valueOf((int) (Long.parseLong(p, 16)));
             }
@@ -125,16 +127,16 @@ public class AddFileAction extends DocumentFilterAction {
             //
         }
 
-        final String extension = request.getParameter("add_ext");
-        final String data64 = request.getParameter("add_filedata");
-        final String fileId = request.getParameter("add_fileid");
-        final String folderId = request.getParameter("add_folderid");
+        final String extension = requestDataHelper.getParameter("add_ext");
+        final String data64 = requestDataHelper.getParameter("add_filedata");
+        final String fileId = requestDataHelper.getParameter("add_fileid");
+        final String folderId = requestDataHelper.getParameter("add_folderid");
         byte[] data = null;
 
         if ((null != fileId) && (null != folderId)) {
-            final DocFileHelper fileHelper = new DocFileHelper(m_services, folderId, fileId);
+            final DocFileHelper fileHelper = new DocFileHelper(m_services);
             final StorageHelper storageHelper = new StorageHelper(m_services, session, folderId);
-            final StreamInfo streamInfo = fileHelper.getDocumentStream(session, storageHelper);
+            final StreamInfo streamInfo = fileHelper.getDocumentStream(session, storageHelper, folderId, fileId);
             final InputStream inputStream = streamInfo.getDocumentStream();
             if (null != inputStream) {
                 try {
