@@ -52,10 +52,12 @@ import java.io.ByteArrayInputStream;
 import java.io.InputStream;
 import java.util.HashMap;
 import java.util.Map;
+
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.documentconverter.Feature;
@@ -69,7 +71,6 @@ import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
 import com.openexchange.file.storage.composition.IDBasedFolderAccess;
 import com.openexchange.file.storage.composition.IDBasedFolderAccessFactory;
-import com.openexchange.folderstorage.FolderService;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
@@ -79,29 +80,25 @@ import com.openexchange.office.json.tools.EmptyDocumentCache;
 import com.openexchange.office.json.tools.FilterExceptionToErrorCode;
 import com.openexchange.office.json.tools.OXDocumentHelper;
 import com.openexchange.office.json.tools.OXDocumentHelper.OfficeDocumentTypeProperties;
-import com.openexchange.office.tools.DocFileHelper;
-import com.openexchange.office.tools.DocumentFormatHelper;
-import com.openexchange.office.tools.DocumentFormatHelper.OfficeDocumentType;
-import com.openexchange.office.tools.DocumentMetaData;
-import com.openexchange.office.tools.DocumentType;
-import com.openexchange.office.tools.ErrorCode;
-import com.openexchange.office.tools.ExceptionToErrorCode;
-import com.openexchange.office.tools.FileDescriptor;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FolderHelper;
-import com.openexchange.office.tools.ResourceManager;
 import com.openexchange.office.tools.StorageHelper;
 import com.openexchange.office.tools.StreamInfo;
-import com.openexchange.office.tools.UserConfigurationHelper;
-import com.openexchange.office.tools.UserConfigurationHelper.Mode;
-import com.openexchange.office.tools.htmldoc.ConfHelper;
+import com.openexchange.office.tools.doc.DocumentFormatHelper;
+import com.openexchange.office.tools.doc.DocumentFormatHelper.OfficeDocumentType;
+import com.openexchange.office.tools.doc.DocumentMetaData;
+import com.openexchange.office.tools.doc.DocumentType;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.error.ExceptionToErrorCode;
+import com.openexchange.office.tools.files.DocFileHelper;
+import com.openexchange.office.tools.files.FileDescriptor;
+import com.openexchange.office.tools.files.FileHelper;
+import com.openexchange.office.tools.files.FolderHelper;
 import com.openexchange.office.tools.htmldoc.GenericHtmlDocumentBuilder;
-import com.openexchange.office.tools.htmldoc.TextTableLimits;
 import com.openexchange.office.tools.message.MessagePropertyKey;
 import com.openexchange.office.tools.monitoring.CreateEvent;
 import com.openexchange.office.tools.monitoring.DocumentEvent;
 import com.openexchange.office.tools.monitoring.ErrorType;
 import com.openexchange.office.tools.monitoring.Statistics;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.session.ServerSession;
 
@@ -340,12 +337,9 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
                 // now we have to create the new document file using the input stream
                 errorCode = FileHelper.createFileAndWriteStream(m_services, session, null, file, inputStream);
                 if (!errorCode.isError()) {
-                    UserConfigurationHelper userConfHelper = new UserConfigurationHelper(m_services, session, "io.ox/office", Mode.WRITE_BACK);
-                    final Boolean fastLoad = userConfHelper.getBoolean("text/documentFastLoad", true);
-                    if (fastLoad != null && fastLoad) {
-                        final TextTableLimits textTableLimits = ConfHelper.getTextTableLimits(userConfHelper);
+                    if (GenericHtmlDocumentBuilder.isFastLoadActive(m_services, session)) {
                         // OperationReducer.reduceOperationCount(documentOperations);
-                        htmlDoc = GenericHtmlDocumentBuilder.buildHtmlDocument(documentOperations, file.getId(), file.getFolderId(), file.getVersion(), textTableLimits);
+                        htmlDoc = GenericHtmlDocumentBuilder.buildHtmlDocument(documentOperations, file, m_services, session);
                         defaultDocCache.setDocData(module,
                             defaultDocCache.new EmptyDocData(htmlDoc,
                                 documentOperations.getJSONArray("operations"),
@@ -510,8 +504,8 @@ public class CreateDefaultDocumentAction extends DocumentFilterAction {
         boolean canCreateFilesInFolder = FolderHelper.folderHasCreateAccess(folderAccess, folderId);
         if ((!canWriteToFolder || !canCreateFilesInFolder) && !preserveFileName) {
             // We cannot create files/write to the folder which contains our source template document (Edit as New).
-            // Create the document in the user's default folder instead.
-            folderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+            // Create the document in the user's documents folder instead.
+            folderId = DocFileHelper.getUserDocumentsFolderId(session, m_services);
         }
 
         // create a new file store item with default content
