@@ -75,24 +75,23 @@ import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
 import com.openexchange.file.storage.composition.IDBasedFolderAccess;
 import com.openexchange.file.storage.composition.IDBasedFolderAccessFactory;
-import com.openexchange.folderstorage.FolderService;
 import com.openexchange.groupware.infostore.InfostoreExceptionCodes;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.IImporter;
 import com.openexchange.office.json.tools.DynamicFields;
-import com.openexchange.office.tools.DocFileHelper;
-import com.openexchange.office.tools.DocumentFormatHelper;
-import com.openexchange.office.tools.DocumentType;
-import com.openexchange.office.tools.ErrorCode;
-import com.openexchange.office.tools.ExceptionToErrorCode;
-import com.openexchange.office.tools.FileDescriptor;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FolderHelper;
-import com.openexchange.office.tools.LocalFileMappingManager;
 import com.openexchange.office.tools.MimeTypeHelper;
 import com.openexchange.office.tools.StorageHelper;
 import com.openexchange.office.tools.StreamInfo;
+import com.openexchange.office.tools.doc.DocumentFormatHelper;
+import com.openexchange.office.tools.doc.DocumentType;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.error.ExceptionToErrorCode;
+import com.openexchange.office.tools.files.DocFileHelper;
+import com.openexchange.office.tools.files.FileDescriptor;
+import com.openexchange.office.tools.files.FileHelper;
+import com.openexchange.office.tools.files.FolderHelper;
+import com.openexchange.office.tools.files.LocalFileMappingManager;
 import com.openexchange.office.tools.message.MessagePropertyKey;
 import com.openexchange.office.tools.monitoring.CreateEvent;
 import com.openexchange.office.tools.monitoring.DocumentEvent;
@@ -149,7 +148,7 @@ public class CreateFromTemplateAction extends DocumentFilterAction {
                     }
                 }
                 if (StringUtils.isBlank(folderId)) {
-                    folderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
+                    folderId = DocFileHelper.getUserDocumentsFolderId(session, m_services);
                 }
 
                 if ((null != fileId) && (null != srcFolderId) && (null != fileAccess) && (null != folderId)) {
