@@ -48,21 +48,29 @@
 
 package com.openexchange.office.json.actions;
 
-import static com.openexchange.realtime.util.EncodeUtility.encodeResourcePart;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.concurrent.TimeUnit;
+
+import org.apache.commons.codec.binary.Base64;
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.ajax.container.ByteArrayFileHolder;
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
+import com.openexchange.documentconverter.NonNull;
 import com.openexchange.exception.OXException;
 import com.openexchange.log.LogFactory;
 import com.openexchange.office.tools.SessionUtils;
+import com.openexchange.office.tools.StatusProperties;
+import com.openexchange.office.tools.actions.ActionParameters;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.rt.EncodeUtility;
 import com.openexchange.realtime.dispatch.MessageDispatcher;
 import com.openexchange.realtime.packet.ID;
 import com.openexchange.realtime.packet.Message;
@@ -73,6 +81,7 @@ import com.openexchange.realtime.payload.PayloadTreeNode;
 import com.openexchange.realtime.util.ActionHandler;
 import com.openexchange.realtime.util.ElementPath;
 import com.openexchange.server.ServiceLookup;
+import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSession;
 
 /**
@@ -90,8 +99,6 @@ public abstract class DocumentFilterAction implements AJAXActionService {
 
     private static final Log LOG = LogFactory.getLog(DocumentFilterAction.class);
 
-    static final String m_officeNamespace = "office";
-
     static final String m_operationsSyntheticUser = "operations";
 
     static final String m_errorKey = "hasErrors";
@@ -109,47 +116,71 @@ public abstract class DocumentFilterAction implements AJAXActionService {
     }
 
     /**
-     * @param inputStm
-     * @param fileName
+     * @param request
+     * @param data
+     * @param format
      * @param mimeType
-     * @return
+     * @param fileName (may be null, if fileName should be retrieved from requestt)
+     * @return AJAXRequestResult
      * @throws OXException
      */
-    static protected AJAXRequestResult createFileRequestResult(AJAXRequestData request, byte[] data, String mimeType, String extension) {
+    static protected AJAXRequestResult createFileRequestResult(@NonNull AJAXRequestData request, @NonNull byte[] data,
+            @NonNull String format, @NonNull String mimeType, String fileName) {
+        AJAXRequestResult requestResult = null;
         final String fileId = request.getParameter("id");
         final String fileVersion = request.getParameter("version");
-        final ByteArrayFileHolder fileHolder = new ByteArrayFileHolder(data);
-        String fileName = request.getParameter("filename");
+        String resultFileName = (null != fileName) ? fileName : request.getParameter("filename");
 
-        if (null == fileName) {
-            fileName = "file";
+        if (null == resultFileName) {
+            resultFileName = "file";
         }
 
-        if (null != extension) {
-            fileName = fileName + '.' + extension;
+        if (format.equals("json")) {
+
+            JSONObject jsonObject = new JSONObject();
+            HashMap<String, String> map = new HashMap<String, String>();
+            StringBuilder dataUrlBuilder = new StringBuilder("data:");
+
+            dataUrlBuilder.append(mimeType).append(";base64,").append(Base64.encodeBase64String(data));
+
+            map.put("mimeType", mimeType);
+            map.put("fileName", resultFileName);
+            map.put("dataUrl", dataUrlBuilder.toString());
+            dataUrlBuilder = null;
+
+            try {
+                jsonObject.put("data", new JSONObject(map));
+            } catch (JSONException e) {
+                LOG.error("Exception detected while trying to create json result for request", e);
+            }
+
+            request.setFormat("json");
+            requestResult = new AJAXRequestResult(jsonObject, "json");
+        } else {
+            final ByteArrayFileHolder fileHolder = new ByteArrayFileHolder(data);
+
+            fileHolder.close();
+            fileHolder.setName(resultFileName);
+            fileHolder.setContentType(mimeType);
+
+            requestResult = new AJAXRequestResult(fileHolder, "file");
+
+            setETag(getETag(fileId, fileVersion), 0, requestResult);
         }
 
-        fileHolder.close();
-        fileHolder.setName(fileName);
-        fileHolder.setContentType(mimeType);
-
-        AJAXRequestResult requestResult = new AJAXRequestResult(fileHolder, "file");
-
-        setETag(getETag(fileId, fileVersion), 0, requestResult);
-
         return requestResult;
     }
 
     /** return JSON which contain information about current user session.
      *
-     *  @note	Retrieving session information is implemented very gracefully.
-     *  		It's because the session API seems not well designed and show
-     *  		different behaviour on different installations.
-     *  		Especially on custom SSO (single sign on) installations the
-     *  		source of context information is not (real) clear.
+     *  @note    Retrieving session information is implemented very gracefully.
+     *          It's because the session API seems not well designed and show
+     *          different behavior on different installations.
+     *          Especially on custom SSO (single sign on) installations the
+     *          source of context information is not (real) clear.
      *
-     * 	@param	serverSession [IN]
-     * 			the server session object where we should retrieve all informations from.
+     *     @param    serverSession [IN]
+     *             the server session object where we should retrieve all informations from.
      *
      *  @return a JSON object which contain all session data we need for RT.
      */
@@ -161,17 +192,17 @@ public abstract class DocumentFilterAction implements AJAXActionService {
         {
             final String sessionId = SessionUtils.getSessionId (session);
             final JSONObject jsonSessionDetails
-            	= new JSONObject()
-            		   .put("protocol" , "ox"           )
-            		   .put("component", JSONObject.NULL)
-            		   .put("resource" , sessionId      );
+                = new JSONObject()
+                       .put("protocol" , "ox"           )
+                       .put("component", JSONObject.NULL)
+                       .put("resource" , sessionId      );
 
             jsonSession = new JSONObject().put("session", jsonSessionDetails);
         }
         catch (final Throwable ex)
         {
             LOG.error(ex.getMessage(), ex);
-        	LOG.error(SessionUtils.dumpSessionToString(session));
+            LOG.error(SessionUtils.dumpSessionToString(session));
             throw new RuntimeException ("Not able to retrieve needed information from session.", ex);
         }
 
@@ -188,23 +219,29 @@ public abstract class DocumentFilterAction implements AJAXActionService {
      */
     protected JSONObject sendConnectionMessageSynchronously(AJAXRequestData request, ServerSession session, String methodName, String elementName, JSONObject messageData) {
         final String fileId = request.getParameter("id");
-        final String folderId = request.getParameter("folder_id");
+        final String sharedResId = request.getParameter("com.openexchange.realtime.resourceID");
+        final String appType = request.getParameter("app");
+        final String rtData = request.getParameter(ActionParameters.KEY_RTDATA);
         final JSONObject requestData = getJSONObjectFromServerSession(session);
         JSONObject resultData = null;
         Stanza resultStanza = null;
 
-        if ( ! StringUtils.isEmpty(fileId) && ! StringUtils.isEmpty(folderId)) {
+        // Check valid must-have parameters before creating a rt-message to be sent via real-time
+        if ((StringUtils.isNotEmpty(fileId) || (StringUtils.isNotEmpty(sharedResId))) && StringUtils.isNotEmpty(rtData)) {
             try {
                 final Message message = new Message();
-                final String resource = (new StringBuilder(encodeResourcePart(folderId))).append('.').append(encodeResourcePart(fileId)).toString();
-
+                final String toEncode = StringUtils.isNotEmpty(sharedResId) ? sharedResId : fileId;
+                final String resource = EncodeUtility.encodeResourcePart(toEncode);
+                
                 message.setTo(new ID(
                     "synthetic",
-                    m_officeNamespace,
+                    appType,
                     m_operationsSyntheticUser,
                     SessionUtils.getContextIdString(session),
                     resource));
                 message.addPayload(ActionHandler.getMethodCall(methodName));
+                message.addPayload(new PayloadTree(PayloadTreeNode.builder().withPayload(
+                    new JSONObject(rtData), "json", ActionParameters.KEY_OFFICE, ActionParameters.KEY_RTDATA).build()));
 
                 if (null != messageData) {
                     for (final Iterator<String> keys = messageData.keys(); keys.hasNext();) {
@@ -214,21 +251,29 @@ public abstract class DocumentFilterAction implements AJAXActionService {
                 }
 
                 message.addPayload(new PayloadTree(PayloadTreeNode.builder().withPayload(
-                    new PayloadElement(requestData, "json", m_officeNamespace, elementName)).build()));
+                    new PayloadElement(requestData, "json", appType, elementName)).build()));
 
                 resultStanza = m_services.getService(MessageDispatcher.class).sendSynchronously(message, 60, TimeUnit.SECONDS);
             } catch (Exception e) {
-                //
+                LOG.error("Exception detected while trying to send message to rt-environment", e);
             }
 
             // check result of the RT connection call and return appropriately
             if (null != resultStanza) {
-                Collection<PayloadTree> payloads = resultStanza.getPayloadTrees(new ElementPath(m_officeNamespace, elementName));
+                Collection<PayloadTree> payloads = resultStanza.getPayloadTrees(new ElementPath(appType, elementName));
 
                 if (!payloads.isEmpty()) {
                     resultData = (JSONObject) payloads.iterator().next().getRoot().getData();
                 }
             }
+        } else {
+            resultData = new JSONObject();
+            try {
+                resultData.put(StatusProperties.KEY_ERRORCODE, ErrorCode.GENERAL_ARGUMENTS_ERROR);
+                resultData.put(m_errorKey, true);
+            } catch (JSONException e) {
+                LOG.error("Exception detected while trying to create error result for request", e);
+            }
         }
 
         return resultData;
@@ -287,6 +332,25 @@ public abstract class DocumentFilterAction implements AJAXActionService {
         return getETag(fileId, fileVersion).equals(testETag);
     }
 
+    /**
+     *
+     * @param rtId
+     * @param session
+     * @return
+     */
+    static protected JSONObject createRTData(final String rtId, final Session session) {
+        final JSONObject rtData = new JSONObject();
+
+        try {
+            rtData.put(ActionParameters.KEY_RTID, rtId);
+            rtData.put(ActionParameters.KEY_SESSION, SessionUtils.getSessionId(session));
+        } catch (Exception e) {
+            // TODO: return null or empty object??
+        }
+
+        return rtData;
+    }
+
     // - Members ---------------------------------------------------------------
 
     protected ServiceLookup m_services = null;
