@@ -52,13 +52,12 @@ package com.openexchange.office.json.actions;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
-
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXActionServiceFactory;
 import com.openexchange.office.json.tools.EmptyDocumentCache;
 import com.openexchange.office.json.tools.TemplateFilesScanner;
-import com.openexchange.office.tools.LocalFileMappingManager;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.files.LocalFileMappingManager;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 
 /**
