@@ -53,7 +53,7 @@ package com.openexchange.office.json.actions;
 
 import com.openexchange.office.IExporter;
 import com.openexchange.office.IImporter;
-import com.openexchange.office.tools.DocumentFormat;
+import com.openexchange.office.tools.doc.DocumentFormat;
 import com.openexchange.server.ServiceLookup;
 
 public class DocumentFilterHelper {
