@@ -60,6 +60,7 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
+
 import javax.imageio.IIOImage;
 import javax.imageio.ImageIO;
 import javax.imageio.ImageReader;
@@ -67,29 +68,23 @@ import javax.imageio.ImageWriteParam;
 import javax.imageio.ImageWriter;
 import javax.imageio.stream.ImageInputStream;
 import javax.imageio.stream.ImageOutputStream;
+
 import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.lang.StringUtils;
 
-import com.openexchange.ajax.container.ByteArrayFileHolder;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.ajax.requesthandler.DispatcherNotes;
 import com.openexchange.documentconverter.Feature;
 import com.openexchange.documentconverter.IManager;
 import com.openexchange.documentconverter.Properties;
-import com.openexchange.file.storage.File;
-import com.openexchange.file.storage.FileStorageFileAccess;
-import com.openexchange.file.storage.composition.IDBasedFileAccess;
-import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
 import com.openexchange.office.tools.ConfigurationHelper;
-import com.openexchange.office.tools.DocFileHelper;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.Resource;
-import com.openexchange.office.tools.ResourceManager;
-import com.openexchange.office.tools.SessionUtils;
 import com.openexchange.office.tools.StorageHelper;
 import com.openexchange.office.tools.StreamInfo;
+import com.openexchange.office.tools.files.DocFileHelper;
+import com.openexchange.office.tools.files.FileHelper;
+import com.openexchange.office.tools.resource.Resource;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.session.ServerSession;
 
@@ -142,32 +137,10 @@ public class GetFileAction extends DocumentFilterAction {
     public AJAXRequestResult perform(AJAXRequestData request, ServerSession session) {
         AJAXRequestResult requestResult = null;
         String resourceName = request.getParameter("get_filename");
-        final String fileId = request.getParameter("id");
-        final String folderId = request.getParameter("folder_id");
+        String getFormat  = request.getParameter("get_format");
 
-        if (StringUtils.isEmpty(fileId) || StringUtils.isEmpty(folderId)) {
-            final AJAXRequestResult result = new AJAXRequestResult();
-            result.setHttpStatusCode(400);
-            return result;
-        }
-
-        boolean canRead = false;
-        try {
-            final IDBasedFileAccessFactory fileAccessFactory = m_services.getService(IDBasedFileAccessFactory.class);
-            final IDBasedFileAccess        fileAccess        = fileAccessFactory.createAccess(session);
-            final File                     metaData          = fileAccess.getFileMetadata(fileId, FileStorageFileAccess.CURRENT_VERSION);
-
-            // we assume that we have at least read access for the file if we can get the meta data
-            canRead = (metaData != null);
-        } catch (Exception e) {
-            // do nothing - an exception is interpreted as missing access rights
-            LOG.error("AjaxRequest GetFileAction exception caught while trying to read meta file for requested document images", e);
-        } finally {
-            if (!canRead) {
-                final AJAXRequestResult result = new AJAXRequestResult();
-                result.setHttpStatusCode(403);
-                return result;
-            }
+        if ((null == getFormat) || !getFormat.equals("json")) {
+            getFormat = "file";
         }
 
         if ((null != resourceName) && (resourceName.length() > 0)) {
@@ -208,17 +181,7 @@ public class GetFileAction extends DocumentFilterAction {
 
                 // if we've got the resource data, set it as request result
                 if (null != resdata) {
-                    try {
-                        final String mimeType = DocFileHelper.getMimeType(resourceName);
-                        final ByteArrayFileHolder fileHolder = new ByteArrayFileHolder(resdata);
-
-                        fileHolder.close();
-                        fileHolder.setName(resourceName);
-                        fileHolder.setContentType(mimeType);
-                        requestResult = new AJAXRequestResult(fileHolder, "file");
-                    } catch (Exception e) {
-                        LOG.error("AjaxRequest GetFileAction exception catched while setting data to resultRequest.",  e);
-                    }
+                    requestResult = createFileRequestResult(request, resdata, getFormat, DocFileHelper.getMimeType(resourceName), resourceName);
                 }
             }
 
@@ -228,6 +191,7 @@ public class GetFileAction extends DocumentFilterAction {
 
                 // always use the current version of the document
                 request.putParameter("version", "0");
+                final String folderId = request.getParameter("folder_id");
                 final StorageHelper storageHelper = new StorageHelper(m_services, session, folderId);
                 final StreamInfo streamInfo = DocFileHelper.getDocumentStream(m_services, session, request, storageHelper);
                 final InputStream documentStm = streamInfo.getDocumentStream();
@@ -249,7 +213,7 @@ public class GetFileAction extends DocumentFilterAction {
                         }
 
                         if (null != buffer) {
-                            requestResult = createFileRequestResult(request, buffer, mimeType[0], null);
+                            requestResult = createFileRequestResult(request, buffer, getFormat, mimeType[0], filename);
                             LOG.trace("AjaxRequest GetFileAction resource data " + filename + " successfully retrieved.");
                         } else {
                             LOG.warn("AjaxRequest GetFileAction resource data " + filename + " not found.");
