@@ -1,3 +1,51 @@
+/*
+ *
+ *    OPEN-XCHANGE legal information
+ *
+ *    All intellectual property rights in the Software are protected by
+ *    international copyright laws.
+ *
+ *
+ *    In some countries OX, OX Open-Xchange, open xchange and OXtender
+ *    as well as the corresponding Logos OX Open-Xchange and OX are registered
+ *    trademarks.
+ *    The use of the Logos is not covered by the GNU General Public License.
+ *    Instead, you are allowed to use these Logos according to the terms and
+ *    conditions of the Creative Commons License, Version 2.5, Attribution,
+ *    Non-commercial, ShareAlike, and the interpretation of the term
+ *    Non-commercial applicable to the aforementioned license is published
+ *    on the web site http://www.open-xchange.com/EN/legal/index.html.
+ *
+ *    Please make sure that third-party modules and libraries are used
+ *    according to their respective licenses.
+ *
+ *    Any modifications to this package must retain all copyright notices
+ *    of the original copyright holder(s) for the original code used.
+ *
+ *    After any such modifications, the original and derivative code shall remain
+ *    under the copyright of the copyright holder(s) and/or original author(s)per
+ *    the Attribution and Assignment Agreement that can be located at
+ *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
+ *    given Attribution for the derivative code and a license granting use.
+ *
+ *     Copyright (C) 2016 OX Software GmbH
+ *     Mail: info@open-xchange.com
+ *
+ *
+ *     This program is free software; you can redistribute it and/or modify it
+ *     under the terms of the GNU General Public License, Version 2 as published
+ *     by the Free Software Foundation.
+ *
+ *     This program is distributed in the hope that it will be useful, but
+ *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+ *     for more details.
+ *
+ *     You should have received a copy of the GNU General Public License along
+ *     with this program; if not, write to the Free Software Foundation, Inc., 59
+ *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ *
+ */
 package com.openexchange.office.json.actions;
 
 import java.util.ArrayList;
@@ -5,7 +53,6 @@ import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.json.JSONArray;
@@ -27,9 +74,9 @@ import com.openexchange.groupware.results.TimedResult;
 import com.openexchange.office.json.tools.FileDescriptorHelper;
 import com.openexchange.office.json.tools.TemplateFilesScanner;
 import com.openexchange.office.tools.ApplicationType;
-import com.openexchange.office.tools.ErrorCode;
-import com.openexchange.office.tools.FolderHelper;
 import com.openexchange.office.tools.RecentFileListManager;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.files.FolderHelper;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.iterator.SearchIterator;
 import com.openexchange.tools.session.ServerSession;
@@ -168,7 +215,6 @@ public class GetTemplateAndRecentListAction extends DocumentFilterAction {
 
         // Iterate through the recent file list and check
         // every file.
-        final Map<String, Boolean> withinTrashFolder = new HashMap<String, Boolean>();
         boolean recentListChanged = false;
         long time1 = System.currentTimeMillis();
 
@@ -181,12 +227,12 @@ public class GetTemplateAndRecentListAction extends DocumentFilterAction {
                 JSONObject newFileMetaData = newMetaDataMap.get(id);
                 if (null != newFileMetaData) {
                     String newFolderId = newFileMetaData.optString("folder_id");
+                    FileStorageFolder folder = folderAccess.getFolder(newFolderId);
                     if (folderId.equals(newFolderId)) {
-                        // check if the parent folder or a higher folder was moved to the trash
-                        if (withinTrashFolder.containsKey(folderId) || isFolderInTrashFolder(folderAccess, folderId, withinTrashFolder)) {
+                        // check if the parent folder was moved to the trash
+                        if (FolderHelper.isFolderTrashOrInTrash(folder)) {
                             removedIds.put(id, true);
                             recentListChanged = true;
-                            withinTrashFolder.put(folderId, true);
                         } else if (FileDescriptorHelper.containsChanges(fileObj, newFileMetaData, m_props)) {
                             // no move - just take over the new meta data
                             fileObj.put("filename", newFileMetaData.optString("filename", ""));
@@ -199,12 +245,10 @@ public class GetTemplateAndRecentListAction extends DocumentFilterAction {
                         }
                     } else {
                         // We have a move - check if the new folder is the trash folder
-                        FileStorageFolder folder = folderAccess.getFolder(newFolderId);
-                        if (withinTrashFolder.containsKey(folderId) || FolderHelper.isTrashFolder(folder)) {
+                        if (FolderHelper.isFolderTrashOrInTrash(folder)) {
                             // file has been moved to the trash => remove from recent files
                             removedIds.put(id, true);
                             recentListChanged = true;
-                            withinTrashFolder.put(newFolderId, true);
                         } else {
                             // move to another folder => take over changes
                             fileObj.put("folder_id", newFolderId);
@@ -312,73 +356,6 @@ public class GetTemplateAndRecentListAction extends DocumentFilterAction {
         return newMetaDataMap;
 	}
 
-    /**
-     * Determines if a folder is within the trash folder hierarchy.
-     *
-     * @param folderAccess
-     *  The folder access instance to be used for the check. Must not be null.
-     *
-     * @param folderId
-     *  The folder id that should be checked.
-     *
-     * @return
-     *  TRUE if the folder is part of the trash folder (directly or via a parent),
-     *  otherwise FALSE.
-     */
-    private boolean isFolderInTrashFolder(final IDBasedFolderAccess folderAccess, String folderId, final Map<String, Boolean> cacheTrashIDs) {
-        boolean result = false;
-        FileStorageFolder trashFolder = null;
-
-        try {
-            trashFolder = folderAccess.getTrashFolder(folderId);
-        } catch (OXException e) {
-            // no trash folder
-        }
-
-        if (null != trashFolder) {
-            // we have a trash folder -> check if the given folder is part of this trash folder
-            final String trashFolderId = trashFolder.getId();
-            // store all folders that we check in the hierarchy, so we can just use a map for
-            // faster access to ask again
-            final ArrayList<String> ids = new ArrayList<String>();
-
-            try {
-                while (true) {
-                    if (cacheTrashIDs.containsKey(folderId)) {
-                        return true;
-                    }
-
-                    if (folderId.equalsIgnoreCase(trashFolderId)) {
-                        for (String id : ids) {
-                            cacheTrashIDs.put(id, true);
-                        }
-                        return true;
-                    }
-                    else {
-                        FileStorageFolder folder = folderAccess.getFolder(folderId);
-                        if (!FolderHelper.isRootFolder(folder)) {
-                            folderId = folder.getParentId();
-                            if (null == folderId) {
-                                return false;
-                            } else {
-                                // normal folder must be stored for faster access
-                                ids.add(folderId);
-                            }
-                        } else {
-                            return false;
-                        }
-                    }
-                }
-            } catch (OXException e) {
-                // in case of an exception -> provide true to remove file from recent list
-                result = true;
-                LOG.error("Exception while checking the folder hierarchy", e);
-            }
-        }
-
-        return result;
-    }
-
     /**
      * Put the file meta data as properties into a JSONObject.
      *
