@@ -5,6 +5,7 @@ import org.json.JSONException;
 import org.json.JSONObject;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
+import com.openexchange.office.json.tools.RequestDataHelper;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.session.ServerSession;
 
@@ -45,9 +46,10 @@ public class RenameDocumentAction extends DocumentFilterAction {
     @Override
     public AJAXRequestResult perform(AJAXRequestData request, ServerSession session) {
         final JSONObject requestData = new JSONObject();
+        final RequestDataHelper helper = new RequestDataHelper(request);
 
         try {
-            requestData.put("fileName", request.getParameter("filename"));
+            requestData.put("fileName", helper.getParameter("filename"));
         } catch (JSONException e) {
             //
         }
