@@ -49,7 +49,7 @@
 package com.openexchange.office.json.tools;
 
 import com.openexchange.office.FilterException;
-import com.openexchange.office.tools.ErrorCode;
+import com.openexchange.office.tools.error.ErrorCode;
 
 public class FilterExceptionToErrorCode {
 
