@@ -62,14 +62,14 @@ import com.openexchange.log.Log;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.IImporter;
-import com.openexchange.office.tools.DocumentFormatHelper.OfficeDocumentType;
-import com.openexchange.office.tools.ErrorCode;
-import com.openexchange.office.tools.DocFileHelper;
 import com.openexchange.office.tools.MimeTypeHelper;
-import com.openexchange.office.tools.Resource;
-import com.openexchange.office.tools.ResourceManager;
 import com.openexchange.office.tools.SessionUtils;
+import com.openexchange.office.tools.doc.DocumentFormatHelper.OfficeDocumentType;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.files.DocFileHelper;
 import com.openexchange.office.tools.message.OperationHelper;
+import com.openexchange.office.tools.resource.Resource;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
