@@ -57,6 +57,7 @@ import java.util.HashMap;
 import java.util.LinkedHashSet;
 import java.util.Set;
 
+import org.apache.commons.lang.StringUtils;
 import org.apache.commons.logging.Log;
 import org.json.JSONArray;
 import org.json.JSONException;
@@ -70,20 +71,21 @@ import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
 import com.openexchange.file.storage.composition.IDBasedFolderAccess;
 import com.openexchange.file.storage.composition.IDBasedFolderAccessFactory;
+import com.openexchange.folderstorage.FolderExceptionErrorMessage;
 import com.openexchange.folderstorage.FolderService;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.results.TimedResult;
 import com.openexchange.office.tools.ApplicationType;
 import com.openexchange.office.tools.ConfigurationHelper;
-import com.openexchange.office.tools.DocFileHelper;
-import com.openexchange.office.tools.DocumentType;
-import com.openexchange.office.tools.ErrorCode;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.FolderHelper;
-import com.openexchange.office.tools.LocalFileMappingManager;
 import com.openexchange.office.tools.MimeTypeHelper;
 import com.openexchange.office.tools.UserConfigurationHelper;
 import com.openexchange.office.tools.UserConfigurationHelper.Mode;
+import com.openexchange.office.tools.doc.DocumentType;
+import com.openexchange.office.tools.error.ErrorCode;
+import com.openexchange.office.tools.files.DocFileHelper;
+import com.openexchange.office.tools.files.FileHelper;
+import com.openexchange.office.tools.files.FolderHelper;
+import com.openexchange.office.tools.files.LocalFileMappingManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 import com.openexchange.tools.iterator.SearchIterator;
@@ -187,15 +189,13 @@ public class TemplateFilesScanner {
     /**
      *
      * @return
-     * @throws OXException 
+     * @throws OXException
      */
     public ErrorCode searchForTemplates(ServerSession session, String type, final JSONArray templates) throws OXException {
         ErrorCode errorCode = ErrorCode.NO_ERROR;
 
         if ((null != session) && (null != type) && (type.length() > 0)) {
             DocumentType searchFilter = DocumentType.NONE;
-            IDBasedFileAccess fileAccess = null;
-            IDBasedFolderAccess folderAccess = null;
 
             // We want to measure the time needed to find the templates
             // including to check for folders that were deleted (moved to
@@ -212,53 +212,59 @@ public class TemplateFilesScanner {
 
             final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
             final IDBasedFolderAccessFactory folderFactory = m_services.getService(IDBasedFolderAccessFactory.class);
-            final String userFolderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), session);
             final User user = session.getUser();
             final String fallbackTemplatePath = "/opt/open-xchange/templates/documents";
             String userLangCode = "en"; // fallback if no language is set
 
-            // create file/folder access
-            fileAccess = fileFactory.createAccess(session);
-            folderAccess = folderFactory.createAccess(session);
-
             if (null != user) {
                 userLangCode = DocFileHelper.mapUserLanguageToLangCode(user.getPreferredLanguage());
             }
-            
-            // scan the user folder and collect available document templates
-            ErrorCode userErrorCode = scanUserFolderForTemplates(fileAccess, templates, userFolderId, searchFilter);
-            if (ErrorCode.NO_ERROR == errorCode) {
-                errorCode = userErrorCode;
-            }
-                        
+
             Set<String> templateFolders = new LinkedHashSet<String>();
             //LinkedHashSet keeps the put order
-            
-            getUserData(session.getUserId(), session, "io.ox/office", "UserTemplateFolders", templateFolders);
-            getUserData(session.getContext().getMailadmin(), session, "io.ox/office", "ContextTemplateFolders", templateFolders);
-                       
+
+            //get User's template folder
+            templateFolders.add(ConfigurationHelper.getCascadeConfigurationValue(m_services, session, "io.ox/files", "folder/templates"));
+
+            getUserData(session.getUserId(), session, "io.ox/office", "MyTemplateFolders", templateFolders, true);
+            getUserData(session.getContext().getMailadmin(), session, "io.ox/office", "ContextTemplateFolders", templateFolders, false);
+
             if (null != templateFolders) {
+
+                // create file/folder access
+                IDBasedFileAccess fileAccess = fileFactory.createAccess(session);
+                IDBasedFolderAccess folderAccess = folderFactory.createAccess(session);
+
                 ErrorCode err = null;
                 for (String folderId: templateFolders){
+
                     try{
                         // check if a user configured template folder is in the trash folder
-                        final String parentFolderId = folderAccess.getFolder(folderId).getParentId();
-                        final FileStorageFolder folder = folderAccess.getFolder(parentFolderId);
+                        final FileStorageFolder folder = folderAccess.getFolder(folderId);
 
-                        if (!FolderHelper.isTrashFolder(folder)) {
+                        if (!FolderHelper.isFolderTrashOrInTrash(folder)) {
                             err = scanUserFolderForTemplates(fileAccess, templates, folderId, searchFilter);
                             if (ErrorCode.NO_ERROR != err && ErrorCode.NO_ERROR == errorCode) {
                                 errorCode = err;
                             }
                         }
-                    } catch (OXException e) { 
+                    } catch (OXException e) {
                         // moved try catch inside the loop, because context admin could set template folder from external (for ex.: Dropbox),
-                        // where only he has access to                        
-                        if (e.getCode() == FileStorageExceptionCodes.ACCOUNT_NOT_FOUND.ordinal()) {
-                            LOG.warn("Error user template paths cannot be checked to be inside the trash folder, probably user has no access to this folder in this context", e);
+                        // where only he has access to
+                        if (isErrorCode(e, FileStorageExceptionCodes.ACCOUNT_NOT_FOUND)) {
+                            LOG.warn( "Error user template paths cannot be checked to be inside the trash folder, probably user has no access to this folder in this context");
+                            LOG.debug("Error user template paths cannot be checked to be inside the trash folder, probably user has no access to this folder in this context", e);
+                        } else if (isErrorCode(e, FileStorageExceptionCodes.ADMIN_FILE_ACCESS_NOT_AVAILABLE)) {
+                            LOG.warn( "templatefolder: user has no read access for this globaltemplatefolder: " + folderId);
+                            LOG.debug("templatefolder: user has no read access for this globaltemplatefolder: " + folderId, e);
+                        } else if (isErrorCode(e, FileStorageExceptionCodes.UNKNOWN_FILE_STORAGE_SERVICE)) {
+                            LOG.warn( "templatefolder: user can not read folder. file storage service is only open for user himself and cannot be used as globaltemplatefolder (" + folderId + ")");
+                            LOG.debug("templatefolder: user can not read folder. file storage service is only open for user himself and cannot be used as globaltemplatefolder (" + folderId + ")", e);
+                        } else if (isErrorCode(e, FolderExceptionErrorMessage.FOLDER_NOT_VISIBLE)) {
+                            LOG.warn( "templatefolder: Folder " + folderId + " not visible for user, he cannot use this folder for global templates");
+                            LOG.debug("templatefolder: Folder " + folderId + " not visible for user, he cannot use this folder for global templates", e);
                         } else {
-                            LOG.error("Error user template paths cannot be checked to be inside the trash folder", e);    
-                            break;
+                            LOG.error("Error user template paths cannot be checked to be inside the trash folder", e);
                         }
                     }
                 }
@@ -283,12 +289,12 @@ public class TemplateFilesScanner {
     }
 
     /**
-     * 
+     *
      * with help of the UserConfigurationHelper, userdata is read.
      * whose userdata is decided by userId & contextId
-     * 
+     *
      * id & key defines the data which is put to target-set
-     * 
+     *
      * @param userId
      * @param contextId
      * @param id
@@ -296,16 +302,45 @@ public class TemplateFilesScanner {
      * @param target
      * @throws OXException
      */
-    private void getUserData(int userId, Session realSession, String id, String key, Collection<String> target) throws OXException{
+    private void getUserData(int userId, Session realSession, String id, String key, Collection<String> target, boolean fromUser) throws OXException{
         Session session = new FakeSession(userId, realSession);
         UserConfigurationHelper userConfHelper = new UserConfigurationHelper(m_services, session, id, Mode.WRITE_BACK);
-        final JSONArray array = userConfHelper.getJSONArray(key);
+        JSONArray array = userConfHelper.getJSONArray(key);
         if (null == array){
-            return;
+            if (fromUser) {
+                try{
+                    //Migration
+                    final String userFolderId = DocFileHelper.getUserFolderId(m_services.getService(FolderService.class), realSession);
+                    if (StringUtils.isEmpty(userFolderId)) {
+                        LOG.warn("DocFileHelper.getUserFolderId() returns no UserFolderId" + userFolderId);
+                        return;
+                    }
+                    array = userConfHelper.getJSONArray("UserTemplateFolders");
+                    if (null == array){
+                        array = new JSONArray();
+                    }
+
+                    Set<String> tmpList = new LinkedHashSet<String>();
+                    for (int i = 0; i < array.length(); i++){
+                        tmpList.add(array.getString(i));
+                    }
+                    tmpList.add(userFolderId);
+                    array = new JSONArray(tmpList);
+
+                    userConfHelper.setValue(key, array);
+                    userConfHelper.setValue("UserTemplateFolders", new JSONArray());
+                    userConfHelper.flushCache();
+                }catch (JSONException e){
+                    throw new OXException(e);
+                }
+            } else {
+                return;
+            }
         }
         try{
             for (int i = 0; i < array.length(); i++){
-                target.add(array.getString(i));
+                String folderId = array.getString(i);
+                target.add(folderId);
             }
         }
         catch (JSONException e){
@@ -703,6 +738,14 @@ public class TemplateFilesScanner {
         }
     }
 
+    private static boolean isErrorCode(OXException e, FileStorageExceptionCodes code) {
+        return e.getCode() == code.getNumber() && e.getPrefix().equals(FileStorageExceptionCodes.PREFIX);
+    }
+
+    private static boolean isErrorCode(OXException e, FolderExceptionErrorMessage code) {
+        return e.getCode() == code.getNumber() && e.getPrefix().equals(FolderExceptionErrorMessage.prefix());
+    }
+
     /**
      * Determine if the provided mime type belongs to a binary document
      * template format of MS Office.
@@ -719,4 +762,5 @@ public class TemplateFilesScanner {
                 (MimeTypeHelper.MIMETYPE_MSPOWERPOINT.equalsIgnoreCase(mimeType)) ||
                 (MimeTypeHelper.MIMETYPE_MSWORD.equalsIgnoreCase(mimeType)));
     }
+
 }
