@@ -2,8 +2,8 @@
  *
  *  Copyright (C) 2016 OX Software GmbH All rights reserved.
  *
- *  This program is proprietary software and licensed to you under
- *  OX Software GmbH's Software License Agreement.
+ *  This program is proprietary software and licensed to you under Open-Xchange
+ *  Inc's Software License Agreement.
  *
  *  In some countries OX, OX Open-Xchange, open-xchange and OXtender as well as
  *  the corresponding Logos OX Open-Xchange and OX are registered trademarks.
@@ -14,9 +14,10 @@ package com.openexchange.office.monitoring;
 
 import javax.management.NotCompliantMBeanException;
 import javax.management.StandardMBean;
+
 import com.openexchange.annotation.NonNull;
 import com.openexchange.office.IMonitoring;
-import com.openexchange.office.tools.DocumentType;
+import com.openexchange.office.tools.doc.DocumentType;
 import com.openexchange.office.tools.monitoring.CalcEngineEventType;
 import com.openexchange.office.tools.monitoring.CloseType;
 import com.openexchange.office.tools.monitoring.OpenType;
@@ -331,6 +332,11 @@ public class OfficeMonitoring extends StandardMBean implements IMonitoring {
         return m_statistics.getCalcEngineMedianJobTime();
     }
 
+    @Override
+    public long getCalcEngine_Peak_JobTime() {
+        return m_statistics.getCalcEnginePeakJobTime();
+    }
+
     @Override
     public long getCalcEngine_Documents_Init() {
         return m_statistics.getCalcEngineData(CalcEngineEventType.DOC_INIT);
