@@ -78,7 +78,7 @@ public abstract class TextContainingElement extends OdfStylableElement {
      * given selection had to be dismissed, sometimes even the parent, when an
      * empty element was deleted).
      */
-    OdfElement appendTextSelection(TextSelection outerSelection) {
+    public OdfElement appendTextSelection(TextSelection outerSelection) {
         if (this.mSelections == null) {
             Comparator c = new SelectionComparator();
             mSelections = new TreeSet<TextSelection>(c);
