@@ -46,6 +46,9 @@ import javax.xml.datatype.DatatypeFactory;
 import javax.xml.xpath.XPath;
 import javax.xml.xpath.XPathConstants;
 
+import org.json.JSONArray;
+import org.odftoolkit.odfdom.component.JsonOperationConsumer;
+import org.odftoolkit.odfdom.component.OdfOperationDocument;
 import org.odftoolkit.odfdom.doc.table.OdfTable;
 import org.odftoolkit.odfdom.dom.OdfContentDom;
 import org.odftoolkit.odfdom.dom.OdfDocumentNamespace;
@@ -54,6 +57,7 @@ import org.odftoolkit.odfdom.dom.OdfSchemaConstraint;
 import org.odftoolkit.odfdom.dom.OdfSchemaDocument;
 import org.odftoolkit.odfdom.dom.attribute.text.TextAnchorTypeAttribute;
 import org.odftoolkit.odfdom.dom.element.draw.DrawPageElement;
+import org.odftoolkit.odfdom.dom.element.office.OfficeAnnotationElement;
 import org.odftoolkit.odfdom.dom.element.office.OfficeBodyElement;
 import org.odftoolkit.odfdom.dom.element.table.TableTableCellElement;
 import org.odftoolkit.odfdom.dom.element.table.TableTableElement;
@@ -91,7 +95,7 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 	private static final Pattern CONTROL_CHAR_PATTERN = Pattern.compile("\\p{Cntrl}");
 	private static final String EMPTY_STRING = "";
 	private Calendar mCreationDate;
-	private static final String FORMER_OPEN_OFFICE_VERSION = "StarOffice/8$Win32 OpenOffice.org_project/680m18$Build-9161";			
+	private static final String FORMER_OPEN_OFFICE_VERSION = "StarOffice/8$Win32 OpenOffice.org_project/680m18$Build-9161";
 
 	// Using static factory instead of constructor
 	protected OdfDocument(OdfPackage pkg, String internalPath, OdfMediaType mediaType) throws SAXException {
@@ -134,14 +138,16 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 		/**
 		 * @return the mediatype String of this document
 		 */
-		public String getMediaTypeString() {
+		@Override
+        public String getMediaTypeString() {
 			return mMediaType;
 		}
 
 		/**
 		 * @return the ODF filesuffix of this document
 		 */
-		public String getSuffix() {
+		@Override
+        public String getSuffix() {
 			return mSuffix;
 		}
 
@@ -210,7 +216,7 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 	}
 
 
-	
+
 	/**
 	 * Loads the ODF root document from the ODF package provided by a Stream.
 	 *
@@ -221,15 +227,15 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 	 * overwriting an input file.</p>
 	 *
 	 * @param inStream - the InputStream of the ODF document.
-	 * @param configuration - key/value pairs of user given run-time settings (configuration)	
+	 * @param configuration - key/value pairs of user given run-time settings (configuration)
 	 * @return the document created from the given InputStream
 	 * @throws java.lang.Exception - if the document could not be created.
 	 */
 	public static OdfDocument loadDocument(InputStream inStream, Map<String, Object> configuration) throws Exception {
 		return loadDocument(OdfPackage.loadPackage(inStream, configuration));
 	}
-			
-			
+
+
 	/**
 	 * Loads the ODF root document from the ODF package provided by a Stream.
 	 *
@@ -772,8 +778,8 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 
 			// OpenOffice 3.4.1 needs this metadata to continue list numbering correctly
 			// https://bugs.open-xchange.com/show_bug.cgi?id=28645
-			metaData.setGenerator(FORMER_OPEN_OFFICE_VERSION);				
-			
+			metaData.setGenerator(FORMER_OPEN_OFFICE_VERSION);
+
 			// set creation date
 			if (mCreationDate != null) {
 				getOfficeMetadata().setCreationDate(mCreationDate);
@@ -1094,17 +1100,58 @@ public abstract class OdfDocument extends OdfSchemaDocument {
 			}
 		}
 	}
-	private Set mFontNames = null;
+	private Set<String> mFontNames = null;
+	private Map<String, OfficeAnnotationElement> annotations = null;
 
 	/**
 	 * This is a temporary hack to know about the fonts within the documents. OX
 	 * Client does not provide font information, therefore the 16 fonts are hard
 	 * coded
 	 */
-	public Set getFontNames() {
+	public Set<String> getFontNames() {
 		if (mFontNames == null) {
-			mFontNames = new HashSet();
+			mFontNames = new HashSet<String>();
 		}
 		return mFontNames;
 	}
+
+    public void addAnnotation(String name, OfficeAnnotationElement element) {
+        if(annotations == null){
+            annotations = new HashMap<String, OfficeAnnotationElement>();
+        }
+        annotations.put(name,  element);
+    }
+    public OfficeAnnotationElement getAnnotation(String name) {
+        if(annotations == null || !annotations.containsKey(name)){
+            return null;
+        }
+        return annotations.get(name);
+    }
+    public void removeAnnotation(String name) {
+        if(annotations != null && annotations.containsKey(name)){
+            annotations.remove(name);
+        }
+    }
+    public String getUniqueAnnotationName() {
+        String prefix = "CmtId";
+        int freeIndex = 0;
+        if(annotations != null ){
+            while(annotations.containsKey(prefix + freeIndex)){
+                ++freeIndex;
+            }
+        }
+        return prefix + freeIndex;
+    }
+
+    public int applyOperations(OdfOperationDocument operationDocument, JSONArray ops)
+    	throws Exception {
+
+    	return JsonOperationConsumer.applyOperations(operationDocument, ops);
+    }
+
+    protected void removeCachedView() {
+        mPackage = getPackage();
+        // removes the LO/AO view caching
+        mPackage.remove("Thumbnails/thumbnail.png");
+    }
 }
