@@ -23,12 +23,12 @@
  */
 package org.odftoolkit.odfdom.dom;
 
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.FOOTER_DEFAULT;
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.FOOTER_EVEN;
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.FOOTER_FIRST;
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.HEADER_DEFAULT;
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.HEADER_EVEN;
-import static org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea.HEADER_FIRST;
+import static org.odftoolkit.odfdom.component.PageArea.FOOTER_DEFAULT;
+import static org.odftoolkit.odfdom.component.PageArea.FOOTER_EVEN;
+import static org.odftoolkit.odfdom.component.PageArea.FOOTER_FIRST;
+import static org.odftoolkit.odfdom.component.PageArea.HEADER_DEFAULT;
+import static org.odftoolkit.odfdom.component.PageArea.HEADER_EVEN;
+import static org.odftoolkit.odfdom.component.PageArea.HEADER_FIRST;
 
 import java.io.InputStream;
 import java.util.ArrayList;
@@ -38,10 +38,12 @@ import java.util.Map;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import org.json.JSONException;
 import org.json.JSONObject;
 import org.odftoolkit.odfdom.component.Component;
 import org.odftoolkit.odfdom.component.JsonOperationProducer;
-import org.odftoolkit.odfdom.component.OdfFileSaxHandler.PageArea;
+import org.odftoolkit.odfdom.component.OdfOperationDocument;
+import org.odftoolkit.odfdom.component.PageArea;
 import org.odftoolkit.odfdom.dom.element.office.OfficeBodyElement;
 import org.odftoolkit.odfdom.dom.element.office.OfficeMasterStylesElement;
 import org.odftoolkit.odfdom.dom.element.style.StyleFooterElement;
@@ -50,9 +52,12 @@ import org.odftoolkit.odfdom.dom.element.style.StyleHeaderElement;
 import org.odftoolkit.odfdom.dom.element.style.StyleHeaderLeftElement;
 import org.odftoolkit.odfdom.dom.element.style.StyleMasterPageElement;
 import org.odftoolkit.odfdom.dom.element.table.TableTableElement;
+import org.odftoolkit.odfdom.dom.style.OdfStyleFamily;
 import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeAutomaticStyles;
 import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeMasterStyles;
 import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeStyles;
+import org.odftoolkit.odfdom.incubator.doc.office.OdfStylesBase;
+import org.odftoolkit.odfdom.incubator.doc.style.OdfStyle;
 import org.odftoolkit.odfdom.incubator.doc.style.OdfStylePageLayout;
 import org.odftoolkit.odfdom.pkg.OdfElement;
 import org.odftoolkit.odfdom.pkg.OdfFileDom;
@@ -177,9 +182,10 @@ public abstract class OdfSchemaDocument extends OdfPackageDocument {
         mJsonOperationQueue = queue;
     }
 
-    // ToDo OX - JSONObject is to be considered..
-    public JSONObject getOperations() throws SAXException{
-        JSONObject ops = null;
+    public JSONObject getOperations(OdfOperationDocument operationDoc)
+    	throws SAXException, JSONException {
+
+    	JSONObject ops = null;
         JsonOperationProducer queue = getJsonOperationQueue();
         if (queue == null) {
             try {
@@ -235,7 +241,7 @@ public abstract class OdfSchemaDocument extends OdfPackageDocument {
      * @return the header or footer element belonging to the given master page
      * style
      */
-    public OdfElement getRootComponentElement(String masterStyleName, PageArea pageArea) {
+    public OdfElement getRootComponentElement(String masterStyleName, PageArea pageArea, boolean createIfNotExisting) {
         OdfElement targetElement = null;
         try {
             OdfStylesDom stylesDom = getStylesDom();
@@ -257,9 +263,12 @@ public abstract class OdfSchemaDocument extends OdfPackageDocument {
                         localName = HEADER_DEFAULT.getLocalName();
                     }
                     targetElement = (OdfElement) masterPage.getChildElement(StyleHeaderElement.ELEMENT_NAME.getUri(), localName, 0);
+                    if(targetElement==null) {
+                    	targetElement = (OdfElement)masterPage.getChildElement(StyleHeaderElement.ELEMENT_NAME.getUri(), "header", 0);
+                    }
                     OdfOfficeAutomaticStyles autoStyles = stylesDom.getOrCreateAutomaticStyles();
                     String pageLayoutName = masterPage.getStylePageLayoutNameAttribute();
-                    if (targetElement == null) { // create a new page area
+                    if (targetElement == null && createIfNotExisting) { // create a new page area
                         if (pageArea.equals(HEADER_FIRST)) {
                             // header-first
                             targetElement = OdfXMLFactory.newOdfElement(stylesDom, STYLE_HEADER_FIRST);
@@ -290,9 +299,12 @@ public abstract class OdfSchemaDocument extends OdfPackageDocument {
                         localName = FOOTER_DEFAULT.getLocalName();
                     }
                     targetElement = (OdfElement) masterPage.getChildElement(StyleFooterElement.ELEMENT_NAME.getUri(), localName, 0);
+                    if(targetElement==null) {
+                    	targetElement = (OdfElement)masterPage.getChildElement(StyleFooterElement.ELEMENT_NAME.getUri(), "footer", 0);
+                    }
                     OdfOfficeAutomaticStyles autoStyles = stylesDom.getOrCreateAutomaticStyles();
                     String pageLayoutName = masterPage.getStylePageLayoutNameAttribute();
-                    if (targetElement == null) { // create a new page area
+                    if (targetElement == null && createIfNotExisting) { // create a new page area
                         if (pageArea.equals(FOOTER_FIRST)) {
                             // header-first
                             targetElement = OdfXMLFactory.newOdfElement(stylesDom, STYLE_FOOTER_FIRST);
@@ -554,6 +566,74 @@ public abstract class OdfSchemaDocument extends OdfPackageDocument {
         return mDocumentStyles;
     }
 
+    public OdfStyle getStyleByName(OdfStyleFamily styleFamily, String styleName)
+    	throws SAXException {
+
+    	OdfStyle odfStyle = getStyleByName(getStylesDom(), styleFamily, styleName);
+    	if(odfStyle==null) {
+        	final OdfContentDom odfContentDom = getContentDom();
+        	if(odfContentDom!=null) {
+        		odfStyle = getStyleByName(odfContentDom.getAutomaticStyles(), styleFamily, styleName);
+        	}
+    	}
+    	return odfStyle;
+    }
+
+    private OdfStyle getStyleByName(OdfStylesDom odfStylesDom, OdfStyleFamily styleFamily, String styleName) {
+    	OdfStyle odfStyle = null;
+    	if(odfStylesDom!=null) {
+    		odfStyle = getStyleByName(odfStylesDom.getOfficeStyles(), styleFamily, styleName);
+    		if(odfStyle==null) {
+    			odfStyle = getStyleByName(odfStylesDom.getAutomaticStyles(), styleFamily, styleName);
+    		}
+    	}
+    	return odfStyle;
+    }
+
+    private OdfStyle getStyleByName(OdfStylesBase odfStylesBase, OdfStyleFamily styleFamily, String styleName) {
+    	OdfStyle odfStyle = null;
+		if(odfStylesBase!=null) {
+			odfStyle = odfStylesBase.getStyle(styleName, styleFamily);
+		}
+    	return odfStyle;
+    }
+
+    public OdfStyle getStyleByDisplayName(OdfStyleFamily styleFamily, String styleDisplayName)
+    	throws SAXException {
+
+    	OdfStyle odfStyle = getStyleByDisplayName(getStylesDom(), styleFamily, styleDisplayName);
+    	if(odfStyle==null) {
+    		final OdfContentDom odfContentDom = getContentDom();
+    		if(odfContentDom!=null) {
+    			odfStyle = getStyleByDisplayName(odfContentDom.getAutomaticStyles(), styleFamily, styleDisplayName);
+    		}
+    	}
+    	return odfStyle;
+    }
+
+    private OdfStyle getStyleByDisplayName(OdfStylesDom odfStylesDom, OdfStyleFamily styleFamily, String styleName) {
+    	OdfStyle odfStyle = null;
+    	if(odfStylesDom!=null) {
+    		odfStyle = getStyleByDisplayName(odfStylesDom.getOfficeStyles(), styleFamily, styleName);
+    		if(odfStyle==null) {
+    			odfStyle = getStyleByDisplayName(odfStylesDom.getAutomaticStyles(), styleFamily, styleName);
+    		}
+    	}
+    	return odfStyle;
+    }
+
+    private OdfStyle getStyleByDisplayName(OdfStylesBase odfStylesBase, OdfStyleFamily styleFamily, String styleName) {
+		if(odfStylesBase!=null) {
+			for(OdfStyle odfStyle:odfStylesBase.getStylesForFamily(styleFamily)) {
+				final String displayName = odfStyle.getStyleDisplayNameAttribute();
+				if(displayName!=null&&displayName.equals(styleName)) {
+					return odfStyle;
+				}
+			}
+		}
+    	return null;
+    }
+
     /**
      * Return a list of table features in this document.
      *
