@@ -41,13 +41,10 @@ import javax.xml.xpath.XPathFactory;
 
 import org.apache.xerces.dom.DocumentImpl;
 import org.odftoolkit.odfdom.dom.OdfContentDom;
-import org.odftoolkit.odfdom.dom.OdfMetaDom;
 import org.odftoolkit.odfdom.dom.OdfSchemaConstraint;
 import org.odftoolkit.odfdom.dom.OdfSchemaDocument;
-import org.odftoolkit.odfdom.dom.OdfSettingsDom;
 import org.odftoolkit.odfdom.dom.OdfStylesDom;
 import org.w3c.dom.DOMException;
-import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
@@ -56,6 +53,8 @@ import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
 import org.xml.sax.helpers.DefaultHandler;
 
+import com.openexchange.office.FilterException;
+
 /**
  * The DOM representation of an XML file within the ODF document.
  */
@@ -107,74 +106,42 @@ public class OdfFileDom extends DocumentImpl implements NamespaceContext {
         pkg.cacheDom(this, internalPath);
     }
 
-    /**
-     * @throws a SAXException if the content.xml or styles.xml files could not
-     * be parsed, although existent.
-     */
-    public static OdfFileDom newFileDom(OdfPackageDocument packageDocument, String packagePath) throws SAXException {
-        OdfFileDom newFileDom = null;
-        // before creating a new dom, make sure that there no DOM opened for this file already
-        Document existingDom = packageDocument.getPackage().getCachedDom(packagePath);
-        if (existingDom == null) {
-            // ToDo: bug 264 - register OdfFileDom to this class
-            if (packagePath.equals("content.xml") || packagePath.endsWith("/content.xml")) {
-                newFileDom = new OdfContentDom((OdfSchemaDocument) packageDocument, packagePath);
-            } else if (packagePath.equals("styles.xml") || packagePath.endsWith("/styles.xml")) {
-                newFileDom = new OdfStylesDom((OdfSchemaDocument) packageDocument, packagePath);
-            } else if (packagePath.equals("meta.xml") || packagePath.endsWith("/meta.xml")) {
-                newFileDom = new OdfMetaDom((OdfSchemaDocument) packageDocument, packagePath);
-            } else if (packagePath.equals("settings.xml") || packagePath.endsWith("/settings.xml")) {
-                newFileDom = new OdfSettingsDom((OdfSchemaDocument) packageDocument, packagePath);
-            } else {
-                newFileDom = new OdfFileDom(packageDocument, packagePath);
-            }
-        } else {
-            if (existingDom instanceof OdfFileDom) {
-                newFileDom = (OdfFileDom) existingDom;
-//ToDO: Issue 264 - Otherwise if NOT an OdfFileDom serialize old DOM AND CREATE A NEW ONE?!
-// Or shall we always reference to the dom, than we can not inherit from Document? Pro/Con?s
-//			}else{
-//				// Create an OdfFileDOM from an existing DOM
-//				newFileDom =
-            }
-
-        }
-        return newFileDom;
-    }
-
-    protected void initialize(DefaultHandler handler, OdfFileDom dom) throws SAXException {
+    protected void initialize(DefaultHandler handler, XMLReader xmlReader, OdfFileDom dom) throws SAXException {
         InputStream fileStream = null;
         try {
             fileStream = mPackage.getInputStream(mPackagePath);
             if (fileStream != null) {
-                XMLReader xmlReader = mPackage.getXMLReader();
                 xmlReader.setContentHandler(handler);
                 InputSource xmlSource = new InputSource(fileStream);
                 xmlReader.parse(xmlSource);
             }
 
-        } catch (Exception ex) {
-            Logger.getLogger(OdfPackageDocument.class.getName()).log(Level.SEVERE, null, ex);
-            if ( ex instanceof SAXException && ((SAXException)ex).getMessage().equals("Too many sheets")) {
-                throw (SAXException)ex;
-            }
-            else if (dom instanceof OdfContentDom || dom instanceof OdfStylesDom) {
-                String fileName;
-                if (dom instanceof OdfContentDom) {
-                    fileName = OdfSchemaDocument.OdfXMLFile.STYLES.getFileName();
-                } else {
-                    fileName = OdfSchemaDocument.OdfXMLFile.CONTENT.getFileName();
-                }
-                OdfValidationException ve = new OdfValidationException(OdfSchemaConstraint.DOCUMENT_WITH_EXISTENT_BUT_UNREADABLE_CONTENT_OR_STYLES_XML, mPackage.getBaseURI(), ex, fileName);
-                ErrorHandler eh = mPackage.getErrorHandler();
-                if (eh != null) {
-                    eh.fatalError(ve);
-                }
-                throw ve;
-            } else if (ex instanceof SAXParseException) {
-                throw (SAXParseException) ex;
-            }
-        } finally {
+        }
+        catch (Exception ex) {
+        	if(ex instanceof FilterException) {
+        		throw (FilterException)ex;
+        	}
+        	else {
+	            Logger.getLogger(OdfPackageDocument.class.getName()).log(Level.SEVERE, null, ex);
+	            if (dom instanceof OdfContentDom || dom instanceof OdfStylesDom) {
+	                String fileName;
+	                if (dom instanceof OdfContentDom) {
+	                    fileName = OdfSchemaDocument.OdfXMLFile.STYLES.getFileName();
+	                } else {
+	                    fileName = OdfSchemaDocument.OdfXMLFile.CONTENT.getFileName();
+	                }
+	                OdfValidationException ve = new OdfValidationException(OdfSchemaConstraint.DOCUMENT_WITH_EXISTENT_BUT_UNREADABLE_CONTENT_OR_STYLES_XML, mPackage.getBaseURI(), ex, fileName);
+	                ErrorHandler eh = mPackage.getErrorHandler();
+	                if (eh != null) {
+	                    eh.fatalError(ve);
+	                }
+	                throw ve;
+	            } else if (ex instanceof SAXParseException) {
+	                throw (SAXParseException) ex;
+	            }
+        	}
+        }
+        finally {
             try {
                 if (fileStream != null) {
                     fileStream.close();
@@ -186,7 +153,7 @@ public class OdfFileDom extends DocumentImpl implements NamespaceContext {
     }
 
     protected void initialize() throws SAXException {
-        initialize(new FileSaxHandler(this), this);
+    	initialize(new FileSaxHandler(this), mPackage.getXMLReader(), this);
     }
 
     /**
