@@ -51,6 +51,7 @@ import java.util.regex.Pattern;
 import java.util.zip.CRC32;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipOutputStream;
+
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
@@ -59,17 +60,24 @@ import javax.xml.parsers.SAXParserFactory;
 import javax.xml.transform.TransformerConfigurationException;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.URIResolver;
-import org.apache.xerces.dom.DOMXSImplementationSourceImpl;
+
+import org.apache.xerces.dom.DOMOutputImpl;
+import org.apache.xml.serializer.SerializationHandler;
+import org.odftoolkit.odfdom.IElementWriter;
 import org.odftoolkit.odfdom.doc.OdfDocument;
 import org.odftoolkit.odfdom.doc.OdfDocument.OdfMediaType;
 import org.odftoolkit.odfdom.pkg.manifest.Algorithm;
 import org.odftoolkit.odfdom.pkg.manifest.EncryptionData;
 import org.odftoolkit.odfdom.pkg.manifest.KeyDerivation;
 import org.odftoolkit.odfdom.pkg.manifest.OdfFileEntry;
+//import org.odftoolkit.odfdom.pkg.serializer.SerializationHandler;
+//import org.odftoolkit.odfdom.pkg.serializer.Serializer;
+//import org.odftoolkit.odfdom.pkg.serializer.dom3.LSSerializerImpl;
 import org.w3c.dom.Document;
-import org.w3c.dom.ls.DOMImplementationLS;
+import org.w3c.dom.Node;
 import org.w3c.dom.ls.LSOutput;
-import org.w3c.dom.ls.LSSerializer;
+import org.w3c.dom.ls.LSParserFilter;
+import org.w3c.dom.ls.LSSerializerFilter;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
@@ -178,6 +186,20 @@ public class OdfPackage implements Closeable {
 		}
 	}
 
+	// is called if a low memory notification was received... then its tried to free as much memory as possible
+	public void freeMemory() {
+		mZipFile = null;
+		mResolver = null;
+		mZipEntries = null;
+		mOriginalZipEntries = null;
+		mManifestEntries = null;
+		mPkgDocuments = null;
+		mPkgDoms = null;
+		mMemoryFileCache = null;
+		mConfiguration = null;
+		mErrorHandler = null;
+	}
+
 	/**
 	 * Creates an OdfPackage from the OpenDocument provided by a File.
 	 *
@@ -396,7 +418,7 @@ public class OdfPackage implements Closeable {
 	 * Run-time configuration such as special logging or maximum table size to create operations from are stored in this map.
 	 * @return key/value pairs of user given run-time settings (configuration)
 	 */
-	public Map getRunTimeConfiguration(){
+	public Map<String, Object> getRunTimeConfiguration(){
 		return mConfiguration;
 	}
 
@@ -736,7 +758,7 @@ public class OdfPackage implements Closeable {
 	 *		path relative to the package root, where the document should be inserted.
 	 * @return an already open W3C XML Documenet via its path, otherwise NULL.
 	 */
-	Document getCachedDom(String internalPath) {
+	public Document getCachedDom(String internalPath) {
 		internalPath = normalizeFilePath(internalPath);
 		return this.mPkgDoms.get(internalPath);
 	}
@@ -1094,7 +1116,7 @@ public class OdfPackage implements Closeable {
 		}
 	}
 
-	XMLReader getXMLReader() throws SAXException {
+	public XMLReader getXMLReader() throws SAXException {
 		// create sax parser
 		SAXParserFactory saxFactory = new org.apache.xerces.jaxp.SAXParserFactoryImpl();
 		saxFactory.setNamespaceAware(true);
@@ -1713,6 +1735,35 @@ public class OdfPackage implements Closeable {
 		return data;
 	}
 
+	private class PackageFilter implements LSSerializerFilter {
+
+		private final SerializationHandler hdl;
+		private int val = 0;
+		public PackageFilter(SerializationHandler o) {
+			hdl = o;
+		}
+
+		@Override
+		public short acceptNode(Node nodeArg) {
+			if(nodeArg instanceof IElementWriter) {
+				// acceptNode is called twice (beginNode and endNode) ... we will
+				// only write the object once
+				if(((++val)&1)!=0) {
+					((IElementWriter)nodeArg).writeObject(hdl);
+				}
+				return LSParserFilter.FILTER_REJECT;
+			}
+			return 0;
+		}
+
+		@Override
+		public int getWhatToShow() {
+			// TODO Auto-generated method stub
+			return LSSerializerFilter.SHOW_ELEMENT;
+		}
+
+	}
+		
 	// Serializes a DOM tree into a byte array.
 	// Providing the counterpart of the generic Namespace handling of OdfFileDom.
 	private byte[] flushDom(Document dom) {
@@ -1728,10 +1779,15 @@ public class OdfPackage implements Closeable {
 			}
 		}
 		ByteArrayOutputStream baos = new ByteArrayOutputStream();
-		DOMXSImplementationSourceImpl dis = new org.apache.xerces.dom.DOMXSImplementationSourceImpl();
-		DOMImplementationLS impl = (DOMImplementationLS) dis.getDOMImplementation("LS");
-		LSSerializer writer = impl.createLSSerializer();
-		LSOutput output = impl.createLSOutput();
+
+		// DOMXSImplementationSourceImpl dis = new org.apache.xerces.dom.DOMXSImplementationSourceImpl();
+		// DOMImplementationLS impl = new org.odftoolkit.odfdom.pkg.serializer.dom3.LSSerializerImpl(null);
+		//(DOMImplementationLS) dis.getDOMImplementation("LS");
+		
+		// creating our own LSSerializerImpl as we need access to the SerializationHandler
+		final LSSerializerImpl writer = new org.odftoolkit.odfdom.pkg.LSSerializerImpl();
+		final LSOutput output = new DOMOutputImpl();
+		writer.setFilter(new PackageFilter(writer.getSerializationHandler()));
 		output.setByteStream(baos);
 		writer.write(dom, output);
 		return baos.toByteArray();
