@@ -49,17 +49,12 @@
 
 package com.openexchange.office.odf;
 
-import java.io.IOException;
 import java.io.InputStream;
 
-import org.apache.commons.logging.Log;
 import org.json.JSONObject;
-import org.xml.sax.SAXException;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
-import com.openexchange.office.FilterException.ErrorCode;
 import com.openexchange.office.IImporter;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
@@ -71,8 +66,6 @@ import com.openexchange.session.Session;
  */
 public class Importer implements IImporter {
 
-    private final static Log LOG = LogFactory.getLog(Importer.class);
-
     final ServiceLookup services;
 
     /**
@@ -91,12 +84,9 @@ public class Importer implements IImporter {
     public JSONObject createOperations(Session session, InputStream inputDocumentStm, DocumentProperties documentProperties) throws FilterException {
 
         JSONObject result = null;
-
         if (null != inputDocumentStm) {
             OdfOperationDoc odfdomDocument = null;
-
             try {
-
                 odfdomDocument = new OdfOperationDoc(session, services, inputDocumentStm, null, documentProperties);
             	// Get uniqueDocumentID when existing otherwise set one..
         		String existingDocumentStatusID = odfdomDocument.readStatusID();
@@ -105,36 +95,17 @@ public class Importer implements IImporter {
         			documentProperties.put(DocumentProperties.PROP_UNIQUE_DOCUMENT_IDENTIFIER, existingDocumentStatusID);
         		}
             	result = odfdomDocument.getOperations();
-
-
-                /*
-                 * document size is no longer needed, the editor is calculating loading time on its own // add documentSize element to
-                 * result if (null != result) { final long documentSize = odfdomDocument.getContentSize(); result.put("documentSize",
-                 * documentSize <= 0 ? -1 : documentSize); }
-                 */
-            } catch (Throwable e) {
-                LOG.error("ODF Filter could not load the given document:", e);
-                if (e instanceof SAXException) {
-                    throw new FilterException("Too many sheets", FilterException.ErrorCode.COMPLEXITY_TOO_HIGH);
-                } else if (e instanceof java.util.zip.ZipException && e.getMessage().contains(
-                    "only DEFLATED entries can have EXT descriptor")) {
-                    throw new FilterException(e, ErrorCode.UNSUPPORTED_ENCRYPTION_USED);
-                } else {
-                    throw new FilterException(e, ErrorCode.CRITICAL_ERROR);
-                }
-            } finally {
-                try {
-                    inputDocumentStm.close();
-                } catch (IOException e) {
-                    LOG.error("Document stream could not be closed:", e);
-                }
-
-                if (null != odfdomDocument) {
+            }
+            catch (Throwable e) {
+            	throw OdfOperationDoc.getRethrowException(e);
+            }
+            finally {
+            	if (null != odfdomDocument) {
                     odfdomDocument.close();
+                    odfdomDocument.removeMemoryListener();
                 }
             }
         }
-
         return result;
     }
 
