@@ -68,6 +68,7 @@ import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.PartName;
 import org.docx4j.openpackaging.parts.PeoplePart;
 import org.docx4j.openpackaging.parts.ThemePart;
+import org.docx4j.openpackaging.parts.WordprocessingML.CommentsPart;
 import org.docx4j.openpackaging.parts.WordprocessingML.DocumentSettingsPart;
 import org.docx4j.openpackaging.parts.WordprocessingML.FontTablePart;
 import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
@@ -79,6 +80,8 @@ import org.docx4j.relationships.Relationships;
 import org.docx4j.w15.CTPeople;
 import org.docx4j.wml.CTLanguage;
 import org.docx4j.wml.CTSettings;
+import org.docx4j.wml.Comments;
+import org.docx4j.wml.Comments.Comment;
 import org.docx4j.wml.ContentAccessor;
 import org.docx4j.wml.DocDefaults;
 import org.docx4j.wml.DocDefaults.RPrDefault;
@@ -89,6 +92,7 @@ import org.docx4j.wml.P;
 import org.docx4j.wml.R;
 import org.docx4j.wml.RPr;
 import org.docx4j.wml.SectPr;
+import org.docx4j.wml.SectPr.PgMar;
 import org.docx4j.wml.Styles;
 import org.docx4j.wml.Styles.LatentStyles;
 import org.docx4j.wml.Styles.LatentStyles.LsdException;
@@ -100,14 +104,15 @@ import org.json.JSONTokener;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.office.ooxml.docx.operations.ApplyOperationHelper;
 import com.openexchange.office.ooxml.docx.operations.CreateOperationHelper;
 import com.openexchange.office.ooxml.docx.tools.Component;
 import com.openexchange.office.ooxml.docx.tools.Component.ParagraphComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.TextRun_Base;
 import com.openexchange.office.ooxml.docx.tools.Component.TrComponent;
+import com.openexchange.office.ooxml.docx.tools.Utils;
 import com.openexchange.office.tools.ConfigurationHelper;
-import com.openexchange.office.tools.ResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
@@ -115,7 +120,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 
     private WordprocessingMLPackage opcPackage;
 
-    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, ResourceManager _resourceManager, DocumentProperties _documentProperties)
+    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, IResourceManager _resourceManager, DocumentProperties _documentProperties)
         throws FilterException {
 
         super(session, _services, _inputDocumentStream, _resourceManager, _documentProperties);
@@ -139,12 +144,27 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         try {
             opcPackage = WordprocessingMLPackage.createPackage();
 
-            MainDocumentPart mainDocumentPart = getPackage().getMainDocumentPart();
-            Styles styles = mainDocumentPart.getStyleDefinitionsPart().getJaxbElement();
+            final MainDocumentPart mainDocumentPart = getPackage().getMainDocumentPart();
+            final Styles styles = mainDocumentPart.getStyleDefinitionsPart().getJaxbElement();
             resetDocumentLanguage(styles);
             insertPredefinedLatentStyles(styles);
-            ThemePart themePart = getDefaultThemePart(new PartName("/word/theme/theme1.xml"));
+            final ThemePart themePart = getDefaultThemePart(new PartName("/word/theme/theme1.xml"));
             mainDocumentPart.addTargetPart(themePart);
+            final DocumentSettingsPart documentSettingsPart = getDefaultSettingsPart(new PartName("/word/settings.xml"));
+            mainDocumentPart.addTargetPart(documentSettingsPart);
+            final SectPr sectPr = Utils.getDocumentProperties(mainDocumentPart, true);
+            PgMar pgMar = sectPr.getPgMar();
+            if(pgMar==null) {
+            	pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+            	pgMar.setParent(sectPr);
+            	sectPr.setPgMar(pgMar);
+            }
+            pgMar.setLeft(BigInteger.valueOf(1440));
+            pgMar.setRight(BigInteger.valueOf(1440));
+            pgMar.setTop(BigInteger.valueOf(1440));
+            pgMar.setBottom(BigInteger.valueOf(1440));
+            pgMar.setHeader(BigInteger.valueOf(708));
+            pgMar.setFooter(BigInteger.valueOf(708));
 
             // creating a fontlist (QuickOffice won't load documents without this part)
             getFontList(true);
@@ -159,6 +179,14 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         if(target.isEmpty()) {
             setContextPart(null);
         }
+        else if (target.startsWith("cmt")) {
+        	// not to be allowed to be zero... a comment has to be inserted
+        	// before it could be used, so this part must be there
+     		final CommentsPart commentsPart = getPackage().getMainDocumentPart().getCommentsPart();
+     		final Comments comments = commentsPart.getJaxbElement();
+     		comments.setCommentId(BigInteger.valueOf(Integer.valueOf(target.substring(3))));
+     		setContextPart(commentsPart);
+        }
         else {
             setContextPart(getPackage().getMainDocumentPart().getRelationshipsPart(false).getPart(target));
         }
@@ -174,58 +202,132 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                 ApplyOperationHelper applyOperationHelper = new ApplyOperationHelper(this);
                 final JSONArray aOperations = new JSONArray(new JSONTokener(applyOperations));
                 for (i = 0; i < aOperations.length(); i++) {
+                	successfulAppliedOperations = i;
                     op = (JSONObject) aOperations.get(i);
                     setContext(op.optString("target", ""));
                     final String opName = op.getString("name");
-                    if (opName.equals("insertParagraph"))
-                        applyOperationHelper.insertParagraph(op.getJSONArray("start"), op.optJSONObject("attrs"));
-                    else if (opName.equals("delete"))
-                        applyOperationHelper.delete(op.getJSONArray("start"), op.optJSONArray("end"));
-                    else if (opName.equals("move"))
-                        applyOperationHelper.move(op.getJSONArray("start"), op.optJSONArray("end"), op.getJSONArray("to"));
-                    else if (opName.equals("splitParagraph"))
-                        applyOperationHelper.splitParagraph(op.getJSONArray("start"));
-                    else if (opName.equals("mergeParagraph"))
-                        applyOperationHelper.mergeParagraph(op.getJSONArray("start"));
-                    else if (opName.equals("insertText"))
-                        applyOperationHelper.insertText(op.getJSONArray("start"), op.getString("text").replaceAll("\\p{C}", " "), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertTab"))
-                        applyOperationHelper.insertTab(op.getJSONArray("start"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertHardBreak"))
-                        applyOperationHelper.insertHardBreak(op.getJSONArray("start"), op.optString("type"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertTable"))
-                        applyOperationHelper.insertTable(op.getJSONArray("start"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertRows"))
-                        applyOperationHelper.insertRows(op.getJSONArray("start"), op.optInt("count", 1), op.optBoolean("insertDefaultCells", false), op.optInt("referenceRow", -1), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertCells"))
-                        applyOperationHelper.insertCells(op.getJSONArray("start"), op.optInt("count", 1), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertColumn"))
-                        applyOperationHelper.insertColumn(op.getJSONArray("start"), op.getJSONArray("tableGrid"), op.getInt("gridPosition"), op.optString("insertMode", "before"));
-                    else if (opName.equals("deleteColumns"))
-                        applyOperationHelper.deleteColumns(op.getJSONArray("start"), op.getInt("startGrid"), op.optInt("endGrid", op.getInt("startGrid")));
-                    else if (opName.equals("setAttributes"))
-                        applyOperationHelper.setAttributes(op.getJSONObject("attrs"), op.getJSONArray("start"), op.optJSONArray("end"));
-                    else if (opName.equals("insertField"))
-                        applyOperationHelper.insertField(op.getJSONArray("start"), op.getString("type"), op.getString("representation"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertDrawing"))
-                        applyOperationHelper.insertDrawing(op.getJSONArray("start"), op.getString("type"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertStyleSheet"))
-                        applyOperationHelper.insertStyleSheet(op.getString("type"), op.getString("styleId"), op.getString("styleName"), op.getJSONObject("attrs"), op.optString("parent"), op.optBoolean("hidden"), op.getInt("uiPriority"), op.optBoolean("default"));
-                    else if (opName.equals("deleteStylesheet"))
-                        applyOperationHelper.deleteStylesheet(op.getString("type"), op.getString("styleId"));
-                    else if (opName.equals("insertListStyle"))
-                        applyOperationHelper.insertListStyle(op.getString("listStyleId"), op.getJSONObject("listDefinition"));
-                    else if (opName.equals("splitTable"))
-                    	applyOperationHelper.splitTable(op.getJSONArray("start"));
-                    else if (opName.equals("mergeTable"))
-                    	applyOperationHelper.mergeTable(op.getJSONArray("start"));
-                    else if (opName.equals("setDocumentAttributes"))
-                    	applyOperationHelper.setDocumentAttributes(op.getJSONObject("attrs"));
-                    else if (opName.equals("createError"))
-                        throw new FilterException("createError operation detected: " + opName, ErrorCode.UNSUPPORTED_OPERATION_USED);
-                    else
-                        logMessage("warn", "Ignoring unsupported operation: " + opName);
+                    switch(opName) {
+                    	case "insertParagraph": {
+                        	applyOperationHelper.insertParagraph(op.getJSONArray("start"), op.optJSONObject("attrs"));
+                        	break;
+                    	}
+                    	case "delete": {
+                    		applyOperationHelper.delete(op.getJSONArray("start"), op.optJSONArray("end"));
+                    		break;
+                    	}
+                    	case "move": {
+                    		applyOperationHelper.move(op.getJSONArray("start"), op.optJSONArray("end"), op.getJSONArray("to"));
+                    		break;
+                    	}
+                    	case "splitParagraph": {
+                    		applyOperationHelper.splitParagraph(op.getJSONArray("start"));
+                    		break;
+                    	}
+                    	case "mergeParagraph": {
+                    		applyOperationHelper.mergeParagraph(op.getJSONArray("start"));
+                    		break;
+                    	}
+                    	case "insertText": {
+                    		applyOperationHelper.insertText(op.getJSONArray("start"), op.getString("text").replaceAll("\\p{C}", " "), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertTab": {
+                    		applyOperationHelper.insertTab(op.getJSONArray("start"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertHardBreak": {
+                    		applyOperationHelper.insertHardBreak(op.getJSONArray("start"), op.optString("type"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertTable": {
+                    		applyOperationHelper.insertTable(op.getJSONArray("start"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertRows": {
+                    		applyOperationHelper.insertRows(op.getJSONArray("start"), op.optInt("count", 1), op.optBoolean("insertDefaultCells", false), op.optInt("referenceRow", -1), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertCells": {
+                    		applyOperationHelper.insertCells(op.getJSONArray("start"), op.optInt("count", 1), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertColumn": {
+                    		applyOperationHelper.insertColumn(op.getJSONArray("start"), op.getJSONArray("tableGrid"), op.getInt("gridPosition"), op.optString("insertMode", "before"));
+                    		break;
+                    	}
+                    	case "deleteColumns": {
+                    		applyOperationHelper.deleteColumns(op.getJSONArray("start"), op.getInt("startGrid"), op.optInt("endGrid", op.getInt("startGrid")));
+                    		break;
+                    	}
+                    	case "setAttributes": {
+                    		applyOperationHelper.setAttributes(op.getJSONObject("attrs"), op.getJSONArray("start"), op.optJSONArray("end"));
+                    		break;
+                    	}
+                    	case "insertField": {
+                    		applyOperationHelper.insertField(op.getJSONArray("start"), op.getString("type"), op.getString("representation"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertDrawing": {
+                    		applyOperationHelper.insertDrawing(op.getJSONArray("start"), op.getString("type"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertStyleSheet": {
+                    		applyOperationHelper.insertStyleSheet(op.getString("type"), op.getString("styleId"), op.getString("styleName"), op.getJSONObject("attrs"), op.optString("parent"), op.optBoolean("hidden"), op.getInt("uiPriority"), op.optBoolean("default"), op.optBoolean("custom"));
+                    		break;
+                    	}
+                    	case "changeStyleSheet": {
+                    		applyOperationHelper.changeStyleSheet(op.getString("type"), op.getString("styleId"), op.optString("styleName"), op.optJSONObject("attrs"), op.optString("parent"), op.optBoolean("hidden"), op.optInt("uiPriority"));
+                    		break;
+                    	}
+                    	case "deleteStyleSheet": {
+                    		applyOperationHelper.deleteStyleSheet(op.getString("type"), op.getString("styleId"));
+                    		break;
+                    	}
+                    	case "insertListStyle": {
+                    		applyOperationHelper.insertListStyle(op.getString("listStyleId"), op.getJSONObject("listDefinition"));
+                    		break;
+                    	}
+                    	case "splitTable": {
+                    		applyOperationHelper.splitTable(op.getJSONArray("start"));
+                    		break;
+                    	}
+                    	case "mergeTable": {
+                    		applyOperationHelper.mergeTable(op.getJSONArray("start"));
+                    		break;
+                    	}
+                    	case "setDocumentAttributes": {
+                    		applyOperationHelper.setDocumentAttributes(op.getJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertHeaderFooter": {
+                    		applyOperationHelper.insertHeaderFooter(op.getString("id"), op.getString("type"));
+                    		break;
+                    	}
+                    	case "deleteHeaderFooter": {
+                    		applyOperationHelper.deleteHeaderFooter(op.getString("id"));
+                    		break;
+                    	}
+                    	case "insertComment": {
+                    		applyOperationHelper.insertComment(op.getJSONArray("start"), op.getString("id"), op.optString("uid"), op.optString("author"), op.optString("date"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertRange": {
+                    		applyOperationHelper.insertRange(op.getJSONArray("start"), op.getString("id"), op.optString("type", "comment"), op.optString("position", "start"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "insertComplexField": {
+                    		applyOperationHelper.insertComplexField(op.getJSONArray("start"), op.getString("instruction"), op.optJSONObject("attrs"));
+                    		break;
+                    	}
+                    	case "createError": {
+                    		throw new FilterException("createError operation detected: " + opName, ErrorCode.UNSUPPORTED_OPERATION_USED);
+                    	}
+                    	default: {
+                    		logMessage("warn", "Ignoring unsupported operation: " + opName);
+                    	}
+                    }
                 }
+                successfulAppliedOperations=aOperations.length();
 
                 // creating fontListPart ...for QuickOffice
                 getFontList(true);
@@ -233,7 +335,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                 // set user language if not already set in the document
                 applyOperationHelper.setDocumentLanguageIfNotSet(getUserLanguage());
 
-                // postfix non conformance of paragraphs in tables and unique ids (maindocumentPart/headerPart/footerPart)
+                // postfix non conformance of paragraphs in tables and unique ids (maindocumentPart/headerPart/footerPart/commentsPart)
                 final AtomicInteger uniqueId = new AtomicInteger(1000000);
                 final MainDocumentPart mainDocumentPart = getPackage().getMainDocumentPart();
                 setContextPart(null);
@@ -249,6 +351,16 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                             final Component.RootComponent headerFooterComponent = new Component.RootComponent((ContentAccessor)getContextPart());
                             applyOperationHelper.postFixNonConformance(headerFooterComponent.getNextChildComponent(null, null), uniqueId);
                         }
+                        else if(type.equals(Namespaces.COMMENTS)) {
+                        	final CommentsPart commentsPart = (CommentsPart)mainRelationshipsPart.getPart(relationship);
+                        	setContextPart(commentsPart);
+                        	final Comments comments = commentsPart.getContents();
+                        	for(Comment c:comments.getComment()) {
+                        		comments.setCommentId(c.getId());
+	                            final Component.RootComponent commentComponent = new Component.RootComponent((ContentAccessor)getContextPart());
+	                            applyOperationHelper.postFixNonConformance(commentComponent.getNextChildComponent(null, null), uniqueId);
+                        	}
+                        }
                     }
                 }
             }
@@ -268,6 +380,12 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 
         final JSONObject aOperations = new JSONObject();
         final JSONArray operationsArray = new JSONArray();
+
+        //set dummy for addSetDocumentAttributes-Operation called in createDocumentDefaults
+        final JSONObject noOpSync = new JSONObject();
+        noOpSync.put("name", "noOp");
+        operationsArray.put(noOpSync);
+
         CreateOperationHelper createOperationHelper = new CreateOperationHelper(this, operationsArray);
         createOperationHelper.CreateFontDescriptions();
         createOperationHelper.CreateStyleOperations();
@@ -336,6 +454,41 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
     	return settingsPart!=null?settingsPart.getJaxbElement():null;
     }
 
+    public CommentsPart getCommentsPart(boolean createIfMissing)
+    	throws InvalidFormatException {
+
+ 		CommentsPart commentsPart = getPackage().getMainDocumentPart().getCommentsPart();
+    	if(commentsPart==null&&createIfMissing) {
+    		commentsPart = new CommentsPart();
+    		commentsPart.setJaxbElement(Context.getWmlObjectFactory().createComments());
+    		getPackage().getMainDocumentPart().addTargetPart(commentsPart);
+    	}
+    	return commentsPart;
+    }
+
+    public Comment getComment(BigInteger id, boolean createIfMissing)
+    	throws InvalidFormatException {
+
+ 		CommentsPart commentsPart = getCommentsPart(createIfMissing);
+    	if(commentsPart!=null) {
+    		final Comments comments = commentsPart.getJaxbElement();
+    		for(Comment comment:comments.getComment()) {
+    			if(comment.getId().equals(id)) {
+    				return comment;
+    			}
+    		}
+    		if(createIfMissing) {
+    			final Comment comment = Context.getWmlObjectFactory().createCommentsComment();
+    			comment.setParent(comments);
+    			comment.setId(id);
+    			comment.setAuthor("");
+    			comments.getComment().add(comment);
+    			return comment;
+    		}
+    	}
+    	return null;
+    }
+
     public List<Font> getFontList(boolean createIfMissing)
         throws Exception {
 
