@@ -55,28 +55,44 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.xml.bind.JAXBException;
+import javax.xml.datatype.DatatypeConfigurationException;
 
-import org.apache.commons.logging.Log;
+import org.docx4j.IndexedNode;
 import org.docx4j.IndexedNodeList;
 import org.docx4j.XmlUtils;
 import org.docx4j.jaxb.Context;
 import org.docx4j.openpackaging.exceptions.InvalidFormatException;
 import org.docx4j.openpackaging.exceptions.PartUnrecognisedException;
 import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
+import org.docx4j.openpackaging.parts.Part;
+import org.docx4j.openpackaging.parts.WordprocessingML.FooterPart;
+import org.docx4j.openpackaging.parts.WordprocessingML.HeaderPart;
+import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
 import org.docx4j.openpackaging.parts.WordprocessingML.NumberingDefinitionsPart;
+import org.docx4j.openpackaging.parts.relationships.RelationshipsPart.AddPartBehaviour;
+import org.docx4j.relationships.Relationship;
 import org.docx4j.wml.BooleanDefaultTrue;
 import org.docx4j.wml.CTLanguage;
+import org.docx4j.wml.CTRel;
 import org.docx4j.wml.CTSettings;
 import org.docx4j.wml.CTShd;
 import org.docx4j.wml.CTTabStop;
 import org.docx4j.wml.CTTblPrBase;
 import org.docx4j.wml.CTTblStylePr;
+import org.docx4j.wml.CommentRangeEnd;
+import org.docx4j.wml.CommentRangeStart;
+import org.docx4j.wml.Comments.Comment;
 import org.docx4j.wml.ContentAccessor;
 import org.docx4j.wml.DocDefaults;
 import org.docx4j.wml.DocDefaults.RPrDefault;
+import org.docx4j.wml.FooterReference;
+import org.docx4j.wml.HdrFtrRef;
+import org.docx4j.wml.HeaderReference;
+import org.docx4j.wml.InstrText;
 import org.docx4j.wml.JcEnumeration;
 import org.docx4j.wml.Lvl;
 import org.docx4j.wml.NumFmt;
@@ -88,8 +104,13 @@ import org.docx4j.wml.Numbering.Num.AbstractNumId;
 import org.docx4j.wml.ObjectFactory;
 import org.docx4j.wml.PPr;
 import org.docx4j.wml.PPrBase.Ind;
+import org.docx4j.wml.R;
 import org.docx4j.wml.RPr;
+import org.docx4j.wml.STPageOrientation;
 import org.docx4j.wml.STTblStyleOverrideType;
+import org.docx4j.wml.SectPr;
+import org.docx4j.wml.SectPr.PgMar;
+import org.docx4j.wml.SectPr.PgSz;
 import org.docx4j.wml.Style;
 import org.docx4j.wml.Styles;
 import org.docx4j.wml.Tabs;
@@ -98,11 +119,12 @@ import org.docx4j.wml.TrPr;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.ooxml.docx.OperationDocument;
 import com.openexchange.office.ooxml.docx.tools.Component;
-import com.openexchange.office.ooxml.docx.tools.Component.ParagraphComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.FldCharComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.SplitMode;
 import com.openexchange.office.ooxml.docx.tools.Component.Type;
 import com.openexchange.office.ooxml.docx.tools.Paragraph;
@@ -111,7 +133,7 @@ import com.openexchange.office.ooxml.docx.tools.Utils;
 
 public class ApplyOperationHelper extends com.openexchange.office.ooxml.operations.ApplyOperationHelper {
 
-    protected final static Log log = LogFactory.getLog(ApplyOperationHelper.class);
+	final private static Logger log = LoggerFactory.getLogger(ApplyOperationHelper.class);
 
     private final OperationDocument operationDocument;
     private final WordprocessingMLPackage wordMLPackage;
@@ -185,7 +207,7 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         }
         Component component = Component.getComponent(getOperationDocument().getContextPart(), start, start.length());
 		component.splitStart(startComponent, SplitMode.DELETE);
-        component.delete((endComponent-startComponent)+1);
+        component.delete(getOperationDocument(), (endComponent-startComponent)+1);
     }
 
     /**
@@ -202,54 +224,28 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         if(startPosition.length()!=endPosition.length())
             throw new JSONException("ooxml export: move operation, size of startPosition != size of endPosition");
 
-        int startIndex = startPosition.getInt(startPosition.length()-1);
-        int endIndex = endPosition.getInt(startPosition.length()-1);
-        int toIndex = toPosition.getInt(toPosition.length()-1);
+        final Component sourceBegComponent = Component.getComponent(getOperationDocument().getContextPart(), startPosition, startPosition.length());
+        final Component sourceEndComponent = sourceBegComponent.getComponent(endPosition.getInt(startPosition.length()-1));
+        final IndexedNodeList<Object> sourceContent = (IndexedNodeList<Object>)((ContentAccessor)sourceBegComponent.getParentComponent().getNode().getData()).getContent();
+        sourceBegComponent.splitStart(startPosition.getInt(startPosition.length()-1), SplitMode.DELETE);
+        sourceEndComponent.splitEnd(endPosition.getInt(startPosition.length()-1), SplitMode.DELETE);
 
-        if((endIndex - startIndex) < 0)
-            throw new JSONException("ooxml export: move operation, component count < 1");
-
-        Component sourceParent = null;
-        Component toParent = null;
-
-        // check if split is necessary (only needed if parent is a paragraph, if length == 1 then the parent is always root)
-        sourceParent = Component.getComponent(getOperationDocument().getContextPart(), startPosition, startPosition.length()-1);
-        if(sourceParent instanceof ParagraphComponent) {
-        	Component c = sourceParent.getChildComponent(startIndex);
-        	if(c!=null) {
-        		c.splitStart(startIndex, SplitMode.ATTRIBUTE);
-        	}
-        	c = c.getComponent(endIndex+1);
-        	if(c!=null) {
-        		c.splitEnd(endIndex, SplitMode.ATTRIBUTE);
-        	}
+        // destComponent is empty if the content is to be appended
+        final Component destComponent = Component.getComponent(getOperationDocument().getContextPart(), toPosition, toPosition.length());
+        Component destParentComponent;
+        if(destComponent!=null) {
+        	destComponent.splitStart(toPosition.getInt(toPosition.length()-1), SplitMode.DELETE);
+        	destParentComponent = destComponent.getParentComponent();
         }
-        toParent = Component.getComponent(getOperationDocument().getContextPart(), toPosition, toPosition.length()-1);
-        if(toParent instanceof ParagraphComponent) {
-        	Component c = toParent.getChildComponent(toIndex);
-        	if(c!=null) {
-        		c.splitStart(toIndex, SplitMode.ATTRIBUTE);
-        	}
+        else {
+        	destParentComponent = Component.getComponent(getOperationDocument().getContextPart(), toPosition, toPosition.length()-1);
         }
+        final int startIndex = sourceBegComponent.getContextChild(null).getNode().getIndex();
+        final int endIndex = sourceEndComponent.getContextChild(null).getNode().getIndex();
 
-        Component startComponent = sourceParent.getChildComponent(startIndex);
-        if(startComponent==null)
-        	throw new JSONException("ooxml export: move operation, could not get component for start position");
-
-        final int parentStartIndex = startComponent.getContextChildNode().getIndex();
-        int parentEndIndex = parentStartIndex;
-
-        while(startComponent!=null&&startComponent.getNextComponentNumber()<=endIndex) {
-            parentEndIndex = startComponent.getContextChildNode().getIndex();
-            startComponent = startComponent.getNextComponent();
-        }
-
-        final IndexedNodeList<Object> sourceContent = (IndexedNodeList<Object>)((ContentAccessor)sourceParent.getObject()).getContent();
-        final IndexedNodeList<Object> destContent = (IndexedNodeList<Object>)((ContentAccessor)toParent.getObject()).getContent();
-
-        final Component toComponent = toParent.getChildComponent(toIndex);
-        final int destIndex = toComponent!=null?toComponent.getContextChildNode().getIndex():destContent.size();
-    	IndexedNodeList.moveNodes(sourceContent, destContent, parentStartIndex, destIndex, (parentEndIndex - parentStartIndex)+1, toParent.getObject());
+        IndexedNodeList.moveNodes(sourceContent, (IndexedNodeList<Object>)((ContentAccessor)destParentComponent.getNode().getData()).getContent(), startIndex,
+        		destComponent!=null?destComponent.getContextChild(null).getNode().getIndex():((ContentAccessor)destParentComponent.getNode().getData()).getContent().size(),
+        				(endIndex - startIndex)+1, destParentComponent.getNode().getData());
     }
 
     public void insertParagraph(JSONArray start, JSONObject attrs)
@@ -392,7 +388,50 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
     		.insertField(operationDocument, start.getInt(start.length()-1), type, representation, attrs);
     }
 
-    public void insertStyleSheet(String type, String styleId, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority, Boolean isDefault)
+    public void changeStyleSheet(String type, String styleId, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority)
+        throws Exception {
+
+        final Styles styles = getOperationDocument().getStyles(true);
+        final List<Style> stylesList = styles.getStyle();
+
+        Style style = null;
+
+        Iterator<Style> styleIter = stylesList.iterator();
+        while(styleIter.hasNext()){
+            Style aStyle = styleIter.next();
+            if ((aStyle.getType().equals(type)) && (aStyle.getStyleId().equals(styleId))) {
+                style = aStyle;
+                break;
+            }
+        };
+
+        if (null != style) {
+
+            if (null != attrs) {
+                //reset attributes
+                style.setPPr(null);
+                style.setRPr(null);
+                style.setNext(null);
+                style.setTblPr(null);
+                style.setTrPr(null);
+                style.setTcPr(null);
+                style.setPPr(null);
+                style.setRPr(null);
+
+                //change attrs
+                handleNewStyleSheet(style, type, styleName, attrs, parentId, hidden, uipriority, null, null);
+            } else {
+                //change name
+                Style.Name name = objectFactory.createStyleName();
+                name.setParent(style);
+                name.setVal(styleName);
+                style.setName(name);
+            }
+
+        }
+    }
+
+    public void insertStyleSheet(String type, String styleId, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority, Boolean isDefault, Boolean custom)
         throws Exception {
 
         final Styles styles = getOperationDocument().getStyles(true);
@@ -408,14 +447,21 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             }
         };
 
-        // always creating a new style...
         Style style = objectFactory.createStyle();
         stylesList.add(style);
+        style.setParent(styles);
 
         // style properties
-        style.setParent(styles);
         style.setType(type);
         style.setStyleId(styleId);
+
+        handleNewStyleSheet(style, type, styleName, attrs, parentId, hidden, uipriority, isDefault, custom);
+    }
+
+    private void handleNewStyleSheet(Style style, String type, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority, Boolean isDefault, Boolean custom)
+        throws Exception {
+
+        // style properties
         if (styleName != null) {
             Style.Name name = objectFactory.createStyleName();
             name.setParent(style);
@@ -443,7 +489,9 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
             basedOn.setVal(parentId);
             style.setBasedOn(basedOn);
         }
-
+        if (custom != null) {
+            style.setCustomStyle(custom);
+        }
         // paragraph properties
         JSONObject paragraphProps = attrs.optJSONObject("paragraph");
         if (paragraphProps != null) {
@@ -570,9 +618,20 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         }
     }
 
-    @SuppressWarnings("unused")
-    public void deleteStylesheet(String type, String styleName) {
-        //
+    public void deleteStyleSheet(String type, String styleId)
+            throws Exception {
+
+        final Styles styles = getOperationDocument().getStyles(true);
+        final List<Style> stylesList = styles.getStyle();
+
+        Iterator<Style> styleIter = stylesList.iterator();
+        while(styleIter.hasNext()){
+            Style aStyle = styleIter.next();
+            if ((aStyle.getType().equals(type)) && (aStyle.getStyleId().equals(styleId))) {
+                styleIter.remove();
+                return;
+            }
+        };
     }
     public void insertListStyle(String listStyleId, JSONObject listDefinition)
         throws InvalidFormatException, JAXBException, JSONException {
@@ -749,9 +808,18 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
         }
     }
 
+
+    private SectPr getDocumentProperties(SectPr sectPr) {
+    	if(sectPr!=null) {
+    		return sectPr;
+    	}
+    	return Utils.getDocumentProperties(getOperationDocument().getPackage().getMainDocumentPart(), true);
+    }
+
     public void setDocumentAttributes(JSONObject attrs)
     	throws Exception {
 
+    	SectPr sectPr = null;
     	final JSONObject documentAttrs = attrs.optJSONObject("document");
     	if(documentAttrs!=null) {
 			final Object changeTracking = documentAttrs.opt("changeTracking");
@@ -767,6 +835,247 @@ public class ApplyOperationHelper extends com.openexchange.office.ooxml.operatio
     			}
     		}
     	}
+    	final JSONObject pageAttrs = attrs.optJSONObject("page");
+    	if(pageAttrs!=null) {
+	    	for(Entry<String, Object> pageAttr : pageAttrs.entrySet()) {
+	    		final String attr = pageAttr.getKey();
+	    		final Object value = pageAttr.getValue();
+	    		if(attr.equals("evenOddPages")) {
+			        final CTSettings settings = getOperationDocument().getSettings(true);
+	    			if(value instanceof Boolean) {
+	    				final BooleanDefaultTrue evenOddHeaders = Context.getWmlObjectFactory().createBooleanDefaultTrue();
+	    				evenOddHeaders.setVal((Boolean)value);
+	    				evenOddHeaders.setParent(settings);
+	    				settings.setEvenAndOddHeaders(evenOddHeaders);
+	    			}
+	    			else {
+	    				settings.setEvenAndOddHeaders(null);
+	    			}
+	    		}
+	    		else if(attr.equals("firstPage")) {
+        			sectPr = getDocumentProperties(sectPr);
+					if(value instanceof Boolean) {
+		        		final BooleanDefaultTrue titlePg = Context.getWmlObjectFactory().createBooleanDefaultTrue();
+		        		titlePg.setVal((Boolean)value);
+		        		titlePg.setParent(sectPr);
+		        		sectPr.setTitlePg(titlePg);
+					}
+					else {
+						sectPr.setTitlePg(null);
+					}
+	    		}
+	    		else if(attr.equals("width")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgSz pgSize = sectPr.getPgSz();
+	    			if(pgSize==null) {
+	    				pgSize = Context.getWmlObjectFactory().createSectPrPgSz();
+	    				pgSize.setParent(sectPr);
+	    				sectPr.setPgSz(pgSize);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgSize.setW(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	    		}
+	    		else if(attr.equals("height")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgSz pgSize = sectPr.getPgSz();
+	    			if(pgSize==null) {
+	    				pgSize = Context.getWmlObjectFactory().createSectPrPgSz();
+	    				pgSize.setParent(sectPr);
+	    				sectPr.setPgSz(pgSize);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgSize.setH(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("marginLeft")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setLeft(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("marginTop")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setTop(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("marginRight")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setRight(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("marginBottom")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setBottom(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	    		}
+	    		else if(attr.equals("marginHeader")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setHeader(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("marginFooter")) {
+        			sectPr = getDocumentProperties(sectPr);
+	    			PgMar pgMar = sectPr.getPgMar();
+	    			if(pgMar==null) {
+	    				pgMar = Context.getWmlObjectFactory().createSectPrPgMar();
+	    				pgMar.setParent(sectPr);
+	    				sectPr.setPgMar(pgMar);
+	    			}
+	    			if(value instanceof Number) {
+	    				pgMar.setFooter(Utils.map100THMMToTwip(((Number)value).longValue()));
+	    			}
+	        	}
+	    		else if(attr.equals("orientation")) {
+	    			sectPr = getDocumentProperties(sectPr);
+	    			PgSz pgSz = sectPr.getPgSz();
+	    			if(pgSz==null) {
+	    				pgSz = Context.getWmlObjectFactory().createSectPrPgSz();
+	    				pgSz.setParent(sectPr);
+	    				sectPr.setPgSz(pgSz);
+	    			}
+	    			if(value instanceof String) {
+	    				pgSz.setOrient(((String)value).equals("landscape") ? STPageOrientation.LANDSCAPE : STPageOrientation.PORTRAIT);
+	    			}
+	    			else {
+	    				pgSz.setOrient(STPageOrientation.PORTRAIT);
+	    			}
+	    		}
+	    	}
+    	}
+    }
+
+    public void insertHeaderFooter(String id, String type)
+    	throws InvalidFormatException {
+
+    	final MainDocumentPart mainDocumentPart = wordMLPackage.getMainDocumentPart();
+    	final String[] typeArray = type.split("_");
+    	final HdrFtrRef hdrFtrRef = typeArray[1].equals("first")?HdrFtrRef.FIRST:typeArray[1].equals("even")?HdrFtrRef.EVEN:HdrFtrRef.DEFAULT;
+    	CTRel ctRel;
+    	Part part;
+    	if(typeArray[0].equals("header")) {
+    		final HeaderPart headerPart = new HeaderPart();
+    		headerPart.setJaxbElement(Context.getWmlObjectFactory().createHdr());
+    		final HeaderReference headerReference = Context.getWmlObjectFactory().createHeaderReference();
+    		headerReference.setType(hdrFtrRef);
+    		ctRel = headerReference;
+    		part = headerPart;
+    	}
+    	else {
+    		final FooterPart footerPart = new FooterPart();
+    		footerPart.setJaxbElement(Context.getWmlObjectFactory().createFtr());
+    		ctRel = Context.getWmlObjectFactory().createFooterReference();
+    		final FooterReference footerReference = Context.getWmlObjectFactory().createFooterReference();
+    		footerReference.setType(hdrFtrRef);
+    		ctRel = footerReference;
+    		part = footerPart;
+    	}
+    	final Relationship rel = mainDocumentPart.addTargetPart(part, AddPartBehaviour.RENAME_IF_NAME_EXISTS, id);
+    	if(!rel.getId().equals(id)) {
+    		log.error("docx: insertHeaderFooter... Id:" + id + " is not unique!!!");
+    	}
+    	final SectPr sectPr = Utils.getDocumentProperties(mainDocumentPart, true);
+    	ctRel.setParent(sectPr);
+    	ctRel.setId(rel.getId());
+    	// adding updating targetToRelId map so that further operations can use its own id
+    	sectPr.getEGHdrFtrReferences().add(ctRel);
+    }
+
+    public void deleteHeaderFooter(String id) {
+    	final MainDocumentPart mainDocumentPart = wordMLPackage.getMainDocumentPart();
+    	mainDocumentPart.getRelationshipsPart(false).getPart(id).remove();
+      	final List<CTRel> hdrFtrReferenceList = Utils.getDocumentProperties(mainDocumentPart, false).getEGHdrFtrReferences();
+      	for(int i=0;i<hdrFtrReferenceList.size();i++) {
+      		final CTRel rel = hdrFtrReferenceList.get(i);
+      		if(rel.getId().equals(id)) {
+      			hdrFtrReferenceList.remove(i);
+      			break;
+      		}
+      	}
+    }
+
+    public void insertComment(JSONArray start, String id, String uid, String author, String date, JSONObject attrs)
+    	throws UnsupportedOperationException, InvalidFormatException, PartUnrecognisedException, JAXBException, JSONException, DatatypeConfigurationException {
+
+    	final Component component = Component.getComponent(getOperationDocument().getContextPart(), start, start.length()-1)
+    		.insertChildComponent(operationDocument, start.getInt(start.length()-1), attrs, Type.COMMENT_REFERENCE);
+
+    	final BigInteger commentId = BigInteger.valueOf(Integer.parseInt(id.substring(3)));
+    	final R.CommentReference commentReference = (R.CommentReference)component.getObject();
+    	commentReference.setId(commentId);
+
+    	final Comment comment = getOperationDocument().getComment(commentId, true);
+    	comment.getContent().clear();
+    	Utils.applyTrackInfoFromJSON(getOperationDocument(), author, date, uid, comment);
+    }
+
+    public void insertRange(JSONArray start, String id, String type, String position, JSONObject attrs)
+    	throws UnsupportedOperationException, InvalidFormatException, PartUnrecognisedException, JAXBException, JSONException {
+
+    	if(type.equals("comment")) {
+        	final BigInteger cId = BigInteger.valueOf(Integer.parseInt(id.substring(3)));
+        	final Component.Type rangeType = position.equals("start") ? Component.Type.COMMENT_RANGE_START : Component.Type.COMMENT_RANGE_END;
+	    	final Component component = Component.getComponent(getOperationDocument().getContextPart(), start, start.length()-1)
+	    		.insertChildComponent(operationDocument, start.getInt(start.length()-1), attrs, rangeType);
+
+	    	final Object commentRangeObject = component.getObject();
+	    	if(commentRangeObject instanceof CommentRangeStart) {
+	    		((CommentRangeStart)commentRangeObject).setId(cId);
+	    	}
+	    	else {
+	    		((CommentRangeEnd)commentRangeObject).setId(cId);
+	    	}
+    	}
+    	else if(type.equals("field")) {
+    		Component.getComponent(getOperationDocument().getContextPart(), start, start.length()-1)
+	    		.insertChildComponent(operationDocument, start.getInt(start.length()-1), attrs, position.equals("start") ? Component.Type.COMPLEX_FIELD_START : Component.Type.COMPLEX_FIELD_END);
+    	}
+    }
+
+    public void insertComplexField(JSONArray start, String instruction, JSONObject attrs)
+    	throws JSONException, JAXBException, InvalidFormatException, PartUnrecognisedException {
+
+    	final FldCharComponent component = (FldCharComponent)Component.getComponent(getOperationDocument().getContextPart(), start, start.length()-1)
+    		.insertChildComponent(operationDocument,  start.getInt(start.length()-1), attrs, Type.COMPLEX_FIELD_SEPARATE);
+
+    	final IndexedNode<Object> rNode = component.getTextRunNode();
+    	final InstrText instrText = new InstrText();
+    	instrText.setParent(rNode.getData());
+    	instrText.setValue(instruction);
+    	((R)rNode.getData()).getContent().addNode(component.getNode(), new IndexedNode<Object>(instrText), true);
     }
 }
-
