@@ -54,13 +54,16 @@ import java.net.URISyntaxException;
 import java.text.ParseException;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Stack;
 
 import javax.xml.bind.JAXBElement;
 
 import org.docx4j.openpackaging.URIHelper;
+import org.docx4j.openpackaging.exceptions.InvalidFormatException;
 import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.WordprocessingML.FontTablePart;
@@ -82,10 +85,14 @@ import org.docx4j.wml.CTTblStylePr;
 import org.docx4j.wml.CTTwipsMeasure;
 import org.docx4j.wml.CTZoom;
 import org.docx4j.wml.Color;
+import org.docx4j.wml.CommentRangeEnd;
+import org.docx4j.wml.CommentRangeStart;
+import org.docx4j.wml.Comments.Comment;
 import org.docx4j.wml.ContentAccessor;
 import org.docx4j.wml.DocDefaults;
 import org.docx4j.wml.DocDefaults.PPrDefault;
 import org.docx4j.wml.DocDefaults.RPrDefault;
+import org.docx4j.wml.FldChar;
 import org.docx4j.wml.FontFamily;
 import org.docx4j.wml.FontPanose;
 import org.docx4j.wml.FontPitch;
@@ -117,9 +124,12 @@ import org.docx4j.wml.PPrBase.Ind;
 import org.docx4j.wml.PPrBase.NumPr;
 import org.docx4j.wml.PPrBase.NumPr.NumId;
 import org.docx4j.wml.Pict;
+import org.docx4j.wml.R;
 import org.docx4j.wml.RFonts;
 import org.docx4j.wml.RPr;
 import org.docx4j.wml.STBrType;
+import org.docx4j.wml.STFldCharType;
+import org.docx4j.wml.STPageOrientation;
 import org.docx4j.wml.STPitch;
 import org.docx4j.wml.STTblStyleOverrideType;
 import org.docx4j.wml.SectPr;
@@ -142,6 +152,10 @@ import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.docx.OperationDocument;
 import com.openexchange.office.ooxml.docx.tools.Character;
 import com.openexchange.office.ooxml.docx.tools.Component;
+import com.openexchange.office.ooxml.docx.tools.Component.CommentRangeEndComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.CommentRangeStartComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.CommentReferenceComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.FldCharComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.FldSimpleComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.HardBreakComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.MultiComponent;
@@ -162,6 +176,26 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 
     private final WordprocessingMLPackage wordprocessingMLPackage;
     private final Map<String, String> conditionalTableAttrNames;
+    private final HashSet<BigInteger> createdComments = new HashSet<BigInteger>();
+    private final Stack<ComplexFieldStackEntry> complexFieldIdStack = new Stack<ComplexFieldStackEntry>();
+    private int nextComplexFieldId = 0;
+
+    private class ComplexFieldStackEntry {
+
+    	final int fldId;
+    	final FldCharComponent fldCharStart;
+
+    	public ComplexFieldStackEntry(int id, FldCharComponent component) {
+    		fldCharStart = component;
+    		fldId = id;
+    	}
+    	public FldCharComponent getFldStart() {
+    		return fldCharStart;
+    	}
+    	public int getFldId() {
+    		return fldId;
+    	}
+    }
 
     public CreateOperationHelper(OperationDocument _operationDocument, JSONArray mOperationsArray)
     	throws FilterException {
@@ -190,15 +224,15 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     public void createOperations()
-        throws JSONException, ParseException, FilterException {
+        throws JSONException, ParseException, FilterException, InvalidFormatException {
 
         createOperations(wordprocessingMLPackage.getMainDocumentPart(), new ArrayList<Integer>(), null);
     }
 
     public void CreateHeaderFooterOperations()
-    	throws JSONException, ParseException, FilterException {
+    	throws JSONException, ParseException, FilterException, InvalidFormatException {
 
-    	final SectPr sectPr = Utils.getDocumentProperties(wordprocessingMLPackage.getMainDocumentPart());
+    	final SectPr sectPr = Utils.getDocumentProperties(wordprocessingMLPackage.getMainDocumentPart(), false);
         if(sectPr!=null) {
         	for(CTRel rel:sectPr.getEGHdrFtrReferences()) {
         		final String id = rel.getId();
@@ -224,13 +258,26 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         }
     }
 
+    public void CreateCommentOperations(Comment comment, BigInteger id)
+        throws JSONException, ParseException, FilterException, InvalidFormatException {
+
+    	if(comment!=null&&!createdComments.contains(id)) {
+    		final Part oldContextPart = operationDocument.getContextPart();
+     		operationDocument.setContextPart(getOperationDocument().getPackage().getMainDocumentPart().getCommentsPart());
+			final String target = "cmt" + id.toString();
+			createOperations(comment, new ArrayList<Integer>(), target);
+    		operationDocument.setContextPart(oldContextPart);
+    		createdComments.add(id);
+    	}
+    }
+
     @Override
     public void createDocumentDefaults(String userLanguage)
         throws Exception {
 
         final JSONObject documentAttributes = new JSONObject();
         final CTSettings settings = getOperationDocument().getSettings(false);
-        final SectPr sectPr = Utils.getDocumentProperties(wordprocessingMLPackage.getMainDocumentPart());
+        final SectPr sectPr = Utils.getDocumentProperties(wordprocessingMLPackage.getMainDocumentPart(), false);
         // document defaults
         final JSONObject jsonDocumentSettings = new JSONObject();
         if (settings!=null) {
@@ -280,6 +327,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             if(pgSz!=null) {
                 jsonPageSettings.put("width", Utils.mapTwipTo100THMM(pgSz.getW()));
                 jsonPageSettings.put("height", Utils.mapTwipTo100THMM(pgSz.getH()));
+                if(pgSz.getOrient()==STPageOrientation.LANDSCAPE) {
+                    jsonPageSettings.put("orientation", "landscape");
+                }
             }
             final PgMar pgMar = sectPr.getPgMar();
             if(pgMar!=null) {
@@ -391,7 +441,6 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     public void AddInsertHeaderFooterOperation(String id, CTRel rel)
     	throws JSONException {
 
-
     	final JSONObject headerFooterOperation = new JSONObject();
     	headerFooterOperation.put("name", "insertHeaderFooter");
     	headerFooterOperation.put("id", id);
@@ -416,6 +465,66 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         operationsArray.put(insertTabObject);
     }
 
+    public void AddInsertCommentOperation(final List<Integer> start, String target, BigInteger commentId, Comment comment)
+        throws JSONException {
+
+        final JSONObject insertComment = Utils.createJSONFromTrackInfo(getOperationDocument(), comment);
+        insertComment.put("name", "insertComment");
+        insertComment.put("start", start);
+        if(target!=null) {
+        	insertComment.put("target", target);
+        }
+        insertComment.put("id", "cmt" + commentId.toString());
+        operationsArray.put(insertComment);
+    }
+
+    public void AddInsertCommentRangeOperation(final List<Integer> start, String target, BigInteger commentId, boolean startRange)
+        throws JSONException {
+
+        final JSONObject insertCommentRangeObject = new JSONObject();
+        insertCommentRangeObject.put("name", "insertRange");
+        insertCommentRangeObject.put("start", start);
+        insertCommentRangeObject.put("type", "comment");
+        if(target!=null) {
+        	insertCommentRangeObject.put("target", target);
+        }
+        insertCommentRangeObject.put("position", startRange?"start":"end");
+        insertCommentRangeObject.put("id", "cmt" + commentId.toString());
+        operationsArray.put(insertCommentRangeObject);
+    }
+
+    // TODO: complex fields within multi components ...
+    public void AddInsertComplexFieldOperation(final List<Integer> start, String target, ComplexFieldStackEntry fieldStackEntry)
+        throws JSONException {
+
+        final JSONObject insertComplexField = new JSONObject();
+        insertComplexField.put("name", "insertComplexField");
+        insertComplexField.put("start", start);
+        if(target!=null) {
+        	insertComplexField.put("target", target);
+        }
+        if(fieldStackEntry!=null) {
+	        insertComplexField.put("id", "fld" + Integer.toString(fieldStackEntry.getFldId()));
+	        insertComplexField.put("instruction", fieldStackEntry.getFldStart().getInstruction());
+        }
+        operationsArray.put(insertComplexField);
+    }
+
+    public void AddInsertComplexFieldRangeOperation(final List<Integer> start, String target, int id, boolean startRange)
+        throws JSONException {
+
+        final JSONObject insertCommentRangeObject = new JSONObject();
+        insertCommentRangeObject.put("name", "insertRange");
+        insertCommentRangeObject.put("start", start);
+        insertCommentRangeObject.put("type", "field");
+        if(target!=null) {
+        	insertCommentRangeObject.put("target", target);
+        }
+        insertCommentRangeObject.put("position", startRange?"start":"end");
+        insertCommentRangeObject.put("id", "fld" + Integer.toString(id));
+        operationsArray.put(insertCommentRangeObject);
+    }
+
     public void AddInsertHardBreakOperation(List<Integer> start, String target, STBrType type)
         throws JSONException {
 
@@ -634,6 +743,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     Boolean hidden = null;
                     Boolean isDefault = style.isDefault();
                     BooleanDefaultTrue defaultTrueHidden = style.getHidden();
+                    Boolean custom = style.isCustomStyle();
                     if (defaultTrueHidden!=null)
                         hidden = defaultTrueHidden.isVal();
                     if(hidden==null||hidden==false)
@@ -652,7 +762,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                             JSONObject character = Character.createCharacterProperties(themeFonts, rPr);
                             attrs.put("character", character);
                         }
-                        operationsArray.put(createInsertStyleSheetOperation("character", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, false));
+                        operationsArray.put(createInsertStyleSheetOperation("character", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, custom, false));
                     }
                     else if (type.equals("paragraph")) {
                         rPr = style.getRPr();
@@ -680,7 +790,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                         if (paragraph!=null) {
                             attrs.put("paragraph", paragraph);
                         }
-                        operationsArray.put(createInsertStyleSheetOperation("paragraph", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, false));
+                        operationsArray.put(createInsertStyleSheetOperation("paragraph", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, custom, false));
                     }
                     else if (type.equals("table")) {
                         JSONObject attrs = new JSONObject();
@@ -693,7 +803,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                                 attrName = conditionalTableAttrNames.get(attrName);
                             CreateConditionalTableStyle(attrs, attrName, conditionalStyle.getTblPr(), conditionalStyle.getTrPr(), conditionalStyle.getTcPr(), conditionalStyle.getPPr(), conditionalStyle.getRPr());
                         }
-                        operationsArray.put(createInsertStyleSheetOperation("table", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, false));
+                        operationsArray.put(createInsertStyleSheetOperation("table", styleId, styleName, attrs, parentId, hidden, uipriority, isDefault, custom, false));
                     }
                 }
             }
@@ -701,7 +811,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     private void createOperations(ContentAccessor parent, List<Integer> parentPosition, String target)
-        throws JSONException, ParseException, FilterException {
+        throws JSONException, ParseException, FilterException, InvalidFormatException {
 
         final Component.RootComponent rootComponent = new Component.RootComponent(parent);
         Component component = rootComponent.getNextChildComponent(null, null);
@@ -730,7 +840,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     private void CreateParagraphOperations(ParagraphComponent paragraphComponent, List<Integer> parentPosition, String target)
-        throws JSONException, ParseException, FilterException {
+        throws JSONException, ParseException, FilterException, InvalidFormatException {
 
         // insert paragraph and apply paragraph attributes
         final List<Integer> paragraphPosition = new ArrayList<Integer>(parentPosition);
@@ -758,6 +868,41 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                 else if (component instanceof FldSimpleComponent) {
                     AddInsertFieldOperation(startPosition, target, ((FldSimpleComponent)component).getInstr(), ((FldSimpleComponent)component).getRepresentation());
                 }
+                else if(component instanceof CommentRangeStartComponent) {
+                    AddInsertCommentRangeOperation(startPosition, target, ((CommentRangeStart)component.getNode().getData()).getId(), true);
+                }
+                else if(component instanceof CommentRangeEndComponent) {
+                    AddInsertCommentRangeOperation(startPosition, target, ((CommentRangeEnd)component.getNode().getData()).getId(), false);
+                }
+                else if(component instanceof CommentReferenceComponent) {
+                	final BigInteger id = ((R.CommentReference)component.getNode().getData()).getId();
+                	final Comment comment = getOperationDocument().getComment(id, false);
+                    AddInsertCommentOperation(startPosition, target, id, comment);
+                    if(comment!=null) {
+                    	CreateCommentOperations(comment, id);
+                    }
+                }
+                else if(component instanceof FldCharComponent) {
+                	final FldChar fldChar = (FldChar)((FldCharComponent)component).getObject();
+                	int id = 0;
+                	if(fldChar.getFldCharType()==STFldCharType.BEGIN) {
+                		complexFieldIdStack.push(new ComplexFieldStackEntry(++nextComplexFieldId, (FldCharComponent)component));
+                    	AddInsertComplexFieldRangeOperation(startPosition, target, nextComplexFieldId, true);
+                	}
+                	else if(fldChar.getFldCharType()==STFldCharType.END) {
+                		if(!complexFieldIdStack.isEmpty()) {
+                			id = complexFieldIdStack.pop().getFldId();
+                		}
+                    	AddInsertComplexFieldRangeOperation(startPosition, target, id, false);
+                	}
+                	else {
+            			ComplexFieldStackEntry fieldStackEntry = null;
+                		if(!complexFieldIdStack.isEmpty()) {
+                			fieldStackEntry = complexFieldIdStack.peek();
+                		}
+                		AddInsertComplexFieldOperation(startPosition, target, fieldStackEntry);
+                	}
+                }
                 else {
                     AddInsertDrawingOperation(startPosition, target, ShapeType.UNDEFINED, null);
                 }
@@ -786,7 +931,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     private void CreateDrawingOperations(Component component, String target, List<Integer> parentPosition)
-    	throws JSONException, ParseException, FilterException {
+    	throws JSONException, ParseException, FilterException, InvalidFormatException {
 
         final List<Integer> position = new ArrayList<Integer>(parentPosition);
         position.add(component.getComponentNumber());
@@ -810,7 +955,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     private void CreateTableOperations(TableComponent tableComponent, List<Integer> parentPosition, String target)
-        throws JSONException, ParseException, FilterException {
+        throws JSONException, ParseException, FilterException, InvalidFormatException {
 
         final List<Integer> tablePosition = new ArrayList<Integer>(parentPosition);
         tablePosition.add(tableComponent.getComponentNumber());
@@ -834,12 +979,15 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         }
     }
 
-    public void AddInsertListOperation( long listIdentifier, JSONObject listDefinition )
+    public void AddInsertListOperation(long listIdentifier, String abstractNumId, JSONObject listDefinition)
         throws JSONException {
 
         final JSONObject insertListObject = new JSONObject();
         insertListObject.put("name", "insertListStyle");
         insertListObject.put("listStyleId", 'L' + String.valueOf(listIdentifier));
+        if(abstractNumId!=null) {
+        	insertListObject.put("baseStyleId", 'A' + abstractNumId);
+        }
         insertListObject.put("listDefinition", listDefinition);
         operationsArray.put(insertListObject);
     }
@@ -1056,9 +1204,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     Lvl level = levelIter.next();
                     createLevel( levelObject, level, picMap );
                 }
-                listObject.put( "listLevel" + index, levelObject );
+                listObject.put("listLevel" + index, levelObject);
             }
-            abstractNumMap.put( new Long(abstractNumId), listObject );
+            abstractNumMap.put(new Long(abstractNumId), listObject);
         }
         //
         Iterator<Num> numListIter = numbering.getNum().iterator();
@@ -1084,7 +1232,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     }
                 }
             }
-            AddInsertListOperation( num.getNumId().longValue(), listObject);
+            AddInsertListOperation(num.getNumId().longValue(), num.getAbstractNumId().getVal().toString(), listObject);
         }
     }
 }
