@@ -55,25 +55,34 @@ import org.docx4j.mce.AlternateContent;
 import org.docx4j.wml.Br;
 import org.docx4j.wml.CTObject;
 import org.docx4j.wml.CTSimpleField;
+import org.docx4j.wml.CommentRangeEnd;
+import org.docx4j.wml.CommentRangeStart;
 import org.docx4j.wml.ContentAccessor;
 import org.docx4j.wml.Drawing;
+import org.docx4j.wml.FldChar;
 import org.docx4j.wml.IText;
+import org.docx4j.wml.InstrText;
 import org.docx4j.wml.P;
 import org.docx4j.wml.Pict;
 import org.docx4j.wml.R;
+import org.docx4j.wml.Tc;
 import org.docx4j.wml.R.Tab;
 import org.docx4j.wml.RunDel;
 import org.docx4j.wml.RunIns;
 import org.docx4j.wml.Tbl;
 
 import com.openexchange.office.ooxml.docx.tools.Component.AlternateContentComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.CommentRangeEndComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.CommentRangeStartComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.DrawingComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.FldCharComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.FldSimpleComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.HardBreakComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.ParagraphComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.PictVMLComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.TabComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.TableComponent;
+import com.openexchange.office.ooxml.docx.tools.Component.TcComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.TextComponent;
 import com.openexchange.office.ooxml.docx.tools.Component.TextRun_Base;
 
@@ -103,11 +112,8 @@ public abstract class ComponentContext {
 
     public static class TextRunContext extends ComponentContext {
 
-        final private IndexedNode<Object> paragraphNode;
-
-        public TextRunContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _paragraphNode) {
+        public TextRunContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            paragraphNode = _paragraphNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
@@ -122,19 +128,27 @@ public abstract class ComponentContext {
                 final Object o = Component.getContentModel(childNode, textRunNode.getData());
                 if(o instanceof IText) {
                     if(((IText)o).getValue().length()>0) {
-                        nextComponent = new TextComponent(this, childNode, paragraphNode, nextComponentNumber);
+                        nextComponent = new TextComponent(this, childNode, nextComponentNumber);
                     }
                 }
                 else if(o instanceof AlternateContent)
-                    nextComponent = new AlternateContentComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new AlternateContentComponent(this, childNode, nextComponentNumber);
                 else if(o instanceof Drawing)
-                    nextComponent = new DrawingComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new DrawingComponent(this, childNode, nextComponentNumber);
                 else if(o instanceof Pict||o instanceof CTObject)
-                    nextComponent = new PictVMLComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new PictVMLComponent(this, childNode, nextComponentNumber);
                 else if(o instanceof Tab)
-                    nextComponent = new TabComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new TabComponent(this, childNode, nextComponentNumber);
                 else if(o instanceof Br)
-                    nextComponent = new HardBreakComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new HardBreakComponent(this, childNode, nextComponentNumber);
+                else if(o instanceof R.CommentReference)
+                	nextComponent = new Component.CommentReferenceComponent(this, childNode, nextComponentNumber);
+                else if(o instanceof FldChar) {
+                	nextComponent = new FldCharComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof InstrText&&previousChildComponent instanceof FldCharComponent) {
+                	((FldCharComponent)previousChildComponent).setInstruction(((InstrText)o).getValue());
+                }
             }
             return nextComponent;
         }
@@ -142,15 +156,13 @@ public abstract class ComponentContext {
 
     public static class HyperlinkContext extends ComponentContext {
 
-        final private IndexedNode<Object> paragraphNode;
-
-        public HyperlinkContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _paragraphNode) {
+        public HyperlinkContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            paragraphNode = _paragraphNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
             final int index = previousChildContext!=null?previousChildContext.getNode().getIndex()+1:0;
+            final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
 
             Component nextComponent = null;
             final IndexedNode<Object> hyperlinkNode = getNode();
@@ -159,17 +171,26 @@ public abstract class ComponentContext {
                 final IndexedNode<Object> childNode = nodeList.getNode(i);
                 final Object o = Component.getContentModel(childNode, hyperlinkNode.getData());
                 if(o instanceof R) {
-                    final TextRunContext textRunContext = new TextRunContext(this, childNode, paragraphNode);
+                    final TextRunContext textRunContext = new TextRunContext(this, childNode);
                     nextComponent = textRunContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof RunIns) {
-                    final ComponentContext.RunInsContext runInsContext = new RunInsContext(this, childNode, paragraphNode);
+                    final ComponentContext.RunInsContext runInsContext = new RunInsContext(this, childNode);
                     nextComponent = runInsContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof RunDel) {
-                    final ComponentContext.RunDelContext runDelContext = new RunDelContext(this, childNode, paragraphNode);
+                    final ComponentContext.RunDelContext runDelContext = new RunDelContext(this, childNode);
                     nextComponent = runDelContext.getNextChildComponent(null, previousChildComponent);
                 }
+                else if(o instanceof CommentRangeStart) {
+                	nextComponent = new CommentRangeStartComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof CommentRangeEnd) {
+                	nextComponent = new CommentRangeEndComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof FldChar) {
+                	nextComponent = new FldCharComponent(this, childNode, nextComponentNumber);
+                }
             }
             return nextComponent;
         }
@@ -177,15 +198,13 @@ public abstract class ComponentContext {
 
     public static class RunInsContext extends ComponentContext {
 
-        final private IndexedNode<Object> paragraphNode;
-
-        public RunInsContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _paragraphNode) {
+        public RunInsContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            paragraphNode = _paragraphNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
             final int index = previousChildContext!=null?previousChildContext.getNode().getIndex()+1:0;
+            final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
 
             Component nextComponent = null;
             final IndexedNode<Object> runInsNode = getNode();
@@ -194,13 +213,22 @@ public abstract class ComponentContext {
                 final IndexedNode<Object> childNode = nodeList.getNode(i);
                 final Object o = Component.getContentModel(childNode, runInsNode.getData());
                 if(o instanceof R) {
-                    final TextRunContext textRunContext = new TextRunContext(this, childNode, paragraphNode);
+                    final TextRunContext textRunContext = new TextRunContext(this, childNode);
                     nextComponent = textRunContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof RunDel) {
-                    final RunDelContext runDelContext = new RunDelContext(this, childNode, paragraphNode);
+                    final RunDelContext runDelContext = new RunDelContext(this, childNode);
                     nextComponent = runDelContext.getNextChildComponent(null, previousChildComponent);
                 }
+                else if(o instanceof CommentRangeStart) {
+                	nextComponent = new CommentRangeStartComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof CommentRangeEnd) {
+                	nextComponent = new CommentRangeEndComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof FldChar) {
+                	nextComponent = new FldCharComponent(this, childNode, nextComponentNumber);
+                }
             }
             return nextComponent;
         }
@@ -208,15 +236,13 @@ public abstract class ComponentContext {
 
     public static class RunDelContext extends ComponentContext {
 
-        final private IndexedNode<Object> paragraphNode;
-
-        public RunDelContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _paragraphNode) {
+        public RunDelContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            paragraphNode = _paragraphNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
             final int index = previousChildContext!=null?previousChildContext.getNode().getIndex()+1:0;
+            final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
 
             Component nextComponent = null;
             final IndexedNode<Object> runDelNode = getNode();
@@ -225,9 +251,18 @@ public abstract class ComponentContext {
                 final IndexedNode<Object> childNode = nodeList.getNode(i);
                 final Object o = Component.getContentModel(childNode, runDelNode.getData());
                 if(o instanceof R) {
-                    final TextRunContext textRunContext = new TextRunContext(this, childNode, paragraphNode);
+                    final TextRunContext textRunContext = new TextRunContext(this, childNode);
                     nextComponent = textRunContext.getNextChildComponent(null, previousChildComponent);
                 }
+                else if(o instanceof CommentRangeStart) {
+                	nextComponent = new CommentRangeStartComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof CommentRangeEnd) {
+                	nextComponent = new CommentRangeEndComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof FldChar) {
+                	nextComponent = new FldCharComponent(this, childNode, nextComponentNumber);
+                }
             }
             return nextComponent;
         }
@@ -235,11 +270,8 @@ public abstract class ComponentContext {
 
     public static class SdtRootContext extends ComponentContext {
 
-        final private IndexedNode<Object> rootNode;
-
-        public SdtRootContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _rootNode) {
+        public SdtRootContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            rootNode = _rootNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
@@ -247,16 +279,16 @@ public abstract class ComponentContext {
             final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
 
             Component nextComponent = null;
-            final IndexedNode<Object> runDelNode = getNode();
-            final IndexedNodeList<Object> nodeList = (IndexedNodeList<Object>)((ContentAccessor)runDelNode.getData()).getContent();
+            final IndexedNode<Object> sdtBlockNode = getNode();
+            final IndexedNodeList<Object> nodeList = (IndexedNodeList<Object>)((ContentAccessor)sdtBlockNode.getData()).getContent();
             for(int i=index; nextComponent==null&&i<nodeList.size(); i++) {
                 final IndexedNode<Object> childNode = nodeList.getNode(i);
-                final Object o = Component.getContentModel(childNode, runDelNode.getData());
+                final Object o = Component.getContentModel(childNode, sdtBlockNode.getData());
                 if(o instanceof P) {
-                    nextComponent = new ParagraphComponent(this, childNode, rootNode, nextComponentNumber);
+                    nextComponent = new ParagraphComponent(this, childNode, nextComponentNumber);
                 }
                 else if(o instanceof Tbl) {
-                    nextComponent = new TableComponent(this, childNode, rootNode, nextComponentNumber);
+                    nextComponent = new TableComponent(this, childNode, nextComponentNumber);
                 }
             }
             return nextComponent;
@@ -265,11 +297,8 @@ public abstract class ComponentContext {
 
     public static class SdtParagraphContext extends ComponentContext {
 
-        final private IndexedNode<Object> paragraphNode;
-
-        public SdtParagraphContext(ComponentContext p, IndexedNode<Object> n, IndexedNode<Object> _paragraphNode) {
+        public SdtParagraphContext(ComponentContext p, IndexedNode<Object> n) {
             super(p, n);
-            paragraphNode = _paragraphNode;
         }
         @Override
         public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
@@ -277,29 +306,60 @@ public abstract class ComponentContext {
             final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
 
             Component nextComponent = null;
-            final IndexedNode<Object> runDelNode = getNode();
-            final IndexedNodeList<Object> nodeList = (IndexedNodeList<Object>)((ContentAccessor)runDelNode.getData()).getContent();
+            final IndexedNode<Object> sdtBlockNode = getNode();
+            final IndexedNodeList<Object> nodeList = (IndexedNodeList<Object>)((ContentAccessor)sdtBlockNode.getData()).getContent();
             for(int i=index; nextComponent==null&&i<nodeList.size(); i++) {
                 final IndexedNode<Object> childNode = nodeList.getNode(i);
-                final Object o = Component.getContentModel(childNode, runDelNode.getData());
+                final Object o = Component.getContentModel(childNode, sdtBlockNode.getData());
                 if(o instanceof R) {
-                    final ComponentContext.TextRunContext textRunContext = new ComponentContext.TextRunContext(this, childNode, paragraphNode);
+                    final ComponentContext.TextRunContext textRunContext = new ComponentContext.TextRunContext(this, childNode);
                     nextComponent = textRunContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof P.Hyperlink) {
-                    final ComponentContext.HyperlinkContext hyperlinkContext = new ComponentContext.HyperlinkContext(this, childNode, paragraphNode);
+                    final ComponentContext.HyperlinkContext hyperlinkContext = new ComponentContext.HyperlinkContext(this, childNode);
                     nextComponent = hyperlinkContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof RunIns) {
-                    final ComponentContext.RunInsContext runInsContext = new ComponentContext.RunInsContext(this, childNode, paragraphNode);
+                    final ComponentContext.RunInsContext runInsContext = new ComponentContext.RunInsContext(this, childNode);
                     nextComponent = runInsContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof RunDel) {
-                    final ComponentContext.RunDelContext runDelContext = new ComponentContext.RunDelContext(this, childNode, paragraphNode);
+                    final ComponentContext.RunDelContext runDelContext = new ComponentContext.RunDelContext(this, childNode);
                     nextComponent = runDelContext.getNextChildComponent(null, previousChildComponent);
                 }
                 else if(o instanceof CTSimpleField) {
-                    nextComponent = new FldSimpleComponent(this, childNode, paragraphNode, nextComponentNumber);
+                    nextComponent = new FldSimpleComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof CommentRangeStart) {
+                	nextComponent = new CommentRangeStartComponent(this, childNode, nextComponentNumber);
+                }
+                else if(o instanceof CommentRangeEnd) {
+                	nextComponent = new CommentRangeEndComponent(this, childNode, nextComponentNumber);
+                }
+            }
+            return nextComponent;
+        }
+    }
+
+    public static class SdtRowContext extends ComponentContext {
+
+        public SdtRowContext(ComponentContext p, IndexedNode<Object> n) {
+            super(p, n);
+        }
+        @Override
+        public Component getNextChildComponent(ComponentContext previousChildContext, Component previousChildComponent) {
+            final int index = previousChildContext!=null?previousChildContext.getNode().getIndex()+1:0;
+            final int nextComponentNumber = previousChildComponent!=null?previousChildComponent.getNextComponentNumber():0;
+            final int nextGridPosition = previousChildComponent instanceof TcComponent?((TcComponent)previousChildComponent).getGridPosition()+1:0;
+
+            Component nextComponent = null;
+            final IndexedNode<Object> sdtBlockNode = getNode();
+            final IndexedNodeList<Object> nodeList = (IndexedNodeList<Object>)((ContentAccessor)sdtBlockNode.getData()).getContent();
+            for(int i=index; nextComponent==null&&i<nodeList.size(); i++) {
+                final IndexedNode<Object> childNode = nodeList.getNode(i);
+                final Object o = Component.getContentModel(childNode, sdtBlockNode.getData());
+                if(o instanceof Tc) {
+                    nextComponent = new TcComponent(this, childNode, nextComponentNumber, nextGridPosition);
                 }
             }
             return nextComponent;
