@@ -71,7 +71,6 @@ import org.docx4j.dml.CTTextBodyProperties;
 import org.docx4j.dml.CTTextShapeAutofit;
 import org.docx4j.dml.Graphic;
 import org.docx4j.dml.GraphicData;
-import org.docx4j.dml.picture.CTPictureNonVisual;
 import org.docx4j.dml.picture.Pic;
 import org.docx4j.dml.wordprocessingDrawing.Anchor;
 import org.docx4j.dml.wordprocessingDrawing.CTPosH;
@@ -109,6 +108,9 @@ import com.openexchange.office.ooxml.docx.tools.Component.IShape;
 import com.openexchange.office.ooxml.docx.tools.Component.ShapeType;
 import com.openexchange.office.ooxml.docx.tools.Component.Type;
 import com.openexchange.office.ooxml.drawingml.DMLHelper;
+import com.openexchange.office.ooxml.drawingml.GroupShape;
+import com.openexchange.office.ooxml.drawingml.Picture;
+import com.openexchange.office.ooxml.drawingml.Shape;
 import com.openexchange.office.ooxml.tools.Commons;
 
 public abstract class DrawingML {
@@ -432,8 +434,7 @@ public abstract class DrawingML {
     	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
     	    throws JSONException {
 
-		    shape.setSpPr(DMLHelper.applyShapePropertiesFromJson(shape.getSpPr(), attrs));
-
+			Shape.applyAttrsFromJSON(operationDocument, attrs, shape);
 		    final JSONObject shapeAttrs = attrs.optJSONObject("shape");
 		    if(shapeAttrs!=null) {
 		        CTTextBodyProperties bodyPr = shape.getBodyPr();
@@ -492,10 +493,9 @@ public abstract class DrawingML {
     	public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
     		throws JSONException, FilterException {
 
-	        DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, shape.getCNvPr());
-	        DMLHelper.createJsonFromShapeProperties(attrs, shape.getSpPr(), createOperationHelper.getOperationDocument().getTheme(), shape.getStyle(), true);
+			Shape.createJSONAttrs(createOperationHelper.getOperationDocument(), attrs, shape);
 
-            final JSONObject initialShapeAttrs = attrs.optJSONObject("shape");
+			final JSONObject initialShapeAttrs = attrs.optJSONObject("shape");
             final JSONObject shapeAttrs = initialShapeAttrs!=null ? initialShapeAttrs : new JSONObject();
             final CTTextBodyProperties bodyPr = shape.getBodyPr();
 	        if(bodyPr!=null) {
@@ -545,63 +545,23 @@ public abstract class DrawingML {
 		}
 		@Override
 		public void postFixNonConformance(AtomicInteger id) {
-			final CTPictureNonVisual picPr = pic.getNvPicPr();
-			if(picPr!=null) {
-				postFixNonConformance(picPr.getCNvPr(), id);
+			final CTNonVisualDrawingProps nonVisualDrawingProps = pic.getCNvPr();
+			if(nonVisualDrawingProps!=null) {
+				postFixNonConformance(nonVisualDrawingProps, id);
 			}
 		}
 		@Override
     	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
     	    throws JSONException, InvalidFormatException, PartUnrecognisedException {
 
-		    pic.setSpPr(DMLHelper.applyShapePropertiesFromJson(pic.getSpPr(), attrs));
-            final JSONObject imageAttrs = attrs.optJSONObject("image");
-            if(imageAttrs!=null) {
-                String imageUrl = Commons.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "word/media/");
-                if(imageUrl==null) {
-                    imageUrl = imageAttrs.optString("imageUrl", null);
-                }
-                if(imageUrl!=null) {
-                    final Relationship relationship = Commons.createGraphicRelation(operationDocument, operationDocument.getContextPart(), imageUrl);
-                    if(relationship!=null) {
-                        CTBlipFillProperties blipFill = pic.getBlipFill();
-                        if(blipFill==null) {
-                            blipFill = Context.getDmlObjectFactory().createCTBlipFillProperties();
-                            pic.setBlipFill(blipFill);
-                        }
-                        CTBlip blip = blipFill.getBlip();
-                        if(blip==null) {
-                            blip = Context.getDmlObjectFactory().createCTBlip();
-                            blip.setParent(blipFill);
-                            blipFill.setBlip(blip);
-                        }
-                        final boolean linked = relationship.getTargetMode() != null && relationship.getTargetMode().equals("External");
-                        if(linked) {
-                            blip.setEmbed(null);
-                            blip.setLink(relationship.getId());
-                        }
-                        else {
-                            blip.setLink(null);
-                            blip.setEmbed(relationship.getId());
-                        }
-                    }
-                }
-            }
+			Picture.applyAttrsFromJSON(operationDocument, attrs, operationDocument.getContextPart(), "word/media/", pic);
 		}
 		@Override
 		public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
-    		throws JSONException {
+    		throws JSONException, FilterException {
 
-			final CTPictureNonVisual pictureNonVisual = pic.getNvPicPr();
-			if(pictureNonVisual!=null) {
-				DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, pictureNonVisual.getCNvPr());
-			}
-	        DMLHelper.createJsonFromShapeProperties(attrs, pic.getSpPr(), true);
-
-	        final CTBlipFillProperties blipFillProperties = pic.getBlipFill();
-    		if(blipFillProperties!=null) {
-    			attrs = DMLHelper.createJsonFromBlipFillProperties(createOperationHelper.getOperationDocument().getContextPart(), attrs, blipFillProperties);
-    		}
+			final OperationDocument operationDocument = createOperationHelper.getOperationDocument();
+			Picture.createJSONAttrs(operationDocument, attrs, operationDocument.getContextPart(), pic);
             return attrs;
     	}
     }
@@ -670,43 +630,13 @@ public abstract class DrawingML {
     	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
     	    throws JAXBException, InvalidFormatException, PartUnrecognisedException, JSONException {
 
-		    CTGroupShapeProperties groupProperties = group.getGrpSpPr();
-			if(groupProperties==null) {
-				groupProperties = Context.getDmlObjectFactory().createCTGroupShapeProperties();
-				group.setGrpSpPr(groupProperties);
-			}
-			CTGroupTransform2D groupTransform2d = groupProperties.getXfrm();
-			if(groupTransform2d==null) {
-			    groupTransform2d = Context.getDmlObjectFactory().createCTGroupTransform2D();
-			    groupProperties.setXfrm(groupTransform2d);
-			}
-			DMLHelper.applyTransform2DFromJson(groupTransform2d, attrs);
-			// now applying the size and pos of the childs
-			final CTPoint2D chPos = Context.getDmlObjectFactory().createCTPoint2D();
-			chPos.setX(0);
-			chPos.setY(0);
-			groupTransform2d.setChOff(chPos);
-			final CTPositiveSize2D ex = groupTransform2d.getExt();
-			if(ex!=null) {
-			    final CTPositiveSize2D chEx = Context.getDmlObjectFactory().createCTPositiveSize2D();
-			    chEx.setCx(ex.getCx());
-			    chEx.setCy(ex.getCy());
-			    groupTransform2d.setChExt(chEx);
-			}
-			DMLHelper.applyFillPropertiesFromJson(groupProperties, attrs);
+			GroupShape.applyAttrsFromJSON(operationDocument, attrs, group);
 		}
 		@Override
     	public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
     		throws JSONException {
 
-	        DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, group.getCNvPr());
-	        final CTGroupShapeProperties groupProperties = group.getGrpSpPr();
-			if(groupProperties!=null) {
-		        final CTGroupTransform2D groupTransform =groupProperties.getXfrm();
-		        if(groupTransform!=null) {
-		        	DMLHelper.createJsonFromTransform2D(attrs, groupTransform);
-		        }
-			}
+			GroupShape.createJSONAttrs(createOperationHelper.getOperationDocument(), attrs, group);
     		return attrs;
     	}
     }
