@@ -53,7 +53,6 @@ import java.math.BigInteger;
 import java.util.Iterator;
 import java.util.List;
 
-import org.apache.commons.logging.Log;
 import org.docx4j.jaxb.Context;
 import org.docx4j.wml.BooleanDefaultTrue;
 import org.docx4j.wml.CTBorder;
@@ -69,19 +68,22 @@ import org.docx4j.wml.PPrBase.OutlineLvl;
 import org.docx4j.wml.PPrBase.PBdr;
 import org.docx4j.wml.STBorder;
 import org.docx4j.wml.STLineSpacingRule;
+import org.docx4j.wml.STTabJc;
+import org.docx4j.wml.STTabTlc;
 import org.docx4j.wml.Tabs;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.docx.OperationDocument;
 import com.openexchange.office.ooxml.tools.Commons;
 
 public class Paragraph {
 
-    protected final static Log log = LogFactory.getLog(Paragraph.class);
+	final private static Logger log = LoggerFactory.getLogger(Paragraph.class);
 
     public static void applyParagraphProperties(OperationDocument operationDocument, JSONObject paragraphProperties, PPrBase pPr)
         throws FilterException, JSONException {
@@ -211,8 +213,10 @@ public class Paragraph {
             else if(attr.equals("listStyleId")) {
                 if(value instanceof String) {
                     String sValue = (String)value;
-                    if(sValue.length()==0)
-                        throw new JSONException("docx export: apply paragraph attributes, listStyleId empty");
+                    if(sValue.length() == 0){
+                        //empty string is just the default state!
+                        continue;
+                    }
 
                     PPrBase.NumPr numPr = pPr.getNumPr();
                     if(numPr==null) {
@@ -240,8 +244,9 @@ public class Paragraph {
                         pPr.setNumPr( null );
                     }
                 }
-                else
+                else {
                     pPr.setNumPr(null);
+                }
             }
             else if(attr.equals("listLevel")) {
                 if(value instanceof Integer) {
@@ -382,6 +387,36 @@ public class Paragraph {
             		pPr.setPageBreakBefore(null);
             	}
             }
+            else if(attr.equals("tabStops")) {
+                if(value instanceof JSONArray) {
+                    JSONArray newTabs = (JSONArray) value;
+                    Tabs tabs = pPr.getTabs();
+                    if (tabs == null) {
+                        tabs = new Tabs();
+                        pPr.setTabs(tabs);
+                    }
+                    List<CTTabStop> tabStops = tabs.getTab();
+                    tabStops.clear();
+    
+                    for (int i = 0; i < newTabs.length(); i++)
+                    {
+                        JSONObject newTab = newTabs.getJSONObject(i);
+                        CTTabStop tab = new CTTabStop();
+    
+                        String fillChar = newTab.optString("fillChar", null);
+                        if(fillChar != null){
+                            tab.setLeader(STTabTlc.fromValue(fillChar));
+                        }
+                        long pos = newTab.getLong("pos");
+                        tab.setPos(Utils.map100THMMToTwip(pos));
+    
+                        String val = newTab.getString("value");
+                        tab.setVal(STTabJc.fromValue(val));
+    
+                        tabStops.add(tab);
+                    }
+                }
+            }
         }
     }
 
