@@ -101,6 +101,7 @@ import com.openexchange.office.ooxml.docx.tools.Component.IShape;
 import com.openexchange.office.ooxml.docx.tools.Component.ShapeType;
 import com.openexchange.office.ooxml.docx.tools.Component.Type;
 import com.openexchange.office.ooxml.tools.Commons;
+import com.openexchange.office.tools.doc.DocumentImageHelper;
 
 public class PictVML {
 
@@ -1183,7 +1184,7 @@ public class PictVML {
 
             final JSONObject imageAttrs = attrs.optJSONObject("image");
             if(imageAttrs!=null) {
-                String imageUrl = Commons.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "word/media/");
+                String imageUrl = DocumentImageHelper.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "word/media/");
                 if(imageUrl==null) {
                     imageUrl = imageAttrs.optString("imageUrl", null);
                 }
