@@ -196,7 +196,7 @@ public class TextUtils{
     public static void setTextRunAttributes(OperationDocument operationDocument, JSONObject attrs, TextRun_Base textRunComponent)
         throws FilterException, JAXBException, JSONException, InvalidFormatException, PartUnrecognisedException {
 
-    	final P paragraph = (P)textRunComponent.getContextObject();
+    	final P paragraph = (P)textRunComponent.getParentComponent().getNode().getData();
 
         int changeTrackFlags = Component.CT_NONE;
         Object attrsInserted = null;
@@ -320,8 +320,8 @@ public class TextUtils{
 	                	if(hyperlinkNode==null) {
 	                        final P.Hyperlink hyperlink = Context.getWmlObjectFactory().createPHyperlink();
 	                        hyperlink.setParent(paragraph);
-	                        hyperlinkNode = new IndexedNode<Object>(hyperlink, childNode.getIndex());
-	                        paragraph.getContent().setNode(childNode.getIndex(), hyperlinkNode);
+	                        hyperlinkNode = new IndexedNode<Object>(hyperlink);
+	                        paragraph.getContent().setNode(childNode, hyperlinkNode);
 	                        hyperlink.getContent().addNode(childNode);
 	                        textRunComponent.insertHyperlink(hyperlinkNode);
 	                        ((Child)childNode.getData()).setParent(hyperlink);
@@ -332,7 +332,7 @@ public class TextUtils{
 	                else {
 	                	// removing url
 	                	if(hyperlinkNode!=null) {
-	                		paragraph.getContent().setNode(hyperlinkNode.getIndex(), childNode);
+	                		paragraph.getContent().setNode(hyperlinkNode, childNode);
 	                		((Child)childNode.getData()).setParent(paragraph);
 	                		textRunComponent.removeHyperlink();
 	                	}
@@ -351,9 +351,9 @@ public class TextUtils{
             		final RunDel runDel = Context.getWmlObjectFactory().createRunDel();
             		final ContentAccessor textRunParent = (ContentAccessor)textRun.getParent();
             		runDel.setParent(textRunParent);
-            		delNode = new IndexedNode<Object>(runDel, textRunNode.getIndex());
+            		delNode = new IndexedNode<Object>(runDel);
             		textRun.setParent(runDel);
-            		((IndexedNodeList<Object>)textRunParent.getContent()).setNode(textRunNode.getIndex(), delNode);
+            		((IndexedNodeList<Object>)textRunParent.getContent()).setNode(textRunNode, delNode);
             		runDel.getContent().addNode(textRunNode);
             		textRunComponent.insertRunDelNode(delNode);
 
@@ -381,7 +381,7 @@ public class TextUtils{
 	    			final IndexedNodeList<Object> delNodeParentContent = (IndexedNodeList<Object>)delNodeParent.getContent();
 	    			final IndexedNodeList<Object> delNodeContent = (IndexedNodeList<Object>)((ContentAccessor)delNode.getData()).getContent();
 	    		    delTextToText(0, delNode.getData());
-	    		    delNodeParentContent.remove(delNode.getIndex());
+	    		    delNodeParentContent.removeNodes(delNode, null);
 	    		    IndexedNodeList.moveNodes(delNodeContent, delNodeParentContent, 0, delNode.getIndex(), delNodeContent.size(), delNodeParent);
 	    			textRunComponent.removeRunDelNode();
     			}
@@ -400,9 +400,9 @@ public class TextUtils{
             		final IndexedNode<Object> newChildNode = delNode!=null?delNode:textRunNode;
             		final ContentAccessor newParent = (ContentAccessor)((Child)(newChildNode.getData())).getParent();
             		runIns.setParent(newParent);
-            		insertNode = new IndexedNode<Object>(runIns, newChildNode.getIndex());
+            		insertNode = new IndexedNode<Object>(runIns);
             		((Child)newChildNode.getData()).setParent(runIns);
-            		((IndexedNodeList<Object>)newParent.getContent()).setNode(newChildNode.getIndex(), insertNode);
+            		((IndexedNodeList<Object>)newParent.getContent()).setNode(newChildNode, insertNode);
             		runIns.getContent().addNode(newChildNode);
             		textRunComponent.insertRunInsNode(insertNode);
             	}
@@ -415,7 +415,7 @@ public class TextUtils{
         			final ContentAccessor insertNodeParent = (ContentAccessor)((Child)insertNode.getData()).getParent();
         			final IndexedNodeList<Object> insertNodeParentContent = (IndexedNodeList<Object>)insertNodeParent.getContent();
         			final IndexedNodeList<Object> insertNodeContent = (IndexedNodeList<Object>)((ContentAccessor)insertNode.getData()).getContent();
-        			insertNodeParentContent.remove(insertNode.getIndex());
+        			insertNodeParentContent.removeNodes(insertNode, null);
         			IndexedNodeList.moveNodes(insertNodeContent, insertNodeParentContent, 0, insertNode.getIndex(), insertNodeContent.size(), insertNodeParent);
         			textRunComponent.removeRunInsNode();
     			}
