@@ -60,6 +60,7 @@ import org.docx4j.IndexedNodeList;
 import org.docx4j.dml.BaseStyles;
 import org.docx4j.dml.CTColorScheme;
 import org.docx4j.dml.Theme;
+import org.docx4j.jaxb.Context;
 import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
 import org.docx4j.w15.CTPeople;
 import org.docx4j.w15.CTPerson;
@@ -340,22 +341,28 @@ public class Utils {
         return tintTransform;
     }
 
-    public static SectPr getDocumentProperties(MainDocumentPart mainDocPart) {
+    public static SectPr getDocumentProperties(MainDocumentPart mainDocPart, boolean createIfMissing) {
 
         SectPr sectPr = null;
         if(mainDocPart!=null) {
-            Document doc = mainDocPart.getJaxbElement();
+            final Document doc = mainDocPart.getJaxbElement();
             if(doc!=null) {
-                Body body = doc.getBody();
-                if(body!=null)
+                final Body body = doc.getBody();
+                if(body!=null) {
                     sectPr = body.getSectPr();
+                }
+                if(sectPr==null&&createIfMissing) {
+                	sectPr = Context.getWmlObjectFactory().createSectPr();
+                	sectPr.setParent(body);
+                	body.setSectPr(sectPr);
+                }
             }
         }
         return sectPr;
     }
 
     public static long getPageWidth(MainDocumentPart mainDocPart, long defaultWidth) {
-    	final SectPr sectPr = getDocumentProperties(mainDocPart);
+    	final SectPr sectPr = getDocumentProperties(mainDocPart, false);
     	if(sectPr!=null) {
 	        PgMar pgMar = sectPr.getPgMar();
 	        PgSz pgSz = sectPr.getPgSz();
@@ -404,24 +411,26 @@ public class Utils {
     	throws JSONException {
 
 		final JSONObject trackInfoDesc = new JSONObject(2);
-		final String author = changeTrack.getAuthor();
-		if(author!=null) {
-			trackInfoDesc.put("author", changeTrack.getAuthor());
-			operationDocument.addChangeTrackAuthor(changeTrack.getAuthor(), changeTrack.getDate());
-
-			final CTPeople ctPeople = operationDocument.getPeopleList(false);
-			if(ctPeople!=null) {
-				final List<CTPerson> peopleList = ctPeople.getPerson();
-				if(peopleList!=null) {
-					for(CTPerson person:peopleList) {
-						if(person.getAuthor()!=null&&person.getAuthor().equals(author)) {
-							final CTPresenceInfo presenceInfo = person.getPresenceInfo();
-							if(presenceInfo!=null) {
-								final String providerId = presenceInfo.getProviderId();
-								final String userId = presenceInfo.getUserId();
-								if(providerId!=null&&userId!=null) {
-									if(providerId.equals("None")&&userId.startsWith("OXText_")) {
-										trackInfoDesc.put("uid", userId.substring(7));
+		if(changeTrack!=null) {
+			final String author = changeTrack.getAuthor();
+			if(author!=null) {
+				trackInfoDesc.put("author", changeTrack.getAuthor());
+				operationDocument.addChangeTrackAuthor(changeTrack.getAuthor(), changeTrack.getDate());
+	
+				final CTPeople ctPeople = operationDocument.getPeopleList(false);
+				if(ctPeople!=null) {
+					final List<CTPerson> peopleList = ctPeople.getPerson();
+					if(peopleList!=null) {
+						for(CTPerson person:peopleList) {
+							if(person.getAuthor()!=null&&person.getAuthor().equals(author)) {
+								final CTPresenceInfo presenceInfo = person.getPresenceInfo();
+								if(presenceInfo!=null) {
+									final String providerId = presenceInfo.getProviderId();
+									final String userId = presenceInfo.getUserId();
+									if(providerId!=null&&userId!=null) {
+										if(providerId.equals("None")&&userId.startsWith("OXText_")) {
+											trackInfoDesc.put("uid", userId.substring(7));
+										}
 									}
 								}
 							}
@@ -429,85 +438,86 @@ public class Utils {
 					}
 				}
 			}
-		}
-		if(changeTrack.getDate()!=null) {
-			trackInfoDesc.put("date", changeTrack.getDate().toXMLFormat());
+			if(changeTrack.getDate()!=null) {
+				trackInfoDesc.put("date", changeTrack.getDate().toXMLFormat());
+			}
 		}
 		return trackInfoDesc;
     }
 
-    public static void applyTrackInfoFromJSON(com.openexchange.office.ooxml.docx.OperationDocument operationDocument, JSONObject trackInfo, CTTrackChange changeTrack) {
+    public static void applyTrackInfoFromJSON(com.openexchange.office.ooxml.docx.OperationDocument operationDocument, JSONObject json, CTTrackChange changeTrack) {
+    	if(json!=null) {
+    		applyTrackInfoFromJSON(operationDocument, json.opt("author"), json.opt("date"), json.opt("uid"), changeTrack);
+    	}
+    }
 
-    		final org.docx4j.w15.ObjectFactory peopleObjectFactory = new org.docx4j.w15.ObjectFactory();
+    public static void applyTrackInfoFromJSON(com.openexchange.office.ooxml.docx.OperationDocument operationDocument, Object author, Object date, Object uid, CTTrackChange changeTrack) {
 
-        	if(trackInfo!=null) {
-            	try {
-            		String uidAuthor = changeTrack.getAuthor();
-	    			final Object author = trackInfo.opt("author");
-	    			if(author!=null) {
-		    			if(author instanceof String) {
-		    				uidAuthor = (String)author;
-		    				changeTrack.setAuthor(uidAuthor);
-		    			}
-		    			else {
-		    				changeTrack.setAuthor(null);
-		    			}
-	    			}
-	    			final Object date = trackInfo.opt("date");
-	    			if(date!=null) {
-		    			if(date instanceof String) {
-		    				final XMLGregorianCalendar xgc=DatatypeFactory.newInstance().newXMLGregorianCalendar((String)date);
-		    				changeTrack.setDate(xgc);
-		    			}
-		    			else {
-		    				changeTrack.setDate(null);
-		    			}
-	    			}
-	    			Object uid = trackInfo.opt("uid");
-	    			if(uid!=null&&uidAuthor!=null&&!uidAuthor.isEmpty()) {
-	    				if(uid instanceof String) {
-	    					final CTPeople ctPeople = operationDocument.getPeopleList(true);
-	    					if(ctPeople!=null) {
-		    					final List<CTPerson> peopleList = ctPeople.getPerson();
-		    					if(peopleList!=null) {
-		    						int i=0;
-		    						for(;i<peopleList.size();i++) {
-		    							final CTPerson person = peopleList.get(i);
-		    							if(person.getAuthor()!=null&&person.getAuthor().equals(uidAuthor)) {
-		    								final CTPresenceInfo presenceInfo = person.getPresenceInfo();
-		    								if(presenceInfo!=null) {
-			    								final String providerId = presenceInfo.getProviderId();
-			    								final String userId = presenceInfo.getUserId();
-			    								if(providerId.equals("None")&&userId.startsWith("OXText_")) {
-			    									if(userId.substring(7).equals((String)uid)) {
-				    									break;
-				    								}
-			    								}
+		final org.docx4j.w15.ObjectFactory peopleObjectFactory = new org.docx4j.w15.ObjectFactory();
+
+    	try {
+    		String uidAuthor = changeTrack.getAuthor();
+			if(author!=null) {
+    			if(author instanceof String) {
+    				uidAuthor = (String)author;
+    				changeTrack.setAuthor(uidAuthor);
+    			}
+    			else {
+    				changeTrack.setAuthor(null);
+    			}
+			}
+			if(date!=null) {
+    			if(date instanceof String) {
+    				final XMLGregorianCalendar xgc=DatatypeFactory.newInstance().newXMLGregorianCalendar((String)date);
+    				changeTrack.setDate(xgc);
+    			}
+    			else {
+    				changeTrack.setDate(null);
+    			}
+			}
+			if(uid!=null&&uidAuthor!=null&&!uidAuthor.isEmpty()) {
+				if(uid instanceof String) {
+					final CTPeople ctPeople = operationDocument.getPeopleList(true);
+					if(ctPeople!=null) {
+    					final List<CTPerson> peopleList = ctPeople.getPerson();
+    					if(peopleList!=null) {
+    						int i=0;
+    						for(;i<peopleList.size();i++) {
+    							final CTPerson person = peopleList.get(i);
+    							if(person.getAuthor()!=null&&person.getAuthor().equals(uidAuthor)) {
+    								final CTPresenceInfo presenceInfo = person.getPresenceInfo();
+    								if(presenceInfo!=null) {
+	    								final String providerId = presenceInfo.getProviderId();
+	    								final String userId = presenceInfo.getUserId();
+	    								if(providerId.equals("None")&&userId.startsWith("OXText_")) {
+	    									if(userId.substring(7).equals((String)uid)) {
+		    									break;
 		    								}
-		    							}
-		    						}
-		    						if(i==peopleList.size()) {
-			    						final CTPerson newPerson = peopleObjectFactory.createCTPerson();
-			    						newPerson.setParent(ctPeople);
-			    						newPerson.setAuthor(uidAuthor);
-			    						peopleList.add(newPerson);
-										final CTPresenceInfo presenceInfo = peopleObjectFactory.createCTPresenceInfo();
-										presenceInfo.setParent(newPerson);
-										presenceInfo.setProviderId("None");
-										presenceInfo.setUserId("OXText_"+(String)uid);
-										newPerson.setPresenceInfo(presenceInfo);
-		    						}
-								}
-		    				}
-	    				}
-	    				else {
-	    					// TODO: removing uid for this author
-	    				}
-	    			}
-            	}
-    	    	catch(DatatypeConfigurationException e) {
-    	    		// ups
-    	    	};
-        	}
-	    }
+	    								}
+    								}
+    							}
+    						}
+    						if(i==peopleList.size()) {
+	    						final CTPerson newPerson = peopleObjectFactory.createCTPerson();
+	    						newPerson.setParent(ctPeople);
+	    						newPerson.setAuthor(uidAuthor);
+	    						peopleList.add(newPerson);
+								final CTPresenceInfo presenceInfo = peopleObjectFactory.createCTPresenceInfo();
+								presenceInfo.setParent(newPerson);
+								presenceInfo.setProviderId("None");
+								presenceInfo.setUserId("OXText_"+(String)uid);
+								newPerson.setPresenceInfo(presenceInfo);
+    						}
+						}
+    				}
+				}
+				else {
+					// TODO: removing uid for this author
+				}
+			}
+    	}
+    	catch(DatatypeConfigurationException e) {
+    		// ups
+    	};
+	}
 }
