@@ -56,7 +56,7 @@ import org.docx4j.jaxb.Context;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.IExporter;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
@@ -84,7 +84,7 @@ public class Exporter implements IExporter {
      * @see com.openexchange.office.IExporter#createDocument(java.io.InputStream, org.json.JSONObject)
      */
     @Override
-    public InputStream createDocument(Session session, InputStream documentStm, String applyOperations, ResourceManager resourceManager, DocumentProperties documentProperties, boolean createFinalDocument)
+    public InputStream createDocument(Session session, InputStream documentStm, String applyOperations, IResourceManager resourceManager, DocumentProperties documentProperties, boolean createFinalDocument)
         throws FilterException {
 
         OperationDocument operationDocument = null;
