@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,11 +51,11 @@ package com.openexchange.office.ooxml.pptx;
 
 import java.io.InputStream;
 
+import org.docx4j.jaxb.Context;
 import org.json.JSONObject;
 
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
-import com.openexchange.office.FilterException.ErrorCode;
 import com.openexchange.office.IImporter;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
@@ -75,6 +75,7 @@ public class Importer implements IImporter {
     public Importer(ServiceLookup _services) {
         super();
         services = _services;
+        Context.getWmlObjectFactory();
     }
 
     /*
@@ -85,17 +86,23 @@ public class Importer implements IImporter {
     public JSONObject createOperations(Session session, InputStream inputDocumentStm, DocumentProperties documentProperties)
         throws FilterException {
 
+        OperationDocument operationDocument = null;
         try {
             boolean debugGetDefaultDocument = false;
             if(debugGetDefaultDocument) {   // just a simple helper method to store our in memory created
                 final OperationDocument emptyDoc = new OperationDocument(services, documentProperties);
                 emptyDoc.writeFile("/tmp/default.oxt");    // template documents...
             }
-            final OperationDocument operationDocument = new OperationDocument(session, services, inputDocumentStm, null, documentProperties);
+            operationDocument = new OperationDocument(session, services, inputDocumentStm, null, documentProperties);
             return operationDocument.getOperations();
         }
         catch(Throwable e) {
-        	throw OperationDocument.rethrowException(e);
+        	throw OperationDocument.getRethrowException(e);
+        }
+        finally {
+            if(operationDocument!=null) {
+                operationDocument.removeMemoryListener();
+            }
         }
     }
 
@@ -103,15 +110,21 @@ public class Importer implements IImporter {
     public InputStream getDefaultDocument(InputStream templateDocument, DocumentProperties documentProperties)
         throws FilterException {
 
+        OperationDocument operationDocument = null;
         if(templateDocument!=null) {
             return templateDocument;
         }
         try {
-	        final OperationDocument operationDocument = new OperationDocument(services, documentProperties);
+	        operationDocument = new OperationDocument(services, documentProperties);
 	        return operationDocument.save();
         }
         catch(Throwable e) {
-        	throw OperationDocument.rethrowException(e);
+        	throw OperationDocument.getRethrowException(e);
+        }
+        finally {
+            if(operationDocument!=null) {
+                operationDocument.removeMemoryListener();
+            }
         }
     }
 }
