@@ -63,7 +63,7 @@ import org.json.JSONTokener;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
@@ -71,7 +71,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
 
     private PresentationMLPackage opcPackage;
 
-    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, ResourceManager _resourceManager, DocumentProperties _documentProperties)
+    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, IResourceManager _resourceManager, DocumentProperties _documentProperties)
         throws FilterException {
 
         super(session, _services, _inputDocumentStream, _resourceManager, _documentProperties);
@@ -107,6 +107,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
             try {
                 final JSONArray aOperations = new JSONArray(new JSONTokener(applyOperations));
                 for (i = 0; i < aOperations.length(); i++) {
+                	successfulAppliedOperations = i;
                     op = (JSONObject) aOperations.get(i);
                     String opName = op.getString("name");
 /*
@@ -150,6 +151,7 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                         applyOperationHelper.insertListStyle(op.getString("listStyleId"), op.getJSONObject("listDefinition"));
 */
                 }
+                successfulAppliedOperations=aOperations.length();
             }
             catch(Exception e) {
                 String message = e.getMessage();
