@@ -49,20 +49,15 @@
 
 package com.openexchange.office.ooxml.xlsx;
 
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
-import java.util.zip.ZipEntry;
-import java.util.zip.ZipInputStream;
 
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.JAXBException;
 
-import org.apache.commons.io.IOUtils;
-import org.apache.commons.logging.Log;
+import org.apache.commons.lang.mutable.MutableInt;
 import org.docx4j.dml.Theme;
 import org.docx4j.openpackaging.exceptions.InvalidFormatException;
 import org.docx4j.openpackaging.packages.OpcPackage;
@@ -79,6 +74,8 @@ import org.docx4j.relationships.Relationship;
 import org.json.JSONArray;
 import org.json.JSONObject;
 import org.json.JSONTokener;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.xlsx4j.jaxb.Context;
 import org.xlsx4j.sml.CTCalcPr;
 import org.xlsx4j.sml.CTDataValidations;
@@ -104,24 +101,25 @@ import org.xlsx4j.sml.Sheets;
 import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
-import com.openexchange.office.ooxml.tools.AttrsMode;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.office.ooxml.xlsx.operations.ApplyOperationHelper;
 import com.openexchange.office.ooxml.xlsx.operations.CreateOperationHelper;
 import com.openexchange.office.ooxml.xlsx.tools.AutoFilterHelper;
+import com.openexchange.office.ooxml.xlsx.tools.ConditionalFormattings;
 import com.openexchange.office.ooxml.xlsx.tools.SheetUtils;
+import com.openexchange.office.ooxml.xlsx.tools.TableHelper;
 import com.openexchange.office.ooxml.xlsx.tools.ValidationUtils;
 import com.openexchange.office.tools.ConfigurationHelper;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.doc.DocumentComplexity;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
 public class OperationDocument extends com.openexchange.office.ooxml.OperationDocument {
 
-    final private int FACTOR_FOR_REQUIRED_HEAPSPACE = 32;
+    final private long FACTOR_FOR_REQUIRED_HEAPSPACE = 32;
     private SpreadsheetMLPackage opcPackage;
     protected CTStylesheet stylesheet = null;
     protected CTSst sharedStrings = null;
@@ -129,15 +127,16 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
     private CreateOperationHelper _createOperationHelper = null;
     private Double defaultFontSize = null;
 
-    protected static Log log = LogFactory.getLog(OperationDocument.class);
+	private final static Logger log = LoggerFactory.getLogger(OperationDocument.class);
 
-    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, ResourceManager _resourceManager, DocumentProperties _documentProperties)
+    public OperationDocument(Session session, ServiceLookup _services, InputStream _inputDocumentStream, IResourceManager _resourceManager, DocumentProperties _documentProperties)
         throws FilterException {
 
         super(session, _services, null, _resourceManager, _documentProperties);
         try {
-
-        	inputDocumentStream = checkDocumentComplexity(_inputDocumentStream, _documentProperties.optLong(DocumentProperties.PROP_MAX_MEMORY_USAGE, -1));
+        	final long maxAllowedXmlSizeMatchingComplexity = ConfigurationHelper.getIntegerOfficeConfigurationValue(getServiceLookup(), getSession(), "//spreadsheet/maxCells", 500000) * 100;
+        	final long maxAllowedXmlSizeMatchingMemory = _documentProperties.optLong(DocumentProperties.PROP_MAX_MEMORY_USAGE, -1) / FACTOR_FOR_REQUIRED_HEAPSPACE;
+        	inputDocumentStream = DocumentComplexity.checkDocumentComplexity(_inputDocumentStream, maxAllowedXmlSizeMatchingComplexity, maxAllowedXmlSizeMatchingMemory);
             opcPackage = (SpreadsheetMLPackage)SpreadsheetMLPackage.load(inputDocumentStream);
 
             // styles
@@ -184,7 +183,6 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
             if(sheetFormatPr==null) {
                 sheetFormatPr = Context.getsmlObjectFactory().createCTSheetFormatPr();
                 worksheet.setSheetFormatPr(sheetFormatPr);
-                sheetFormatPr.setParent(worksheet);
                 sheetFormatPr.setBaseColWidth(new Long(10L));
                 sheetFormatPr.setDefaultRowHeight(14.5);
             }
@@ -197,7 +195,8 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         }
     }
 
-    public Part getContextPart() {
+    @Override
+	public Part getContextPart() {
     	return contextPart!=null ? contextPart : getPackage().getWorkbookPart();
     }
 
@@ -344,89 +343,230 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                 applyOperationHelper = new ApplyOperationHelper(this);
                 final JSONArray aOperations = new JSONArray(new JSONTokener(applyOperations));
                 for (i = 0; i < aOperations.length(); i++) {
+                	successfulAppliedOperations = i;
                     op = (JSONObject) aOperations.get(i);
                     final String opName = op.getString("name");
-                    if (opName.equals("setCellContents"))
-                        applyOperationHelper.setCellContents(op.getInt("sheet"), op.getJSONArray("start"), op.getJSONArray("contents"));
-                    else if(opName.equals("fillCellRange"))
-                        applyOperationHelper.fillCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.opt("value"), op.opt("result"), op.optJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)), op.optInt("shared", -1), op.optJSONArray("ref"));
-                    else if(opName.equals("clearCellRange"))
-                        applyOperationHelper.clearCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), true, true);
-                    else if (opName.equals("setRowAttributes"))
-                        applyOperationHelper.setRowAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)));
-                    else if (opName.equals("setColumnAttributes"))
-                        applyOperationHelper.setColumnAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"), AttrsMode.getEnum(op.optString("attrsMode", null)));
-                    else if (opName.equals("insertRows"))
-                        applyOperationHelper.insertRows(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
-                    else if (opName.equals("deleteRows"))
-                        applyOperationHelper.deleteRows(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
-                    else if (opName.equals("insertColumns"))
-                        applyOperationHelper.insertColumns(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
-                    else if (opName.equals("deleteColumns"))
-                        applyOperationHelper.deleteColumns(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
-                    else if (opName.equals("insertSheet"))
-                        applyOperationHelper.insertSheet(op.getInt("sheet"), op.getString("sheetName"), op.optJSONObject("attrs"));
-                    else if (opName.equals("moveSheet"))
-                        applyOperationHelper.moveSheet(op.getInt("sheet"), op.getInt("to"));
-                    else if (opName.equals("deleteSheet"))
-                        applyOperationHelper.deleteSheet(op.getInt("sheet"));
-                    else if (opName.equals("copySheet"))
-                    	applyOperationHelper.copySheet(op.getInt("sheet"), op.getInt("to"), op.getString("sheetName"));
-                    else if (opName.equals("setSheetName"))
-                        applyOperationHelper.setSheetName(op.getInt("sheet"), op.getString("sheetName"));
-                    else if (opName.equals("setSheetAttributes"))
-                        applyOperationHelper.setSheetAttributes(op.getInt("sheet"), op.getJSONObject("attrs"));
-                    else if (opName.equals("mergeCells"))
-                        applyOperationHelper.mergeCells(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.optBoolean("keepContent", false), op.optString("type", "merge"));
-                    else if (opName.equals("insertDrawing"))
-                        applyOperationHelper.insertDrawing(op.getJSONArray("start"), op.getString("type"), op.optJSONObject("attrs"));
-                    else if (opName.equals("deleteDrawing"))
-                        applyOperationHelper.deleteDrawing(op.getJSONArray("start"));
-                    else if (opName.equals("setDrawingAttributes"))
-                        applyOperationHelper.applyDrawingAttributes(op.getJSONArray("start"), op.getJSONObject("attrs"));
-                    else if (opName.equals("insertStyleSheet"))
-                        applyOperationHelper.insertStyleSheet(op.getString("type"), op.getString("styleId"), op.getString("styleName"), op.getJSONObject("attrs"), op.optString("parent", null), op.optBoolean("hidden", false), op.optInt("uiPriority", 0), op.optBoolean("default", false), op.optBoolean("auto", false));
-                    else if (opName.equals("insertChartDataSeries"))
-                        applyOperationHelper.insertChartDataSeries(op.getJSONArray("start"), op.getInt("series"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertChartDataSeries"))
-                        applyOperationHelper.insertChartDataSeries(op.getJSONArray("start"), op.getInt("series"), op.optJSONObject("attrs"));
-                    else if (opName.equals("setChartDataSeriesAttributes"))
-                        applyOperationHelper.setChartDataSeriesAttributes(op.getJSONArray("start"), op.getInt("series"), op.optJSONObject("attrs"));
-                    else if (opName.equals("deleteChartDataSeries"))
-                        applyOperationHelper.deleteChartDataSeries(op.getJSONArray("start"), op.getInt("series"));
-                    else if (opName.equals("insertName") || opName.equals("changeName"))
-                        applyOperationHelper.insertName(op.optLong("sheet", -1), op.getString("exprName"), op.getString("formula"), null);
-                    else if (opName.equals("deleteName"))
-                        applyOperationHelper.deleteName(op.optLong("sheet"), op.getString("exprName"));
-                    else if (opName.equals("setChartAxisAttributes"))
-                        applyOperationHelper.setChartAxisAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
-                    else if (opName.equals("setChartGridlineAttributes"))
-                        applyOperationHelper.setChartGridlineAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
-                    else if (opName.equals("setChartTitleAttributes"))
-                        applyOperationHelper.setChartTitleAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
-                    else if (opName.equals("setChartLegendAttributes"))
-                        applyOperationHelper.setChartLegendAttributes(op.getJSONArray("start"), op.optJSONObject("attrs"));
-                    else if (opName.equals("insertValidation"))
-                    	ValidationUtils.insertValidation(this, op);
-                    else if (opName.equals("changeValidation"))
-                    	ValidationUtils.changeValidation(this, op);
-                    else if (opName.equals("deleteValidation"))
-                    	ValidationUtils.deleteValidation(this, op.getInt("sheet"), op.getInt("index"));
-                    else if (opName.equals("setDocumentAttributes"))
-                    	applyOperationHelper.setDocumentAttributes(op.getJSONObject("attrs"));
-                    else if (opName.equals("insertTable"))
-                        AutoFilterHelper.insertTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"));
-                    else if (opName.equals("changeTable"))
-                        AutoFilterHelper.changeTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"));
-                    else if (opName.equals("deleteTable"))
-                        AutoFilterHelper.deleteTable(applyOperationHelper, op.getInt("sheet"), op.optString("table", null));
-                    else if (opName.equals("changeTableColumn"))
-                        AutoFilterHelper.changeTableColumn(applyOperationHelper, op.getInt("sheet"), op.optString("table", null), op.getLong("col"), op.getJSONObject("attrs"));
-                    else if (opName.equals("createError"))
-                        throw new FilterException("createError operation detected: " + opName, ErrorCode.UNSUPPORTED_OPERATION_USED);
-                    else
-                        logMessage("warn", "Ignoring unsupported operation: " + opName);
+                    switch(opName) {
+	                    case "setCellContents": {
+	                        applyOperationHelper.setCellContents(op.getInt("sheet"), op.getJSONArray("start"), op.getJSONArray("contents"));
+	                        break;
+	                    }
+	                    case "fillCellRange": {
+	                        applyOperationHelper.fillCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.opt("value"), op.opt("result"), op.optJSONObject("attrs"), op.optInt("shared", -1), op.optJSONArray("ref"));
+	                        break;
+	                    }
+	                    case "clearCellRange": {
+	                        applyOperationHelper.clearCellRange(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.optBoolean("remove", false));
+	                        break;
+	                    }
+	                    case "setRowAttributes": {
+	                        applyOperationHelper.setRowAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "setColumnAttributes": {
+	                        applyOperationHelper.setColumnAttributes(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1), op.getJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "insertRows": {
+	                        applyOperationHelper.insertRows(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
+	                        break;
+	                    }
+	                    case "deleteRows": {
+	                        applyOperationHelper.deleteRows(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
+	                        break;
+	                    }
+	                    case "insertColumns": {
+	                        applyOperationHelper.insertColumns(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
+	                        break;
+	                    }
+	                    case "deleteColumns": {
+	                        applyOperationHelper.deleteColumns(op.getInt("sheet"), op.getInt("start"), op.optInt("end", -1));
+	                        break;
+	                    }
+	                    case "insertSheet": {
+	                        applyOperationHelper.insertSheet(op.getInt("sheet"), op.getString("sheetName"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "moveSheet": {
+	                        applyOperationHelper.moveSheet(op.getInt("sheet"), op.getInt("to"));
+	                        break;
+	                    }
+	                    case "deleteSheet": {
+	                        applyOperationHelper.deleteSheet(op.getInt("sheet"));
+	                        break;
+	                    }
+	                    case "copySheet": {
+	                    	applyOperationHelper.copySheet(op.getInt("sheet"), op.getInt("to"), op.getString("sheetName"));
+	                    	break;
+	                    }
+	                    case "setSheetName": {
+	                        applyOperationHelper.setSheetName(op.getInt("sheet"), op.getString("sheetName"));
+	                        break;
+	                    }
+	                    case "setSheetAttributes": {
+	                        applyOperationHelper.setSheetAttributes(op.getInt("sheet"), op.getJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "mergeCells": {
+	                        applyOperationHelper.mergeCells(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.optBoolean("keepContent", false), op.optString("type", "merge"));
+	                        break;
+	                    }
+	                    case "insertDrawing": {
+	                        applyOperationHelper.insertDrawing(op.getJSONArray("start"), op.getString("type"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "deleteDrawing": {
+	                        applyOperationHelper.deleteDrawing(op.getJSONArray("start"));
+	                        break;
+	                    }
+	                    case "setDrawingAttributes": {
+	                        applyOperationHelper.applyDrawingAttributes(op.getJSONArray("start"), op.getJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "insertStyleSheet": {
+	                        applyOperationHelper.insertStyleSheet(op.getString("type"), op.getString("styleId"), op.getString("styleName"), op.getJSONObject("attrs"), op.optString("parent", null), op.optBoolean("hidden", false), op.optInt("uiPriority", 0), op.optBoolean("default", false), op.optBoolean("auto", false));
+	                        break;
+	                    }
+	                    case "insertChartDataSeries": {
+	                        applyOperationHelper.insertChartDataSeries(op.getJSONArray("start"), op.getInt("series"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "setChartDataSeriesAttributes": {
+	                        applyOperationHelper.setChartDataSeriesAttributes(op.getJSONArray("start"), op.getInt("series"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "deleteChartDataSeries": {
+	                        applyOperationHelper.deleteChartDataSeries(op.getJSONArray("start"), op.getInt("series"));
+	                        break;
+	                    }
+	                    case "insertName": {
+	                        applyOperationHelper.insertName(op.optLong("sheet", -1), op.getString("exprName"), op.getString("formula"), null);
+	                        break;
+	                    }
+	                    case "changeName": {
+	                        applyOperationHelper.changeName(op.optLong("sheet", -1), op.getString("exprName"), op.optString("formula", null), op.optString("newName", null));
+	                        break;
+	                    }
+	                    case "deleteName": {
+	                        applyOperationHelper.deleteName(op.optLong("sheet", -1), op.getString("exprName"));
+	                        break;
+	                    }
+	                    case "setChartAxisAttributes": {
+	                        applyOperationHelper.setChartAxisAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "setChartGridlineAttributes": {
+	                        applyOperationHelper.setChartGridlineAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "setChartTitleAttributes": {
+	                        applyOperationHelper.setChartTitleAttributes(op.getJSONArray("start"), op.getString("axis"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "setChartLegendAttributes": {
+	                        applyOperationHelper.setChartLegendAttributes(op.getJSONArray("start"), op.optJSONObject("attrs"));
+	                        break;
+	                    }
+	                    case "insertValidation": {
+	                    	ValidationUtils.insertValidation(this, op);
+	                    	break;
+	                    }
+	                    case "changeValidation": {
+	                    	ValidationUtils.changeValidation(this, op);
+	                    	break;
+	                    }
+	                    case "deleteValidation": {
+	                    	ValidationUtils.deleteValidation(this, op.getInt("sheet"), op.getInt("index"));
+	                    	break;
+	                    }
+	                    case "setDocumentAttributes": {
+	                    	applyOperationHelper.setDocumentAttributes(op.getJSONObject("attrs"));
+	                    	break;
+	                    }
+	                    case "insertTable": {
+	                    	final String tableName = op.optString("table", null);
+	                    	if(tableName==null||tableName.isEmpty()) {
+	                    		AutoFilterHelper.insertTable(applyOperationHelper, op.getInt("sheet"), null, op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"));
+	                    	}
+	                    	else {
+	                    		TableHelper.changeOrInsertTable(applyOperationHelper, op.getInt("sheet"), tableName, op.getJSONArray("start"), op.getJSONArray("end"), op.optJSONObject("attrs"), true);
+	                    	}
+	                        break;
+	                    }
+	                    case "changeTable": {
+	                    	final String tableName = op.optString("table", null);
+	                    	if(tableName==null||tableName.isEmpty()) {
+	                    		AutoFilterHelper.changeTable(applyOperationHelper, op.getInt("sheet"), null, op.optJSONArray("start"), op.optJSONArray("end"), op.optJSONObject("attrs"));
+	                    	}
+	                    	else {
+	                    		TableHelper.changeOrInsertTable(applyOperationHelper, op.getInt("sheet"), tableName, op.optJSONArray("start"), op.optJSONArray("end"), op.optJSONObject("attrs"), false);
+	                    	}
+	                        break;
+	                    }
+	                    case "deleteTable": {
+	                    	final String tableName = op.optString("table", null);
+	                    	if(tableName==null||tableName.isEmpty()) {
+	                    		AutoFilterHelper.deleteTable(applyOperationHelper, op.getInt("sheet"), null);
+	                    	}
+	                    	else {
+	                    		TableHelper.deleteTable(applyOperationHelper, op.getInt("sheet"), tableName);
+	                    	}
+	                        break;
+	                    }
+	                    case "changeTableColumn": {
+	                    	final String tableName = op.optString("table", null);
+	                    	if(tableName==null||tableName.isEmpty()) {
+	                    		AutoFilterHelper.changeTableColumn(applyOperationHelper, op.getInt("sheet"), null, op.getLong("col"), op.getJSONObject("attrs"));
+	                    	}
+	                    	else {
+	                    		TableHelper.changeTableColumn(applyOperationHelper, op.getInt("sheet"), tableName, op.getLong("col"), op.getJSONObject("attrs"));
+	                    	}
+	                        break;
+	                    }
+	                    case "insertCondFormat": {
+	                    	ConditionalFormattings.insertCondFormat(this, op.getInt("sheet"), op.getInt("index"), op.getJSONArray("ranges"));
+	                    	break;
+	                    }
+	                    case "changeCondFormat": {
+	                    	ConditionalFormattings.changeCondFormat(this, op.getInt("sheet"), op.getInt("index"), op.getJSONArray("ranges"));
+	                    	break;
+	                    }
+	                    case "deleteCondFormat": {
+	                    	ConditionalFormattings.deleteCondFormat(this, op.getInt("sheet"), op.getInt("index"));
+	                    	break;
+	                    }
+	                    case "insertCondFormatRule": {
+	                    	ConditionalFormattings.changeOrinsertCondFormatRule(this, true, op.getInt("sheet"), op.getInt("index"), op.getInt("ruleIndex"), op.optString("type", "formula"),
+	                    		op.has("value1") ? op.get("value1") : "", op.optString("value2", ""), op.optInt("priority"), op.optBoolean("stop", false), op.has("attrs") ? op.getJSONObject("attrs") : new JSONObject());
+	                    	break;
+	                    }
+	                    case "changeCondFormatRule": {
+	                    	ConditionalFormattings.changeOrinsertCondFormatRule(this, false, op.getInt("sheet"), op.getInt("index"), op.getInt("ruleIndex"), op.optString("type", null),
+	                    		op.opt("value1"), op.optString("value2", null), op.optInt("priority"), op.optBoolean("stop"), op.optJSONObject("attrs"));
+	                    	break;
+	                    }
+	                    case "deleteCondFormatRule": {
+	                    	ConditionalFormattings.deleteCondFormatRule(this, op.getInt("sheet"), op.getInt("index"), op.getInt("ruleIndex"));
+	                    	break;
+	                    }
+	                    case "insertHyperlink": {
+	                    	applyOperationHelper.insertHyperlink(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"), op.getString("url"));
+	                    	break;
+	                    }
+	                    case "deleteHyperlink": {
+	                    	applyOperationHelper.deleteHyperlink(op.getInt("sheet"), op.getJSONArray("start"), op.optJSONArray("end"));
+	                    	break;
+	                    }
+	                    case "createError": {
+	                        throw new FilterException("createError operation detected: " + opName, ErrorCode.UNSUPPORTED_OPERATION_USED);
+	                    }
+	                    default: {
+	                        logMessage("warn", "Ignoring unsupported operation: " + opName);
+	                        break;
+	                    }
+                    }
                 }
+                successfulAppliedOperations=aOperations.length();
                 final WorkbookPart workbookPart = getPackage().getWorkbookPart();
                 final Workbook workbook = workbookPart.getJaxbElement();
                 CTCalcPr calcPr = workbook.getCalcPr();
@@ -449,15 +589,17 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                     }
                 }
 
-                // removing tabSelected from other worksheets
+                // removing tabSelected from other worksheets and finalizing tables
                 final Sheets sheets = workbook.getSheets();
                 final List<Sheet> sheetList = sheets.getSheet();
+                final MutableInt uniqueTableId = new MutableInt(1000);
                 for(int sheetIndex = 0; sheetIndex < sheetList.size(); sheetIndex++) {
                     final Sheet sheet = sheetList.get(sheetIndex);
                     final Part part = getSheetPart(getPackage(), sheet);
                     final String sheetType = getSheetType(part);
                     if(sheetType.equals("worksheet")) {
                         final Worksheet worksheet = ((WorksheetPart)part).getJaxbElement();
+                        TableHelper.finalizeTables(this, (WorksheetPart)part, uniqueTableId);
                         final SheetViews sheetViews = worksheet.getSheetViews();
                         if(sheetViews!=null) {
                             final List<SheetView> sheetViewList = sheetViews.getSheetView();
@@ -489,23 +631,22 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
             final Part part = workbookRelationshipsPart.getPart(sheet.getId());
             if(part instanceof WorksheetPart) {
                 final WorksheetPart sheetPart = (WorksheetPart)part;
-                final RelationshipsPart sheetRelationshipsPart = sheetPart.getRelationshipsPart();
-                if(sheetRelationshipsPart!=null) {
-                    final Worksheet worksheet = sheetPart.getJaxbElement();
-                    final SheetData sheetData = worksheet.getSheetData();
-                    if(sheetData!=null) {
-                        final Iterator<Row> rowIterator = sheetData.createRowIterator();
-                        while(rowIterator.hasNext()) {
-                            final Row row = rowIterator.next();
-                            final Iterator<Cell> cellIterator = row.createCellIterator();
-                            while(cellIterator.hasNext()) {
-                                final Cell cell = cellIterator.next();
-                                if(cell.getF()!=null&&cell.getV()==null) {
-                                	log.info("XLSX: Cell:" + cell.getColumn() + " formula without result!");
-                                }
-                            }
-                        }
-                    }
+                if(sheetPart.isUnmarshalled()) {
+	                final Worksheet worksheet = sheetPart.getJaxbElement();
+	                final SheetData sheetData = worksheet.getSheetData();
+	                if(sheetData!=null) {
+	                    final Iterator<Row> rowIterator = sheetData.createRowIterator();
+	                    while(rowIterator.hasNext()) {
+	                        final Row row = rowIterator.next();
+	                        final Iterator<Cell> cellIterator = row.createCellIterator();
+	                        while(cellIterator.hasNext()) {
+	                            final Cell cell = cellIterator.next();
+	                            if(cell.getF()!=null&&cell.getV()==null) {
+	                            	log.info("XLSX: Cell:" + cell.getColumn() + " formula without result!");
+	                            }
+	                        }
+	                    }
+	                }
                 }
             }
         }
@@ -588,9 +729,9 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
                 if(stylesheet!=null) {
                     final CTFonts fonts = stylesheet.getFonts();
                     if(fonts!=null&&!fonts.getFont().isEmpty()) {
-                        final Object fontSize = fonts.getFont().get(0).getProperty("sz");
-                        if(fontSize!=null&&fontSize instanceof CTFontSize) {
-                            defaultFontSize = ((CTFontSize)fontSize).getVal();
+                        final CTFontSize fontSize = fonts.getFont().get(0).getSz();
+                        if(fontSize!=null) {
+                            defaultFontSize = fontSize.getVal();
                         }
                     }
                 }
@@ -605,6 +746,12 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return defaultFontSize.doubleValue();
     }
 
+    public Worksheet getWorksheet(int sheetIndex) {
+        final Sheet sheet = SheetUtils.getSheet(getPackage(), sheetIndex);
+        final RelationshipsPart relationshipPart = getPackage().getWorkbookPart().getRelationshipsPart();
+        return ((WorksheetPart)relationshipPart.getPart(sheet.getId())).getJaxbElement();
+    }
+
     public double getDefaultColumnWidth(Worksheet worksheet) {
 
         final CTSheetFormatPr sheetFormatPr = worksheet.getSheetFormatPr();
@@ -624,69 +771,6 @@ public class OperationDocument extends com.openexchange.office.ooxml.OperationDo
         return sheetFormatPr!=null ? sheetFormatPr.getDefaultRowHeight() : 14.5d;
     }
 
-    private InputStream checkDocumentComplexity(InputStream _inputStream, long maxMemoryUsage)
-    	throws FilterException {
-
-    	InputStream returnValue = null;
-
-    	final int maxCellCount  = ConfigurationHelper.getIntegerOfficeConfigurationValue(getServiceLookup(), getSession(), "//spreadsheet/maxCells", 500000);
-        if(maxCellCount<=0) {
-        	return _inputStream;
-        }
-
-        int xmlFileSize = 0;
-
-        byte[] bytes;
-
-        try {
-			bytes = IOUtils.toByteArray(_inputStream);
-		} catch (IOException e1) {
-            throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
-		} finally {
-	        IOUtils.closeQuietly(_inputStream);
-		}
-
-        final InputStream _zipInputStream = new ByteArrayInputStream(bytes);
-        final ZipInputStream zipInputStream = new ZipInputStream(_zipInputStream);
-
-        ZipEntry zipEntry = null;
-        final byte[] buffer = new byte[0x10000];  // 64kb buffer
-
-        try {
-			while((zipEntry = zipInputStream.getNextEntry())!=null) {
-
-				int read;
-
-				final String zipEntryName = zipEntry.getName();
-				if(zipEntryName.endsWith(".xml")) {
-		        	while((read = zipInputStream.read(buffer))>=0) {
-		        		xmlFileSize += read;
-		        	}
-		        }
-			    zipInputStream.closeEntry();
-			    if(xmlFileSize>maxCellCount*100) {
-		            throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
-			    }
-			}
-            if(maxMemoryUsage>0) {
-                if(xmlFileSize*FACTOR_FOR_REQUIRED_HEAPSPACE>maxMemoryUsage) {
-                    throw new FilterException("", ErrorCode.MEMORY_USAGE_TOO_HIGH, (long)(xmlFileSize*FACTOR_FOR_REQUIRED_HEAPSPACE));
-                }
-            }
-			returnValue = new ByteArrayInputStream(bytes);
-        } catch (IOException e) {
-            throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
-		} finally {
-			try {
-				zipInputStream.close();
-			} catch (IOException e) {
-
-			}
-			IOUtils.closeQuietly(_zipInputStream);
-		}
-        return returnValue;
-    }
-
     public static Part getSheetPart(SpreadsheetMLPackage spreadsheetPackage, final Sheet sheet) {
         final RelationshipsPart relationshipPart = spreadsheetPackage.getWorkbookPart().getRelationshipsPart();
         return relationshipPart.getPart(sheet.getId());
