@@ -55,7 +55,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.commons.logging.Log;
 import org.docx4j.openpackaging.packages.SpreadsheetMLPackage;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.SpreadsheetML.WorkbookPart;
@@ -73,6 +72,7 @@ import org.xlsx4j.sml.CTCellStyles;
 import org.xlsx4j.sml.CTCellXfs;
 import org.xlsx4j.sml.CTDefinedName;
 import org.xlsx4j.sml.CTDrawing;
+import org.xlsx4j.sml.CTFont;
 import org.xlsx4j.sml.CTHyperlink;
 import org.xlsx4j.sml.CTHyperlinks;
 import org.xlsx4j.sml.CTRst;
@@ -95,7 +95,6 @@ import org.xlsx4j.sml.SmlUtils;
 import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
@@ -104,10 +103,13 @@ import com.openexchange.office.ooxml.xlsx.OperationDocument;
 import com.openexchange.office.ooxml.xlsx.tools.AutoFilterHelper;
 import com.openexchange.office.ooxml.xlsx.tools.CellUtils;
 import com.openexchange.office.ooxml.xlsx.tools.ColumnUtils;
+import com.openexchange.office.ooxml.xlsx.tools.ConditionalFormattings;
 import com.openexchange.office.ooxml.xlsx.tools.Drawings;
 import com.openexchange.office.ooxml.xlsx.tools.MergeCellHelper;
 import com.openexchange.office.ooxml.xlsx.tools.RowUtils;
 import com.openexchange.office.ooxml.xlsx.tools.SheetUtils;
+import com.openexchange.office.ooxml.xlsx.tools.TableHelper;
+import com.openexchange.office.ooxml.xlsx.tools.Utils;
 import com.openexchange.office.ooxml.xlsx.tools.ValidationUtils;
 import com.openexchange.office.tools.ConfigurationHelper;
 import com.openexchange.server.ServiceLookup;
@@ -115,20 +117,38 @@ import com.openexchange.session.Session;
 
 public class CreateOperationHelper extends com.openexchange.office.ooxml.operations.CreateOperationHelper {
 
-    private final static Log log = LogFactory.getLog(CreateOperationHelper.class);
-
     private final SpreadsheetMLPackage spreadsheetMLPackage;
-
     private final Map<Long, String> indexToStyleId = new HashMap<Long, String>();
+    final int maxCellCount;
+    final int maxSheetCount;
+    int currentCellCount = 0;
+
+    private final static int MAX_EMPTY_CELL_REPEAT_CONTENT = 10;
 
     public CreateOperationHelper(OperationDocument _operationDocument, JSONArray mOperationsArray)
     	throws FilterException {
 
     	super(_operationDocument, mOperationsArray);
         spreadsheetMLPackage = _operationDocument.getPackage();
+        final ServiceLookup services = _operationDocument.getServiceLookup();
+    	final Session session = _operationDocument.getSession();
+        maxCellCount = ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxCells", 500000);
+        maxSheetCount = ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxSheets", 256);
 
-        checkComplexity();
-        CTStylesheet stylesheet = getOperationDocument().getStylesheet(false);
+        final WorkbookPart workbookPart = spreadsheetMLPackage.getWorkbookPart();
+        final Workbook workbook = workbookPart.getJaxbElement();
+        final Sheets sheets = workbook.getSheets();
+        if(sheets!=null) {
+            final List<Sheet> sheetList = sheets.getSheet();
+            if(maxSheetCount>=0) {
+                if(sheetList.size()>maxSheetCount) {
+                	final FilterException filterException = new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
+                	filterException.setSubType("MAX_SHEET_COUNT_EXCEEDED");
+                    throw filterException;
+                }
+            }
+        }
+        final CTStylesheet stylesheet = getOperationDocument().getStylesheet(false);
         if(stylesheet!=null) {
             // creating styles
             final CTCellStyleXfs cellStyleXfs = stylesheet.getCellStyleXfs();
@@ -164,7 +184,8 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                         String styleName = styleId;
                         JSONObject jsonProperties = new JSONObject();
                         Commons.mergeJsonObjectIfUsed("cell", jsonProperties, CellUtils.createCellProperties(cellStyleXf, stylesheet, true));
-                        Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(cellStyleXf, stylesheet, true));
+                        final CTFont ctFont = stylesheet.getFontByIndex(CellUtils.getAttributeIndex(cellStyleXf.getFontId(), cellStyleXf.isApplyFont(), true));
+                        Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(new JSONObject(), stylesheet, ctFont));
                         final JSONObject jsonApplyProperties = new JSONObject(0);
                         if(cellStyleXf.isApplyBorder()!=null&&!cellStyleXf.isApplyBorder().booleanValue()) {
                             jsonApplyProperties.put("border", false);
@@ -192,6 +213,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                         Boolean hidden = null;
                         Integer uiPriority = null;
                         Boolean isDefault = null;
+                        Boolean custom = null;
 
                         // check for proper UI styleName
                         CTCellStyles cellStyles = stylesheet.getCellStyles();
@@ -211,7 +233,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                             }
                         }
                         if(hidden==null||!hidden.booleanValue()) {
-                        	operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, false));
+                        	operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, custom, false));
                         }
                     }
                 }
@@ -226,8 +248,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     final String styleId = "a" + Integer.toString(i);
                     String styleName = "";
                     JSONObject jsonProperties = new JSONObject();
-                    Commons.mergeJsonObjectIfUsed("cell", jsonProperties, CellUtils.createCellProperties(xf, stylesheet, true));
-                    Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(xf, stylesheet, true));
+                    Commons.mergeJsonObjectIfUsed("cell", jsonProperties, CellUtils.createCellProperties(xf, stylesheet, false));
+                    final CTFont ctFont = stylesheet.getFontByIndex(CellUtils.getAttributeIndex(xf.getFontId(), xf.isApplyFont(), false));
+                    Commons.mergeJsonObjectIfUsed("character", jsonProperties, CellUtils.createCharacterProperties(new JSONObject(), stylesheet, ctFont));
                     JSONObject attrs = jsonProperties.length()>0?jsonProperties:null;
                     String parentId = null;
                     final Long xfId = xf.getXfId();
@@ -239,8 +262,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     }
                     Boolean hidden = true;
                     Integer uiPriority = null;
-                    Boolean isDefault = false;
-                    operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, true));
+                    Boolean isDefault = i==0;
+                    Boolean custom = null;
+                    operationsArray.put(createInsertStyleSheetOperation("cell", styleId, styleName, attrs, parentId, hidden, uiPriority, isDefault, custom, true));
                 }
             }
         }
@@ -266,7 +290,9 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    	final String sheetType = OperationDocument.getSheetType(part);
 	        createInsertSheetOperation(sheet, sheetType, part, sheetIndex);
 	        if(sheetType.equals("worksheet")) {
+	        	updateCellCount(((WorksheetPart)part).getJaxbElement());
         		createWorksheetOperations((WorksheetPart)part, sheetIndex, false);
+    	        ((WorksheetPart)part).setJaxbElement((Worksheet)null);
 	        }
 	    }
     }
@@ -288,11 +314,14 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    	final Sheet sheet = sheetList.get(sheetIndex);
 	    	final Part part = OperationDocument.getSheetPart(spreadsheetMLPackage, sheet);
 	    	final String sheetType = OperationDocument.getSheetType(part);
-	        createInsertSheetOperation(sheet, sheetType, part, sheetIndex);
-	        if(sheetType.equals("worksheet")) {
-	        	if(activeSheetIndex==sheetIndex) {
-	        		createWorksheetOperations((WorksheetPart)part, sheetIndex, true);
-	        	}
+	        if(sheetType.equals("worksheet")&&(activeSheetIndex==sheetIndex)) {
+	        	updateCellCount(((WorksheetPart)part).getJaxbElement());
+		        createInsertSheetOperation(sheet, sheetType, part, sheetIndex);
+        		createWorksheetOperations((WorksheetPart)part, sheetIndex, true);
+		        ((WorksheetPart)part).setJaxbElement((Worksheet)null);
+	        }
+	        else {
+		        createInsertSheetOperation(sheet, sheetType, null, sheetIndex);
 	        }
 	    }
         documentProperties.put(DocumentProperties.PROP_SPREADSHEET_SHEET_COUNT, sheetList.size());
@@ -312,55 +341,32 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 	    	final Sheet sheet = sheetList.get(sheetIndex);
 	    	final Part part = OperationDocument.getSheetPart(spreadsheetMLPackage, sheet);
 	    	final String sheetType = OperationDocument.getSheetType(part);
-	        if(sheetType.equals("worksheet")) {
-	        	if(activeSheetIndex!=sheetIndex) {
-	        		createWorksheetOperations((WorksheetPart)part, sheetIndex, false);
-	        	}
+	        if(sheetType.equals("worksheet")&&(activeSheetIndex!=sheetIndex)) {
+	        	final Worksheet worksheet = ((WorksheetPart)part).getJaxbElement();
+	        	updateCellCount(worksheet);
+		        createSetSheetAttributesOperation(sheet, worksheet, sheetIndex);
+        		createWorksheetOperations((WorksheetPart)part, sheetIndex, false);
+		        ((WorksheetPart)part).setJaxbElement((Worksheet)null);
 	        }
 	    }
     }
 
-    private void checkComplexity()
+    private void updateCellCount(Worksheet worksheet)
         throws FilterException {
 
-        final WorkbookPart workbookPart = spreadsheetMLPackage.getWorkbookPart();
-        final Workbook workbook = workbookPart.getJaxbElement();
-        final ServiceLookup services = getOperationDocument().getServiceLookup();
-        final Session session = getOperationDocument().getSession();
-
-        final int maxCellCount  =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxCells", 500000);
-        final int maxSheetCount =      ConfigurationHelper.getIntegerOfficeConfigurationValue(services, session, "//spreadsheet/maxSheets", 256);
-
-        final Sheets sheets = workbook.getSheets();
-        if(sheets!=null) {
-            final List<Sheet> sheetList = sheets.getSheet();
-            if(maxSheetCount>=0) {
-                if(sheetList.size()>maxSheetCount) {
-                    throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
-                }
-            }
-            if(maxCellCount>0) {
-                int cellCount = maxCellCount;
-                final RelationshipsPart relationshipPart = spreadsheetMLPackage.getWorkbookPart().getRelationshipsPart();
-                for(Sheet sheet:sheetList) {
-                    final Part part = relationshipPart.getPart(sheet.getId());
-                    if(part instanceof WorksheetPart) {
-                        final Worksheet worksheet = ((WorksheetPart)part).getJaxbElement();
-                        final SheetData sheetData = worksheet.getSheetData();
-                        if(sheetData!=null) {
-                            final Iterator<Row> rowIterator = sheetData.createRowIterator();
-                            while(rowIterator.hasNext()) {
-                                final Row row = rowIterator.next();
-                                cellCount -= row.getCellCount();
-                                if(cellCount<0) {
-                                    throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
-                                }
-                            }
-                        }
+    	if(maxCellCount>0) {
+            final SheetData sheetData = worksheet.getSheetData();
+            if(sheetData!=null) {
+                final Iterator<Row> rowIterator = sheetData.createRowIterator();
+                while(rowIterator.hasNext()) {
+                    final Row row = rowIterator.next();
+                    currentCellCount += row.getCellCount();
+                    if(currentCellCount>maxCellCount) {
+                        throw new FilterException("", ErrorCode.COMPLEXITY_TOO_HIGH);
                     }
                 }
             }
-        }
+    	}
     }
 
     /* returns the active sheet */
@@ -398,7 +404,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         }
         final HashMap<String, CTDefinedName> definedNameMap = definedNames.getDefinedNameMap();
         for(CTDefinedName definedName:definedNameMap.values()) {
-            addInsertNameOperation(definedName.getLocalSheetId(), definedName.getName(), definedName.getValue()!=null ? definedName.getValue() : "");
+            addInsertNameOperation(definedName.getLocalSheetId(), definedName.getName(), definedName.getValue()!=null ? definedName.getValue() : "", definedName.isHidden());
         }
     }
 
@@ -488,26 +494,50 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
     }
 
     /*
-     * if != null the topLeftCell parameter is filled with the current sheet settings
+     * part can be zero, then only a sheet insertion without attributes is done
      */
     private void createInsertSheetOperation(Sheet sheet, String sheetType, Part part, int sheetIndex)
     	throws JSONException {
 
-        if(sheetType.equals("worksheet")) {
+        final JSONObject attrs = new JSONObject();
+        if(part!=null&&sheetType.equals("worksheet")) {
             final WorksheetPart worksheetPart = (WorksheetPart)part;
             final Worksheet worksheet = worksheetPart.getJaxbElement();
 
-            final JSONObject properties = new JSONObject();
-            properties.put("sheet", SheetUtils.createWorksheetProperties(sheet, worksheet));
-            properties.put("column", SheetUtils.createWorksheetColumnProperties(getOperationDocument(), worksheet));
-            properties.put("row",  SheetUtils.createWorksheetRowProperties(worksheet));
-            addInsertSheetOperation(sheetIndex, sheet.getName(), sheetType, properties);
+            final JSONObject sheetProps = SheetUtils.createWorksheetProperties(sheet, worksheet);
+            if(sheetProps!=null&&!sheetProps.isEmpty()) {
+            	attrs.put("sheet", sheetProps);
+            }
+            final JSONObject columnProps = SheetUtils.createWorksheetColumnProperties(getOperationDocument(), worksheet);
+            if(columnProps!=null&&!columnProps.isEmpty()) {
+            	attrs.put("column", columnProps);
+            }
+            final JSONObject rowProps = SheetUtils.createWorksheetRowProperties(worksheet);
+            if(rowProps!=null&&!rowProps.isEmpty()) {
+            	attrs.put("row", rowProps);
+            }
+            addInsertSheetOperation(sheetIndex, sheet.getName(), sheetType, attrs);
         }
         else {
-            addInsertSheetOperation(sheetIndex, sheet.getName(), sheetType, Commons.surroundJSONObject("sheet", SheetUtils.createSheetProperties(sheet)));
+        	final JSONObject sheetProps = SheetUtils.createSheetProperties(sheet);
+        	if(sheetProps!=null&&!sheetProps.isEmpty()) {
+        		attrs.put("sheet", sheetProps);
+        	}
+            addInsertSheetOperation(sheetIndex, sheet.getName(), sheetType, attrs);
         }
     }
 
+    private void createSetSheetAttributesOperation(Sheet sheet, Worksheet worksheet, int sheetIndex)
+    	throws JSONException {
+
+    	final JSONObject properties = new JSONObject();
+        properties.put("sheet", SheetUtils.createWorksheetProperties(sheet, worksheet));
+        properties.put("column", SheetUtils.createWorksheetColumnProperties(getOperationDocument(), worksheet));
+        properties.put("row",  SheetUtils.createWorksheetRowProperties(worksheet));
+        addSetSheetAttributesOperation(sheetIndex, properties);
+    }
+
+
     private void createWorksheetOperations(final WorksheetPart worksheetPart, int sheetIndex, boolean preview)
         throws Exception {
 
@@ -519,6 +549,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 
         MergeCellHelper.createMergeCellOperations(operationsArray, sheetIndex, worksheet.getMergeCells());
         AutoFilterHelper.createAutoFilterOperations(operationsArray, sheetIndex, worksheet.getAutoFilter());
+        TableHelper.createTableOperations(operationsArray, worksheetPart, sheetIndex, worksheet.getTableParts());
 
         // hyperlinks
         final CTHyperlinks hyperlinks = worksheet.getHyperlinks();
@@ -526,15 +557,19 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             final List<CTHyperlink> hyperlinkList = hyperlinks.getHyperlink();
             for(int i=0; i<hyperlinkList.size();i++) {
                 final CTHyperlink hyperlink = hyperlinkList.get(i);
-                final SmlUtils.CellRef cellRef = SmlUtils.createCellRef(hyperlink.getRef());
-                if(cellRef!=null) {
+                final SmlUtils.CellRefRange cellRefRange = hyperlink.getCellRefRange(false);
+                if(cellRefRange!=null) {
                     final String hyperlinkUrl = Commons.getUrl(worksheetPart, hyperlink.getId());
                     if(hyperlinkUrl!=null&&!hyperlinkUrl.isEmpty()) {
-                        final JSONObject attrs = new JSONObject();
-                        final JSONObject characterAttributes = new JSONObject();
-                        characterAttributes.put("url", hyperlinkUrl);
-                        attrs.put("character", characterAttributes);
-                        addSetCellAttributesOperation(sheetIndex, cellRef, attrs);
+                        final JSONObject insertHyperlinkOperation = new JSONObject(5);
+                        insertHyperlinkOperation.put("name", "insertHyperlink");
+                        insertHyperlinkOperation.put("sheet", sheetIndex);
+                        insertHyperlinkOperation.put("start", cellRefRange.getStart().getJSONArray());
+                        if(!cellRefRange.getStart().equals(cellRefRange.getEnd())) {
+                        	insertHyperlinkOperation.put("end", cellRefRange.getEnd().getJSONArray());
+                        }
+                        insertHyperlinkOperation.put("url", hyperlinkUrl);
+                        operationsArray.put(insertHyperlinkOperation);
                     }
                 }
             }
@@ -556,29 +591,24 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
 
         // validations
         final List<IDataValidations> validations = OperationDocument.getDataValidations(worksheet);
+        int validationIndex = 0;
         for(IDataValidations dataValidations:validations) {
             for(IDataValidation dataValidation:dataValidations.getDataValidation()) {
-                createDataValidation(sheetIndex, dataValidation);
+                createDataValidation(sheetIndex, validationIndex++, dataValidation);
             }
         }
+
+        // conditional formattings
+        ConditionalFormattings.createOperations(operationsArray, worksheet, sheetIndex, getOperationDocument().getStylesheet(false));
     }
 
-    private void createDataValidation(int sheetIndex, IDataValidation dataValidation)
+    private void createDataValidation(int sheetIndex, int validationIndex, IDataValidation dataValidation)
         throws JSONException {
 
-        final List<SmlUtils.CellRefRange> cellRefRangeList = new ArrayList<SmlUtils.CellRefRange>();
-        final List<String> sqRefList = dataValidation.getsqref();
-        for(String sqRef:sqRefList) {
-            final SmlUtils.CellRefRange cellRefRange = SmlUtils.createCellRefRange(sqRef);
-            if(cellRefRange!=null) {
-                cellRefRangeList.add(cellRefRange);
-            }
-        }
-        if(cellRefRangeList.size()>0) {
-            ValidationUtils.addInsertValidationOperation(operationsArray, sheetIndex, cellRefRangeList, dataValidation.getType(), dataValidation.getOperator(), dataValidation.getFormula1(), dataValidation.getFormula2(), dataValidation.isShowInputMessage(),
-                dataValidation.getPromptTitle(), dataValidation.getPrompt(), dataValidation.isShowErrorMessage(), dataValidation.getErrorTitle(), dataValidation.getError(), dataValidation.getErrorStyle(),
-                    dataValidation.isShowDropDown()==false, dataValidation.isAllowBlank());
-        }
+        final List<SmlUtils.CellRefRange> cellRefRangeList = Utils.createCellRefRangeListFromSqrefList(dataValidation.getsqref());
+        ValidationUtils.addInsertValidationOperation(operationsArray, sheetIndex, validationIndex, cellRefRangeList, dataValidation.getType(), dataValidation.getOperator(), dataValidation.getFormula1(), dataValidation.getFormula2(), dataValidation.isShowInputMessage(),
+            dataValidation.getPromptTitle(), dataValidation.getPrompt(), dataValidation.isShowErrorMessage(), dataValidation.getErrorTitle(), dataValidation.getError(), dataValidation.getErrorStyle(),
+                dataValidation.isShowDropDown()==false, dataValidation.isAllowBlank());
     }
 
     private boolean compareCell(Cell source, Cell dest) {
@@ -656,8 +686,7 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         while(rowIterator.hasNext()) {
         	final Row row = rowIterator.next();
 
-        	final JSONArray rowArray = new JSONArray();
-            final JSONArray cellArray = new JSONArray();
+            JSONArray cellArray = new JSONArray();
             SmlUtils.CellRef topLeftCellRef = null;
 
             Cell 		lastCell = null;
@@ -673,20 +702,23 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
                     topLeftCellRef = cellRef;
                 }
                 if(lastCell!=null) {
-	                // checking empty cells
+                	// checking empty cells
 	                final int cellDifference = cellRef.getColumn() - lastCellX;
 	                if(cellDifference > 1) {
-	                	if(lastCellObject!=null) {
-	                		cellArray.put(lastCellObject);
-	                	}
-	                	// we have empty cells
-	                	if(cellDifference==2) {
-	                		cellArray.put((Object)null);
+	                	// we have empty cells ... at least one
+                		cellArray.put(lastCellObject);
+	                	if(cellDifference<=(MAX_EMPTY_CELL_REPEAT_CONTENT-1)) {
+	                		for(int i = cellDifference-1; i!=0; i--) {
+	                			cellArray.put((Object)null);
+	                		}
 	                	}
 	                	else {
-	                		final JSONObject emptyCells = new JSONObject(1);
-	                		emptyCells.put("repeat", cellDifference - 1);
-	                		cellArray.put(emptyCells);
+	                		// too many empty cells, we will create a new setCellContentOperation...
+                        	final JSONArray rowArray = new JSONArray(1);
+                			rowArray.put(cellArray);
+                			addSetCellContentsOperation(sheetIndex, topLeftCellRef, rowArray);
+	                		cellArray = new JSONArray();
+	                		topLeftCellRef = cellRef;
 	                	}
 	                }
 	                else {
@@ -708,9 +740,11 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             if(lastCellObject!=null) {
                 cellArray.put(lastCellObject);
             }
-            rowArray.put(cellArray);
-            if(topLeftCellRef!=null)
+            if(!cellArray.isEmpty()) {
+            	final JSONArray rowArray = new JSONArray(1);
+                rowArray.put(cellArray);
                 addSetCellContentsOperation(sheetIndex, topLeftCellRef, rowArray);
+            }
         }
     }
 
@@ -755,12 +789,22 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
             insertSheetObject.put("type", type);
         }
         insertSheetObject.put("sheetName", sheetName);
-        if(attrs!=null)
+        if(attrs!=null&&!attrs.isEmpty()) {
             insertSheetObject.put("attrs", attrs);
-
+        }
         operationsArray.put(insertSheetObject);
     }
 
+    public void addSetSheetAttributesOperation(int sheetIndex, final JSONObject attrs)
+        throws JSONException {
+
+        final JSONObject setSheetAttributesObject = new JSONObject();
+        setSheetAttributesObject.put("name", "setSheetAttributes");
+        setSheetAttributesObject.put("sheet", sheetIndex);
+    	setSheetAttributesObject.put("attrs", attrs);
+        operationsArray.put(setSheetAttributesObject);
+    }
+
     final static List<Object> objectList = new ArrayList<Object>();
 
     public void addSetCellContentsOperation(int sheetIndex, SmlUtils.CellRef cellRef, JSONArray cellContent)
@@ -826,18 +870,20 @@ public class CreateOperationHelper extends com.openexchange.office.ooxml.operati
         operationsArray.put(addSetColumnAttributesObject);
     }
 
-    public void addInsertNameOperation(Long sheetIndex, String name, String formula)
+    public void addInsertNameOperation(Long sheetIndex, String name, String formula, boolean hidden)
         throws JSONException {
 
-        final JSONObject addInsertNameObject = new JSONObject();
+        final JSONObject addInsertNameObject = new JSONObject(5);
         addInsertNameObject.put("name", "insertName");
         if(sheetIndex!=null) {
             addInsertNameObject.put("sheet", sheetIndex);
         }
+        if(hidden) {
+        	addInsertNameObject.put("hidden", true);
+        }
         addInsertNameObject.put("exprName", name);
         addInsertNameObject.put("formula", formula);
 
         operationsArray.put(addInsertNameObject);
     }
-
 }
