@@ -59,7 +59,6 @@ import org.xlsx4j.sml.Col;
 import org.xlsx4j.sml.Worksheet;
 
 import com.openexchange.office.FilterException;
-import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
 
 public class ColumnUtils {
@@ -73,7 +72,7 @@ public class ColumnUtils {
 		return null;
 	}
 
-    public static void applyColumnProperties(OperationDocument operationDocument, JSONObject columnProperties, AttrsMode attrsMode, Col col, Worksheet worksheet, CTStylesheet stylesheet, Map<Long, Long> columnStyleCache)
+    public static void applyColumnProperties(OperationDocument operationDocument, JSONObject columnProperties, Col col, Worksheet worksheet, CTStylesheet stylesheet)
         throws FilterException, JSONException {
 
         // Column properties
@@ -109,13 +108,12 @@ public class ColumnUtils {
                 }
             }
         }
-        col.setStyle(CellUtils.applyAutoStyle(operationDocument, col.getStyle(), columnProperties, attrsMode, stylesheet, columnStyleCache, null, null, null));
+        col.setStyle(CellUtils.applyAutoStyle(col.getStyle(), columnProperties));
 
         // Excel 2013 is having problems with empty columns
         if(col.getWidth()==null) {
             col.setWidth(operationDocument.getDefaultColumnWidth(worksheet));
         }
-        return;
     }
 
     public static double excelColumnWidthTo100thmm(OperationDocument operationDocument, double columnWidth) {
