@@ -52,23 +52,23 @@ package com.openexchange.office.ooxml.xlsx.tools;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 
-import org.apache.commons.logging.Log;
-import org.docx4j.dml.CTNonVisualDrawingProps;
 import org.docx4j.dml.CTPoint2D;
 import org.docx4j.dml.CTPositiveSize2D;
 import org.docx4j.dml.CTShapeProperties;
+import org.docx4j.dml.IShape;
 import org.docx4j.dml.STShapeType;
 import org.docx4j.dml.chart.CTChartSpace;
 import org.docx4j.dml.chart.CTStyle;
-import org.docx4j.dml.chart.chartStyle.CTChartColor;
+import org.docx4j.dml.chartStyle2012.CTColorStyle;
 import org.docx4j.dml.spreadsheetdrawing.CTAbsoluteAnchor;
+import org.docx4j.dml.spreadsheetdrawing.CTConnector;
 import org.docx4j.dml.spreadsheetdrawing.CTGraphicalObjectFrame;
 import org.docx4j.dml.spreadsheetdrawing.CTGraphicalObjectFrameNonVisual;
 import org.docx4j.dml.spreadsheetdrawing.CTMarker;
 import org.docx4j.dml.spreadsheetdrawing.CTOneCellAnchor;
 import org.docx4j.dml.spreadsheetdrawing.CTPicture;
-import org.docx4j.dml.spreadsheetdrawing.CTPictureNonVisual;
 import org.docx4j.dml.spreadsheetdrawing.CTShape;
 import org.docx4j.dml.spreadsheetdrawing.CTShapeNonVisual;
 import org.docx4j.dml.spreadsheetdrawing.CTTwoCellAnchor;
@@ -76,25 +76,29 @@ import org.docx4j.dml.spreadsheetdrawing.ICellAnchor;
 import org.docx4j.dml.spreadsheetdrawing.STEditAs;
 import org.docx4j.jaxb.Context;
 import org.docx4j.openpackaging.parts.Part;
-import org.docx4j.openpackaging.parts.DrawingML.Chart;
-import org.docx4j.openpackaging.parts.DrawingML.ChartColor;
-import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
-import org.docx4j.relationships.Relationship;
 import org.json.JSONArray;
+import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
+import com.openexchange.office.ooxml.drawingml.ChartWrapper;
 import com.openexchange.office.ooxml.drawingml.DMLChartSpace;
 import com.openexchange.office.ooxml.drawingml.DMLGraphic;
 import com.openexchange.office.ooxml.drawingml.DMLHelper;
+import com.openexchange.office.ooxml.drawingml.Picture;
+import com.openexchange.office.ooxml.drawingml.Shape;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
+import com.openexchange.office.tools.doc.DocumentImageHelper;
 
 public class Drawings {
 
-    protected static Log log = LogFactory.getLog(Drawings.class);
+    private final static String CHART_KEY = "chartPart";
+
+	private final static Logger log = LoggerFactory.getLogger(Drawings.class);
 
     public static void createDrawingOperations(OperationDocument operationDocument, JSONArray operationsArray, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, int sheetIndex)
         throws Exception {
@@ -163,78 +167,69 @@ public class Drawings {
                 throw new FilterException("XLSX import: unknown anchor type for drawing.", ErrorCode.CRITICAL_ERROR);
             }
             attrs.put("drawing", drawingProperties);
-            final HashMap<String, Object> drawingPropertyMap = DMLGraphic.getDrawingType(drawingsPart, anchor);
+            final HashMap<String, Object> drawingPropertyMap = DMLGraphic.getDrawingType(operationDocument, drawingsPart, anchor);
             final String drawingType = drawingPropertyMap.containsKey("type") ? (String)drawingPropertyMap.get("type") : "undefined";
             if(drawingType.equals("image")) {
-                DrawingPic.createProperties(operationDocument, attrs, drawingsPart, anchor.getPic());
+                Picture.createJSONAttrs(operationDocument, attrs, drawingsPart, anchor.getPic());
             }
-            final JSONObject insertDrawingOperation = new JSONObject();
+            else if(drawingType.equals("shape")) {
+            	// a shape might also be a connector ..
+            	final IShape iShape = anchor.getSp();
+            	if(iShape!=null) {
+            		Shape.createJSONAttrs(operationDocument, attrs, anchor.getSp());
+            	}
+            }
+            else if(drawingPropertyMap.containsKey(CHART_KEY) && drawingPropertyMap.containsKey("chartType")) {
+
+                final JSONObject chartProperties = new JSONObject();
+            	ChartWrapper chart = getChart(drawingPropertyMap);
+
+            	chartProperties.put("type", drawingPropertyMap.get("chartType"));
+
+                final CTChartSpace cs = chart.getChartSpace();
+                final CTStyle chartStyle = cs.getStyle();
+                if(chartStyle!=null){
+                    chartProperties.put("chartStyleId", chartStyle.getVal());
+                } else{
+                    //2 is the default style of excel
+                    chartProperties.put("chartStyleId", 2);
+                }
+
+                final String chartGroup = (String) drawingPropertyMap.get("chartGroup");
+                if(chartGroup != null){
+                    chartProperties.putOpt("stacking", chartGroup);
+                }
+
+                final ChartWrapper chartPart = getChart(drawingPropertyMap);
+                final CTColorStyle colorStyle = chartPart.getChartColorStyle();
+                if(colorStyle!=null) {
+                    final JSONObject jsonChartColors = DMLChartSpace.createJsonFromChartColor(colorStyle);
+                    if(jsonChartColors!=null) {
+                        chartProperties.put("chartColors", jsonChartColors);
+                    }
+                }
+                attrs.put("chart", chartProperties);
+                DMLHelper.createJsonFromShapeProperties(attrs, cs.getSpPr(), chartPart.getTheme(), chartPart.getChartStyle().getPlotArea(), false);
+            }
+
+            final JSONObject insertDrawingOperation = new JSONObject(4);
             insertDrawingOperation.put("name", "insertDrawing");
-            final JSONArray jsonStart = new JSONArray();
+            final JSONArray jsonStart = new JSONArray(2);
             jsonStart.put(sheetIndex);
             jsonStart.put(i);
             insertDrawingOperation.put("start", jsonStart);
             insertDrawingOperation.put("type", drawingType);
-            if(drawingPropertyMap.containsKey("chartObject")) {
-                if(drawingPropertyMap.containsKey("chartType")) {
-
-
-                    final JSONObject chartProperties = new JSONObject();
-                    chartProperties.put("type", drawingPropertyMap.get("chartType"));
-
-                    final CTChartSpace cs = (CTChartSpace)drawingPropertyMap.get("chartObject");
-                    final CTStyle chartStyle = cs.getStyle();
-                    if(chartStyle!=null){
-                        chartProperties.put("chartStyleId", chartStyle.getVal());
-                    } else{
-                        //2 is the default style of excel
-                        chartProperties.put("chartStyleId", 2);
-                    }
-
-                    final String chartGroup = (String) drawingPropertyMap.get("chartGroup");
-                    if(chartGroup != null){
-                        chartProperties.putOpt("stacking", chartGroup);
-                    }
-
-                    final Chart chartPart = (Chart)drawingPropertyMap.get("chartPart");
-                    final RelationshipsPart chartRelPart = chartPart.getRelationshipsPart(false);
-                    if (chartRelPart != null) {
-                        // final Relationship chartStyleRel =
-                        // chartRelPart.getRelationshipByType("http://schemas.microsoft.com/office/2011/relationships/chartStyle");
-                        // final Part chartStylePart = chartRelPart.getPart(chartStyleRel);
-                        final Relationship chartColorRel = chartRelPart.getRelationshipByType("http://schemas.microsoft.com/office/2011/relationships/chartColorStyle");
-                        if(chartColorRel!=null) {
-                            try {
-                                final ChartColor chartColorPart = (ChartColor) chartRelPart.getPart(chartColorRel);
-
-                                final CTChartColor el = chartColorPart.getJaxbElement();
-                                final JSONObject jsonChartColors = DMLHelper.createJsonFromChartColor(el);
-                                if(jsonChartColors!=null) {
-                                    chartProperties.put("chartColors", jsonChartColors);
-                                }
-                            } catch (Exception e) {
-                                log.error("error while handling chart colors", e);
-                            }
-                        }
-                    }
-                    attrs.put("chart", chartProperties);
-                    DMLHelper.createJsonFromShapeProperties(attrs, cs.getSpPr(), false);
-                }
-            }
             insertDrawingOperation.put("attrs", attrs);
             operationsArray.put(insertDrawingOperation);
 
-            if(drawingPropertyMap.containsKey("chartObject")) {
-                DMLChartSpace.createChartSpaceOperations(operationsArray, jsonStart, (CTChartSpace)drawingPropertyMap.get("chartObject"));
-                DMLChartSpace.createShapeOperations(attrs, jsonStart, (CTChartSpace)drawingPropertyMap.get("chartObject"));
+            if(drawingPropertyMap.containsKey(CHART_KEY)) {
+                DMLChartSpace.createChartSpaceOperations(operationsArray, jsonStart, getChart(drawingPropertyMap));
             }
-
-
         }
         return;
     }
 
-    public static ICellAnchor applyAnchorAttributes(int drawingIndex, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, JSONObject drawingProperties)
+    public static ICellAnchor applyAnchorAttributes(int drawingIndex, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, JSONObject attrs)
         throws Exception {
 
         final org.docx4j.dml.spreadsheetdrawing.CTDrawing drawings = drawingsPart.getJaxbElement();
@@ -242,6 +237,7 @@ public class Drawings {
         if (drawingIndex>=anchors.size()) {
             throw new RuntimeException("xlsx::applyDrawingOperation: drawing not available");
         }
+        final JSONObject drawingProperties = attrs.optJSONObject("drawing");
         if(drawingProperties==null) {
             return anchors.get(drawingIndex);
         }
@@ -250,37 +246,16 @@ public class Drawings {
         return cellAnchor;
     }
 
-    public static void applyDrawingAttributes(ICellAnchor anchor, JSONObject drawingProperties) {
+    public static void applyDrawingAttributes(ICellAnchor anchor, JSONObject attrs)
+    	throws JSONException {
 
-        CTNonVisualDrawingProps nonVisualDrawingProps = null;
         if(anchor.getPic()!=null) {
-            final CTPicture pic = anchor.getPic();
-            CTPictureNonVisual nonVisualPicProps = pic.getNvPicPr();
-            if(nonVisualPicProps==null) {
-                nonVisualPicProps = Context.getDmlSpreadsheetDrawingObjectFactory().createCTPictureNonVisual();
-                pic.setNvPicPr(nonVisualPicProps);
-            }
-            nonVisualDrawingProps = nonVisualPicProps.getCNvPr();
-            if(nonVisualDrawingProps==null) {
-                nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
-                nonVisualPicProps.setCNvPr(nonVisualDrawingProps);
-            }
+            DMLHelper.applyNonVisualDrawingProperties(anchor.getPic(), attrs.optJSONObject("drawing"));
+    	    anchor.getPic().setSpPr(DMLHelper.applyShapePropertiesFromJson(anchor.getPic().getSpPr(), attrs));
         }
         else if(anchor.getGraphicFrame()!=null) {
-            final CTGraphicalObjectFrame graphicFrame = anchor.getGraphicFrame();
-            CTGraphicalObjectFrameNonVisual nonVisualGraphicProps = graphicFrame.getNvGraphicFramePr();
-            if(nonVisualGraphicProps==null) {
-                nonVisualGraphicProps = Context.getDmlSpreadsheetDrawingObjectFactory().createCTGraphicalObjectFrameNonVisual();
-                graphicFrame.setNvGraphicFramePr(nonVisualGraphicProps);
-            }
-            nonVisualDrawingProps = nonVisualGraphicProps.getCNvPr();
-            if(nonVisualDrawingProps==null) {
-                nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
-                nonVisualGraphicProps.setCNvPr(nonVisualDrawingProps);
-            }
+            DMLHelper.applyNonVisualDrawingProperties(anchor.getGraphicFrame(), attrs.optJSONObject("drawing"));
         }
-
-        DMLHelper.applyNonVisualDrawingProperties(drawingProperties, nonVisualDrawingProps);
     }
 
     public static void insertDrawing(OperationDocument operationDocument, int drawingIndex, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingPart, String drawingType, JSONObject attrs)
@@ -289,16 +264,15 @@ public class Drawings {
         ICellAnchor anchor = Context.getDmlSpreadsheetDrawingObjectFactory().createCTTwoCellAnchor();
         anchor.setClientData(Context.getDmlSpreadsheetDrawingObjectFactory().createCTAnchorClientData());
         drawingPart.getJaxbElement().getEGAnchor().add(drawingIndex, anchor);
-        final JSONObject drawingProperties = attrs!=null ? attrs.optJSONObject("drawing") : null;
 
-        anchor = applyAnchorAttributes(drawingIndex, drawingPart, drawingProperties);
+        anchor = applyAnchorAttributes(drawingIndex, drawingPart, attrs);
 
         // we have to create a valid drawing otherwise excel wont open... at least we are creating an default empty rect shape
         final JSONObject imageAttrs = attrs.optJSONObject("image");
         if(drawingType.equals("image")) {
             String imageUrl = null;
             if(imageAttrs!=null) {
-                imageUrl = Commons.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "xl/media/");
+                imageUrl = DocumentImageHelper.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "xl/media/");
                 if(imageUrl==null) {
                     imageUrl = imageAttrs.optString("imageUrl", null);
                 }
@@ -316,7 +290,7 @@ public class Drawings {
         else {
             anchor.setSp(createDummyShape());
         }
-        applyDrawingAttributes(anchor, drawingProperties);
+        applyDrawingAttributes(anchor, attrs);
     }
 
     private static CTGraphicalObjectFrame createChart(OperationDocument operationDocument, Part drawingPart, JSONObject chartAttrs)
@@ -339,7 +313,8 @@ public class Drawings {
         shapeProperties.setXfrm(DMLHelper.createTransform2D(0, 0, 500, 500));
         shapeProperties.setPrstGeom(DMLHelper.createPresetGeometry2D(STShapeType.RECT));
         picture.setSpPr(shapeProperties);
-        picture.setNvPicPr(createPictureNonVisual("image"));
+        picture.setCNvPicPr(Context.getDmlObjectFactory().createCTNonVisualPictureProperties());
+        picture.setCNvPr(DMLHelper.createNonVisualDrawingProps("image"));
 
         // picture
         picture.setBlipFill(DMLHelper.createBlipFillProperties(operationDocument, drawingPart, imageUrl));
@@ -368,13 +343,6 @@ public class Drawings {
         return graphicalObjectFrameNonVisual;
     }
 
-    private static CTPictureNonVisual createPictureNonVisual(String name) {
-        final CTPictureNonVisual pictureNonVisual = Context.getDmlSpreadsheetDrawingObjectFactory().createCTPictureNonVisual();
-        pictureNonVisual.setCNvPicPr(Context.getDmlObjectFactory().createCTNonVisualPictureProperties());
-        pictureNonVisual.setCNvPr(DMLHelper.createNonVisualDrawingProps(name));
-        return pictureNonVisual;
-    }
-
     private static CTShapeNonVisual createShapeNonVisual(String name) {
         final CTShapeNonVisual shapeNonVisual = Context.getDmlSpreadsheetDrawingObjectFactory().createCTShapeNonVisual();
         shapeNonVisual.setCNvSpPr(Context.getDmlObjectFactory().createCTNonVisualDrawingShapeProps());
@@ -516,4 +484,8 @@ public class Drawings {
         }
         return cellAnchor;
     }
+
+    public static ChartWrapper getChart(Map<String, Object> drawingPropertyMap){
+        return (ChartWrapper) drawingPropertyMap.get(CHART_KEY);
+    }
 }
