@@ -58,12 +58,11 @@ import org.xlsx4j.sml.CTStylesheet;
 import org.xlsx4j.sml.Row;
 
 import com.openexchange.office.FilterException;
-import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
 
 public class RowUtils {
 
-    public static void applyRowProperties(OperationDocument operationDocument, JSONObject rowProperties, AttrsMode attrsMode, Row row, CTStylesheet stylesheet, Map<Long, Long> rowStyleCache)
+    public static void applyRowProperties(OperationDocument operationDocument, JSONObject rowProperties, Row row, CTStylesheet stylesheet)
         throws FilterException, JSONException {
 
         // Row properties
@@ -103,8 +102,7 @@ public class RowUtils {
                 }
             }
         }
-        row.setS(Long.valueOf(CellUtils.applyAutoStyle(operationDocument, row.getStyle(), rowProperties, attrsMode, stylesheet, rowStyleCache, null, null, null)));
-        return;
+        row.setS(Long.valueOf(CellUtils.applyAutoStyle(row.getStyle(), rowProperties)));
     }
 
     public static double map100thmmToPoint(double rowHeight) {
