@@ -51,7 +51,6 @@ package com.openexchange.office.ooxml.xlsx.tools;
 
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 
 import org.docx4j.openpackaging.packages.SpreadsheetMLPackage;
 import org.docx4j.openpackaging.parts.SpreadsheetML.WorkbookPart;
@@ -79,7 +78,6 @@ import org.xlsx4j.sml.SmlUtils;
 import org.xlsx4j.sml.Workbook;
 import org.xlsx4j.sml.Worksheet;
 
-import com.openexchange.office.ooxml.tools.AttrsMode;
 import com.openexchange.office.ooxml.tools.Commons;
 import com.openexchange.office.ooxml.xlsx.OperationDocument;
 
@@ -337,7 +335,6 @@ public class SheetUtils {
 		}
 		if(forceCreate) {
 			CTSelection selection = Context.getsmlObjectFactory().createCTSelection();
-			selection.setParent(selectionList);
 			selection.setPane(activePane);
 			selectionList.add(selection);
 			return selection;
@@ -350,7 +347,6 @@ public class SheetUtils {
     	SheetViews sheetViews = worksheet.getSheetViews();
     	if(sheetViews==null) {
     		sheetViews = Context.getsmlObjectFactory().createSheetViews();
-    		sheetViews.setParent(worksheet);
     		worksheet.setSheetViews(sheetViews);
     	}
     	final List<SheetView> sheetViewList = sheetViews.getSheetView();
@@ -366,7 +362,6 @@ public class SheetUtils {
     	CTPane pane = sheetView.getPane();
     	if(pane==null) {
     		pane = Context.getsmlObjectFactory().createCTPane();
-    		pane.setParent(sheetView);
     		sheetView.setPane(pane);
     	}
     	return pane;
@@ -389,13 +384,11 @@ public class SheetUtils {
         BookViews bookViews = workbook.getBookViews();
         if(bookViews==null) {
             bookViews = Context.getsmlObjectFactory().createBookViews();
-            bookViews.setParent(workbook);
             workbook.setBookViews(bookViews);
         }
         final List<CTBookView> workbookViewList = bookViews.getWorkbookView();
         if(workbookViewList.isEmpty()) {
             final CTBookView bookView = Context.getsmlObjectFactory().createCTBookView();
-            bookView.setParent(bookViews);
             workbookViewList.add(bookView);
         }
         for(CTBookView bookView:workbookViewList) {
@@ -554,7 +547,6 @@ public class SheetUtils {
     	CTSheetFormatPr sheetFormatPr = worksheet.getSheetFormatPr();
     	if(sheetFormatPr==null) {
     		sheetFormatPr = Context.getsmlObjectFactory().createCTSheetFormatPr();
-    		sheetFormatPr.setParent(worksheet);
     		worksheet.setSheetFormatPr(sheetFormatPr);
     	}
     	return sheetFormatPr;
@@ -677,7 +669,7 @@ public class SheetUtils {
      * applying attrs to each physical available cell within the row
      *
      */
-    public static void applyRowProperties(OperationDocument operationDocument, SheetData sheetData, int start, JSONObject attrs, AttrsMode attrsMode, Map<Long, Long> rowCellStyleCache)
+    public static void applyRowProperties(OperationDocument operationDocument, SheetData sheetData, int start, JSONObject attrs)
         throws Exception {
 
         Row row = sheetData.getRow(start, false);
@@ -686,7 +678,7 @@ public class SheetUtils {
         }
         final Iterator<Cell> cellIterator = row.createCellIterator();
         while(cellIterator.hasNext()) {
-            CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, attrsMode, row, cellIterator.next(), operationDocument.getStylesheet(true), rowCellStyleCache);
+            CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, row, cellIterator.next());
         }
     }
 
@@ -694,7 +686,7 @@ public class SheetUtils {
      * applying attrs to each physical available cell within the column
      *
      */
-    public static void applyColumnProperties(OperationDocument operationDocument, SheetData sheetData, int start, int end, JSONObject attrs, AttrsMode attrsMode, Map<Long, Long> columnCellStyleCache)
+    public static void applyColumnProperties(OperationDocument operationDocument, SheetData sheetData, int start, int end, JSONObject attrs)
         throws Exception {
 
         final Iterator<Row> rowIterator = sheetData.createRowIterator();
@@ -703,7 +695,7 @@ public class SheetUtils {
             for(int i=start; i<=end; i++) {
                 final Cell cell = row.getCell(i, false);
                 if(cell!=null) {
-                    CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, attrsMode, row, cell, operationDocument.getStylesheet(true), columnCellStyleCache);
+                    CellUtils.applyCellPropertiesToCell(operationDocument, null, sheetData, attrs, row, cell);
                 }
             }
         }
