@@ -49,9 +49,9 @@
 
 package com.openexchange.office.ooxml.xlsx.tools;
 
+import java.util.ArrayList;
 import java.util.List;
 
-import org.docx4j.XmlUtils;
 import org.docx4j.dml.BaseStyles;
 import org.docx4j.dml.CTColorScheme;
 import org.docx4j.dml.Theme;
@@ -66,6 +66,7 @@ import org.xlsx4j.sml.CTCellProtection;
 import org.xlsx4j.sml.CTColor;
 import org.xlsx4j.sml.CTColors;
 import org.xlsx4j.sml.CTFill;
+import org.xlsx4j.sml.CTGradientStop;
 import org.xlsx4j.sml.CTIndexedColors;
 import org.xlsx4j.sml.CTNumFmt;
 import org.xlsx4j.sml.CTPatternFill;
@@ -75,7 +76,9 @@ import org.xlsx4j.sml.Col;
 import org.xlsx4j.sml.Cols;
 import org.xlsx4j.sml.STBorderStyle;
 import org.xlsx4j.sml.STHorizontalAlignment;
+import org.xlsx4j.sml.STPatternType;
 import org.xlsx4j.sml.STVerticalAlignment;
+import org.xlsx4j.sml.SmlUtils;
 
 import com.openexchange.office.FilterException;
 import com.openexchange.office.ooxml.tools.Commons;
@@ -107,7 +110,17 @@ public class Utils {
         throws JSONException {
 
         if(fill!=null) {
-            createPatternFill(dest, stylesheet, fill.getPatternFill());
+            if (null == fill.getPatternFill() && null != fill.getGradientFill()) {
+                //fallback for gradients! taking its first color
+                List<CTGradientStop> stop = fill.getGradientFill().getStop();
+                if (null != stop && stop.size() > 0) {
+                    CTColor color = stop.get(0).getColor();
+                    JSONObject jsonColor = createColor(stylesheet, color);
+                    dest.put("fillColor", jsonColor);
+                }
+            }else {
+                createPatternFill(dest, stylesheet, fill.getPatternFill());
+            }
 //          createGradientFill(dest, fill.getGradientFill());
         }
     }
@@ -116,7 +129,11 @@ public class Utils {
         throws JSONException {
 
         if(patternFill!=null) {
-            switch(patternFill.getPatternType()) {
+        	STPatternType patternType = patternFill.getPatternType();
+        	if(patternType==null) {
+        		patternType = STPatternType.SOLID;
+        	}
+            switch(patternType) {
                 case NONE: {
                     JSONObject jsonColor = new JSONObject(1);
                     jsonColor.put("type", "auto");
@@ -125,7 +142,7 @@ public class Utils {
                 }
                 case SOLID :
                 default: {
-                    JSONObject jsonColor = createColor(stylesheet, patternFill.getFgColor());
+                    JSONObject jsonColor = createColor(stylesheet, patternFill.getFgColor()!=null?patternFill.getFgColor():patternFill.getBgColor());
                     if(jsonColor!=null) {
                         dest.put("fillColor", jsonColor);
                     }
@@ -246,6 +263,15 @@ public class Utils {
         return null;
     }
 
+    public static JSONObject createDefaultColor()
+    	throws JSONException {
+
+    	JSONObject jsonColor = new JSONObject(2);
+        jsonColor.put("type",  "scheme");
+        jsonColor.put("value", themeColors[1]);
+        return jsonColor;
+    }
+
     private static JSONArray createTransformations(CTColor color, byte alpha)
         throws JSONException {
 
@@ -621,7 +647,7 @@ public class Utils {
                     size -= colLength;
                 }
                 else if(colLength>size) {
-                    Col newCol = XmlUtils.deepCopy(col, Context.getJcSML());
+                    final Col newCol = col.clone();
                     col.setMax(col.getMin()+size-1);
                     newCol.setMin(col.getMax()+1);
                     colList.add(i+1, newCol);
@@ -635,7 +661,7 @@ public class Utils {
             }
             // colMin() < start
             else if(col.getMax()>=start) {
-                Col newCol = XmlUtils.deepCopy(col, Context.getJcSML());
+                final Col newCol = col.clone();
                 newCol.setMin(start);
                 newCol.setMax(col.getMax());
                 col.setMax(start-1);
@@ -649,4 +675,51 @@ public class Utils {
             colList.add(i, newCol);
         }
     }
+
+    public static void applySqrefRangeFromJsonArray(List<String> sqrefList, Object value)
+    	throws JSONException {
+
+    	if(value instanceof JSONArray) {
+            final JSONArray ranges = (JSONArray)value;
+            sqrefList.clear();
+            for(int i=0; i<ranges.length();i++) {
+                final JSONObject range = ranges.getJSONObject(i);
+                final JSONArray start = range.getJSONArray("start");
+                final JSONArray end = range.optJSONArray("end");
+                final SmlUtils.CellRefRange cellRefRange = new SmlUtils.CellRefRange(
+                		new SmlUtils.CellRef(start.getInt(0), start.getInt(1)),
+                		end!=null ? new SmlUtils.CellRef(end.getInt(0), end.getInt(1)) : new SmlUtils.CellRef(start.getInt(0), start.getInt(1)));
+                sqrefList.add(SmlUtils.getCellRefRange(cellRefRange));
+            }
+    	}
+    	else {
+    		sqrefList.clear();
+    	}
+    }
+
+    public static JSONArray createJsonArrayFromCellRefRange(List<SmlUtils.CellRefRange> rangeList)
+    	throws JSONException {
+
+    	final JSONArray ranges = new JSONArray();
+        for(SmlUtils.CellRefRange cellRefRange:rangeList) {
+            final JSONObject range = new JSONObject(2);
+            range.put("start", cellRefRange.getStart().getJSONArray());
+            if((cellRefRange.getEnd().getColumn()!=cellRefRange.getStart().getColumn())||(cellRefRange.getEnd().getRow()!=cellRefRange.getStart().getRow())) {
+                range.put("end", cellRefRange.getEnd().getJSONArray());
+            }
+            ranges.put(range);
+        }
+        return ranges;
+    }
+
+    public static List<SmlUtils.CellRefRange> createCellRefRangeListFromSqrefList(List<String> sqrefList) {
+		final List<SmlUtils.CellRefRange> cellRefRangeList = new ArrayList<SmlUtils.CellRefRange>();
+	    for(String sqRef:sqrefList) {
+	        final SmlUtils.CellRefRange cellRefRange = SmlUtils.createCellRefRange(sqRef);
+	        if(cellRefRange!=null) {
+	            cellRefRangeList.add(cellRefRange);
+	        }
+	    }
+	    return cellRefRangeList;
+    }
 }
