@@ -53,8 +53,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.StringTokenizer;
 
-import org.docx4j.openpackaging.parts.SpreadsheetML.WorksheetPart;
-import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -68,7 +66,6 @@ import org.xlsx4j.sml.IDataValidations;
 import org.xlsx4j.sml.STDataValidationErrorStyle;
 import org.xlsx4j.sml.STDataValidationOperator;
 import org.xlsx4j.sml.STDataValidationType;
-import org.xlsx4j.sml.Sheet;
 import org.xlsx4j.sml.SmlUtils;
 import org.xlsx4j.sml.Worksheet;
 
@@ -110,7 +107,7 @@ public class ValidationUtils {
     	return value;
     }
 
-    public static void addInsertValidationOperation(JSONArray operationsArray, int sheetIndex, List<SmlUtils.CellRefRange> rangeList, STDataValidationType type, STDataValidationOperator operator, String value1, String value2,
+    public static void addInsertValidationOperation(JSONArray operationsArray, int sheetIndex, int validationIndex, List<SmlUtils.CellRefRange> rangeList, STDataValidationType type, STDataValidationOperator operator, String value1, String value2,
                                             boolean showInfo, String infoTitle, String infoText, boolean showError, String errorTitle, String errorText, STDataValidationErrorStyle errorStyle, boolean showDropDown, boolean ignoreBlank)
         throws JSONException {
 
@@ -120,16 +117,8 @@ public class ValidationUtils {
         final JSONObject insertValidationObject = new JSONObject();
         insertValidationObject.put("name", "insertValidation");
         insertValidationObject.put("sheet", sheetIndex);
-        final JSONArray ranges = new JSONArray();
-        for(SmlUtils.CellRefRange cellRefRange:rangeList) {
-            final JSONObject range = new JSONObject();
-            range.put("start", cellRefRange.getStart().getJSONArray());
-            if((cellRefRange.getEnd().getColumn()!=cellRefRange.getStart().getColumn())||(cellRefRange.getEnd().getRow()!=cellRefRange.getStart().getRow())) {
-                range.put("end", cellRefRange.getEnd().getJSONArray());
-            }
-            ranges.put(range);
-        }
-        insertValidationObject.put("ranges", ranges);
+        insertValidationObject.put("index", validationIndex);
+        insertValidationObject.put("ranges", Utils.createJsonArrayFromCellRefRange(rangeList));
         int val = type.ordinal();
         if(type==STDataValidationType.LIST) {
             if(!value1.isEmpty()&&value1.charAt(0)!='"') {
@@ -182,14 +171,8 @@ public class ValidationUtils {
         operationsArray.put(insertValidationObject);
     }
 
-    private static Worksheet getWorksheet(OperationDocument operationDocument, int sheetIndex) {
-        final Sheet sheet = SheetUtils.getSheet(operationDocument.getPackage(), sheetIndex);
-        final RelationshipsPart relationshipPart = operationDocument.getPackage().getWorkbookPart().getRelationshipsPart();
-        return ((WorksheetPart)relationshipPart.getPart(sheet.getId())).getJaxbElement();
-    }
-
     private static List<IDataValidations> getDataValidations(OperationDocument operationDocument, int sheetIndex) {
-        return OperationDocument.getDataValidations(getWorksheet(operationDocument, sheetIndex));
+        return OperationDocument.getDataValidations(operationDocument.getWorksheet(sheetIndex));
     }
 
     private static IDataValidation getDataValidation(OperationDocument operationDocument, int sheetIndex, int index) {
@@ -218,23 +201,7 @@ public class ValidationUtils {
 	            final String attr = keys.next();
 	            final Object value = op.get(attr);
 	            if(attr.equals("ranges")) {
-	            	if(value instanceof JSONArray) {
-		                final JSONArray ranges = (JSONArray)value;
-		                final List<String> sqrefList = iDataValidation.getsqref();
-		                sqrefList.clear();
-		                for(int i=0; i<ranges.length();i++) {
-			                final JSONObject range = ranges.getJSONObject(i);
-			                final JSONArray start = range.getJSONArray("start");
-			                final JSONArray end = range.optJSONArray("end");
-			                final SmlUtils.CellRefRange cellRefRange = new SmlUtils.CellRefRange(
-			                		new SmlUtils.CellRef(start.getInt(0), start.getInt(1)),
-			                		end!=null ? new SmlUtils.CellRef(end.getInt(0), end.getInt(1)) : new SmlUtils.CellRef(start.getInt(0), start.getInt(1)));
-			                sqrefList.add(SmlUtils.getCellRefRange(cellRefRange));
-		                }
-	            	}
-	            	else {
-	            		iDataValidation.getsqref().clear();
-	            	}
+	            	Utils.applySqrefRangeFromJsonArray(iDataValidation.getsqref(), value);
 	            }
 	            else if(attr.equals("type")) {
             		STDataValidationType dataValidationType = STDataValidationType.NONE;
@@ -357,7 +324,7 @@ public class ValidationUtils {
 				iDataValidations = dataValidationsList.get(dataValidationsList.size()-1);
 			}
 			else {
-				final Worksheet worksheet = getWorksheet(operationDocument, sheet);
+				final Worksheet worksheet = operationDocument.getWorksheet(sheet);
 				final CTDataValidations dataValidations = Context.getsmlObjectFactory().createCTDataValidations();
 				dataValidations.setParent(worksheet);
 				worksheet.setDataValidations(dataValidations);
