@@ -75,7 +75,6 @@ import javax.xml.datatype.DatatypeConstants;
 import javax.xml.datatype.XMLGregorianCalendar;
 
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.logging.Log;
 import org.docx4j.dml.Theme;
 import org.docx4j.docProps.core.CoreProperties;
 import org.docx4j.docProps.core.dc.elements.SimpleLiteral;
@@ -90,17 +89,20 @@ import org.docx4j.openpackaging.parts.DocPropsCorePart;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.PartName;
 import org.docx4j.openpackaging.parts.ThemePart;
+import org.docx4j.openpackaging.parts.WordprocessingML.DocumentSettingsPart;
 import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
+import org.docx4j.wml.CTSettings;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.openexchange.log.LogFactory;
 import com.openexchange.office.DocumentProperties;
 import com.openexchange.office.FilterException;
 import com.openexchange.office.FilterException.ErrorCode;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.office.tools.ConfigurationHelper;
-import com.openexchange.office.tools.ResourceManager;
 import com.openexchange.office.tools.memory.MemoryObserver;
 import com.openexchange.office.tools.memory.MemoryObserver.MemoryListener;
 import com.openexchange.server.ServiceLookup;
@@ -151,10 +153,9 @@ abstract public class OperationDocument {
 		}
 	};
 
+	final private static Logger log = LoggerFactory.getLogger(OperationDocument.class);
 
-    private final static Log LOG = LogFactory.getLog(OperationDocument.class);
-
-    private final ResourceManager       resourceManager;
+    private final IResourceManager      resourceManager;
     private final DocumentProperties    documentProperties;
     protected 	  InputStream           inputDocumentStream;
     private final ServiceLookup         services;
@@ -164,10 +165,11 @@ abstract public class OperationDocument {
     private Set<ChangeTrackAuthor> 		changeTrackAuthors = null;
     protected Part 						contextPart = null;
     private MemoryListener              memoryListener = null;
+    protected static int 				successfulAppliedOperations = 0;
 
     static final int DEFAULT_BUFFER_SIZE = 8192;
 
-    protected OperationDocument(Session session, ServiceLookup services, InputStream inputDocumentStream, ResourceManager resourceManager, DocumentProperties documentProperties) {
+    protected OperationDocument(Session session, ServiceLookup services, InputStream inputDocumentStream, IResourceManager resourceManager, DocumentProperties documentProperties) {
         this.services = services;
         this.session = session;
         this.inputDocumentStream = inputDocumentStream;
@@ -176,6 +178,7 @@ abstract public class OperationDocument {
         saveDebugOperations = Boolean.parseBoolean(ConfigurationHelper.getOfficeConfigurationValue(services, session, "debugoperations", "false"));
         registerMemoryListener();
         Context.setLowMemoryAbort(false);
+        successfulAppliedOperations = 0;
     };
 
     public void registerMemoryListener() {
@@ -207,7 +210,7 @@ abstract public class OperationDocument {
     }
 
     private void notifyLowMemory() {
-        LOG.info("OOXML Filter: Low memory notification received");
+        log.info("OOXML Filter: Low memory notification received");
         Context.setLowMemoryAbort(true);
         final OpcPackage opcPackage = getPackage();
         if(opcPackage!=null) {
@@ -261,7 +264,7 @@ abstract public class OperationDocument {
         	throw e;
         }
         catch(Throwable e) {
-            LOG.error("OOXML Filter could not save the given document:", e);
+            log.error("OOXML Filter could not save the given document:", e);
             throw (e instanceof FilterException) ? (FilterException)e : new FilterException(e, ErrorCode.CRITICAL_ERROR);
         }
     }
@@ -290,15 +293,15 @@ abstract public class OperationDocument {
 
     protected void logMessage(final String msgClass, final String logMessage) {
         if ("fatal".equalsIgnoreCase(logMessage))
-            LOG.fatal(logMessage);
+            log.error(logMessage);
         else if ("error".equalsIgnoreCase(msgClass))
-            LOG.error(logMessage);
+            log.error(logMessage);
         else if ("warning".equalsIgnoreCase(msgClass))
-            LOG.warn(logMessage);
+            log.warn(logMessage);
         else if ("debug".equalsIgnoreCase(msgClass))
-            LOG.debug(logMessage);
+            log.debug(logMessage);
         else // "info"
-            LOG.info(logMessage);
+            log.info(logMessage);
     }
 
     /**
@@ -315,10 +318,10 @@ abstract public class OperationDocument {
             getPackage().getContentTypeManager().addNonPersistContentFile("oxoffice/extendedData.json",
                 jsonObject.toString().getBytes("UTF-8"),"json", "application/octet-stream");
         } catch (UnsupportedEncodingException e) {
-            LOG.error("OOXML Filter could not save the unique document indentifier:", e);
+            log.error("OOXML Filter could not save the unique document indentifier:", e);
             return false;
         } catch (JSONException e) {
-            LOG.error("OOXML Filter could not create JSON object to store the unique document indentifier:", e);
+            log.error("OOXML Filter could not create JSON object to store the unique document indentifier:", e);
             return false;
         }
 
@@ -329,7 +332,7 @@ abstract public class OperationDocument {
      * that stores each information necessary to repeat the last save operation that was done.
      * (original document, the operations just applied and the content of the resource manager. */
 
-    private InputStream debugSave(String debugOperations, java.io.InputStream debugDocument, ResourceManager debugResourceManager)
+    private InputStream debugSave(String debugOperations, java.io.InputStream debugDocument, IResourceManager debugResourceManager)
         throws FilterException {
 
         try {
@@ -437,7 +440,7 @@ abstract public class OperationDocument {
                 outputStream.flush();
             }
         } catch (Throwable e) {
-            LOG.error("OOXML Filter could not save the given document:", e);
+            log.error("OOXML Filter could not save the given document:", e);
             throw new FilterException(e, ErrorCode.CRITICAL_ERROR);
         } finally {
             IOUtils.closeQuietly(inputStream);
@@ -452,7 +455,7 @@ abstract public class OperationDocument {
     abstract public Theme getTheme()
     	throws FilterException;
 
-    public ResourceManager getResourceManager() {
+    public IResourceManager getResourceManager() {
         return resourceManager;
     }
 
@@ -465,21 +468,22 @@ abstract public class OperationDocument {
     }
 
     public static FilterException getRethrowException(Throwable e) {
+    	FilterException ret;
         if(Context.isLowMemoryAbort()) {
-            return new FilterException("FilterException...", ErrorCode.MEMORY_USAGE_MIN_FREE_HEAP_SPACE_REACHED);
+            ret = new FilterException("FilterException...", ErrorCode.MEMORY_USAGE_MIN_FREE_HEAP_SPACE_REACHED);
         }
-    	if(e instanceof FilterException) {
+        else if(e instanceof FilterException) {
     		final FilterException filterException = (FilterException)e;
     		if(filterException.getErrorcode()==ErrorCode.COMPLEXITY_TOO_HIGH) {
-    			LOG.warn("could not load/save document, the document complexity is too high");
+    			log.warn("could not load/save document, the document complexity is too high");
     		}
     		else if(filterException.getErrorcode()==ErrorCode.CRITICAL_ERROR) {
-    			LOG.error(e.getMessage(), e);
+    			log.error(e.getMessage(), e);
     		}
-    		return (FilterException)e;
+    		ret = (FilterException)e;
     	}
     	else if (e instanceof OutOfMemoryError) {
-    		return new FilterException(e, ErrorCode.MEMORY_USAGE_MIN_FREE_HEAP_SPACE_REACHED);
+    		ret = new FilterException(e, ErrorCode.MEMORY_USAGE_MIN_FREE_HEAP_SPACE_REACHED);
     	}
     	else if(e instanceof Docx4JException) {
             ErrorCode errorCode = ErrorCode.CRITICAL_ERROR;
@@ -489,12 +493,14 @@ abstract public class OperationDocument {
             else if(e.getCause() instanceof org.apache.poi.EncryptedDocumentException) {
                 errorCode = ErrorCode.UNSUPPORTED_ENCRYPTION_USED;
             }
-            return new FilterException(e, errorCode);
+            ret = new FilterException(e, errorCode);
     	}
     	else {
-    		LOG.error(e.getMessage(), e);
-    		return new FilterException(e, ErrorCode.CRITICAL_ERROR);
+    		log.error(e.getMessage(), e);
+    		ret = new FilterException(e, ErrorCode.CRITICAL_ERROR);
     	}
+    	ret.setOperationCount(OperationDocument.successfulAppliedOperations);
+    	return ret;
     }
 
     public long getFreeHeapSpace() {
@@ -851,4 +857,84 @@ abstract public class OperationDocument {
         }
         return themePart;
     }
+    
+    public DocumentSettingsPart getDefaultSettingsPart(PartName partName) {
+    	
+    	final String documentSettingsStr =
+    		"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n" + 
+    		"<w:settings\n" + 
+    		"	xmlns:mc=\"http://schemas.openxmlformats.org/markup-compatibility/2006\"\n" + 
+    		"	xmlns:o=\"urn:schemas-microsoft-com:office:office\"\n" + 
+    		"	xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"\n" + 
+    		"	xmlns:m=\"http://schemas.openxmlformats.org/officeDocument/2006/math\"\n" + 
+    		"	xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:w10=\"urn:schemas-microsoft-com:office:word\"\n" + 
+    		"	xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\"\n" + 
+    		"	xmlns:w14=\"http://schemas.microsoft.com/office/word/2010/wordml\"\n" + 
+    		"	xmlns:w15=\"http://schemas.microsoft.com/office/word/2012/wordml\"\n" + 
+    		"	xmlns:sl=\"http://schemas.openxmlformats.org/schemaLibrary/2006/main\"\n" + 
+    		"	mc:Ignorable=\"w14 w15\">\n" + 
+    		"	<w:zoom w:percent=\"100\" />\n" + 
+    		"	<w:defaultTabStop w:val=\"720\" />\n" + 
+    		"	<w:characterSpacingControl w:val=\"doNotCompress\" />\n" + 
+    		"	<w:compat>\n" + 
+    		"		<w:compatSetting w:name=\"compatibilityMode\"\n" + 
+    		"			w:uri=\"http://schemas.microsoft.com/office/word\" w:val=\"15\" />\n" + 
+    		"		<w:compatSetting w:name=\"overrideTableStyleFontSizeAndJustification\"\n" + 
+    		"			w:uri=\"http://schemas.microsoft.com/office/word\" w:val=\"1\" />\n" + 
+    		"		<w:compatSetting w:name=\"enableOpenTypeFeatures\"\n" + 
+    		"			w:uri=\"http://schemas.microsoft.com/office/word\" w:val=\"1\" />\n" + 
+    		"		<w:compatSetting w:name=\"doNotFlipMirrorIndents\"\n" + 
+    		"			w:uri=\"http://schemas.microsoft.com/office/word\" w:val=\"1\" />\n" + 
+    		"		<w:compatSetting w:name=\"differentiateMultirowTableHeaders\"\n" + 
+    		"			w:uri=\"http://schemas.microsoft.com/office/word\" w:val=\"1\" />\n" + 
+    		"	</w:compat>\n" + 
+    		"	<w:rsids>\n" + 
+    		"		<w:rsidRoot w:val=\"008463FB\" />\n" + 
+    		"		<w:rsid w:val=\"002B1CD8\" />\n" + 
+    		"		<w:rsid w:val=\"008463FB\" />\n" + 
+    		"	</w:rsids>\n" + 
+    		"	<m:mathPr>\n" + 
+    		"		<m:mathFont m:val=\"Cambria Math\" />\n" + 
+    		"		<m:brkBin m:val=\"before\" />\n" + 
+    		"		<m:brkBinSub m:val=\"--\" />\n" + 
+    		"		<m:smallFrac m:val=\"0\" />\n" + 
+    		"		<m:dispDef />\n" + 
+    		"		<m:lMargin m:val=\"0\" />\n" + 
+    		"		<m:rMargin m:val=\"0\" />\n" + 
+    		"		<m:defJc m:val=\"centerGroup\" />\n" + 
+    		"		<m:wrapIndent m:val=\"1440\" />\n" + 
+    		"		<m:intLim m:val=\"subSup\" />\n" + 
+    		"		<m:naryLim m:val=\"undOvr\" />\n" + 
+    		"	</m:mathPr>\n" + 
+    		"	<w:themeFontLang w:val=\"en-US\" />\n" + 
+    		"	<w:clrSchemeMapping w:bg1=\"light1\" w:t1=\"dark1\"\n" + 
+    		"		w:bg2=\"light2\" w:t2=\"dark2\" w:accent1=\"accent1\" w:accent2=\"accent2\"\n" + 
+    		"		w:accent3=\"accent3\" w:accent4=\"accent4\" w:accent5=\"accent5\" w:accent6=\"accent6\"\n" + 
+    		"		w:hyperlink=\"hyperlink\" w:followedHyperlink=\"followedHyperlink\" />\n" + 
+    		"	<w:shapeDefaults>\n" + 
+    		"		<o:shapedefaults v:ext=\"edit\" spidmax=\"1026\" />\n" + 
+    		"		<o:shapelayout v:ext=\"edit\">\n" + 
+    		"			<o:idmap v:ext=\"edit\" data=\"1\" />\n" + 
+    		"		</o:shapelayout>\n" + 
+    		"	</w:shapeDefaults>\n" + 
+    		"	<w:decimalSymbol w:val=\",\" />\n" + 
+    		"	<w:listSeparator w:val=\";\" />\n" + 
+    		"</w:settings>";
+
+    	DocumentSettingsPart documentSettingsPart = null;
+        try {
+        	documentSettingsPart = new DocumentSettingsPart(partName);
+            java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();
+
+            final CTSettings documentSettings = (CTSettings)((JAXBElement<?>)org.docx4j.XmlUtils.unmarshallFromTemplate(documentSettingsStr, mappings)).getValue();
+            documentSettingsPart.setJaxbElement(documentSettings);
+        }
+        catch (JAXBException e) {
+            e.printStackTrace();
+        }
+        catch (InvalidFormatException e) {
+            e.printStackTrace();
+        }
+        return documentSettingsPart;
+    }
 }
