@@ -51,7 +51,9 @@ package com.openexchange.office.ooxml.drawingml;
 
 import java.util.HashMap;
 import java.util.List;
+
 import javax.xml.bind.JAXBElement;
+
 import org.docx4j.dml.Graphic;
 import org.docx4j.dml.GraphicData;
 import org.docx4j.dml.chart.CTChartSpace;
@@ -66,6 +68,7 @@ import org.docx4j.openpackaging.parts.relationships.RelationshipsPart.AddPartBeh
 import org.docx4j.relationships.Relationship;
 import org.json.JSONException;
 import org.json.JSONObject;
+
 import com.openexchange.office.ooxml.OperationDocument;
 
 final public class DMLGraphic {
@@ -80,7 +83,7 @@ final public class DMLGraphic {
      *
      * @throws JSONException
      */
-    public static void getGraphicProperties(Part part, Graphic graphic, HashMap<String, Object> map) {
+    public static void getGraphicProperties(OperationDocument operationDocument, Part part, Graphic graphic, HashMap<String, Object> map) {
 
         if(graphic!=null) {
             final GraphicData graphicData = graphic.getGraphicData();
@@ -98,10 +101,9 @@ final public class DMLGraphic {
                                     if(relationshipsPart!=null) {
                                         final Part relPart = relationshipsPart.getPart(relId);
                                         if(relPart.getRelationshipType().equals("http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart")) {
-                                            final CTChartSpace chartSpace = ((Chart)relPart).getJaxbElement();
-                                            map.put("chartObject", chartSpace);
-                                            map.put("chartPart", relPart);
-                                            DMLChartSpace.getChartType(chartSpace, map);
+                                            final ChartWrapper chart = new ChartWrapper((Chart)relPart, operationDocument);
+                                            map.put("chartPart", chart);
+                                            DMLChartSpace.getChartType(chart, map);
                                         }
                                     }
                                 }
@@ -114,7 +116,7 @@ final public class DMLGraphic {
         }
     }
 
-    public static HashMap<String, Object> getDrawingType(org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, ICellAnchor cellAnchor) {
+    public static HashMap<String, Object> getDrawingType(OperationDocument operationDocument, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, ICellAnchor cellAnchor) {
 
         final HashMap<String, Object> drawingProperties = new HashMap<String, Object>();
         String drawingType = "undefined";
@@ -131,7 +133,7 @@ final public class DMLGraphic {
             drawingType = "image";
         }
         else if(cellAnchor.getGraphicFrame()!=null) {
-            DMLGraphic.getGraphicProperties(drawingsPart, cellAnchor.getGraphicFrame().getGraphic(), drawingProperties);
+            DMLGraphic.getGraphicProperties(operationDocument, drawingsPart, cellAnchor.getGraphicFrame().getGraphic(), drawingProperties);
         }
         if(!drawingProperties.containsKey("type")) {
             drawingProperties.put("type", drawingType);
