@@ -78,10 +78,8 @@ import org.docx4j.dml.CTSRgbColor;
 import org.docx4j.dml.CTScRgbColor;
 import org.docx4j.dml.CTSchemeColor;
 import org.docx4j.dml.CTShapeProperties;
-import org.docx4j.dml.CTShapeStyle;
 import org.docx4j.dml.CTSolidColorFillProperties;
 import org.docx4j.dml.CTStyleMatrix;
-import org.docx4j.dml.CTStyleMatrixReference;
 import org.docx4j.dml.CTSystemColor;
 import org.docx4j.dml.CTTextBody;
 import org.docx4j.dml.CTTextBodyProperties;
@@ -94,40 +92,19 @@ import org.docx4j.dml.CTTextParagraphProperties;
 import org.docx4j.dml.CTTransform2D;
 import org.docx4j.dml.IColorChoice;
 import org.docx4j.dml.IFillProperties;
+import org.docx4j.dml.INonVisualDrawingProperties;
+import org.docx4j.dml.IShapeStyle;
+import org.docx4j.dml.IStyleMatrixReference;
 import org.docx4j.dml.ITransform2D;
 import org.docx4j.dml.STPresetColorVal;
 import org.docx4j.dml.STPresetLineDashVal;
 import org.docx4j.dml.STSchemeColorVal;
 import org.docx4j.dml.STShapeType;
-import org.docx4j.dml.STTextAnchoringType;
-import org.docx4j.dml.STTextStrikeType;
-import org.docx4j.dml.STTextUnderlineType;
-import org.docx4j.dml.STTextVertOverflowType;
-import org.docx4j.dml.STTextVerticalType;
-import org.docx4j.dml.STTextWrappingType;
 import org.docx4j.dml.TextFont;
 import org.docx4j.dml.Theme;
-import org.docx4j.dml.chart.CTBoolean;
-import org.docx4j.dml.chart.CTCatAx;
-import org.docx4j.dml.chart.CTChartLines;
-import org.docx4j.dml.chart.CTLayout;
-import org.docx4j.dml.chart.CTNumFmt;
-import org.docx4j.dml.chart.CTPlotArea;
-import org.docx4j.dml.chart.CTScaling;
-import org.docx4j.dml.chart.CTTickLblPos;
-import org.docx4j.dml.chart.CTTickMark;
-import org.docx4j.dml.chart.CTTitle;
-import org.docx4j.dml.chart.CTTx;
-import org.docx4j.dml.chart.CTValAx;
-import org.docx4j.dml.chart.IAxisDescription;
-import org.docx4j.dml.chart.IListSer;
-import org.docx4j.dml.chart.STTickLblPos;
-import org.docx4j.dml.chart.STTickMark;
-import org.docx4j.dml.chart.chartStyle.CTChartColor;
 import org.docx4j.jaxb.Context;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.relationships.Relationship;
-import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 
@@ -137,7 +114,7 @@ import com.openexchange.office.ooxml.tools.json.JSONHelper;
 
 final public class DMLHelper {
 
-	private static final CTTextCharacterProperties GERMAN = new CTTextCharacterProperties();
+	static final CTTextCharacterProperties GERMAN = new CTTextCharacterProperties();
 	static {
 		GERMAN.setLang("de-DE");
 	}
@@ -159,7 +136,7 @@ final public class DMLHelper {
 			final JSONObject initialImageAttrs = attrs.optJSONObject("image");
 			final JSONObject imageAttrs = initialImageAttrs!=null ? initialImageAttrs : new JSONObject();
 
-            imageAttrs.put("imageUrl", DMLHelper.getBlipUrl(contextPart, blipFillProperties.getBlip()));
+            imageAttrs.put("imageUrl", getBlipUrl(contextPart, blipFillProperties.getBlip()));
 
             final CTRelativeRect srcRect = blipFillProperties.getSrcRect();
             if (srcRect!=null) {
@@ -320,12 +297,6 @@ final public class DMLHelper {
     	}
     }
 
-    public static void createJsonFromShapeProperties(JSONObject attrs, CTShapeProperties shapeProperties, boolean createTransform)
-        throws JSONException {
-
-        createJsonFromShapeProperties(attrs, shapeProperties, null, null, createTransform);
-    }
-
     private static enum ShapeStyleType {
         FILL,
         LINE,
@@ -334,7 +305,7 @@ final public class DMLHelper {
     }
 
     // styleMatrixReference can be zero, if available then the color from the styleMatrix is used
-    private static void createJsonFromFillStyle(Object source, CTStyleMatrixReference styleMatrixReference, JSONObject dest)
+    private static void createJsonFromFillStyle(Object source, IStyleMatrixReference styleMatrixReference, JSONObject dest)
         throws JSONException {
 
         if(source instanceof CTSolidColorFillProperties) {
@@ -374,7 +345,7 @@ final public class DMLHelper {
     }
 
     // styleMatrixReference can be zero, if available then the color from the styleMatrix is used
-    private static void createJsonFromLineStyle(CTLineProperties lineProperties, CTStyleMatrixReference styleMatrixReference, JSONObject dest)
+    private static void createJsonFromLineStyle(CTLineProperties lineProperties, IStyleMatrixReference styleMatrixReference, JSONObject dest)
         throws JSONException {
 
         if(lineProperties.getNoFill()!=null) {
@@ -418,10 +389,10 @@ final public class DMLHelper {
             }
         }
     }
-    private static void createJsonFromStyleMatrix(Theme theme, CTShapeStyle shapeStyle, ShapeStyleType type, JSONObject dest)
+    private static void createJsonFromStyleMatrix(Theme theme, IShapeStyle shapeStyle, ShapeStyleType type, JSONObject dest)
         throws JSONException {
 
-        if(shapeStyle==null&&theme==null) {
+        if(shapeStyle==null||theme==null) {
             return;
         }
         final BaseStyles themeElements = theme.getThemeElements();
@@ -433,7 +404,7 @@ final public class DMLHelper {
             return;
         }
         if(type==ShapeStyleType.FILL) {
-            final CTStyleMatrixReference fillStyleReference = shapeStyle.getFillRef();
+            final IStyleMatrixReference fillStyleReference = shapeStyle.getFillRef();
             if(fillStyleReference!=null) {
                 final long idx = fillStyleReference.getIdx();
                 if(idx==0||idx==1000) {
@@ -464,7 +435,7 @@ final public class DMLHelper {
             }
         }
         else if(type==ShapeStyleType.LINE) {
-           final CTStyleMatrixReference lineStyleReference = shapeStyle.getLnRef();
+           final IStyleMatrixReference lineStyleReference = shapeStyle.getLnRef();
            if(lineStyleReference!=null) {
                final int idx = (int)lineStyleReference.getIdx();
                final CTLineStyleList lineStyleList = fmtScheme.getLnStyleLst();
@@ -479,21 +450,33 @@ final public class DMLHelper {
         return;
     }
 
-    public static void createJsonFromShapeProperties(JSONObject attrs, CTShapeProperties shapeProperties, Theme theme, CTShapeStyle shapeStyle, boolean createTransform)
+    public static void createJsonFromShapeProperties(JSONObject attrs, CTShapeProperties shapeProperties, Theme theme, IShapeStyle shapeStyle, boolean createTransform)
+            throws JSONException {
+
+        if(shapeProperties!=null) {
+            if(createTransform) {
+                createJsonFromTransform2D(attrs, shapeProperties.getXfrm());
+            }
+            createJsonFromLineProperties(attrs, theme, shapeStyle, shapeProperties.getLn());
+        } else {
+            createJsonFromLineProperties(attrs, theme, shapeStyle, null);
+        }
+        createJsonFromFillProperties(attrs, shapeProperties, theme, shapeStyle, createTransform);
+    }
+
+    public static void createJsonFromFillProperties(JSONObject attrs, CTShapeProperties shapeProperties, Theme theme, IShapeStyle shapeStyle, boolean createTransform)
         throws JSONException {
 
         final JSONObject initialFillAttrs = attrs.optJSONObject("fill");
         final JSONObject fillAttrs = initialFillAttrs!=null ? initialFillAttrs : new JSONObject(1);
 
+        if(shapeStyle!=null) {
+            // try to apply the style first...
+            createJsonFromStyleMatrix(theme, shapeStyle, ShapeStyleType.FILL, fillAttrs);
+        }
+
         if(shapeProperties!=null) {
-        	if(createTransform) {
-	            createJsonFromTransform2D(attrs, shapeProperties.getXfrm());
-        	}
-        	createJsonFromLineProperties(attrs, theme, shapeStyle, shapeProperties.getLn());
-        	if(shapeStyle!=null) {
-        	    // try to apply the style first...
-        	    createJsonFromStyleMatrix(theme, shapeStyle, ShapeStyleType.FILL, fillAttrs);
-        	}
+
         	// then each hard applied fill attribute
         	if(shapeProperties.getNoFill()!=null) {
         	    createJsonFromFillStyle(shapeProperties.getNoFill(), null, fillAttrs);
@@ -516,20 +499,6 @@ final public class DMLHelper {
         	}
         	*/
         }
-        // TODO: special chart handling... should be removed if frontend is no longer depenending to a type solid default
-        if(shapeStyle==null) {
-            if(!fillAttrs.has("type")) {
-                fillAttrs.put("type", "solid");
-            }
-            JSONObject lineAttrs = attrs.optJSONObject("line");
-            if (null == lineAttrs) {
-                lineAttrs = new JSONObject();
-                attrs.put("line", lineAttrs);
-            }
-            if (!lineAttrs.has("type")) {
-                lineAttrs.put("type", "auto");
-            }
-        }
         if(initialFillAttrs==null&&!fillAttrs.isEmpty()) {
             attrs.put("fill", fillAttrs);
         }
@@ -552,7 +521,7 @@ final public class DMLHelper {
 		return shapeProperties;
 	}
 
-    public static void createJsonFromLineProperties(JSONObject attrs, Theme theme, CTShapeStyle shapeStyle, CTLineProperties lineProperties)
+    public static void createJsonFromLineProperties(JSONObject attrs, Theme theme, IShapeStyle shapeStyle, CTLineProperties lineProperties)
         throws JSONException {
 
         final JSONObject initialLineAttrs = attrs.optJSONObject("line");
@@ -566,11 +535,6 @@ final public class DMLHelper {
     	    createJsonFromLineStyle(lineProperties, null, lineAttrs);
     	}
 
-    	// TODO: the following code is superfluous if line type "auto" is removed and the default is "none"
-    	if(!lineAttrs.has("type")) {
-    	    lineAttrs.put("type", "none");
-    	}
-
     	if(initialLineAttrs==null&&!lineAttrs.isEmpty()) {
             attrs.put("line", lineAttrs);
         }
@@ -584,33 +548,24 @@ final public class DMLHelper {
 			if(lineProperties==null) {
 				lineProperties = Context.getDmlObjectFactory().createCTLineProperties();
 			}
-			final Object type = lineAttrs.opt("type");
-			if(type!=null) {
+			final Object typeO = lineAttrs.opt("type");
+			if(null!=typeO) {
+				final String type = typeO instanceof String ? (String)typeO : "none";
                 final CTSolidColorFillProperties oldSolidFill = lineProperties.getSolidFill();
 			    lineProperties.setNoFill(null);
                 lineProperties.setSolidFill(null);
                 lineProperties.setGradFill(null);
                 lineProperties.setPattFill(null);
-			    if(type instanceof String) {
-                    if(((String)type).equals("none")||((String)type).equals("auto")) {
-                        lineProperties.setNoFill(Context.getDmlObjectFactory().createCTNoFillProperties());
+                if(type.equals("none")) {
+                    lineProperties.setNoFill(Context.getDmlObjectFactory().createCTNoFillProperties());
+                }
+                else if(type.equals("solid")) {
+                    if(oldSolidFill!=null) {
+                        lineProperties.setSolidFill(oldSolidFill);
+                    } else {
+                        lineProperties.setSolidFill(Context.getDmlObjectFactory().createCTSolidColorFillProperties());
                     }
-                    else if(((String)type).equals("solid")) {
-                        if(oldSolidFill!=null) {
-                            lineProperties.setSolidFill(oldSolidFill);
-                        }
-                        else {
-                            final CTSolidColorFillProperties solidColorFillProperties = Context.getDmlObjectFactory().createCTSolidColorFillProperties();
-                            final CTPresetColor presetColor = Context.getDmlObjectFactory().createCTPresetColor();
-                            presetColor.setVal(STPresetColorVal.BLACK);
-                            solidColorFillProperties.setPrstClr(presetColor);
-                            lineProperties.setSolidFill(solidColorFillProperties);
-                        }
-                    }
-			    }
-			    else {
-			        lineProperties.setNoFill(Context.getDmlObjectFactory().createCTNoFillProperties());
-			    }
+                }
 			}
 			final Object color = lineAttrs.opt("color");
 			if(color!=null) {
@@ -806,8 +761,8 @@ final public class DMLHelper {
 				value = "followedHyperlink";
 				break;
 			case PH_CLR:
-				value = "style";
-				break;
+			    //nothing can be interpreted as AUTO color
+				return;
 			}
         }
         dest.put("color", JSONHelper.makeColor("scheme", value, color.getEGColorTransform()));
@@ -836,8 +791,6 @@ final public class DMLHelper {
 			colorVal = STSchemeColorVal.HLINK;
 		} else if (value.equals("followedHyperlink")) {
 			colorVal = STSchemeColorVal.FOL_HLINK;
-		} else if (value.equals("style")) {
-			colorVal = STSchemeColorVal.PH_CLR;
 		} else {
 			STSchemeColorVal tmpVal;
 			try {
@@ -869,40 +822,10 @@ final public class DMLHelper {
         dest.put("color", JSONHelper.makeColor("preset", color.getVal().value(), color.getEGColorTransform()));
     }
 
-    public static JSONObject createJsonFromChartColor(CTChartColor color)
-        throws JSONException {
-
-        JSONObject jsonChartColor = null;
-        if(color!=null) {
-            jsonChartColor = new JSONObject();
-            jsonChartColor.put("id", color.getId());
-            jsonChartColor.put("meth", color.getMeth());
-            final List<CTSchemeColor> schemeClrList = color.getSchemeClr();
-            if(schemeClrList!=null) {
-                final JSONArray jsonArray = new JSONArray();
-                for(final CTSchemeColor schemeClr:schemeClrList) {
-                    final JSONObject jsonSchemeColor = new JSONObject();
-                    createJsonFromSchemeClr(jsonSchemeColor, schemeClr);
-                    jsonArray.put(jsonSchemeColor);
-                }
-                jsonChartColor.put("schemeClr", jsonArray);
-            }
-            final List<CTSchemeColor> variationList = color.getVariation();
-            if(variationList!=null ) {
-                final JSONArray jsonArray = new JSONArray();
-                for(final CTSchemeColor schemeClr:variationList) {
-                    final JSONObject resColor = new JSONObject();
-                	createJsonFromSchemeClr(resColor, schemeClr);
-					if (resColor != null){
-						jsonArray.put(resColor);
-					}
-                }
-				if (!jsonArray.isEmpty()){
-					jsonChartColor.put("variation", jsonArray);
-				}
-            }
-        }
-        return jsonChartColor!=null&&!jsonChartColor.isEmpty()?jsonChartColor:null;
+    public static CTNonVisualDrawingProps createNonVisualDrawingProps(String name) {
+        final CTNonVisualDrawingProps nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
+        nonVisualDrawingProps.setName(name);
+        return nonVisualDrawingProps;
     }
 
     public static void createJsonFromNonVisualDrawingProperties(JSONObject attrs, CTNonVisualDrawingProps nonVisualDrawingProps)
@@ -922,35 +845,44 @@ final public class DMLHelper {
         if(description!=null&&!description.isEmpty()) {
             drawingAttrs.put("description", description);
         }
+        if(nonVisualDrawingProps.isHidden()) {
+        	drawingAttrs.put("hidden", true);
+        }
         if(initialDrawingAttrs==null&&!drawingAttrs.isEmpty()) {
 			attrs.put("drawing", drawingAttrs);
 		}
     }
 
-    public static void applyNonVisualDrawingProperties(JSONObject drawingProperties, CTNonVisualDrawingProps nonVisualDrawingProps) {
+    public static void applyNonVisualDrawingProperties(INonVisualDrawingProperties iNonVisualDrawingProperties, JSONObject drawingProperties) {
 
-        if(nonVisualDrawingProps==null) {
-            return;
-        }
-
-        if(drawingProperties.has("name")) {
-            final Object optName = drawingProperties.opt("name");
-            if(optName instanceof String) {
-                nonVisualDrawingProps.setName((String)optName);
-            }
-            else {
-                // name is a required xml attribute... so we are setting a dummy
-                nonVisualDrawingProps.setName("...");
-            }
-        }
-        if(drawingProperties.has("description")) {
-            final Object optDescription = drawingProperties.opt("description");
-            if(optDescription instanceof String) {
-                nonVisualDrawingProps.setDescr((String)optDescription);
-            }
-            else {
-                nonVisualDrawingProps.setDescr(null);
-            }
+    	if(drawingProperties==null) {
+    		return;
+    	}
+    	final Object optName = drawingProperties.opt("name");
+        final Object optDescription = drawingProperties.opt("description");
+        if(optName!=null||optDescription!=null) {
+        	CTNonVisualDrawingProps nonVisualDrawingProperties = iNonVisualDrawingProperties.getCNvPr();
+        	if(nonVisualDrawingProperties==null) {
+        		nonVisualDrawingProperties = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
+        		iNonVisualDrawingProperties.setCNvPr(nonVisualDrawingProperties);
+        	}
+        	if(optName!=null) {
+	            if(optName instanceof String) {
+	            	nonVisualDrawingProperties.setName((String)optName);
+	            }
+	            else {
+	                // name is a required xml attribute... so we are setting a dummy
+	            	nonVisualDrawingProperties.setName("...");
+	            }
+        	}
+        	if(optDescription!=null) {
+	            if(optDescription instanceof String) {
+	            	nonVisualDrawingProperties.setDescr((String)optDescription);
+	            }
+	            else {
+	            	nonVisualDrawingProperties.setDescr(null);
+	            }
+	        }
         }
     }
 
@@ -994,12 +926,6 @@ final public class DMLHelper {
         return blipFillProperties;
     }
 
-    public static CTNonVisualDrawingProps createNonVisualDrawingProps(String name) {
-        final CTNonVisualDrawingProps nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
-        nonVisualDrawingProps.setName(name);
-        return nonVisualDrawingProps;
-    }
-
     public static void addTo(CTRegularTextRun textRun, StringBuffer stringBuffer) {
     	stringBuffer.append(textRun.getT());
     }
@@ -1037,164 +963,12 @@ final public class DMLHelper {
 		return buf.toString();
 	}
 
-	public static JSONObject createJsonFromProperties(CTTitle title) throws JSONException {
-		if (title == null) {
-			return null;
-		}
-		final CTTx tx = title.getTx();
-		if (tx == null) {
-			return null;
-		}
-		final CTTextBody rich = tx.getRich();
-		if (rich == null) {
-			return null;
-		}
-		final String first = getText(rich);
-
-		final JSONObject attrs = new JSONObject();
-		final JSONArray link = new JSONArray();
-		link.put(first);
-		final JSONObject text = new JSONObject();
-		text.put("link", link);
-		attrs.put("text", text);
-
-		final JSONObject character = DMLChartSpace.createJsonFromProperties(rich);
-		if (character != null) {
-			attrs.put("character", character);
-		}
-		return attrs;
-	}
-
-	public static void setTitleFromAttrs(CTTitle title, JSONObject attrs) throws JSONException {
-		handleStandardTypes(title);
-		final CTTx tx = title.getTx();
-		final CTTextBody rich = tx.getRich();
-		final CTTextBodyProperties body = rich.getBodyPr();
-		final CTTextBody txPr = title.getTxPr();
-		final CTTextBodyProperties body2 = txPr.getBodyPr();
-		body.setRot(0);
-		body2.setRot(0);
-
-		final JSONObject text = attrs.optJSONObject("text");
-		if(text==null){
-			return;
-		}
-
-
-		final Object olink = text.opt("link");
-
-		if (olink == null || !(olink instanceof JSONArray))
-		{
-
-			if (olink == null || olink.equals(JSONObject.NULL))
-			{
-				return;
-			}
-			else
-			{
-				Logger.getAnonymousLogger().warning("title text has no correct content " + text.opt("link"));
-				return;
-			}
-		}
-
-		final JSONArray links = (JSONArray) olink;
-		if (links.length() > 0)
-		{
-
-
-			final List<CTTextParagraph> paras = rich.getP();
-			paras.clear();
-			for (int i = 0; i < links.length(); i++)
-			{
-				final CTTextParagraph p = new CTTextParagraph();
-				setTextFromAttrs(p, attrs);
-				paras.add(p);
-				final List<Object> textHolder = p.getEGTextRun();
-				textHolder.clear();
-
-				final String link = links.getString(i);
-				final String[] textes = link.split("\n");
-				for (int j = 0; j < textes.length; j++)
-				{
-					final CTRegularTextRun reg = new CTRegularTextRun();
-					reg.setRPr(GERMAN);
-					reg.setT(textes[j]);
-					textHolder.add(reg);
-					if (j < textes.length - 1)
-					{
-						final CTTextLineBreak lb = new CTTextLineBreak();
-						lb.setRPr(GERMAN);
-						textHolder.add(lb);
-					}
-				}
-			}
-		}
-	}
-
-	public static void refreshTitleRotations(
-		CTPlotArea area)
-	{
-		final List<IAxisDescription> axes = area.getValAxOrCatAxOrDateAx();
-		final IListSer chart = area.getAreaChartOrArea3DChartOrLineChart().get(0);
-		final String dir = chart.getBarDirection();
-		for (int i = 0; i < axes.size(); i++)
-		{
-			boolean rotate = false;
-			if (i == 1)
-			{
-				rotate = true;
-			}
-			if (dir == null || dir.equals("col"))
-			{
-				// normal
-			}
-			else
-			{
-				rotate = !rotate;
-			}
-			int rot = 0;
-			if (rotate)
-			{
-				rot = -5400000;
-			}
-			final IAxisDescription axis = axes.get(i);
-			final CTTitle title = axis.getTitle();
-
-			if (title != null && title.getTx() != null)
-			{
-				final CTTx tx = title.getTx();
-				final CTTextBody rich = tx.getRich();
-				final CTTextBodyProperties body = rich.getBodyPr();
-				body.setRot(rot);
-
-				final CTTextBody txPr = title.getTxPr();
-				if (null != txPr) {
-				    final CTTextBodyProperties body2 = txPr.getBodyPr();
-				    body2.setRot(rot);
-				}
-			}
-
-			final CTTextBody lbl = axis.getTxPr();
-			if (lbl != null && lbl.getBodyPr() != null)
-			{
-				final CTTextBodyProperties body = lbl.getBodyPr();
-				body.setRot(0);
-			}
-		}
-	}
-
 	public static void handleStandardTypes(CTTextBody txPr) {
 		CTTextBodyProperties body = txPr.getBodyPr();
 		if(body==null){
 			body = new CTTextBodyProperties();
 			txPr.setBodyPr(body);
 		}
-		body.setSpcFirstLastPara(true);
-		body.setVertOverflow(STTextVertOverflowType.ELLIPSIS);
-		body.setVert(STTextVerticalType.HORZ);
-		body.setWrap(STTextWrappingType.SQUARE);
-		body.setAnchor(STTextAnchoringType.CTR);
-		body.setAnchorCtr(true);
 
 		CTTextListStyle lst2 = txPr.getLstStyle();
 		if (lst2 == null) {
@@ -1203,211 +977,13 @@ final public class DMLHelper {
 		}
 	}
 
-	private static void handleStandardTypes(CTTitle title) {
-		CTTx tx = title.getTx();
-		if (tx == null) {
-			tx = new CTTx();
-			title.setTx(tx);
-		}
-		CTTextBody rich = tx.getRich();
-		if (rich == null) {
-			rich = new CTTextBody();
-			tx.setRich(rich);
-		}
-		handleStandardTypes(rich);
-
-		CTLayout layout = title.getLayout();
-		if (layout == null) {
-			layout = new CTLayout();
-			title.setLayout(layout);
-		}
-		CTBoolean overlay = title.getOverlay();
-		if (overlay == null) {
-			overlay = new CTBoolean();
-			title.setOverlay(overlay);
-		}
-		overlay.setVal(false);
-		CTShapeProperties spr = title.getSpPr();
-		if (spr == null) {
-			spr = getHiddenShapeProperties();
-		}
-		CTTextBody txpr = title.getTxPr();
-		if (txpr == null) {
-			txpr = new CTTextBody();
-			title.setTxPr(txpr);
-		}
-		handleStandardTypes(txpr);
-
-		final List<CTTextParagraph> ps = rich.getP();
-		ps.clear();
-
-		final List<CTTextParagraph> par2 = txpr.getP();
-		par2.clear();
-
-		final CTTextParagraph p = new CTTextParagraph();
-		p.setPPr(getTextProp());
-		par2.add(p);
-		ps.add(p);
-
-	}
-
 	public static CTTextParagraphProperties getTextProp() {
 		final CTTextParagraphProperties props = new CTTextParagraphProperties();
 		final CTTextCharacterProperties def = new CTTextCharacterProperties();
 		props.setDefRPr(def);
-
-		def.setSz(1100);
-		def.setB(false);
-		def.setI(false);
-		def.setU(STTextUnderlineType.NONE);
-		def.setStrike(STTextStrikeType.NO_STRIKE);
-		def.setKern(1200);
-		def.setBaseline(0);
-
-		final TextFont latin = new TextFont();
-		latin.setTypeface("+mn-lt");
-		def.setLatin(latin);
-		final TextFont ea = new TextFont();
-		ea.setTypeface("+mn-ea");
-		def.setEa(ea);
-		final TextFont cs = new TextFont();
-		cs.setTypeface("+mn-cs");
-		def.setCs(cs);
-
-		def.setSolidFill(null);
-		def.setNoFill(null);
-
 		return props;
 	}
 
-	public static void addToOperations(CTPlotArea plotArea, JSONArray operations, JSONArray position) throws JSONException {
-		final List<IAxisDescription> axes = plotArea.getValAxOrCatAxOrDateAx();
-		if (axes != null && axes.size() > 0) {
-			for (int i = 0; i < axes.size(); i++) {
-				final IAxisDescription axe = axes.get(i);
-
-				final String axisParam;
-				if (i == 0) {
-					axisParam = "x";
-				} else if (i == 1) {
-					axisParam = "y";
-				} else if (i == 2) {
-					axisParam = "z";
-				} else {
-					return;
-				}
-
-				getAxeOperation(axe, operations, axisParam, position);
-				getGridOperation(axe, operations, axisParam, position);
-			}
-		}
-	}
-
-	private static final void getAxeOperation(IAxisDescription axe, JSONArray operations, String axisParam, JSONArray position) throws JSONException {
-
-		final JSONObject axis = new JSONObject();
-        final CTScaling scaling = axe.getScaling();
-        if(scaling!=null) {
-            if (scaling.getMin()!=null) {
-                axis.put("min", scaling.getMin().getVal());
-            }
-            if(scaling.getMax()!=null) {
-                axis.put("max", scaling.getMax().getVal());
-            }
-        }
-        JSONObject character = null;
-		final JSONObject axisOp = new JSONObject();
-		final CTBoolean del = axe.getDelete();
-		if (del != null && del.isVal()) {
-			final JSONObject line = new JSONObject();
-			line.put("type", "none");
-			axisOp.put("line", line);
-		} else {
-			final CTShapeProperties shape = axe.getSpPr();
-			if (shape != null) {
-                DMLHelper.createJsonFromShapeProperties(axisOp, shape, false);
-			}
-			if (!axisOp.has("line")) {
-				axisOp.put("line", makeStandardShapeType());
-			}
-
-			final CTTickLblPos label = axe.getTickLblPos();
-			if (label != null && label.getVal() != STTickLblPos.NONE) {
-				axis.put("label", true);
-				character = DMLChartSpace.createJsonFromProperties(axe.getTxPr());
-			}
-
-			getAxeTitleOperation(axe, operations, axisParam, position);
-		}
-
-		if (axis.length() > 0) {
-			axisOp.put("axis", axis);
-		}
-		if (character != null) {
-			axisOp.put("character", character);
-		}
-
-
-		if (axisOp.length() > 0) {
-			final JSONObject op = new JSONObject();
-			op.put("name", "setChartAxisAttributes");
-			op.put("start", position);
-			op.put("axis", axisParam);
-			op.put("attrs", axisOp);
-			operations.put(op);
-		}
-	}
-
-	public static void setAxisAttributes(CTPlotArea plotArea, String axis, JSONObject attrs) throws JSONException {
-		final IAxisDescription ax = getAxis(plotArea, axis);
-		if (ax == null) {
-			return;
-		}
-		final CTShapeProperties shape = applyShapePropertiesFromJson(ax.getSpPr(), attrs);
-		ax.setSpPr(shape);
-		CTTickMark tickmark = ax.getMajorTickMark();
-		if (tickmark == null)
-		{
-			tickmark = new CTTickMark();
-			ax.setMajorTickMark(tickmark);
-		}
-		if (shape.getLn() != null && shape.getLn().getNoFill() != null)
-		{
-			tickmark.setVal(STTickMark.NONE);
-		}
-		else
-		{
-			tickmark.setVal(STTickMark.OUT);
-		}
-
-		final JSONObject a = attrs.optJSONObject("axis");
-		if (a != null) {
-			CTTickLblPos lblPos = ax.getTickLblPos();
-			if (lblPos == null) {
-				lblPos = new CTTickLblPos();
-				ax.setTickLblPos(lblPos);
-			}
-			final boolean label = a.optBoolean("label", false);
-
-			if (label) {
-				lblPos.setVal(STTickLblPos.LOW);
-
-
-				CTTextBody txPr = ax.getTxPr();
-
-				if (txPr == null) {
-					txPr = new CTTextBody();
-					ax.setTxPr(txPr);
-					DMLHelper.handleStandardTypes(txPr);
-				}
-				setCharacterFromAttrs(txPr, attrs);
-
-			} else {
-				lblPos.setVal(STTickLblPos.NONE);
-			}
-		}
-	}
-
 	public static void setCharacterFromAttrs(CTTextBody txPr, JSONObject attrs) throws JSONException {
 		final List<CTTextParagraph> ps = txPr.getP();
 		ps.clear();
@@ -1416,131 +992,41 @@ final public class DMLHelper {
 		setTextFromAttrs(p, attrs);
 	}
 
-	public static void setTextFromAttrs(CTTextParagraph p, JSONObject attrs) throws JSONException {
-		final JSONObject character = attrs.optJSONObject("character");
+    public static void setTextFromAttrs(CTTextParagraph p, JSONObject attrs) throws JSONException {
+        final JSONObject character = attrs.optJSONObject("character");
 
+        CTTextParagraphProperties prop = p.getPPr();
+        if (prop == null)
+        {
+            prop = getTextProp();
+            p.setPPr(prop);
+        }
 
-		CTTextParagraphProperties prop = p.getPPr();
-		if(prop == null){
-			prop = getTextProp();
-			p.setPPr(prop);
-		}
+        if (character != null)
+        {
 
-		if (character != null)
-		{
+            final CTTextCharacterProperties def = prop.getDefRPr();
 
-			final CTTextCharacterProperties def = prop.getDefRPr();
+            def.setSz(ptTofontSize(character.optDouble("fontSize", 10)));
 
-			def.setSz(ptTofontSize(character.optDouble("fontSize", 10)));
+            final JSONObject color = character.optJSONObject("color");
+            if (color != null)
+            {
 
-			CTSolidColorFillProperties solid = def.getSolidFill();
-			if (solid == null)
-			{
-				solid = new CTSolidColorFillProperties();
-				def.setSolidFill(solid);
-			}
+                CTSolidColorFillProperties solid = def.getSolidFill();
+                if (solid == null)
+                {
+                    solid = new CTSolidColorFillProperties();
+                    def.setSolidFill(solid);
+                }
 
-			final JSONObject color = character.optJSONObject("color");
-			if (color != null)
-			{
-				if (!createSolidColorFillPropertiesFromJson(def.getSolidFill(), color))
-				{
-					def.setSolidFill(null);
-				}
-			}
-		}
-	}
-
-	private static IAxisDescription getAxis(CTPlotArea plotArea, String axis) {
-		final int index;
-		if (axis.equals("x")) {
-			index = 0;
-		} else if (axis.equals("y")) {
-			index = 1;
-		} else if (axis.equals("z")) {
-			index = 2;
-		} else {
-			return null;
-		}
-		final List<IAxisDescription> axes = plotArea.getValAxOrCatAxOrDateAx();
-
-		if (index >= axes.size()) {
-			return null;
-		}
-
-		IAxisDescription res = axes.get(index);
-
-		if (res == null) {
-			if (axis.equals("x")) {
-				final CTCatAx cat = new CTCatAx();
-				cat.setMinorGridlines(getHiddenChartLines());
-				cat.setMinorTickMark(getHiddenTickMark());
-				cat.setNumFmt(createNumFmt());
-				res = cat;
-			} else {
-				final CTValAx val = new CTValAx();
-				val.setMinorGridlines(getHiddenChartLines());
-				val.setMinorTickMark(getHiddenTickMark());
-				val.setNumFmt(createNumFmt());
-				res = val;
-			}
-
-			res.getAxId().setVal(index);
-			axes.set(index, res);
-		}
-		if (res.getDelete() == null || res.getDelete().isVal()) {
-			hideAxis(res);
-		}
-
-		return res;
-	}
-
-	private static void hideAxis(
-		IAxisDescription axis)
-	{
-		final CTBoolean delete = new CTBoolean();
-		delete.setVal(false);
-		axis.setDelete(delete);
-
-		axis.setSpPr(getHiddenShapeProperties());
-
-		axis.setMajorGridlines(getHiddenChartLines());
-		axis.setTickLblPos(getHiddenLblPos());
-
-		axis.setTitle(null);
-		axis.setTxPr(null);
-
-	}
-
-	public static CTNumFmt createNumFmt()
-	{
-		final CTNumFmt res = new CTNumFmt();
-		res.setFormatCode("0");
-		res.setSourceLinked(true);
-		return res;
-	}
-
-
-	private static CTChartLines getHiddenChartLines()
-	{
-		final CTChartLines line = new CTChartLines();
-		line.setSpPr(getHiddenShapeProperties());
-		return line;
-	}
-
-	private static CTTickLblPos getHiddenLblPos()
-	{
-		final CTTickLblPos lbl = new CTTickLblPos();
-		lbl.setVal(STTickLblPos.NONE);
-		return lbl;
-	}
-
-	private static CTTickMark getHiddenTickMark()
-	{
-		final CTTickMark mark = new CTTickMark();
-		mark.setVal(STTickMark.NONE);
-		return mark;
-	}
+                if (!createSolidColorFillPropertiesFromJson(def.getSolidFill(), color))
+                {
+                    def.setSolidFill(null);
+                }
+            }
+        }
+    }
 
 	public static CTShapeProperties getHiddenShapeProperties()
 	{
@@ -1552,72 +1038,7 @@ final public class DMLHelper {
 		return shape;
 	}
 
-	private static final void getAxeTitleOperation(IAxisDescription axe, JSONArray operations, String axisParam, JSONArray position) throws JSONException {
-		final CTTitle title = axe.getTitle();
-
-		final JSONObject attrs = DMLHelper.createJsonFromProperties(title);
-		if (attrs != null) {
-
-			final JSONObject op = new JSONObject();
-			op.put("name", "setChartTitleAttributes");
-			op.put("start", position);
-			op.put("axis", axisParam);
-			op.put("attrs", attrs);
-			operations.put(op);
-		}
-
-	}
-
-	public static void setChartTitleAttributes(CTPlotArea plotArea, String axis, JSONObject attrs) throws JSONException {
-		final IAxisDescription ax = getAxis(plotArea, axis);
-		if (ax == null) {
-			return;
-		}
-		CTTitle title = ax.getTitle();
-		if (title == null) {
-			title = new CTTitle();
-			ax.setTitle(title);
-		}
-		setTitleFromAttrs(title, attrs);
-		refreshTitleRotations(plotArea);
-	}
-
-	private static final void getGridOperation(IAxisDescription axe, JSONArray operations, String axisParam, JSONArray position) throws JSONException {
-
-		final CTChartLines grid = axe.getMajorGridlines();
-		if (grid != null) {
-			final JSONObject gridOp = new JSONObject();
-			final CTShapeProperties shape = grid.getSpPr();
-			if (shape != null) {
-				createJsonFromShapeProperties(gridOp, shape, false);
-			}
-			if (!gridOp.has("line")) {
-				gridOp.put("line", makeStandardShapeType());
-			}
-			final JSONObject op = new JSONObject();
-			op.put("name", "setChartGridlineAttributes");
-			op.put("start", position);
-			op.put("axis", axisParam);
-			op.put("attrs", gridOp);
-			operations.put(op);
-		}
-	}
-
-	public static void setChartGridlineAttributes(CTPlotArea plotArea, String axis, JSONObject attrs) throws JSONException {
-		final IAxisDescription ax = getAxis(plotArea, axis);
-		if (ax == null) {
-			return;
-		}
-
-		CTChartLines grids = ax.getMajorGridlines();
-		if (grids == null) {
-			grids = new CTChartLines();
-			ax.setMajorGridlines(grids);
-		}
-		grids.setSpPr(applyShapePropertiesFromJson(grids.getSpPr(), attrs));
-	}
-
-    /**
+	/**
      * @param part (partName Uri should be /word/document.xml)
      * @return the imageUrl in following form: word/media/xxxx.png
      */
@@ -1651,13 +1072,57 @@ final public class DMLHelper {
     	return (int)(w*360);
 	}
 
-	//FIXME: nessecary?
-	private static JSONObject makeStandardShapeType() throws JSONException {
-		final JSONObject shape = new JSONObject();
-		shape.put("type", "solid");
-		final JSONObject color = new JSONObject();
-		color.put("type", "auto");
-		shape.put("color", color);
-		return shape;
-	}
+	public static JSONObject createJsonFromProperties(CTTextBody props) throws JSONException {
+    	JSONObject character = null;
+    	if (props != null) {
+
+    		final List<CTTextParagraph> paras = props.getP();
+    		if (paras != null && paras.size() > 0) {
+    			final CTTextParagraph para = paras.get(0);
+
+    			final CTTextParagraphProperties prop = para.getPPr();
+    			if (prop != null) {
+    				final CTTextCharacterProperties defPr = prop.getDefRPr();
+    				if (defPr != null) {
+
+    					// color
+    					if (defPr.getSolidFill() != null) {
+    					    character = new JSONObject();
+    						createJsonColorFromSolidColorFillProperties(character, defPr.getSolidFill());
+    					} else if (defPr.getNoFill() != null) {
+    						character = new JSONObject();
+    						Logger.getAnonymousLogger().warning("CtTextbody has a no fill property, result is set to null");
+    					}
+    					if (character == null) {
+    						character = new JSONObject();
+    					}
+
+    					// font-family
+    					final TextFont latn = defPr.getLatin();
+    					if (latn != null) {
+    						final String ff = latn.getTypeface();
+    						if (ff != null && !ff.equals("+mn-lt")) {
+    							character.put("fontName", ff);
+    						}
+
+    					}
+
+    					final float size = fontSizeToPt(defPr.getSz());
+    					character.put("fontSize", size);
+    					final Boolean bold = defPr.isB();
+    					final Boolean italic = defPr.isI();
+    					if (bold != null && bold.booleanValue()) {
+    						character.put("bold", true);
+    					}
+    					if (italic != null && italic.booleanValue()) {
+    						character.put("italic", true);
+    					}
+    				}
+
+    			}
+    		}
+    	}
+
+    	return character;
+    }
 }
