@@ -145,7 +145,7 @@ abstract public class CreateOperationHelper {
         operationsArray.put(insertThemeObject);
     }
 
-    public JSONObject createInsertStyleSheetOperation(String type, String styleId, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority, Boolean isDefault, boolean auto)
+    public JSONObject createInsertStyleSheetOperation(String type, String styleId, String styleName, JSONObject attrs, String parentId, Boolean hidden, Integer uipriority, Boolean isDefault, Boolean custom, boolean auto)
         throws JSONException {
 
         final JSONObject insertStylesheetObject = new JSONObject();
@@ -166,6 +166,9 @@ abstract public class CreateOperationHelper {
         if(auto==true) {
             insertStylesheetObject.put("auto", auto);
         }
+        if(custom!=null) {
+            insertStylesheetObject.put("custom", custom);
+        }
         return insertStylesheetObject;
     }
 }
