@@ -49,6 +49,9 @@
 
 package com.openexchange.office.ooxml.tools;
 
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.io.InputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Collections;
@@ -62,13 +65,16 @@ import javax.xml.bind.DatatypeConverter;
 import javax.xml.bind.JAXBElement;
 
 import org.apache.commons.io.FilenameUtils;
-import org.apache.commons.logging.Log;
+import org.apache.commons.io.IOUtils;
 import org.docx4j.dml.CTColor;
 import org.docx4j.openpackaging.URIHelper;
 import org.docx4j.openpackaging.contenttype.ContentTypeManager;
 import org.docx4j.openpackaging.contenttype.ContentTypes;
+import org.docx4j.openpackaging.exceptions.Docx4JException;
 import org.docx4j.openpackaging.exceptions.InvalidFormatException;
 import org.docx4j.openpackaging.exceptions.PartUnrecognisedException;
+import org.docx4j.openpackaging.io3.stores.PartStore;
+import org.docx4j.openpackaging.io3.stores.ZipPartStore;
 import org.docx4j.openpackaging.parts.Part;
 import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPart;
 import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPartAbstractImage;
@@ -79,15 +85,16 @@ import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 
-import com.openexchange.log.LogFactory;
+import com.openexchange.office.IResourceManager;
 import com.openexchange.office.ooxml.OperationDocument;
-import com.openexchange.office.tools.FileHelper;
-import com.openexchange.office.tools.Resource;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.files.FileHelper;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 final public class Commons {
 
-    final private static Log log = LogFactory.getLog(Commons.class);
+	final private static Logger log = LoggerFactory.getLogger(Commons.class);
 
     final static String hexArray = "0123456789ABCDEF";
 
@@ -478,45 +485,21 @@ final public class Commons {
         return rId;
     }
 
-    // image Folder: "word/media/"
-    public static String getImageUrlFromImageData(ResourceManager resourceManager, String imageData, String imageFolder) {
-        String imageUrl = null;
-        if(imageData != null) {
-            long uid = resourceManager.addBase64(imageData);
-            if(uid!=0) {
-                imageUrl = imageFolder + "uid" + Long.toHexString(uid);
-                final String basePattern = "base64,";
-                final String dataPattern = "data:image/";
-
-                int pos = imageData.indexOf(basePattern);
-                if (imageData.startsWith(dataPattern)) {
-                    imageUrl += '.' + imageData.substring(dataPattern.length(), pos - 1);
-                }
-            }
-        }
-        return imageUrl;
-    }
-
-    public static byte[] retrieveResource(String imageUrl, ResourceManager resourceManager) {
+    public static byte[] retrieveResource(String imageUrl, IResourceManager resourceManager) {
 
         byte[] buf = null;
-        Resource resource = null;
         final String filename = FilenameUtils.getBaseName(imageUrl);
         if(filename.startsWith("uid")) {
             try {
                 long uid = Long.parseLong(filename.substring(3), 16);
-                resource = resourceManager.getResource(uid);
+                buf = resourceManager.getResourceBuffer(uid);
             }
             catch(NumberFormatException e) {
                 //
             }
         }
-        if(resource!=null) {
-            buf = resource.getBuffer();
-
-            if(buf==null) {
-                log.error("ooxml export: could not get buffer of resource " + filename);
-            }
+        if(buf==null) {
+            log.error("ooxml export: could not get buffer of resource " + filename);
         }
         return buf;
     }
@@ -570,7 +553,6 @@ final public class Commons {
        // check if the image is already part of the document
        Relationship rel = Commons.getRelationship(part, imageUrl);
        if(rel==null) {
-
            // Ensure the relationships part exists
            RelationshipsPart relationshipsPart = part.getRelationshipsPart();
            if(relationshipsPart==null) {
@@ -603,10 +585,31 @@ final public class Commons {
                    contentType = ContentTypes.IMAGE_PICT;
            }
 
-           final byte[] imageData = Commons.retrieveResource(imageUrl, operationDocument.getResourceManager());
+           byte[] imageData = Commons.retrieveResource(imageUrl, operationDocument.getResourceManager());
+           if(imageData==null) {
+        	   // check if we can find the image in the source part store
+        	   final PartStore sourcePartStore = operationDocument.getPackage().getSourcePartStore();
+        	   if(sourcePartStore instanceof ZipPartStore) {
+        		   final String url = imageUrl.charAt(0)=='/' ? imageUrl.substring(1) : imageUrl;
+        		   if(((ZipPartStore)sourcePartStore).partExists(url)) {
+        			   try {
+        				   final InputStream in = ((ZipPartStore)sourcePartStore).loadPart(url);
+        				   if(in!=null) {
+        					   imageData = IOUtils.toByteArray(in);
+        				   }
+        			   }
+        			   catch (Docx4JException e) {
+        				   
+        			   }
+        			   catch (IOException e) {
+
+        			   }
+        		   }
+        	   }
+           }
            if(imageData==null) {
 
-               // we will create an external link to the graphic
+        	   // we will create an external link to the graphic
                Part imagePart = ctm.newPartForContentType(contentType, BinaryPartAbstractImage.createImageName(operationDocument.getPackage(), part, proposedRelId, "ext"), null);
                rel = part.addTargetPart(imagePart);
                rel.setTargetMode("External");
@@ -616,7 +619,7 @@ final public class Commons {
            }
            else {
 
-               // the graphic is embedded and will be inserted added to the zip archive
+        	   // the graphic is embedded and will be inserted added to the zip archive
                Part imagePart = ctm.newPartForContentType(contentType, "/" + imageUrl, null);
                if(imagePart instanceof BinaryPart)
                    ((BinaryPart)imagePart).setBinaryData(imageData);
