@@ -49,9 +49,11 @@
 
 package com.openexchange.office.realtime.impl;
 
+import javax.annotation.concurrent.NotThreadSafe;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import com.openexchange.office.tools.StatusProperties;
 
 /**
  * {@link ConnectionStatus}
@@ -65,40 +67,32 @@ import org.json.JSONObject;
  * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
  * @author <a href="mailto:carsten.driesner@open-xchange.com">Carsten Driesner</a>
  */
-public class ConnectionStatus {
-
-    static final String KEY_EDIT_USER = "editUser";
-
-    static final String KEY_EDIT_USERID = "editUserId";
-
-    static final String KEY_HAS_ERRORS = "hasErrors";
-
-    static final String KEY_WRITE_PROTECTED = "writeProtected";
-
-    static final String KEY_LOCKED = "locked";
-
-    static final String KEY_LOCKED_BY_USER = "lockedByUser";
-
-    static final String KEY_FAILSAFESAVEDONE = "failSafeSaveDone";
-
-    static final String KEY_WANTSEDIT_USER = "wantsToEditUser";
-
-    static final String KEY_WANTSEDIT_USERID = "wantsToEditUserId";
-
-    static final String KEY_SERVER_OSN = "serverOSN";
-
-    static final String KEY_ACTIVE_USERS = "activeUsers";
-
-    static final String KEY_LASTMODIFYING_USERID = "lastModifyingUserId";
-
-    static final String KEY_FINALLOADUPDATE = "finalLoadUpdate";
-
-    static final String KEY_SYNCLOAD = "syncLoad";
-
-    static final String KEY_ERRORCODE = "error";
+@NotThreadSafe
+public class ConnectionStatus implements Cloneable {
+    public static final String EMPTY_USERID = "";
+    public static final String EMPTY_USERNAME = "";
+    public static final int NO_USERID = -1;
 
+    static final org.apache.commons.logging.Log LOG = com.openexchange.log.LogFactory.getLog(ConnectionStatus.class);
     static final long INACTIVITY_TRESHOLD_TIME = 25 * 60; // 25 minutes, stored as seconds
 
+    // - Members ---------------------------------------------------------------
+
+    private boolean m_hasErrors = false;
+    private boolean m_locked = false;
+    private boolean m_finalLoad = false;
+    private boolean m_syncLoad = false;
+    private String m_currentEditingUserName = EMPTY_USERNAME;
+    private String m_currentEditingUserId = EMPTY_USERID;
+    private String m_wantsEditRightsUserId = EMPTY_USERID;
+    private String m_wantsEditRightsUserName = EMPTY_USERNAME;
+    private int m_operationStateNumber = 0;
+    private String m_lockedByUser = "";
+    private int m_lockedById = NO_USERID;
+    private boolean m_failSafeSaveDone = false;
+    private JSONArray m_activeUsers = new JSONArray();
+    private Boolean m_optWriteProtected = null;
+
     // - Implementation --------------------------------------------------------
 
     /**
@@ -113,7 +107,7 @@ public class ConnectionStatus {
      *
      * @return The m_hasErrors
      */
-    boolean isHasErrors() {
+    public boolean isHasErrors() {
         return m_hasErrors;
     }
 
@@ -122,16 +116,28 @@ public class ConnectionStatus {
      *
      * @param hasErrors The m_hasErrors to set
      */
-    void setHasErrors(boolean hasErrors) {
+    public void setHasErrors(boolean hasErrors) {
         this.m_hasErrors = hasErrors;
     }
 
+    /**
+     * Sets a temporary write protected state.
+     * ATTENTION: This must be a temporary state as the read/write
+     * state depends on the client accessing the file. Don't use it
+     * to modify the global connection status!!
+     *
+     * @param isWriteProtected
+     */
+    public void setWriteProtected(boolean isWriteProtected) {
+        this.m_optWriteProtected = new Boolean(isWriteProtected);
+    }
+
     /**
      * Get the lock state
      *
      * @return The current lock state
      */
-    boolean isLocked() {
+    public boolean isLocked() {
         return m_locked;
     }
 
@@ -140,7 +146,7 @@ public class ConnectionStatus {
      *
      * @param locked The new lock state
      */
-    void setLockState(boolean locked) {
+    public void setLockState(boolean locked) {
         this.m_locked = locked;
     }
 
@@ -151,7 +157,7 @@ public class ConnectionStatus {
      * or empty string if user is unknown or nobody set
      * a lock.
      */
-    String getLockedByUser() {
+    public String getLockedByUser() {
         return m_lockedByUser;
     }
 
@@ -162,26 +168,28 @@ public class ConnectionStatus {
      * or empty string if unknown or nobody set
      * a lock.
      */
-    void setLockedByUser(String lockedByUser) {
+    public void setLockedByUser(String lockedByUser) {
         this.m_lockedByUser = lockedByUser;
     }
 
     /**
-     * Gets the m_isWriteProtected
+     * Get the user id of the user who locked the file.
      *
-     * @return The m_isWriteProtected
+     * @return The id of the user who locked the file. NO_USERID, if no
+     * user locked the file.
      */
-    boolean isWriteProtected() {
-        return m_isWriteProtected;
+    public int getLockedByUserId() {
+        return this.m_lockedById;
     }
 
     /**
-     * Sets the m_isWriteProtected
+     * Sets the user id of the user who locked the file.
      *
-     * @param isWriteProtected The isWriteProtected to set
+     * @param userId The id of the user who locked the file. NO_USERID, if no
+     * user locked the file.
      */
-    void setWriteProtected(boolean isWriteProtected) {
-        this.m_isWriteProtected = isWriteProtected;
+    public void setLockedByUserId(int userId) {
+        this.m_lockedById = userId;
     }
 
     /**
@@ -189,7 +197,7 @@ public class ConnectionStatus {
      *
      * @return The m_currentEditingUserName
      */
-    void setCurrentEditingUser(String currentEditingUserId, String currentEditingUserName) {
+    public void setCurrentEditingUser(String currentEditingUserId, String currentEditingUserName) {
         m_currentEditingUserId = currentEditingUserId;
         m_currentEditingUserName = currentEditingUserName;
     }
@@ -199,7 +207,7 @@ public class ConnectionStatus {
      *
      * @return The m_currentEditingUserName
      */
-    String getCurrentEditingUserId() {
+    public String getCurrentEditingUserId() {
         return m_currentEditingUserId;
     }
 
@@ -208,14 +216,14 @@ public class ConnectionStatus {
      *
      * @param currentEditingUserName The currentEditingUserName to set
      */
-    String getCurrentEditingUserName() {
+    public String getCurrentEditingUserName() {
         return m_currentEditingUserName;
     }
 
     /**
      * Gets the user id who wants to receive the edit rights
      */
-    String getWantsEditRightsUserId() {
+    public String getWantsEditRightsUserId() {
     	return m_wantsEditRightsUserId;
     }
 
@@ -223,7 +231,7 @@ public class ConnectionStatus {
      * Gets the user name who wants to receive the edit rights
      * @return
      */
-    String getWantsEditRightsUserName() {
+    public String getWantsEditRightsUserName() {
     	return m_wantsEditRightsUserName;
     }
 
@@ -232,7 +240,7 @@ public class ConnectionStatus {
      *
      * @param userId
      */
-    void setWantsEditRightsUser(String userId, String userName) {
+    public void setWantsEditRightsUser(String userId, String userName) {
     	m_wantsEditRightsUserId = userId;
     	m_wantsEditRightsUserName = userName;
     }
@@ -242,7 +250,7 @@ public class ConnectionStatus {
      *
      * @return The number of active clients.
      */
-    int getActiveClients() {
+    public int getActiveClients() {
         return (null != m_activeUsers) ? m_activeUsers.length() : 0;
     }
 
@@ -251,7 +259,7 @@ public class ConnectionStatus {
      *
      * @return The m_operationStateNumber
      */
-    int getOperationStateNumber() {
+    public int getOperationStateNumber() {
         return this.m_operationStateNumber;
     }
 
@@ -260,7 +268,7 @@ public class ConnectionStatus {
      *
      * @param operationStateNumber The operation state number to set
      */
-    void setOperationStateNumber(int operationStateNumber) {
+    public void setOperationStateNumber(int operationStateNumber) {
         this.m_operationStateNumber = operationStateNumber;
     }
 
@@ -272,7 +280,7 @@ public class ConnectionStatus {
      *  TRUE if there recently was a successful
      *  fail safe save operation, otherwise FALSE.
      */
-    boolean isFailSafeSaveDone() {
+    public boolean isFailSafeSaveDone() {
         return m_failSafeSaveDone;
     }
 
@@ -307,19 +315,18 @@ public class ConnectionStatus {
     /**
      * Add a user to the activeUsers map
      *
-     * @param userId
-     *  ID of the user
-     * @param userDisplayName
-     *  The UI name of the new user
-     * @param userData
-     *  JSON Object containing user data, e.g. {userName:'Rocky Balboa', selection:{...}, ...}
+     * @param userId ID of the user
+     * @param userDisplayName The UI name of the new user
+     * @param guest Specifies, if the user is a guest or not
+     * @param userData JSON Object containing user data, e.g. {userName:'Rocky Balboa', selection:{...}, ...}
      * @throws JSONException
      */
-    public void addActiveUser(String userId, String userDisplayName, int id, JSONObject userData){
+    public void addActiveUser(String userId, String userDisplayName, int id, boolean guest, JSONObject userData){
         JSONObject userObject = new JSONObject();
         try {
             userObject.put("userId", userId);
             userObject.put("userDisplayName", userDisplayName);
+            userObject.put("guest", guest);
             userObject.put("userData", userData);
             userObject.put("active", true);
             userObject.put("id", id);
@@ -628,31 +635,6 @@ public class ConnectionStatus {
         }
     }
 
-    /**
-     * Provides the userid that made the latest change to the
-     * document.
-     *
-     * @return
-     *  The userid or an empty string if no modification has been
-     *  done yet.
-     */
-    public String getLastModifyingUserId() {
-        return m_lastModifyingUserId;
-    }
-
-    /**
-     * Set the userid of the user who made the latest change to
-     * the document.
-     *
-     * @param userId
-     *  The userid as a string. Must not be null or empty.
-     */
-    public void setLastModifyingUserId(String userId) {
-        if (null != userId) {
-            m_lastModifyingUserId = userId;
-        }
-    }
-
     /**
      * Sets the 'finalLoad' attribute, which notifies the client that the
      * asynchronous loading process is finished and the 'update' with
@@ -705,16 +687,16 @@ public class ConnectionStatus {
      *  A shallow clone.
      */
     @Override
-    public Object clone() {
+    public ConnectionStatus clone() {
         ConnectionStatus clone = new ConnectionStatus();
         clone.setCurrentEditingUser(m_currentEditingUserId, m_currentEditingUserName);
         clone.setFailSafeSaveDone(m_failSafeSaveDone);
         clone.setHasErrors(m_hasErrors);
         clone.setLockedByUser(m_lockedByUser);
         clone.setLockState(m_locked);
+        clone.setLockedByUserId(m_lockedById);
         clone.setOperationStateNumber(m_operationStateNumber);
         clone.setWantsEditRightsUser(m_wantsEditRightsUserId, m_wantsEditRightsUserName);
-        clone.setWriteProtected(m_isWriteProtected);
         JSONArray users = new JSONArray();
         try {
 	        for (int i = 0; i < m_activeUsers.length(); i++) {
@@ -727,7 +709,6 @@ public class ConnectionStatus {
         	LOG.debug("Exception while cloning ConnectionStatus", e);
         }
         clone.setActiveUsers(users);
-        clone.setLastModifyingUserId(m_lastModifyingUserId);
         return clone;
     }
 
@@ -745,24 +726,27 @@ public class ConnectionStatus {
         final JSONObject ret = new JSONObject();
 
         try {
-            ret.put(KEY_HAS_ERRORS, m_hasErrors);
-            ret.put(KEY_WRITE_PROTECTED, m_isWriteProtected);
-            ret.put(KEY_EDIT_USERID, m_currentEditingUserId);
-            ret.put(KEY_EDIT_USER, m_currentEditingUserName);
-            ret.put(KEY_LOCKED, m_locked);
-            ret.put(KEY_LOCKED_BY_USER, m_lockedByUser);
-            ret.put(KEY_FAILSAFESAVEDONE, m_failSafeSaveDone);
-            ret.put(KEY_WANTSEDIT_USER, m_wantsEditRightsUserName);
-            ret.put(KEY_WANTSEDIT_USERID, m_wantsEditRightsUserId);
-            ret.put(KEY_SERVER_OSN, m_operationStateNumber);
+            ret.put(StatusProperties.KEY_HAS_ERRORS, m_hasErrors);
+            ret.put(StatusProperties.KEY_EDIT_USERID, m_currentEditingUserId);
+            ret.put(StatusProperties.KEY_EDIT_USER, m_currentEditingUserName);
+            ret.put(StatusProperties.KEY_LOCKED, m_locked);
+            ret.put(StatusProperties.KEY_LOCKED_BY_USER, m_lockedByUser);
+            ret.put(StatusProperties.KEY_FAILSAFESAVEDONE, m_failSafeSaveDone);
+            ret.put(StatusProperties.KEY_WANTSEDIT_USER, m_wantsEditRightsUserName);
+            ret.put(StatusProperties.KEY_WANTSEDIT_USERID, m_wantsEditRightsUserId);
+            ret.put(StatusProperties.KEY_SERVER_OSN, m_operationStateNumber);
+            ret.put(StatusProperties.KEY_LOCKED_BY_ID, m_lockedById);
+            if (null != m_optWriteProtected) {
+                ret.put(StatusProperties.KEY_WRITE_PROTECTED, m_optWriteProtected);
+            }
             if (m_finalLoad) {
-            	ret.put(KEY_FINALLOADUPDATE, m_finalLoad);
+                ret.put(StatusProperties.KEY_FINALLOADUPDATE, m_finalLoad);
             }
             if (m_syncLoad) {
-            	ret.put(KEY_SYNCLOAD, m_syncLoad);
+                ret.put(StatusProperties.KEY_SYNCLOAD, m_syncLoad);
             }
             // send all collaboration user (client must filter/process list)
-            ret.put(KEY_ACTIVE_USERS, m_activeUsers);
+            ret.put(StatusProperties.KEY_ACTIVE_USERS, m_activeUsers);
         } catch (JSONException e) {
             LOG.error("Exception while creating JSON object of the current connection status", e);
         }
@@ -786,53 +770,53 @@ public class ConnectionStatus {
         boolean found = false;
 
         try {
-            if (jsonData.has(KEY_HAS_ERRORS)) {
-                queueStatus.m_hasErrors = jsonData.getBoolean(KEY_HAS_ERRORS);
+            if (jsonData.has(StatusProperties.KEY_HAS_ERRORS)) {
+                queueStatus.m_hasErrors = jsonData.getBoolean(StatusProperties.KEY_HAS_ERRORS);
                 found = true;
             }
 
-            if (jsonData.has(KEY_WRITE_PROTECTED)) {
-                queueStatus.m_isWriteProtected = jsonData.getBoolean(KEY_WRITE_PROTECTED);
+            if (jsonData.has(StatusProperties.KEY_EDIT_USERID)) {
+                queueStatus.m_currentEditingUserId = jsonData.getString(StatusProperties.KEY_EDIT_USERID);
                 found = true;
             }
 
-            if (jsonData.has(KEY_EDIT_USERID)) {
-                queueStatus.m_currentEditingUserId = jsonData.getString(KEY_EDIT_USERID);
+            if (jsonData.has(StatusProperties.KEY_EDIT_USER)) {
+                queueStatus.m_currentEditingUserName = jsonData.getString(StatusProperties.KEY_EDIT_USER);
                 found = true;
             }
 
-            if (jsonData.has(KEY_EDIT_USER)) {
-                queueStatus.m_currentEditingUserName = jsonData.getString(KEY_EDIT_USER);
+            if (jsonData.has(StatusProperties.KEY_LOCKED)) {
+                queueStatus.m_locked = jsonData.getBoolean(StatusProperties.KEY_LOCKED);
                 found = true;
             }
 
-            if (jsonData.has(KEY_LOCKED)) {
-                queueStatus.m_locked = jsonData.getBoolean(KEY_LOCKED);
+            if (jsonData.has(StatusProperties.KEY_LOCKED_BY_USER)) {
+                queueStatus.m_lockedByUser = jsonData.getString(StatusProperties.KEY_LOCKED_BY_USER);
                 found = true;
             }
 
-            if (jsonData.has(KEY_LOCKED_BY_USER)) {
-                queueStatus.m_lockedByUser = jsonData.getString(KEY_LOCKED_BY_USER);
+            if (!jsonData.has(StatusProperties.KEY_FAILSAFESAVEDONE)) {
+                queueStatus.m_failSafeSaveDone = jsonData.getBoolean(StatusProperties.KEY_FAILSAFESAVEDONE);
                 found = true;
             }
 
-            if (!jsonData.has(KEY_FAILSAFESAVEDONE)) {
-                queueStatus.m_failSafeSaveDone = jsonData.getBoolean(KEY_FAILSAFESAVEDONE);
+            if (!jsonData.has(StatusProperties.KEY_WANTSEDIT_USER)) {
+                queueStatus.m_wantsEditRightsUserName = jsonData.getString(StatusProperties.KEY_WANTSEDIT_USER);
                 found = true;
             }
 
-            if (!jsonData.has(KEY_WANTSEDIT_USER)) {
-                queueStatus.m_wantsEditRightsUserName = jsonData.getString(KEY_WANTSEDIT_USER);
+            if (!jsonData.has(StatusProperties.KEY_WANTSEDIT_USERID)) {
+                queueStatus.m_wantsEditRightsUserId = jsonData.getString(StatusProperties.KEY_WANTSEDIT_USERID);
                 found = true;
             }
 
-            if (!jsonData.has(KEY_WANTSEDIT_USERID)) {
-                queueStatus.m_wantsEditRightsUserId = jsonData.getString(KEY_WANTSEDIT_USERID);
+            if (!jsonData.has(StatusProperties.KEY_ACTIVE_USERS)) {
+                queueStatus.m_activeUsers = jsonData.getJSONArray(StatusProperties.KEY_ACTIVE_USERS);
                 found = true;
             }
 
-            if (!jsonData.has(KEY_ACTIVE_USERS)) {
-                queueStatus.m_activeUsers = jsonData.getJSONArray(KEY_ACTIVE_USERS);
+            if (!jsonData.has(StatusProperties.KEY_LOCKED_BY_ID)) {
+                queueStatus.m_lockedById = jsonData.getInt(StatusProperties.KEY_LOCKED_BY_ID);
                 found = true;
             }
         } catch (JSONException e) {
@@ -842,21 +826,19 @@ public class ConnectionStatus {
         return (found ? queueStatus : null);
     }
 
-    // - Members ---------------------------------------------------------------
+    static public boolean isEmptyUser(final String user) {
+        return ((user != null) && EMPTY_USERID.equals(user));
+    }
 
-    static protected final org.apache.commons.logging.Log LOG = com.openexchange.log.LogFactory.getLog(ConnectionStatus.class);
-    private boolean m_hasErrors = false;
-    private boolean m_isWriteProtected = false;
-    private boolean m_locked = false;
-    private boolean m_finalLoad = false;
-    private boolean m_syncLoad = false;
-    private String m_currentEditingUserName = "";
-    private String m_currentEditingUserId = "";
-    private String m_wantsEditRightsUserId = "";
-    private String m_wantsEditRightsUserName = "";
-    private String m_lastModifyingUserId = "";
-    private int m_operationStateNumber = 0;
-    private String m_lockedByUser = "";
-    private boolean m_failSafeSaveDone = false;
-    private JSONArray m_activeUsers = new JSONArray();
+    static public boolean isEmptyUserName(final String userName) {
+        return ((userName != null) && EMPTY_USERNAME.equals(userName));
+    }
+
+    static public boolean isNotEmptyUser(final String user) {
+        return ((user != null) && !(EMPTY_USERID.equals(user)));
+    }
+
+    static public boolean isNotEmptyUserName(final String userName) {
+        return ((userName != null) && !(EMPTY_USERNAME.equals(userName)));
+    }
 }
