@@ -49,10 +49,19 @@
 
 package com.openexchange.office.realtime.impl;
 
+import org.apache.commons.logging.Log;
 import org.json.JSONException;
 import org.json.JSONObject;
 
-
+/**
+ * Contains data that is needed for synchronization purposes between
+ * client and server.
+ *
+ * {@link Sync}
+ *
+ * @author <a href="mailto:carsten.driesner@open-xchange.com">Carsten Driesner</a>
+ * @since v7.6.2
+ */
 public class Sync {
 
     private final int SYNC_UNKNOWN_DOCOSN = -1;
@@ -65,7 +74,17 @@ public class Sync {
 
     public static final String SYNC_INFO = "syncInfo";
 
-    public Sync(String folderId, String fileId) {
+    // members
+    private int m_documentOperationStateNumber = SYNC_UNKNOWN_DOCOSN;
+    private String m_version;
+    private final String m_folderId;
+    private final String m_fileId;
+    private String m_fileName;
+    private boolean m_supportsVersions = true;
+    private final Log m_log;
+
+    public Sync(final Log log, final String folderId, final String fileId) {
+        m_log = log;
         m_folderId = folderId;
         m_fileId = fileId;
         m_supportsVersions = true;
@@ -96,28 +115,20 @@ public class Sync {
         m_fileName = fileName;
     }
 
-    public JSONObject toJSON() throws JSONException {
-        JSONObject jsonObject = new JSONObject();
+    public JSONObject toJSON() {
+        final JSONObject jsonObject = new JSONObject();
 
-        jsonObject.put(SYNC_FOLDERID, m_folderId);
-        jsonObject.put(SYNC_FILEID, m_fileId);
-        jsonObject.put(SYNC_FILEVERSION, (m_version == null) ? JSONObject.NULL : m_version);
-        jsonObject.put(SYNC_DOCOSN, m_documentOperationStateNumber);
-        jsonObject.put(SYNC_SUPPORTSVERSIONS, m_supportsVersions);
-        jsonObject.put(SYNC_FILENAME, (m_fileName == null) ? JSONObject.NULL : m_fileName);
+        try {
+            jsonObject.put(SYNC_FOLDERID, m_folderId);
+            jsonObject.put(SYNC_FILEID, m_fileId);
+            jsonObject.put(SYNC_FILEVERSION, (m_version == null) ? JSONObject.NULL : m_version);
+            jsonObject.put(SYNC_DOCOSN, m_documentOperationStateNumber);
+            jsonObject.put(SYNC_SUPPORTSVERSIONS, m_supportsVersions);
+            jsonObject.put(SYNC_FILENAME, (m_fileName == null) ? JSONObject.NULL : m_fileName);
+        } catch (JSONException e) {
+            m_log.error("Exception catched while setting sync data", e);
+        }
 
         return jsonObject;
     }
-
-    private int m_documentOperationStateNumber = SYNC_UNKNOWN_DOCOSN;
-
-    private String m_version;
-
-    private final String m_folderId;
-
-    private final String m_fileId;
-
-    private String m_fileName;
-
-    private boolean m_supportsVersions = true;
 }
