@@ -52,30 +52,28 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
 import java.util.Queue;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.ExecutorService;
 import java.util.concurrent.atomic.AtomicBoolean;
-
-import com.openexchange.config.ConfigurationService;
-import com.openexchange.office.realtime.impl.Connection;
 import com.openexchange.server.ServiceLookup;
-import com.openexchange.threadpool.ThreadPoolService;
 
+/**
+ * Encapsulates a request queue for requests regarding documents. The request
+ * queue awaits requests and tries to process them using threads/ThreadExecutor.
+ *
+ * @author Carsten Driesner
+ * @since 7.6.2
+ */
 public class AsyncLoadRequestQueue implements Runnable {
 
     static protected final org.apache.commons.logging.Log LOG = com.openexchange.log.LogFactory.getLog(AsyncLoadRequestQueue.class);
 
 	private ServiceLookup services = null;
-    private Queue<Request> requestQueue = new LinkedList<Request>();
-    private HashMap<String, Request> runningRequests = new HashMap<String, Request>();
-    private AtomicBoolean bTerminate = new AtomicBoolean(false);
-    private Boolean queueNotEmpty = false;
+    private final Queue<Request> requestQueue = new LinkedList<Request>();
+    private final HashMap<String, Request> runningRequests = new HashMap<String, Request>();
+    private final AtomicBoolean bTerminate = new AtomicBoolean(false);
+    private final Boolean queueNotEmpty = false;
     private RequestExecutor requestExecutor = null;
-    private AtomicBoolean bStarted = new AtomicBoolean(false);
+    private final AtomicBoolean bStarted = new AtomicBoolean(false);
     private Thread queueThread = null;
 
     public AsyncLoadRequestQueue(ServiceLookup services) {
@@ -83,6 +81,10 @@ public class AsyncLoadRequestQueue implements Runnable {
 		requestExecutor = new RequestExecutor(services);
 	}
 
+    public boolean isStarted() {
+        return this.bStarted.get();
+    }
+
     /**
      * Enables the queue to process requests asynchronously via background
      * threads. This method does nothing if call more than once. A queue
@@ -105,7 +107,7 @@ public class AsyncLoadRequestQueue implements Runnable {
     public void destroyQueue() {
 	    boolean terminated = bTerminate.getAndSet(true);
 	    boolean started = bStarted.get();
-		
+
 		try {
 			if (started && !terminated) {
 				synchronized(requestQueue) {
@@ -119,7 +121,7 @@ public class AsyncLoadRequestQueue implements Runnable {
 			}
 		} catch (InterruptedException e) {
             LOG.debug("InterruptedException preparing shutdown of the CalcLoadRequestQueue", e);
-		} 
+		}
 	}
 
 	/**
@@ -130,7 +132,7 @@ public class AsyncLoadRequestQueue implements Runnable {
 	 *  threads.
 	 */
     public void putRequest(final Request request) throws IllegalStateException {
-    	
+
 		if (bStarted.get() && !bTerminate.get()) {
 			synchronized(requestQueue) {
 			    requestQueue.add(request);
@@ -164,7 +166,7 @@ public class AsyncLoadRequestQueue implements Runnable {
   	    	requestQueue.removeAll(requestsToBeRemoved);
     	}
     }
-    
+
 	/**
 	 * Central run method to be executed by a special thread to
 	 * process the Requests put into the CalcLoadRequestQueue.
@@ -195,7 +197,7 @@ public class AsyncLoadRequestQueue implements Runnable {
 						}
 					}
 				}
-				
+
 				if (null == requestToBeProcessed) {
 					synchronized(queueNotEmpty) {
 					    queueNotEmpty.wait(100);
@@ -206,7 +208,8 @@ public class AsyncLoadRequestQueue implements Runnable {
 					// set new request as running
 					runningRequests.put(requestToBeProcessed.getDocumentId(), requestToBeProcessed);
 					requestExecutor.execute(new Runnable() {
-						public void run() {
+						@Override
+                        public void run() {
 							try {
 								IRequestProcessor processor = processRequest.getProcessor();
 								if (null != processor) {
@@ -245,14 +248,14 @@ public class AsyncLoadRequestQueue implements Runnable {
 	    	runningRequests.remove(id);
 	    }
 	}
-	
+
 	/**
 	 * Determines if a request for a certain document is currently running or
 	 * not. This method is NOT thread-safe and must only be called from the run()
 	 * method!
 	 *
 	 * @param docId
-	 *  The unique document identifier, consists of the <folderid>.<fileid>
+	 *  The unique document identifier, consists of the document resource id.
 	 *
 	 * @return
 	 *  TRUE if there is a request running, otherwise FALSE.
