@@ -50,18 +50,19 @@
 package com.openexchange.office.realtime.impl.calc;
 
 import java.util.concurrent.atomic.AtomicBoolean;
-
 import org.json.JSONArray;
 import org.json.JSONObject;
-
-import com.openexchange.office.realtime.impl.OXDocument;
-import com.openexchange.office.tools.DocumentMetaData;
+import com.openexchange.office.realtime.doc.OXDocument;
+import com.openexchange.office.tools.doc.DocumentMetaData;
 import com.openexchange.realtime.packet.ID;
 
 /**
- * Encapsulates an asynchronous load request to be processed by a request processor.
+ * Encapsulates an asynchronous load request to be processed by a request processor. It
+ * provides a isFinished method which can be used to determine, if the request has been
+ * processed.
  *
  * @author Carsten Driesner
+ * @since 7.6.2
  */
 public class Request {
 
@@ -77,9 +78,7 @@ public class Request {
 
     private int documentOSN = 0;
 
-    private String folderId = null;
-
-    private String fileId = null;
+    private String rtResourceID = null;
 
     private DocumentMetaData docMetaData = null;
 
@@ -100,10 +99,9 @@ public class Request {
      * @param processor
      * @param documentOSN
      */
-    public Request(String folderId, String fileId, ID fromId, IRequestProcessor processor, int documentOSN, final DocumentMetaData docMetaData) {
+    public Request(final String rtResourceID, final ID fromId, IRequestProcessor processor, int documentOSN, final DocumentMetaData docMetaData) {
         this.type = Type.REQUEST_LOAD_SYNC;
-        this.folderId = folderId;
-        this.fileId = fileId;
+        this.rtResourceID = rtResourceID;
         this.fromId = fromId;
         this.processor = processor;
         this.documentOSN = documentOSN;
@@ -123,10 +121,9 @@ public class Request {
      * @param documentOSN
      * @throws Exception
      */
-    public Request(String folderId, String fileId, ID fromId, final JSONArray operations, IRequestProcessor processor, int documentOSN, final DocumentMetaData docMetaData) throws Exception {
+    public Request(final String rtResourceID, final ID fromId, final JSONArray operations, IRequestProcessor processor, int documentOSN, final DocumentMetaData docMetaData) throws Exception {
         this.type = Type.REQUEST_LOAD;
-        this.folderId = folderId;
-        this.fileId = fileId;
+        this.rtResourceID = rtResourceID;
         this.fromId = fromId;
         this.processor = processor;
         this.documentOSN = documentOSN;
@@ -147,10 +144,9 @@ public class Request {
      * @param documentOSN
      * @throws Exception
      */
-    public Request(String folderId, String fileId, ID fromId, final OXDocument documentLoader, IRequestProcessor processor, int documentOSN, int previewUIOps) throws Exception {
+    public Request(final String rtResourceID, final ID fromId, final OXDocument documentLoader, IRequestProcessor processor, int documentOSN, int previewUIOps) throws Exception {
         this.type = Type.REQUEST_LOAD_REMAINING;
-        this.folderId = folderId;
-        this.fileId = fileId;
+        this.rtResourceID = rtResourceID;
         this.fromId = fromId;
         this.processor = processor;
         this.documentOSN = documentOSN;
@@ -220,7 +216,7 @@ public class Request {
      * @return
      */
     public String getDocumentId() {
-        return new StringBuilder(folderId).append(".").append(fileId).toString();
+        return rtResourceID;
     }
 
     /**
@@ -262,11 +258,10 @@ public class Request {
     /**
      * Creates the document id.
      *
-     * @param folderId
-     * @param fileId
+     * @param resourceId the resource id from the connection id
      * @return
      */
-    static public String buildDocumentId(final String folderId, final String fileId) {
-        return new StringBuilder(folderId).append(".").append(fileId).toString();
+    static public String buildDocumentId(final String resourceId) {
+        return new StringBuilder("document:" + resourceId).toString();
     }
 }
