@@ -65,7 +65,12 @@ import com.openexchange.folderstorage.FolderService;
 import com.openexchange.jslob.JSlobService;
 import com.openexchange.office.IResourceProvider;
 import com.openexchange.office.calcengine.client.CalcEngineClientFactory;
+import com.openexchange.office.hazelcast.doc.DocumentDirectory;
+import com.openexchange.office.realtime.impl.ConnectionBackgroundProcessorManager;
 import com.openexchange.office.realtime.impl.ConnectionComponent;
+import com.openexchange.office.realtime.impl.FailSafeSaveProcessor;
+import com.openexchange.office.realtime.impl.calc.CalcComponent;
+import com.openexchange.office.realtime.impl.text.TextComponent;
 import com.openexchange.office.realtime.tools.JSON2MessageData;
 import com.openexchange.office.realtime.tools.MessageData2JSON;
 import com.openexchange.osgi.HousekeepingActivator;
@@ -79,6 +84,8 @@ import com.openexchange.user.UserService;
 //The activator does the service wiring, as usual.
 public class ConnectionActivator extends HousekeepingActivator {
 
+    private ConnectionBackgroundProcessorManager m_backgroundProcesses = null;
+
     @Override
     protected Class<?>[] getNeededServices() {
         return new Class<?>[] {
@@ -105,7 +112,8 @@ public class ConnectionActivator extends HousekeepingActivator {
             GlobalRealtimeCleanup.class,
             JSlobService.class,
             ThreadPoolService.class,
-            ContextService.class
+            ContextService.class,
+            DocumentDirectory.class
         };
     }
 
@@ -124,24 +132,38 @@ public class ConnectionActivator extends HousekeepingActivator {
      */
     @Override
     protected void startBundle() throws Exception {
-        final Log log = com.openexchange.log.Log.loggerFor(ConnectionActivator.class);
-
         Services.setServiceLookup(this);
 
-        log.info("starting bundle: " + m_serviceName);
-
-        registerService(Component.class, component = new ConnectionComponent(this));
-        // Register the Component as RealtimeJanitor that can be tracked via OSGI and used when a cleanup is needed
-        registerService(RealtimeJanitor.class, component, component.getServiceProperties());
-        registerService(SimplePayloadConverter.class, new MessageData2JSON());
-        registerService(SimplePayloadConverter.class, new JSON2MessageData());
+        LOG.info("starting bundle: " + m_serviceName);
 
+        // Register the components
+        registerService(Component.class, textComponent = new TextComponent(this));
+        registerService(Component.class, calcComponent = new CalcComponent(this));
+        // Register the components as RealtimeJanitor that can be tracked via OSGI and used when a cleanup is needed
+        registerService(RealtimeJanitor.class, textComponent, textComponent.getServiceProperties());
+        registerService(RealtimeJanitor.class, calcComponent, calcComponent.getServiceProperties());
         // You have to provide these converters if you want to put non JSON classes into
         // Payload trees.
-
         // Also, if you want to receive non-json data, which is usually very convenient, you
         // can declare that a payload with a certain namespace and element name should always be
         // Converted into a specific format. For that see the PayloadTreeConverter interface
+        registerService(SimplePayloadConverter.class, new MessageData2JSON());
+        registerService(SimplePayloadConverter.class, new JSON2MessageData());
+
+        // Register document specific background processor dependent on
+        // configuration entries.
+        final ConfigurationService configuration = this.getService(com.openexchange.config.ConfigurationService.class);
+        boolean backgroundSave = configuration.getBoolProperty("io.ox/office//module/backgroundSaveDoc", true);
+        LOG.debug("Background document save = " + backgroundSave);
+        if (backgroundSave) {
+            m_backgroundProcesses = new ConnectionBackgroundProcessorManager(this);
+            final FailSafeSaveProcessor failSafeSaveProcessor = new FailSafeSaveProcessor(this);
+            m_backgroundProcesses.registerProcessor(failSafeSaveProcessor);
+            // only add connection pool if we know that at least one processor has been added
+            m_backgroundProcesses.addConnectionPool(textComponent);
+            m_backgroundProcesses.addConnectionPool(calcComponent);
+            m_backgroundProcesses.start();
+        }
 
         openTrackers();
     }
@@ -150,12 +172,16 @@ public class ConnectionActivator extends HousekeepingActivator {
     protected void stopBundle() throws Exception {
         if (!shutdownProcessed) {
 
-            final Log log = com.openexchange.log.Log.loggerFor(ConnectionActivator.class);
+            LOG.info("stopping bundle: " + m_serviceName);
 
-            log.info("stopping bundle: " + m_serviceName);
-
-            if (null != component) {
-                component.shutdown();
+            if (null != m_backgroundProcesses) {
+                m_backgroundProcesses.shutdown();
+            }
+            if (null != textComponent) {
+                textComponent.shutdown();
+            }
+            if (null != calcComponent) {
+                calcComponent.shutdown();
             }
 
             Services.setServiceLookup(null);
@@ -172,17 +198,15 @@ public class ConnectionActivator extends HousekeepingActivator {
      * @see com.openexchange.osgi.HousekeepingActivator#handleUnavailability()
      */
     protected void handleUnavailability(final Class<?> clazz) {
-        final Log log = com.openexchange.log.Log.loggerFor(ConnectionActivator.class);
-
         lock.lock();
         try {
             if (!shutdownProcessed) {
-                log.info("bundle: " + m_serviceName + " detected that a required service was removed.");
+                LOG.info("bundle: " + m_serviceName + " detected that a required service was removed.");
                 stopBundle();
                 shutdownProcessed = true;
             }
         } catch (Exception e) {
-            log.error(e.getMessage(), e);
+            LOG.error(e.getMessage(), e);
         } finally {
             lock.unlock();
         }
@@ -190,11 +214,15 @@ public class ConnectionActivator extends HousekeepingActivator {
 
     // - Members ---------------------------------------------------------------
 
+    private final Log LOG = com.openexchange.log.Log.loggerFor(ConnectionActivator.class);
+
     private final String m_serviceName = "com.openexchange.office.realtime";
 
     private boolean shutdownProcessed = false;
 
     private final Lock lock = new ReentrantLock();
 
-    private ConnectionComponent component = null;
+    private ConnectionComponent textComponent = null;
+
+    private ConnectionComponent calcComponent = null;
 }
