@@ -1,7 +1,7 @@
 package com.openexchange.office.tools.htmldoc;
 
 import java.util.List;
-
+import org.apache.commons.lang.StringEscapeUtils;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -54,7 +54,9 @@ public class GenDocHelper
         if (jquerydata.length() > 0)
         {
             document.append(" jquerydata='");
-            document.append(jquerydata.toString());
+            String str = StringEscapeUtils.escapeHtml(jquerydata.toString());
+            str = str.replace("'", "&#39;"); // escaping all single quotes (49847)
+            document.append(str);
             document.append("' ");
         }
     }
