@@ -60,7 +60,13 @@ import org.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.openexchange.config.ConfigurationService;
+import com.openexchange.file.storage.File;
+import com.openexchange.office.tools.UserConfigurationHelper;
+import com.openexchange.office.tools.UserConfigurationHelper.Mode;
 import com.openexchange.office.tools.json.JSONValueComparator;
+import com.openexchange.server.ServiceLookup;
+import com.openexchange.session.Session;
 
 public class GenericHtmlDocumentBuilder
 {
@@ -83,17 +89,43 @@ public class GenericHtmlDocumentBuilder
         return LOG;
     }
 
+    private static UserConfigurationHelper getUserHelper(ServiceLookup services, Session session) {
+        return new UserConfigurationHelper(services, session, "io.ox/office", Mode.WRITE_BACK);
+    }
+
+    public static boolean isFastLoadActive(ServiceLookup services, Session session) {
+        ConfigurationService service = services.getService(ConfigurationService.class);
+        if(service.getBoolProperty("io.ox/office//module/debugavailable", false)){
+            return getUserHelper(services, session).getBoolean("text/useFastLoad", true);
+        } else {
+            return true;
+        }
+    }
+
     static public String buildHtmlDocument(
+        JSONObject documentOperations,
+        File file,
+        ServiceLookup services,
+        Session session)
+    {
+        final TextTableLimits textTableLimits = ConfHelper.getTextTableLimits(getUserHelper(services, session));
+
+        return buildHtmlDocument(documentOperations, file.getId(), file.getFolderId(), file.getVersion(), file.getFileName(), textTableLimits);
+    }
+
+    static private String buildHtmlDocument(
         JSONObject documentOperations,
         String fileId,
         String folderId,
         String fileVersion,
+        String fileName,
         TextTableLimits textTableLimits)
     {
 
         String result = null;
 
-        Map<String, NodeHolder> nodes = new HashMap<String, NodeHolder>();
+        Map<String, NodeHolder> headerFooterNodes = new HashMap<String, NodeHolder>();
+        Map<String, NodeHolder> commentNodes = new HashMap<String, NodeHolder>();
         DocumentNode doc = new DocumentNode();
 
         try
@@ -122,7 +154,7 @@ public class GenericHtmlDocumentBuilder
                     if (opName.equalsIgnoreCase("insertHeaderFooter"))
                     {
                         String id = operation.getString("id");
-                        nodes.put(id, new HeaderFooter(id, operation.getString("type")));
+                        headerFooterNodes.put(id, new HeaderFooter(id, operation.getString("type"), attrs));
                         continue;
                     }
 
@@ -134,7 +166,13 @@ public class GenericHtmlDocumentBuilder
                     }
                     else
                     {
-                        documentNode = nodes.get(target);
+                    	if (headerFooterNodes.get(target) != null) {
+                    		documentNode = headerFooterNodes.get(target);
+                    	} else if (commentNodes.get(target) != null) {
+                    		documentNode = commentNodes.get(target);
+                    	} else {
+                    		documentNode = doc; // Must not occur
+                    	}
                     }
 
                     if (null == start)
@@ -211,6 +249,27 @@ public class GenericHtmlDocumentBuilder
                             final int textIndex = start.getInt(start.length() - 1);
                             insert = new Field(textIndex, operation.optString("representation", ""), operation.optString("type", null));
                         }
+                        else if (opName.equalsIgnoreCase("insertComplexField"))
+                        {
+                            final int textIndex = start.getInt(start.length() - 1);
+                            String id = operation.getString("id");
+                            insert = new ComplexField(textIndex, id, operation.optString("instruction", ""));
+                        }
+                        else if (opName.equalsIgnoreCase("insertComment"))
+                        {
+                            String id = operation.getString("id");
+                            final int textIndex = start.getInt(start.length() - 1);
+                            commentNodes.put(id, new Comment(id, operation.optString("author"), operation.optString("uid"), operation.optString("date"), operation.optString("target")));
+                            insert = new CommentPlaceHolder(textIndex, id);
+                        }
+                        else if (opName.equalsIgnoreCase("insertRange"))
+                        {
+                            final int textIndex = start.getInt(start.length() - 1);
+                            final String id = operation.optString("id");
+                            final String type = operation.optString("type");
+                            final String pos = operation.optString("position");
+                            insert = new RangeMarker(textIndex, id, type, pos);
+                        }
                         else
                         {
                             LOG.warn(opName + " not implemented yet");
@@ -243,9 +302,10 @@ public class GenericHtmlDocumentBuilder
             doc.appendContent(mainDocument);
             res.put("mainDocument", mainDocument);
 
+            // handling header/footer string
             JSONObject headerFooter = new JSONObject();
 
-            for (Entry<String, NodeHolder> e : nodes.entrySet())
+            for (Entry<String, NodeHolder> e : headerFooterNodes.entrySet())
             {
                 final StringBuilder document = new StringBuilder();
 
@@ -256,6 +316,20 @@ public class GenericHtmlDocumentBuilder
 
             res.put("headerFooter", headerFooter);
 
+            // handling comments string
+            JSONObject allComments = new JSONObject();
+
+            for (Entry<String, NodeHolder> e : commentNodes.entrySet())
+            {
+                final StringBuilder document = new StringBuilder();
+
+                e.getValue().appendContent(document);
+
+                allComments.put(e.getKey(), document);
+            }
+
+            res.put("comments", allComments);
+
             // set result
             result = res.toString();
             documentOperations.put("operations", reducedOperations);
@@ -263,11 +337,11 @@ public class GenericHtmlDocumentBuilder
             // result = result.replace("'", "\"");
 
             LOG.debug("RT connection: fastLoad optimizations used for loading document: " + ((null != folderId) ? folderId : "") + "." + ((null != fileId) ? fileId : ""));
-            LOG.debug("RT connection: creating the generic html document took " + (System.currentTimeMillis() - time) + "ms");
+            LOG.trace("RT connection: TIME creating the generic html document took: " + (System.currentTimeMillis() - time) + "ms");
         }
         catch (final Exception e)
         {
-            LOG.warn("RT connection: Eexception while creating the generic html document string from operations.", e);
+            LOG.error("RT connection: Exception while creating the generic html document string from operations. docDetails: " + fileName + " fileId: " + fileId + " folderId: " + folderId + " fileVersion: " + fileVersion, e);
         }
         return result;
     }
