@@ -1,5 +1,6 @@
 package com.openexchange.office.tools.htmldoc;
 
+import org.json.JSONArray;
 import org.json.JSONObject;
 
 public class HeaderFooter
@@ -10,10 +11,16 @@ public class HeaderFooter
     private final String id;
 
     public HeaderFooter(
-            String id, String type)
+            String id, String type, JSONObject attrs)
+        throws Exception
     {
         this.type = type;
         this.id = id;
+        
+        if (null != attrs)
+        {
+            setAttribute(new JSONArray(), null, attrs);
+        }
 
     }
 
@@ -31,13 +38,10 @@ public class HeaderFooter
 
         document.append("<div class='inactive-selection " + type.split("_")[0] + " " + id + " " + type + "'");
 
-        JSONObject params = new JSONObject();
-        params.put("id", id);
-        params.put("type", type);
-
-        document.append(" jquerydata='");
-        document.append(params.toString());
-        document.append("' ");
+        final JSONObject adapter = GenDocHelper.getJQueryData(getAttribute(), false);
+        adapter.put("id", id);
+        adapter.put("type", type);
+        GenDocHelper.appendJQueryData(document, adapter);
 
         document.append(">");
 
