@@ -54,7 +54,7 @@ import org.json.JSONException;
 
 /**
  * A set of functions to support the work with JSONObjects and JSONArrays.
- * 
+ *
  * @author Carsten Driesner
  */
 public class JSONHelper {
@@ -62,7 +62,7 @@ public class JSONHelper {
     /**
      * Creates a shallow copy of the JSONArray, means that the function provides a new JSONArray with references to the entries within the
      * source array.
-     * 
+     *
      * @param array The source array which should be shallow cloned.
      * @return A new JSONArray containing the references to the old entries.
      * @throws Exception
@@ -82,7 +82,7 @@ public class JSONHelper {
 
     /**
      * Appends a JSONArray to an existing JSONArray.
-     * 
+     *
      * @param target The target JSONArray.
      * @param source The source JSONArray.
      * @return The target array contains at the end all entries of the source if both arguments are set, otherwise null will be returned.
@@ -103,7 +103,7 @@ public class JSONHelper {
      * to JSONArrays. This function creates a local JSONArray, that is filled with the values of the JSONArray, that is a parameter of it. A
      * second parameter is required to specify, which element shall be removed. Its value must be between 0 and the length of the JSONArray
      * - 1. The locally created JSONArray is returned from this function.
-     * 
+     *
      * @param arr The array, from which one element shall be removed
      * @param pos The position of the item, that shall be removed from the array. This value must be between 0 and arr.length - 1
      * @return If a problem occurred, the JSONArray will not be modified and returned. Otherwise a reduced version of the JSONArray is
@@ -133,7 +133,7 @@ public class JSONHelper {
 
     /**
      * Creates a new array which contains the entries from the source beginning with the index startIndex until the end of source.
-     * 
+     *
      * @param source The source JSONArray where entries should be
      * @param startIndex
      * @return
@@ -156,7 +156,7 @@ public class JSONHelper {
 
     /**
      * Creates a new array which contains a subset of entries specified by startIndex and endIndex.
-     * 
+     *
      * @param source
      * @param startIndex
      * @param endIndex
@@ -179,4 +179,31 @@ public class JSONHelper {
 
         return subArray;
     }
+
+    /**
+     * Creates a new array which contains a subset of entries specified by startIndex and length.
+     *
+     * @param source
+     * @param startIndex
+     * @param length
+     * @return
+     */
+    static public final JSONArray subArrayLength(final JSONArray source, int startIndex, int length) throws JSONException {
+        JSONArray subArray = null;
+
+        if ((null != source) && (startIndex >= 0) && (length > 0)) {
+            try {
+                int endIndex = Math.min(Math.max(0, (startIndex + length)), Math.max(0, source.length() - 1));
+                subArray = new JSONArray();
+
+                for (int i = startIndex; i < endIndex; i++) {
+                    subArray.put(source.get(i));
+                }
+            } catch (JSONException e) {
+                // nothing to do
+            }
+        }
+
+        return subArray;
+     }
 }
