@@ -64,10 +64,11 @@ import org.apache.commons.io.IOUtils;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-import com.openexchange.office.tools.Resource;
-import com.openexchange.office.tools.ResourceManager;
+import com.openexchange.office.tools.error.ErrorCode;
 import com.openexchange.office.tools.json.JSONDebugWriter;
 import com.openexchange.office.tools.json.JSONHelper;
+import com.openexchange.office.tools.resource.Resource;
+import com.openexchange.office.tools.resource.ResourceManager;
 import com.openexchange.tools.encoding.Base64;
 
 /**
@@ -77,8 +78,9 @@ import com.openexchange.tools.encoding.Base64;
  */
 public class OperationHelper {
 
-    final static private String m_actionsKey = "actions";
-    final static private String m_operationsKey = "operations";
+    final static private String KEY_ACTIONS = "actions";
+    final static private String KEY_OPERATIONS = "operations";
+    final static private String OP_NAME_NOOP = "noOp";
 
     /**
      * Comparing, if two positions in the form of JSONArrays have the same
@@ -164,8 +166,11 @@ public class OperationHelper {
         String nextText = next.getString("text");
         JSONArray nextStart = next.getJSONArray("start");
         JSONArray lastStart = last.getJSONArray("start");
+        // additional requirement is to check if operations have target, and if they are equal
+        String lastTarget = last.optString("target", "");
+        String nextTarget = next.optString("target", "");
 
-        if ((lastStart != null) && (nextStart != null) && (hasSameParentComponent(lastStart, nextStart, parentLevel))) {
+        if ((lastStart != null) && (nextStart != null) && (hasSameParentComponent(lastStart, nextStart, parentLevel) && lastTarget.equals(nextTarget))) {
 
             int lastTextPosStart = lastStart.getInt(lastStart.length() - 1);
             int nextTextPosStart = nextStart.getInt(nextStart.length() - 1);
@@ -241,6 +246,9 @@ public class OperationHelper {
         JSONArray lastStart = last.getJSONArray("start");
         // delete has an optional end parameter
         JSONArray nextEnd = null;
+        // additional requirement is to check if operations have target, and if they are equal
+        String lastTarget = last.optString("target", "");
+        String nextTarget = next.optString("target", "");
 
         try {
             nextEnd = next.getJSONArray("end");
@@ -255,7 +263,7 @@ public class OperationHelper {
         if ((lastStart != null) && (nextStart != null) && (hasSameParentComponent(lastStart, nextStart, parentLevel)) && (hasSameParentComponent(
             nextStart,
             nextEnd,
-            parentLevel))) {
+            parentLevel)) && (lastTarget.equals(nextTarget))) {
 
             int lastTextPosStart = lastStart.getInt(lastStart.length() - 1);
             int nextTextPosStart = nextStart.getInt(nextStart.length() - 1);
@@ -302,9 +310,9 @@ public class OperationHelper {
      */
     static private void optimizeOperationsArray(JSONObject jsonResult) {
 
-        if ((null != jsonResult) && (jsonResult.has(m_operationsKey))) {
+        if ((null != jsonResult) && (jsonResult.has(KEY_OPERATIONS))) {
             try {
-                JSONArray operations = jsonResult.getJSONArray(m_operationsKey);
+                JSONArray operations = jsonResult.getJSONArray(KEY_OPERATIONS);
                 JSONArray operationArr = new JSONArray();
 
                 if (!operations.isEmpty()) {
@@ -371,7 +379,7 @@ public class OperationHelper {
                         }
                     }
 
-                    jsonResult.put(m_operationsKey, operationArr);
+                    jsonResult.put(KEY_OPERATIONS, operationArr);
                 }
             } catch (JSONException e) {
                 LOG.error("RT connection: Exception caught while setting up JSON result within optimizeOperationsArray", e);
@@ -412,10 +420,10 @@ public class OperationHelper {
                     final String curKey = keys.next();
                     final Object curObject = jsonAppender.get(curKey);
 
-                    if (curKey.equals(m_actionsKey)) {
+                    if (curKey.equals(KEY_ACTIONS)) {
                         if (!keepActions) {
-                            if (!jsonResult.has(m_operationsKey)) {
-                                jsonResult.put(m_operationsKey, new JSONArray());
+                            if (!jsonResult.has(KEY_OPERATIONS)) {
+                                jsonResult.put(KEY_OPERATIONS, new JSONArray());
                             }
 
                             if (curObject instanceof JSONArray) {
@@ -425,18 +433,18 @@ public class OperationHelper {
                                     // [{"operations":[{"text":"a","start":[0,1],"name":"insertText"}]},{"operations":[{"text":"l","start":[0,2],"name":"insertText"}]}]
                                     JSONObject operationsObject = operationsGroups.getJSONObject(i);
 
-                                    if (operationsObject.has(m_operationsKey)) {
-                                        final JSONArray newOperations = operationsObject.getJSONArray(m_operationsKey);
+                                    if (operationsObject.has(KEY_OPERATIONS)) {
+                                        final JSONArray newOperations = operationsObject.getJSONArray(KEY_OPERATIONS);
 
                                         for (int j = 0; j < newOperations.length(); j++) {
-                                            jsonResult.append(m_operationsKey, newOperations.get(j));
+                                            jsonResult.append(KEY_OPERATIONS, newOperations.get(j));
                                         }
                                     } else {
-                                        jsonResult.append(m_operationsKey, operationsGroups.get(i));
+                                        jsonResult.append(KEY_OPERATIONS, operationsGroups.get(i));
                                     }
                                 }
                             } else {
-                                jsonResult.append(m_operationsKey, curObject);
+                                jsonResult.append(KEY_OPERATIONS, curObject);
                             }
                         } else {
                             appendJSONElement(jsonResult, curKey, curObject);
@@ -453,6 +461,88 @@ public class OperationHelper {
         return jsonResult;
     }
 
+    /**
+     * Appends operations of a message chunk to a JSONObject which represents
+     * the complete operations of the current modification session. The
+     * jsonResult will contain the combinations of both operations, while
+     * the function will try to remove superfluous no-op operations, which
+     * were inserted for synchronization purposes.
+     *
+     * @param jsonResult The json object which contains the operations of the
+     *                   current modification session. Can be be empty.
+     * @param senderOperations generated operations without osn/opl. Can be null.
+     * @param chunk      A message chunk containing operations or not. Must not be null.
+     * @param serverOSN  The current server OSN.
+     */
+    public static void appendPackedMessageChunk(final JSONObject jsonResult, final JSONArray senderOperations, final MessageChunk chunk, int serverOSN) {
+        final JSONObject chunkOpsObject = chunk.getOperations();
+
+        try {
+            final JSONArray actionsArray = chunkOpsObject.optJSONArray(MessagePropertyKey.KEY_ACTIONS);
+
+            if (null != actionsArray && actionsArray.length() > 0) {
+                int actions = actionsArray.length();
+
+                if (!jsonResult.has(KEY_OPERATIONS)) {
+                    jsonResult.put(KEY_OPERATIONS, new JSONArray());
+                } else {
+                    JSONArray oldOperations = jsonResult.getJSONArray(KEY_OPERATIONS);
+                    if(oldOperations.length() > 0){
+                        JSONObject lastOp = oldOperations.getJSONObject(oldOperations.length() - 1);
+                        lastOp.remove("opl");
+                        lastOp.remove("osn");
+                    }
+                }
+
+                if (null != senderOperations) {
+                    for( int sIdx = 0; sIdx < senderOperations.length(); ++sIdx) {
+                        final JSONObject opsObject = senderOperations.getJSONObject(sIdx);
+                        final String opName = opsObject.optString("name");
+
+                        if (!opName.equals(OP_NAME_NOOP)) {
+                            jsonResult.append(KEY_OPERATIONS, new JSONObject(opsObject));
+                        }
+                    }
+                }
+
+                for (int i=0; i < actions; i++) {
+                    final JSONObject actionsObject = actionsArray.getJSONObject(i);
+                    final JSONArray operationsArray = actionsObject.optJSONArray(KEY_OPERATIONS);
+
+                    if ((null != operationsArray) && (operationsArray.length() > 0)) {
+                        final int operations = operationsArray.length();
+
+                        for (int j=0; j < operations; j++) {
+                            final JSONObject opsObject = new JSONObject(operationsArray.getJSONObject(j));
+                            final String opName = opsObject.optString("name");
+                            opsObject.remove("opl");
+                            opsObject.remove("osn");
+
+                            if (!opName.equals(OP_NAME_NOOP)) {
+                                jsonResult.append(KEY_OPERATIONS, opsObject);
+                            }
+
+                        }
+                    }
+                }
+
+                JSONObject lastResultOpsObject = null;
+                final JSONArray operationsArray = jsonResult.optJSONArray(KEY_OPERATIONS);
+                if ((null != operationsArray) && (operationsArray.length() > 0)) {
+                    lastResultOpsObject = operationsArray.getJSONObject(operationsArray.length() - 1);
+                }
+
+                // modify last operation in the combined list to reflect the correct osn
+                if ((null != lastResultOpsObject)) {
+                    lastResultOpsObject.put("opl", 1);
+                    lastResultOpsObject.put("osn", serverOSN - 1);
+                }
+            }
+        } catch (final Exception e) {
+            LOG.warn("RT connection: appendPackedMessageChunk catches exception trying to append packed operations.", e);
+        }
+    }
+
     /**
      * @param jsonResult
      * @param key
@@ -460,19 +550,19 @@ public class OperationHelper {
      */
     static private void appendJSONElement(JSONObject jsonResult, String key, Object element) {
         try {
-            if (key.equals(m_operationsKey)) {
-                if (!jsonResult.has(m_operationsKey)) {
-                    jsonResult.put(m_operationsKey, new JSONArray());
+            if (key.equals(KEY_OPERATIONS)) {
+                if (!jsonResult.has(KEY_OPERATIONS)) {
+                    jsonResult.put(KEY_OPERATIONS, new JSONArray());
                 }
 
                 if (element instanceof JSONArray) {
                     final JSONArray jsonArray = (JSONArray) element;
 
                     for (int i = 0; i < jsonArray.length(); ++i) {
-                        jsonResult.append(m_operationsKey, jsonArray.get(i));
+                        jsonResult.append(KEY_OPERATIONS, jsonArray.get(i));
                     }
                 } else {
-                    jsonResult.append(m_operationsKey, element);
+                    jsonResult.append(KEY_OPERATIONS, element);
                 }
             } else {
                 jsonResult.put(key, element);
@@ -660,11 +750,11 @@ public class OperationHelper {
      *  The MessageChunk which should be checked.
      *
      * @return
-     *  TRUE if the operations inside the message chunk comply
-     *  to the requirements or FALSE if not.
+     *  The error code of the check. ErrorCode.NO_ERROR, if nothing
+     *  is wrong.
      */
-    public static boolean hasValidOperations(MessageChunk chunk, int serverOSN) {
-        boolean result = false;
+    public static ErrorCode hasValidOperations(MessageChunk chunk, int serverOSN) {
+        ErrorCode errorCode = ErrorCode.NO_ERROR;
         JSONObject chunkOpsObject = chunk.getOperations();
 
         try {
@@ -685,20 +775,18 @@ public class OperationHelper {
 
                             if (opsOSN != serverOSN) {
                                 // found inconsistent osn in chunk
-                                return false;
+                                return ErrorCode.HANGUP_INVALID_OSN_DETECTED_ERROR;
                             }
                             serverOSN = opsOSN + opsOPL;
                         }
                     }
                 }
             }
-
-            result = true;
         } catch (final Exception e) {
             LOG.warn("RT connection: hasValidOperations catches exception trying to retrieve operations from chunkList. ", e);
         }
 
-        return result;
+        return errorCode;
     }
 
     /**
