@@ -49,7 +49,7 @@
 
 package com.openexchange.office.tools.monitoring;
 
-import com.openexchange.office.tools.DocumentType;
+import com.openexchange.office.tools.doc.DocumentType;
 
 
 /**
