@@ -1,4 +1,8 @@
-/*oftware are protected by
+/*
+ *
+ *    OPEN-XCHANGE legal information
+ *
+ *    All intellectual property rights in the Software are protected by
  *    international copyright laws.
  *
  *
@@ -47,15 +51,19 @@ package com.openexchange.office.tools.monitoring;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.EnumMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.Timer;
 import java.util.TimerTask;
 import java.util.concurrent.atomic.AtomicLong;
+
 import org.apache.commons.logging.Log;
+
 import com.openexchange.log.LogFactory;
-import com.openexchange.office.tools.DocumentType;
+import com.openexchange.office.tools.doc.DocumentType;
 
 /**
  * {@link Statistics
@@ -133,7 +141,7 @@ public class Statistics {
             }
 
             // initialize periodic members
-            m_documentsSizes.put(documentType, new ArrayList<AtomicLong>());
+            m_documentsSizes.put(documentType, Collections.synchronizedList(new ArrayList<AtomicLong>()));
             m_documentsSizesResult.put(documentType, new AtomicLong());
         }
 
@@ -164,7 +172,7 @@ public class Statistics {
     /**
      * @return
      */
-    public synchronized long getDocumentsCreated(com.openexchange.office.tools.DocumentType documentType) {
+    public synchronized long getDocumentsCreated(com.openexchange.office.tools.doc.DocumentType documentType) {
         long ret = 0;
 
         if (null == documentType) {
@@ -226,7 +234,14 @@ public class Statistics {
      *
      */
     public synchronized long getCalcEngineMedianJobTime() {
-        return m_calcEngineJobTimesResult.get();
+        return m_calcEngineMedianJobTimeResult.get();
+    }
+
+    /**
+    *
+    */
+    public synchronized long getCalcEnginePeakJobTime() {
+        return m_calcEnginePeakJobTimeResult.get();
     }
 
     // - Implementation --------------------------------------------------------
@@ -341,7 +356,6 @@ public class Statistics {
 
             // the periodically called reset task
             m_resetTask = new TimerTask() {
-
                 /*
                  * (non-Javadoc)
                  * @see com.openexchange.documentconverter.impl.Statistics.ResetTask#run()
@@ -374,8 +388,13 @@ public class Statistics {
             m_documentsSizes.get(documentType).clear();
         }
 
+        // get median job time for the last measurement period
+        m_calcEngineMedianJobTimeResult.set(getMedianValue(m_calcEngineJobTimes));
 
-        m_calcEngineJobTimesResult.set(getMedianValue(m_calcEngineJobTimes));
+        // get peak job time for the last measurement period
+        m_calcEnginePeakJobTimeResult.set(getPeakValue(m_calcEngineJobTimes));
+
+        // clear list of times to be filled during the next period
         m_calcEngineJobTimes.clear();
     }
 
@@ -411,13 +430,17 @@ public class Statistics {
      * @return
      */
     protected long getDocumentsMedianSizeValue(DocumentType documentType) {
-        ArrayList<AtomicLong> workList = null;
+        List<AtomicLong> workList = null;
 
         if (null == documentType) {
             workList = new ArrayList<AtomicLong>();
 
             for (DocumentType curDocumentType : DocumentType.values()) {
-                workList.addAll(m_documentsSizes.get(curDocumentType));
+                final List<AtomicLong> curList = m_documentsSizes.get(curDocumentType);
+
+                synchronized (curList) {
+                    workList.addAll(curList);
+                }
             }
         } else {
             workList = m_documentsSizes.get(documentType);
@@ -430,30 +453,54 @@ public class Statistics {
      * @param workList
      * @return
      */
-    protected static long getMedianValue(ArrayList<AtomicLong> workList) {
+    protected static long getMedianValue(List<AtomicLong> workList) {
         double ret = 0.0;
 
         if (null != workList) {
-            final long[] longArray = new long[workList.size()];
+            synchronized (workList) {
+                final long[] longArray = new long[workList.size()];
 
-            if (longArray.length > 0) {
-                final int medianIndex = longArray.length >> 1;
-                int i = 0;
+                if (longArray.length > 0) {
+                    final int medianIndex = longArray.length >> 1;
+                    int i = 0;
 
-                for (AtomicLong curEntry : workList) {
-                    longArray[i++] = curEntry.get();
+                    for (AtomicLong curEntry : workList) {
+                        longArray[i++] = curEntry.get();
+                    }
+
+                    Arrays.sort(longArray);
+
+                    // calculate average of left and right values wrt. the median position for even size lists
+                    ret = (0 == (longArray.length % 2)) ? ((longArray[medianIndex - 1] + longArray[medianIndex]) * 0.5) : longArray[medianIndex];
                 }
-
-                Arrays.sort(longArray);
-
-                // calculate average of left and right values wrt. the median position for even size lists
-                ret = (0 == (longArray.length % 2)) ? ((longArray[medianIndex - 1] + longArray[medianIndex]) * 0.5) : longArray[medianIndex];
             }
         }
 
         return Math.round(ret);
     }
 
+    /**
+     * @param workList
+     * @return
+     */
+    protected static long getPeakValue(List<AtomicLong> workList) {
+        long ret = 0;
+
+        if (null != workList) {
+            synchronized (workList) {
+                for (AtomicLong curEntry : workList) {
+                    final long curValue = curEntry.get();
+
+                    if (curValue > ret) {
+                        ret = curValue;
+                    }
+                }
+            }
+        }
+
+        return ret;
+    }
+
 
     // - Members ---------------------------------------------------------------
 
@@ -481,7 +528,7 @@ public class Statistics {
     private Map<DocumentType, Map<? extends OpType, AtomicLong>> m_documentsOperations = new EnumMap<DocumentType, Map<? extends OpType, AtomicLong>>(
         DocumentType.class);
 
-    private Map<DocumentType, ArrayList<AtomicLong>> m_documentsSizes = new EnumMap<DocumentType, ArrayList<AtomicLong>>(DocumentType.class);
+    private Map<DocumentType, List<AtomicLong>> m_documentsSizes = new EnumMap<DocumentType, List<AtomicLong>>(DocumentType.class);
 
     private Map<DocumentType, AtomicLong> m_documentsSizesResult = new EnumMap<DocumentType, AtomicLong>(DocumentType.class);
 
@@ -489,7 +536,9 @@ public class Statistics {
 
     private Map<CalcEngineEventType, AtomicLong> m_calcEngineEvents = new EnumMap<CalcEngineEventType, AtomicLong>(CalcEngineEventType.class);
 
-    private ArrayList<AtomicLong> m_calcEngineJobTimes = new ArrayList<AtomicLong>();
+    private List<AtomicLong> m_calcEngineJobTimes = Collections.synchronizedList(new ArrayList<AtomicLong>());
 
-    private AtomicLong m_calcEngineJobTimesResult = new AtomicLong();
+    private AtomicLong m_calcEngineMedianJobTimeResult = new AtomicLong();
+
+    private AtomicLong m_calcEnginePeakJobTimeResult = new AtomicLong();
 }
