@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2016 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -58,7 +58,7 @@ import java.util.HashMap;
  */
 public class DocumentProperties extends HashMap<String, Object> {
 
-	// TYPE String
+    // TYPE String
     static final public String PROP_CREATOR = "Creator";
 
     // TYPE String
@@ -84,33 +84,46 @@ public class DocumentProperties extends HashMap<String, Object> {
 
     static final public String PROP_PREVIEWDATA = "PreviewData";
 
+    static final public String PROP_SPREADHSEET_ACTIVE_SHEET_INDEX = "ActiveSheetIndex";
+
+    static final public String PROP_SPREADSHEET_DOCUMENT = "SpreadsheetDocument";
+
+    static final public String PROP_SPREADSHEET_SHEET_COUNT = "SheetCount";
+
     // TYPE Long
     static final public String PROP_MAX_MEMORY_USAGE = "MaxMemoryUsage";
 
-    // TYPE Long
-    static final public String PROP_MIN_FREE_HEAPSPACE = "MinFreeHeapSpace";
+    static final public String PROP_MEMORYOBSERVER = "MemoryObserver";
 
     public String optString(String key, String _default) {
         final Object val = get(key);
-        if(val instanceof String) {
-            return (String)val;
+        if (val instanceof String) {
+            return (String) val;
         }
         return _default;
     }
 
     public boolean optBoolean(String key, boolean _default) {
-    	final Object val = get(key);
-    	if(val instanceof Boolean) {
-    		return ((Boolean)val).booleanValue();
-    	}
-    	return _default;
-	}
+        final Object val = get(key);
+        if (val instanceof Boolean) {
+            return ((Boolean) val).booleanValue();
+        }
+        return _default;
+    }
+
+    public Integer optInteger(String key, int _default) {
+        final Object val = get(key);
+        if (val instanceof Integer) {
+            return (Integer) val;
+        }
+        return _default;
+    }
 
     public long optLong(String key, long _default) {
-    	final Object val = get(key);
-    	if(val instanceof Long) {
-    		return ((Long)val).longValue();
-    	}
-    	return _default;
+        final Object val = get(key);
+        if (val instanceof Long) {
+            return ((Long) val).longValue();
+        }
+        return _default;
     }
 }
