@@ -105,6 +105,7 @@ public class SpellChecker implements ISpellChecker {
     @Override
     public JSONArray checkSpelling(String text, String locale, int offset) {
         final String separatorPattern = "[\t\n\u00a0\f\r \\.,;\\?!\":(){}\\[\\] ]";
+        final String theDot = ".";
         text = text + ' ';
         JSONArray result = new JSONArray();
         Hunspell.Dictionary dict = dictionaries != null ? dictionaries.getDictionary( locale ) : null;
@@ -115,7 +116,9 @@ public class SpellChecker implements ISpellChecker {
                 String substring = text.substring(pos, pos + 1);
                 if( Pattern.matches(separatorPattern, substring) ){
                     if( len > 0 ){
-                        String word = text.substring(startPos, startPos + len);
+                    	// #40631# A dot might also be used for abbreviations.
+                    	int extra = substring.equals(theDot) ? 1 : 0;
+                    	String word = text.substring(startPos, startPos + len + extra);
                         if(dict.misspelled(word)){
                             JSONObject error = new JSONObject();
                             try {
